// Imports
const log = require("../util/Log");
const Ami = require("../ami");
const { AgenteShow } = require("../ws/handlers");

exports.register = (server, options, next) => {

    //#region Constants
    const pausedAgents = [];
    const MOTIVO_AUTO_PAUSA = "AutoPausa";

    const StatusAgente = {
        desconhecido: 0,
        disponivel: 1,
        ocupado1: 2,
        ocupado2: 3,
        invalido: 4,
        indisponivel: 5,
        tocando: 6,
        ring_in_use: 7,
        aguardando: 8,
        pausa: 10
    };
    //#endregion

    //#region Event listeners
    /**
     * @listens AgentConnect|BlindTransfer|AttendedTransfer|NewState
     *
     * @param {string} evento Descreve o evento recebido
     * @param {any} ligacao Objeto contendo informações do evento
     */
    function onReceiveCall(evento, ligacao) {
        log.debug(`[AUTO-PAUSA] Evento '${evento}' recebido!`);
        // console.log(`Ligação -> `, ligacao);

        // Pega o ramal do agente
        let channel = ligacao.destchannel || ligacao.channel || ligacao.transfereechannel;
        let ramal = channel.split('-')[0];

        // Busca os dados do agente pelo ramal
        let agente = Ami.getAgenteLogadoByRamal(ramal).find(x => x);

        // Verifica se o agente possui auto-pausa ativa
        if (agente && agente.data && agente.data.tempo_auto_pausa) {

            // Define os status inválidos
            let statusInvalidos = [
                StatusAgente.ocupado1,
                StatusAgente.ocupado2,
                StatusAgente.invalido,
                StatusAgente.indisponivel,
                StatusAgente.tocando,
                StatusAgente.ring_in_use,
                StatusAgente.aguardando,
                StatusAgente.pausa
            ];

            // Verifica se o agente está em pausa
            if (statusInvalidos.indexOf(agente.status)) return log.debug(`[AUTO-PAUSA] Ignorando o agente '${agente.nome}'... Já está em pausa!`);

            // Pausa o agente
            pauseAgent(ramal, MOTIVO_AUTO_PAUSA);

            // Adiciona o agente para a listagem, assim esperando o evento de hangup para iniciar a contagem de despausa
            pausedAgents.push(agente);
            log.debug(`[AUTO-PAUSA] Agente '${agente.nome}' iniciou o estado de auto-pausa!`);
        } else {
            log.debug(`[AUTO-PAUSA] Ignorando o ramal '${ramal}'... Não possui auto-pausa definida!`);
        }
    }

    /**
     * @listens Hangup
     *
     * @param {string} evento Descreve o evento recebido
     * @param {any} ligacao Objeto contendo informações do evento
     */
    function onReceiveHangup(evento, ligacao) {
        log.debug(`[AUTO-PAUSA] Evento '${evento}' recebido!`);
        // console.log(`Ligação -> `, ligacao);

        // Pega o ramal do agente
        let channel = ligacao.destchannel || ligacao.channel || ligacao.transfereechannel;
        let ramal = channel.split('-')[0];

        // Pega o agente pelo ramal
        let agente = pausedAgents.find(x => x.ramal === ramal);

        // Verifica se o agente foi encontrado
        if (agente && agente.data && agente.data.tempo_auto_pausa) {

            // Remove o agente da listagem
            pausedAgents.splice(pausedAgents.indexOf(agente), 1);

            // Calcula, em minutos, o tempo para permanecer em auto pausa
            let tempoPausa = parseInt(agente.data.tempo_auto_pausa) * 1000;

            // Inicia o timer para despausar o agente
            setTimeout(scheduleAutomaticUnpause.bind(this, agente), tempoPausa);

            log.debug(`[AUTO-PAUSA] Ignorando o ramal '${ramal}'... Iniciou o processo de finalização da auto-pausa!`);
        } else {
            log.debug(`[AUTO-PAUSA] Ignorando o ramal '${ramal}'... Não iniciou o processo de auto-pausa!`);
        }
    }
    //#endregion

    //#region Functions
    function pauseAgent(ramal, motivo) {
        return new Promise((resolve, reject) => {
            // Busca as filas do agente
            let filas = Ami.getFilasLogadasDoAgente(ramal);

            // Coloca o ramal em pausa em todas as filas
            filas.forEach(f => {
                let options = { ramal, fila: f.nome, motivo };

                // Envia o comando para o AMI
                Ami.QueuePause(options, e => {
                    if (e.response === "Error") reject(e.message);
                    else resolve(e.message);
                });
            });
        });
    }

    function unpauseAgent(ramal, motivo="") {
        return new Promise((resolve, reject) => {
            // Busca as filas do agente
            let filas = Ami.getFilasLogadasDoAgente(ramal);

            // Coloca o ramal em pausa em todas as filas
            filas.forEach(f => {
                let options = { ramal, fila: f.nome, motivo };

                // Envia o comando para o AMI
                Ami.QueueUnpause(options, e => {
                    if (e.response === "Error") reject(e.message);
                    else resolve(e.message);
                });
            });
        });
    }

    function scheduleAutomaticUnpause(agente) {
        try {
            log.debug(`[AUTO-PAUSA] Agente '${agente.nome}' saiu do estado de auto-pausa!`);

            // Verifica se o agente está pausado, se estiver, ignora
            if (agente.status !== StatusAgente.pausa || agente.pausa_motivo !== MOTIVO_AUTO_PAUSA) {
                return log.debug(`[AUTO-PAUSA] Ignorando o agente '${agente.nome}' porque se encontra disponível.`);
            }

            // Pausa o agente
            unpauseAgent(agente.ramal);

        } catch (error) {
            console.trace(error);
            log.error(`[AUTO-PAUSA] Erro ao despausar agente '${agente.ramal}'!\n${error}`);
        }
    }

    function registerEventListeners() {
        // Define os eventos a escutar
        Ami.XCeventos.on("Atendeu", onReceiveCall.bind(this, "AgentConnect"));
        Ami.XCeventos.on("BlindTransfer", onReceiveCall.bind(this, "BlindTransfer"));
        Ami.XCeventos.on("AttendedTransfer", onReceiveCall.bind(this, "AttendedTransfer"));
        Ami.XCeventos.on("Ligacao", onReceiveCall.bind(this, "NewState"));
        Ami.XCeventos.on("Desliga", onReceiveHangup.bind(this, "Hangup"));
    }
    //#endregion

    //#region Execution
    registerEventListeners();
    next();
    //#endregion

};

exports.register.attributes = {
    name: 'xcontact-autopausa'
};