"use strict";

//#region Imports
const SimpleNodeLogger = require('simple-node-logger');
const moment = require('moment-timezone');
const db = require('../util/db');
const ws = require('../ws');
const Config = require('../config.json');
//#endregion

//#region Simple node logger setup
const log = SimpleNodeLogger.createSimpleLogger({
    logFilePath: '/var/log/xcontact.log',
    timestampFormat: 'DD-MM-YYYY HH:mm:ss.SSS'
});
log.setLevel(Config.log || 'error');
//#endregion

class DepuradorDiscador {

    constructor () {
        this.campanhas = {};
        this.clientes = {};
        this.ativado = false;

        log.info(`[DEPURADOR_DISCADOR] ${this.ativado ? "ATIVADO!" : "DESATIVADO!"}`);
    }

    //#region Utility
    getTipoCampanha(campanha) {
        switch (campanha.tipo_campanha) {
            case 0:
                return "Voz";
            case 1:
                return "Whatsapp";
            default:
                return "Desconhecido";
        }
    }

    criarPlaceholderCampanha(campanha) {
        return {
            rodando: true,
            turnoValido: true,
            agentes: null,
            status_descricao: campanha && campanha.status === "F" ? "Finalizada" : "Processando...",
            registros: {
                total: 0,
                falhas: 0
            },
            contatos: {
                total: 0,
                numeros: 0,
                discados: 0
            }
        };
    }

    compararArrays(a, b) {
        try {
            if (!a) a = [];
            if (!b) b = [];

            if (a.length != b.length || !a.every((value, i) => value === b[i])) {
                return false;
            } else {
                return true;
            }
        } catch (e) {
            console.error(e);
            console.trace(e);
            log.error("[DEPURADOR_DISCADOR] Erro ao verificarConteudoArrays -> " + e);
        }

        return false;
    }
    //#endregion

    async notificarProcessamento(campanha) {
        // Verifica se o depurador se encontra ativado
        if (!this.ativado) return;

        // Verifica se a campanha se enconta em depuração
        if (!this.campanhas.hasOwnProperty(campanha.id)) return;

        let object = Object.assign({}, campanha);
        object.rodando = true;
        object.registros = await this.buscarRegistrosDaCampanha(object);
        object.contatos = await this.buscarContatosDaCampanha(object);

        this.atualizarCampanha(object.id, object);
    }

    buscarRegistrosDaCampanha(campanha) {
        return new Promise(async (resolve, reject) => {
            try {
                // Verifica se o depurador se encontra ativado
                if (!this.ativado) return;

                // Verifica se a campanha se enconta em depuração
                if (!this.campanhas.hasOwnProperty(campanha.id)) return;

                let result = await db.knex.select("campanhas_registro.falha", "campanhas_registro.data_status")
                    .from("campanhas_registro")
                    .leftJoin("campanhas", "campanhas_registro.id_campanha", "campanhas.id")
                    .where("id_campanha", "=", campanha.id);

                if (result) {
                    // Se a campanha estiver ativa, filtra os registros para apenas os que ocorreram após a última mudança de status da campanha
                    if (campanha.status === "A") {
                        let dataStatusCampanha = moment(campanha.data_status);
                        result = result.filter(x => moment(x.data_status).isSameOrAfter(dataStatusCampanha));
                    }

                    // Pega as somas e retorna
                    resolve({
                        total: result.length || 0,
                        falhas: result.filter(x => x.falha || x.falha === 'TRUE').length || 0
                    });
                } else {
                    // Retorna dados zerados
                    resolve({
                        total: 0,
                        falhas: 0
                    });
                }
            } catch (e) {
                log.error(`[DEPURADOR_DISCADOR] Erro ao buscar registros da campanha ${campanha.id}. ` + e);
                reject(e);
            }
        });
    }

    buscarContatosDaCampanha(campanha) {
        return new Promise(async (resolve, reject) => {
            try {
                // Verifica se o depurador se encontra ativado
                if (!this.ativado) return;

                // Verifica se a campanha se enconta em depuração
                if (!this.campanhas.hasOwnProperty(campanha.id)) return;

                // Define as sub-queries a serem executadas
                let queryTotalContatos = /* sql */`select co.id_campanha, count(*) as total_contatos from campanhas_contato co group by co.id_campanha`;
                let queryTotalNumeros = /* sql */`select aa.id, count(*) as total_numeros, sum(case when b.sucesso is true then aa.tentativas::integer when b.retentativas >= aa.tentativas then aa.tentativas::integer else b.tentativas end) as total_discados
                                                    from campanhas_contato_numeros b
                                                    join campanhas aa on aa.id = b.id_campanha
                                                    where aa.id = b.id_campanha
                                                    group by aa.id`;

                // Define a query principal que vai ser executada
                let query = /* sql */`select c.total_numeros, c.total_discados, contatos.total_contatos from campanhas a
                                            left join (${queryTotalContatos}) contatos on contatos.id_campanha = a.id
                                            left join (${queryTotalNumeros}) c on a.id = c.id
                                            where a.id=${campanha.id}`;

                // Executa a query
                let result = await db.query(query);

                // Valida o resultado
                if (result && result.rows && result.rows.length > 0) {
                    // Pega a primeira linha válida proveniente da query
                    let row = result.rows.find(x => x);
                    // Retorna dados zerados
                    resolve({
                        total: row.total_contatos || 0,
                        numeros: row.total_numeros || 0,
                        discados: row.total_discados || 0
                    });
                } else {
                    // Retorna dados zerados
                    resolve({
                        total: 0,
                        numeros: 0,
                        discados: 0
                    });
                }
            } catch (e) {
                log.error(`[DEPURADOR_DISCADOR] Erro ao buscar registros da campanha ${campanha.id}. ` + e);
                reject(e);
            }
        });
    }

    registrarLigacaoCompletada(campanha, ligacao, motivo) {
        // Verifica se o depurador se encontra ativado
        if (!this.ativado) return;

        // Verifica se a campanha se enconta em depuração
        if (!this.campanhas.hasOwnProperty(campanha.id)) return;

        log.info("DISCADOR: LIGACAO COMPLETADA -----------------------");
        if (this.campanhas[campanha.id]) {
            // Puxa o array de ligações completadas da campanha salva
            let ligacoes_completadas = this.campanhas[campanha.id].ligacoes_completadas || [];

            // Adiciona ligação especificada
            ligacoes_completadas.push(Object.assign({ motivo }, ligacao));

            // Atualiza o objeto da campanha
            this.campanhas[campanha.id].ligacoes_completadas = ligacoes_completadas;
            let changes = this.campanhas[campanha.id].changes || [];
            changes.push("ligacoes_completadas");
            changes.push("ligacoes_ativas");
            this.campanhas[campanha.id].changes = changes;

            // Envia as atualizações
            this.emitirAtualizacoes(campanha.id);
        }
    }

    removerCampanha(id) {
        // Verifica se o depurador se encontra ativado
        if (!this.ativado) return;

        if (this.campanhas[id]) {
            log.info(`[DEPURADOR_DISCADOR] Removendo a campanha ${id}...`);
            delete this.campanhas[id];
        }
    }

    adicionarCampanha(campanha) {
        try {
            // Verifica se o depurador se encontra ativado
            if (!this.ativado) return;

            // Verifica se a campanha já existe
            if (this.campanhas[campanha.id]) {
                // log.info(`[DEPURADOR_DISCADOR] Campanha ${campanha.id} já existente! Alterando valores...`);

                // Atualiza as propriedades informadas
                for (let property in campanha) {
                    if (!campanha.hasOwnProperty(property)) continue;

                    // Altera o valor da propriedade
                    this.campanhas[campanha.id][property] = campanha[property];
                    // Notifica como alterada
                    this.campanhas[campanha.id].changes.push(property);
                }

                // Aproveitando que a campanha já foi definida, emite as atualizações feitas acima
                this.emitirAtualizacoes(campanha.id);
            } else {
                log.info(`[DEPURADOR_DISCADOR] Campanha ${campanha.id} adicionada!`);

                // Guarda o objeto inteiro
                this.campanhas[campanha.id] = Object.assign(this.criarPlaceholderCampanha(campanha), campanha);
                this.campanhas[campanha.id].changes = [];
            }
        } catch (error) {
            log.error(`[DEPURADOR_DISCADOR] Erro ao adicionar campanha ${campanha.id}!\n` + error);
            console.trace(error);
        }
    }

    atualizarCampanha(id, propriedades) {
        // Verifica se o depurador se encontra ativado
        if (!this.ativado) return;

        // Verifica se a campanha se enconta em depuração
        if (!this.campanhas.hasOwnProperty(id)) return;

        if (this.campanhas.hasOwnProperty(id)) {
            // log.info(`[DEPURADOR_DISCADOR] Campanha ${JSON.stringify(id)} já existente! Alterando valores...`);

            // Atualiza as propriedades informadas
            for (let property in propriedades) {
                if (!propriedades.hasOwnProperty(property)) continue;

                // Verifica se os valores diferem
                if (Array.isArray(this.campanhas[id][property]) || Array.isArray(propriedades[property])) {
                    // Se os valores são arrays, o procedimento para comparação de dados é um pouco diferente
                    // Devemos verificar cada elemento do array
                    if (!this.compararArrays(this.campanhas[id][property], propriedades[property])) {
                        // Altera o valor da propriedade
                        this.campanhas[id][property] = propriedades[property];
                        // Notifica como alterada
                        this.campanhas[id].changes.push(property);
                    }
                } else if (this.campanhas[id][property] != propriedades[property]) {
                    // Altera o valor da propriedade
                    this.campanhas[id][property] = propriedades[property];
                    // Notifica como alterada
                    this.campanhas[id].changes.push(property);
                }
            }

            // Aproveitando que a campanha já foi definida, emite as atualizações feitas acima
            this.emitirAtualizacoes(id);
        } else {
            log.info(`[DEPURADOR_DISCADOR] Campanha ${id} adicionada!`);

            // Guarda o objeto inteiro
            this.campanhas[id] = Object.assign(this.criarPlaceholderCampanha(null), propriedades);
            this.campanhas[id].changes = [];
        }
    }

    resetar() {
        this.campanhas = {};
    }

    emitirEvento(sala, evento, payload) {
        try {
            if (this.clientes && this.clientes.hasOwnProperty(sala) && this.clientes[sala]) {
                (this.clientes[sala] || []).forEach(x => {
                    x.emit(evento, payload);
                });
            }
        } catch (error) {
            log.error(`[DEPURADOR_DISCADOR] Erro ao emitir eventos para os sockets conectados!\n${error}`);
            console.trace(error);
        }
    }

    /**
     * Este método emite via WS todas as alterações feitas na campanha (Provida pelo ID {@link id})
     * @param {Any} id Id da campanha
     **/
    emitirAtualizacoes(id) {
        // Verifica se o depurador se encontra ativado
        if (!this.ativado) return;

        // Verifica se a campanha se enconta em depuração
        if (!this.campanhas.hasOwnProperty(id)) return;

        // Verifica se a campanha existe
        if (this.campanhas[id]) {
            // Verifica se existem alterações ainda não sincronizadas
            if (this.campanhas[id].changes && this.campanhas[id].changes.length > 0) {
                // Cria um 'buffer' para guardar todos os valores que foram alterados
                let changeBuffer = {};
                for (let property of this.campanhas[id].changes) {
                    changeBuffer[property] = this.campanhas[id][property];
                }
                log.info("[DEPURADOR_DISCADOR] Changes " + this.campanhas[id].changes);
                // Reseta o array de alterações, porque todas as mudanças foram sincronizadas
                this.campanhas[id].changes = [];

                // Monta um objeto com todas as informações do evento
                let object = {
                    event: "StatusDiscador",
                    tipo: this.getTipoCampanha(this.campanhas[id]),
                    id: id,
                    alteracoes: changeBuffer
                };

                // Emite via socket
                this.emitirAtualizacoes(id, "message", JSON.stringify(object));
                // ws.broadcastEvent("message", JSON.stringify(object));
            } else {
                log.info("[DEPURADOR_DISCADOR] Changes up-to-date");
            }
        } else {
            log.warn("[DEPURADOR_DISCADOR] Não foi possível emitir atualizações porque a campanha não foi definida! campanha_id:" + id);
        }
    }

    /**
     * Este método emite via WS todo o objeto da campanha (Provida pelo ID {@link id})
     * @param {Any} id Id da campanha
     **/
    emitirCampanha(id) {
        // Verifica se o depurador se encontra ativado
        if (!this.ativado) return;

        // Verifica se a campanha se enconta em depuração
        if (!this.campanhas.hasOwnProperty(id)) return;

        // Verifica se a campanha existe
        if (this.campanhas[id]) {
            // Reseta o array de alterações, porque todas as mudanças foram sincronizadas
            this.campanhas[id].changes = [];

            // Monta um objeto com todas as informações do evento
            let object = {
                event: "StatusDiscador",
                tipo: this.getTipoCampanha(this.campanhas[id]),
                alteracoes: this.campanhas[id],
                id
            };

            // Emite via socket
            this.emitirAtualizacoes(id, "message", JSON.stringify(object));
            // ws.broadcastEvent("message", JSON.stringify(object));
        } else {
            log.warn("[DEPURADOR_DISCADOR] Não foi possível emitir atualizações porque a campanha não foi definida! campanha_id:" + id);
        }
    }

    iniciarDepuracao(socket, campanhaId) {
        // Verifica se o depurador se encontra ativado
        if (!this.ativado) return;

        // Coloca o socket na room da campanha, para receber todos os eventos da mesma
        socket.join(`depurador-${campanhaId}`);

        // Adiciona no array de clientes conectados
        if (!this.clientes.hasOwnProperty(campanhaId)) this.clientes[campanhaId] = [];
        if (this.clientes[campanhaId].indexOf(socket) === -1) this.clientes[campanhaId].push(socket);

        // Verifica se a campanha já está sendo depurada
        if (!this.campanhas.hasOwnProperty(campanhaId)) {
            // Adiciona a campanha como zerada
            this.adicionarCampanha({ id: campanhaId });
        } else {
            log.info(`[DEPURADOR_DISCADOR] Cliente iniciou depuração na campanha ${campanhaId}...`);
        }
    }

    finalizarDepuracao(socket, campanhaId) {
        // Verifica se o depurador se encontra ativado
        if (!this.ativado) return;

        // Remove o socket da room da campanha, assim parando de receber eventos da mesma
        socket.leave(`depurador-${campanhaId}`);

        // Remove do array de clientes conectados
        if (!this.clientes.hasOwnProperty(campanhaId) || !this.clientes[campanhaId]) this.clientes[campanhaId] = [];

        let index = this.clientes[campanhaId].indexOf(socket);
        if (index !== -1) this.clientes[campanhaId].splice(index, 1);

        // Verifica se nenhum cliente está escutando os eventos da campanha
        if (this.clientes[campanhaId].length <= 0) {
            // Remove a campanha do depurador
            this.removerCampanha(campanhaId);
        } else {
            log.info(`[DEPURADOR_DISCADOR] Cliente finalizou depuração na campanha ${campanhaId}...`);
        }
    }

}

module.exports = new DepuradorDiscador();