const db = require('../util/db');
const moment = require('moment-timezone');
const ConfigFile = require("../config.json");
const ws = require('../ws');
const Horarios = require("../models/Horarios");

const log = require('simple-node-logger').createSimpleLogger({
    logFilePath: '/var/log/xcontact.log',
    timestampFormat: 'DD-MM-YYYY HH:mm:ss.SSS'
});
const log_level = ConfigFile.log ? ConfigFile.log : 'error';
log.setLevel(log_level);

exports.verificarTurnos = function (campanha) {
    return new Promise(async (resolve, reject) => {
        if (campanha && campanha.turnos && campanha.turnos.length > 0) {
            let turnosValidos = await Horarios.verificarTurnos(campanha.turnos);
            resolve(turnosValidos);
        } else {
            log.info(`[DiscadorCommon] Ignorando verificação de turnos na campanha ${campanha.nome} - ${campanha.id}`);
            resolve(true);
        }
    });
};

exports.verificarAgendamento = function (campanha) {
    if (campanha && (campanha.data_ini || campanha.data_fim)) {
        let hoje = moment();

        // Verifica se a data inicial ainda não chegou
        if (campanha.data_ini) {
            let inicio = moment(campanha.data_ini).startOf("day");

            if (hoje.isBefore(inicio)) {
                log.error(`[DiscadorCommon] AGENDAMENTO - a data inicial na campanha ${campanha.nome} - ${campanha.id} ainda não chegou.`);
                return false
            };
        }

        // Verifica se a data final já passou
        if (campanha.data_fim) {
            let final = moment(campanha.data_fim).endOf("day");

            if (hoje.isAfter(final)) {
                log.error(`[DiscadorCommon] AGENDAMENTO - a data final na campanha ${campanha.nome} - ${campanha.id} já passou.`)
                return false
            };
        }
    }

    log.error(`[DiscadorCommon] AGENDAMENTO - O agendamento para a campanha ${campanha.nome} - ${campanha.id} está ok.`)
    return true;
};

exports.finalizarCampanha = function (campanha) {
    return new Promise((resolve, reject) => {
        // Altera o status da campanha para 'F' de 'Finalizada'
        campanha.status = "F";
        // Registra o timestamp de quando o status da campanha foi alterado
        campanha.data_status = moment.tz(ConfigFile.servidor.timezone).format('YYYY-MM-DD HH:mm:ss');

        ws.broadcast({ event: 'CampanhaFim', campanha: campanha });

        if (campanha.notificar_email && campanha.notificar_email.length > 0) {
            let transporter = nodemailer.createTransport({
                host: ConfigFile.email.host,
                port: ConfigFile.email.port,
                secure: ConfigFile.email.secure,
                auth: {
                    user: ConfigFile.email.user,
                    pass: ConfigFile.email.pass
                }
            });

            let mailOptions = {
                from: '"' + ConfigFile.email.from_name + '" <' + ConfigFile.email.from_email + '>',
                to: campanha.notificar_email,
                subject: 'Xcontact - A campanha ' + campanha.nome + ' foi finalizada',
                text: 'A campanha ' + campanha.nome + ' foi finalizada.',
            };

            transporter.sendMail(mailOptions, (error, info) => {
                if (error) {
                    log.error('NOTIFICA_EMAIL: Falha ao enviar email.');
                    log.error(error);
                } else {
                    log.info('Message sent: ' + info.messageId);
                }
            });
        }

        db.query("UPDATE campanhas set status='F',data_status = $2,descricao = 'Campanha finalizada pelo discador.' WHERE id = $1", [campanha.id, campanha.data_status])
            .then(x => {
                log.info('[DISCADOR-COMMON] FECHACAMPANHA: Campanha finalizada ID = ' + campanha.id);
                resolve();
            })
            .catch(error => {
                log.error('[DISCADOR-COMMON] FECHACAMPANHA: Falha finalizar campanha.');
                log.error(e);
                reject(e);
            });
    });
}

exports.notificarConclusaoDaCampanha = function (registro) {
    return new Promise((resolve, reject) => {
        // Fecha Numero
        db.query("UPDATE campanhas_contato_numeros SET sucesso = true WHERE id_contato = $1", [registro.ContatoID])
            .then(res => {
                log.info('[DISCADOR-COMMON] REGISTRACOMPLETADA: Numero Finalizado para contatoID = ' + registro.ContatoID);
                resolve(res);
            })
            .catch(e => {
                log.error('[DISCADOR-COMMON] REGISTRACOMPLETADA: Falha ao registrar chamada completada.');
                log.error(e);
                reject(e);
            });
    });
}

exports.notificarRetentativaDeContato = function (contatoId) {
    return new Promise((resolve, reject) => {
        db.query("UPDATE campanhas_contato_numeros SET tentativas = tentativas - 1, retentativas = retentativas + 1 WHERE id = $1", [contatoId])
            .then(res => {
                log.info('[DISCADOR-COMMON] DEC_TENTATIVA_INC_RETENTATIVA: Numero ID = ' + contatoId + ' teve tentatativas decrementada e retentativas aumentada.');
                resolve(res);
            })
            .catch(e => {
                log.error('[DISCADOR-COMMON] DEC_TENTATIVA_INC_RETENTATIVA: Falha ao incrementar retentativas.');
                log.error(e);
                reject(e);
            });
    });
}

exports.inserirRegistroDaCampanha = function (registro) {
    return new Promise((resolve, reject) => {
        const duracao = registro.falando ? registro.falando : '00:00:00';
        db.query("INSERT INTO campanhas_registro(id_campanha,id_contato,id_numero,numero,callid,actionid,falha,id_status,campos,duracao) VALUES($1,$2,$3,$4,$5,$6,$7,$8,$9,$10)", [registro.CampanhaID, registro.ContatoID, registro.NumeroID, registro.Destino, registro.Linkedid, registro.ActionID, registro.Falha, registro.Status, registro.Campos, duracao])
            .then(res => {
                log.info("[DISCADOR-COMMON] REGISTRALOG: Log da campanha ID = " + registro.CampanhaID + " registrado.");
                resolve(res);
            })
            .catch(e => {
                log.error('[DISCADOR-COMMON] REGISTRALOG: Falha ao registrar o log para camapanhaID = ' + registro.CampanhaID);
                log.error(e);
                reject(e);
            });
    });
}