define(["require", "exports", "./plan", "./context", "./request", "./target", "../bindings/binding_count", "./reflection_utils", "./metadata", "../constants/error_msgs", "../constants/metadata_keys", "../constants/literal_types", "../utils/serialization"], function (require, exports, plan_1, context_1, request_1, target_1, binding_count_1, reflection_utils_1, metadata_1, ERROR_MSGS, METADATA_KEY, literal_types_1, serialization_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    function getBindingDictionary(cntnr) {
        return cntnr._bindingDictionary;
    }
    exports.getBindingDictionary = getBindingDictionary;
    function _createTarget(isMultiInject, targetType, serviceIdentifier, name, key, value) {
        var metadataKey = isMultiInject ? METADATA_KEY.MULTI_INJECT_TAG : METADATA_KEY.INJECT_TAG;
        var injectMetadata = new metadata_1.Metadata(metadataKey, serviceIdentifier);
        var target = new target_1.Target(targetType, name, serviceIdentifier, injectMetadata);
        if (key !== undefined) {
            var tagMetadata = new metadata_1.Metadata(key, value);
            target.metadata.push(tagMetadata);
        }
        return target;
    }
    function _getActiveBindings(avoidConstraints, context, parentRequest, target) {
        var bindings = getBindings(context.container, target.serviceIdentifier);
        var activeBindings = [];
        if (avoidConstraints === false) {
            activeBindings = bindings.filter(function (binding) {
                var request = new request_1.Request(binding.serviceIdentifier, context, parentRequest, binding, target);
                return binding.constraint(request);
            });
        }
        else {
            activeBindings = bindings;
        }
        _validateActiveBindingCount(target.serviceIdentifier, activeBindings, target, context.container);
        return activeBindings;
    }
    function _validateActiveBindingCount(serviceIdentifier, bindings, target, container) {
        switch (bindings.length) {
            case binding_count_1.BindingCount.NoBindingsAvailable:
                if (target.isOptional() === true) {
                    return bindings;
                }
                else {
                    var serviceIdentifierString = serialization_1.getServiceIdentifierAsString(serviceIdentifier);
                    var msg = ERROR_MSGS.NOT_REGISTERED;
                    msg += serialization_1.listMetadataForTarget(serviceIdentifierString, target);
                    msg += serialization_1.listRegisteredBindingsForServiceIdentifier(container, serviceIdentifierString, getBindings);
                    throw new Error(msg);
                }
            case binding_count_1.BindingCount.OnlyOneBindingAvailable:
                if (target.isArray() === false) {
                    return bindings;
                }
            case binding_count_1.BindingCount.MultipleBindingsAvailable:
            default:
                if (target.isArray() === false) {
                    var serviceIdentifierString = serialization_1.getServiceIdentifierAsString(serviceIdentifier), msg = ERROR_MSGS.AMBIGUOUS_MATCH + " " + serviceIdentifierString;
                    msg += serialization_1.listRegisteredBindingsForServiceIdentifier(container, serviceIdentifierString, getBindings);
                    throw new Error(msg);
                }
                else {
                    return bindings;
                }
        }
    }
    function _createSubRequests(metadataReader, avoidConstraints, serviceIdentifier, context, parentRequest, target) {
        try {
            var activeBindings = void 0;
            var childRequest_1;
            if (parentRequest === null) {
                activeBindings = _getActiveBindings(avoidConstraints, context, null, target);
                childRequest_1 = new request_1.Request(serviceIdentifier, context, null, activeBindings, target);
                var thePlan = new plan_1.Plan(context, childRequest_1);
                context.addPlan(thePlan);
            }
            else {
                activeBindings = _getActiveBindings(avoidConstraints, context, parentRequest, target);
                childRequest_1 = parentRequest.addChildRequest(target.serviceIdentifier, activeBindings, target);
            }
            activeBindings.forEach(function (binding) {
                var subChildRequest = null;
                if (target.isArray()) {
                    subChildRequest = childRequest_1.addChildRequest(binding.serviceIdentifier, binding, target);
                }
                else {
                    subChildRequest = childRequest_1;
                }
                if (binding.type === literal_types_1.BindingTypeEnum.Instance && binding.implementationType !== null) {
                    var dependencies = reflection_utils_1.getDependencies(metadataReader, binding.implementationType);
                    dependencies.forEach(function (dependency) {
                        _createSubRequests(metadataReader, false, dependency.serviceIdentifier, context, subChildRequest, dependency);
                    });
                }
            });
        }
        catch (error) {
            if (error instanceof RangeError && parentRequest !== null) {
                serialization_1.circularDependencyToException(parentRequest.parentContext.plan.rootRequest);
            }
            else {
                throw new Error(error.message);
            }
        }
    }
    function getBindings(container, serviceIdentifier) {
        var bindings = [];
        var bindingDictionary = getBindingDictionary(container);
        if (bindingDictionary.hasKey(serviceIdentifier)) {
            bindings = bindingDictionary.get(serviceIdentifier);
        }
        else if (container.parent !== null) {
            bindings = getBindings(container.parent, serviceIdentifier);
        }
        return bindings;
    }
    function plan(metadataReader, container, isMultiInject, targetType, serviceIdentifier, key, value, avoidConstraints) {
        if (avoidConstraints === void 0) { avoidConstraints = false; }
        var context = new context_1.Context(container);
        var target = _createTarget(isMultiInject, targetType, serviceIdentifier, "", key, value);
        _createSubRequests(metadataReader, avoidConstraints, serviceIdentifier, context, null, target);
        return context;
    }
    exports.plan = plan;
    function createMockRequest(container, serviceIdentifier, key, value) {
        var target = new target_1.Target(literal_types_1.TargetTypeEnum.Variable, "", serviceIdentifier, new metadata_1.Metadata(key, value));
        var context = new context_1.Context(container);
        var request = new request_1.Request(serviceIdentifier, context, null, [], target);
        return request;
    }
    exports.createMockRequest = createMockRequest;
});
