"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const events_1 = require("events");
const fs_1 = require("fs");
const add_seconds_1 = __importDefault(require("date-fns/add_seconds"));
class Client extends events_1.EventEmitter {
    constructor(socket, { filepath, maxWait = 60, data, highWaterMark, withStats = false }, eventNamespace = 'akuma') {
        super();
        this.socket = socket;
        this.eventNamespace = eventNamespace;
        this.filesize = 0;
        this.chunks = 0;
        this.id = null;
        this.bytesPerChunk = 102400; //100kb
        this.isPaused = false;
        this.event = '';
        this.isResume = true;
        this.isFirst = true;
        this.maxWaitCounter = 0;
        this.maxWaitTimer = null;
        this.filepath = filepath;
        this.data = data;
        this.bytesPerChunk = highWaterMark || this.bytesPerChunk;
        this.withStats = withStats;
        this.maxWait = maxWait;
    }
    __getId() {
        this.socket.emit(`__${this.eventNamespace}_::new::id__`, (id) => {
            if (this.id)
                return;
            this.id = id;
            this.emit('ready');
        });
    }
    __read(start, end) {
        if (this.isPaused)
            return;
        if (this.filesize <= this.bytesPerChunk) {
            let chunk = fs_1.readFileSync(this.filepath);
            this.socket.emit(`__${this.eventNamespace}_::data::${this.id}__`, {
                chunk,
                info: {
                    size: this.filesize,
                    data: this.data
                },
                event: this.event
            });
            this.emit('progress', { size: this.filesize, total: chunk.length });
            this.__maxWaitMonitor();
            return;
        }
        const stream = fs_1.createReadStream(this.filepath, {
            highWaterMark: this.bytesPerChunk,
            start,
            end
        });
        stream.read(end - start);
        //
        stream.once('data', (chunk) => {
            // avoid resending extra infos after first upload
            if (this.isFirst || this.isResume) {
                this.socket.emit(`__${this.eventNamespace}_::data::${this.id}__`, {
                    chunk,
                    info: {
                        size: this.filesize,
                        data: this.data
                    },
                    event: this.event
                });
            }
            else {
                this.socket.emit(`__${this.eventNamespace}_::data::${this.id}__`, { chunk });
            }
            stream.close();
            this.emit('progress', { size: this.filesize, total: this.chunks });
            this.isFirst = false;
            this.isResume = false;
            this.__maxWaitMonitor();
        });
    }
    __maxWaitMonitor() {
        if (this.maxWaitTimer)
            clearInterval(this.maxWaitTimer);
        this.maxWaitTimer = setInterval(() => {
            this.maxWaitCounter += 1;
            if (this.isPaused) {
                this.__clearMaxWaitMonitor();
                return;
            }
            if (this.maxWaitCounter >= this.maxWait) {
                this.socket.emit(`__${this.eventNamespace}_::stop::__`, this.id);
                this.emit('cancel', '[local] Response timeout id=' + this.id);
                this.__destroy();
            }
        }, 1000);
    }
    __clearMaxWaitMonitor() {
        if (this.maxWaitTimer) {
            clearInterval(this.maxWaitTimer);
            this.maxWaitTimer = null;
            this.maxWaitCounter = 0;
        }
    }
    __start(cb) {
        this.filesize = fs_1.statSync(this.filepath).size;
        this.__read(0, this.bytesPerChunk);
        // listen for new request
        this.socket
            .on(`__${this.eventNamespace}_::more::${this.id}__`, (chunks) => {
            if (!chunks)
                return;
            this.chunks = chunks;
            let toChunk = Math.min(this.bytesPerChunk, this.filesize - chunks);
            this.__clearMaxWaitMonitor();
            this.__read(chunks, toChunk + chunks);
        })
            // listen for resume request
            .on(`__${this.eventNamespace}_::resume::${this.id}__`, (chunks) => {
            this.isResume = true;
            this.__maxWaitMonitor();
            if (typeof chunks === 'number') {
                this.chunks = chunks;
                let toChunk = Math.min(this.bytesPerChunk, this.filesize - chunks);
                this.__read(chunks, toChunk + chunks);
            }
            else
                this.__read(0, this.bytesPerChunk);
        })
            // listen for end event
            .on(`__${this.eventNamespace}_::end::${this.id}__`, ({ total, payload }) => {
            this.emit('progress', { size: this.filesize, total });
            let data = { size: this.filesize, total, payload };
            this.emit('done', data);
            this.__clearMaxWaitMonitor();
            if (typeof cb === 'function') {
                if (this.withStats)
                    cb(data);
                else
                    cb(...payload);
            }
            this.__destroy();
        });
    }
    upload(event, cb) {
        this.event = event;
        if (typeof this.filepath === 'string') {
            if (fs_1.existsSync(this.filepath)) {
                if (this.id)
                    this.__start(cb);
                else {
                    // get an id from server
                    this.__getId();
                    let whenToAbort = add_seconds_1.default(new Date(), 30).getTime(), timer = setInterval(() => {
                        if (this.id)
                            clearInterval(timer);
                        else {
                            this.__getId();
                            if (Date.now() >= whenToAbort) {
                                this.__destroy();
                                this.emit('cancel', 'Get id timeout id=' + this.id);
                            }
                        }
                    }, 5000);
                    this.once('ready', () => {
                        clearInterval(timer);
                        this.__start(cb);
                    });
                }
            }
            else {
                let text = `${this.filepath} does not exist.`;
                throw new Error(text);
            }
        }
        else {
            let text = `${this.filepath} must be typeof string.`;
            throw new Error(text);
        }
        return this;
    }
    pause() {
        this.isPaused = true;
        this.emit('pause');
    }
    resume() {
        if (!this.id)
            return;
        this.isPaused = false;
        this.emit('resume');
        this.socket.emit(`__${this.eventNamespace}_::resume::__`, this.id);
    }
    stop() {
        this.socket.emit(`__${this.eventNamespace}_::stop::__`, this.id);
        this.__destroy();
        this.emit('cancel', 'Stopped id=' + this.id);
    }
    __destroy() {
        this.socket.off(`__${this.eventNamespace}_::more::${this.id}__`, () => { });
        this.socket.off(`__${this.eventNamespace}_::data::${this.id}__`, () => { });
        this.socket.off(`__${this.eventNamespace}_::resume::${this.id}__`, () => { });
        this.socket.off(`__${this.eventNamespace}_::end::${this.id}__`, () => { });
        this.data = null;
        this.id = null;
        this.__clearMaxWaitMonitor();
    }
}
exports.default = Client;
