'use strict'
const Joi = require('joi');
const CRUD = require('../../util/CRUD');

const CreateReturnSchema = {
    error: Joi.boolean().example(false).description('Indica se o processo gerou algum erro.'),
    id: Joi.number().integer().example('1').description('ID do registro criado'),
    message: Joi.string().example('Registro criado com aucesso.')
};

module.exports = [
  {
    method: 'GET',
    path: '/api/v2/agendas',
    handler: (request, reply) => {
      const crud = new CRUD('public.agenda');      
      crud.filtros = [];
      if(request.query.nome) crud.filtros.push({campo: 'LOWER(nome)', operador: 'LIKE', valor: '%'+decodeURIComponent(request.query.nome).toLowerCase()+'%'});
      if(request.query.numero) crud.filtros.push({campo: 'numero', valor: decodeURIComponent(request.query.numero)});
      crud.getAll(request, reply);
    },
    config: {
      description: 'Obtem as agendas cadastradas',
      tags: ['api','v2','Agenda'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
                campo: Joi.string().description('Nome da coluna'),
                ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          nome: Joi.string().description('Filtra por nome'),
          numero: Joi.string().description('Filtra por numero')
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/agenda/{ID}',
    handler: (request, reply) => {
      const crud = new CRUD('public.agenda');
      crud.getOne(request, reply);
    },
    config: {
      description: 'Obtem os dados da agenda pelo ID',
      tags: ['api','v2','Agenda'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v2/agenda',
    handler: (request, reply) => {
      const crud = new CRUD('public.agenda');
      crud.create(request, reply);
    },
    config: {
      description: 'Cria uma nova agenda',
      tags: ['api','v2','Agenda'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      response: {
        schema: CreateReturnSchema
      },
      validate: {
        payload: Joi.object({
          numero: Joi.string().required().example('123456').description('Número'),
          nome: Joi.string().required().example('João').description('Nome'),
          id_grupo: Joi.number().allow(null).allow('').integer().example('1').description('ID do grupo de agenda'),    
        })
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v2/agenda/{ID}',
    handler: (request, reply) => {
      const crud = new CRUD('public.agenda');
      crud.update(request, reply);
    },
    config: {
      description: 'Atualiza uma agenda',
      tags: ['api','v2','Agenda'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            ID: Joi.number().integer().required()
        },
        payload: Joi.object({
          numero: Joi.string().example('123456').description('Número'),
          nome: Joi.string().example('João').description('Nome'),
          id_grupo: Joi.number().allow(null).allow('').integer().example('1').description('ID do grupo de agenda'),    
        })
      }
    }
  },
  {
    method: 'DELETE',
    path: '/api/v2/agenda/{ID}',
    handler: (request, reply) => {
      const crud = new CRUD('public.agenda');
      crud.delete(request, reply);
    },
    config: {
      description: 'Apaga uma agenda',
      tags: ['api','v2','Agenda'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/agendas/grupos',
    handler: (request, reply) => {
      const crud = new CRUD('public.agenda_grupos'); 
      crud.filtros = [];
      if(request.query.nome) crud.filtros.push({campo: 'LOWER(nome)', operador: 'LIKE', valor: '%'+decodeURIComponent(request.query.nome).toLowerCase()+'%'});     
      crud.getAll(request, reply);
    },
    config: {
      description: 'Obtem os grupos de agendas cadastradas',
      tags: ['api','v2','Agenda'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
                campo: Joi.string().description('Nome da coluna'),
                ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          nome: Joi.string().description('Filtra por nome'),
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/agenda/grupo/{ID}',
    handler: (request, reply) => {
      const crud = new CRUD('public.agenda_grupos');
      crud.getOne(request, reply);
    },
    config: {
      description: 'Obtem os dados grupo da agenda pelo ID',
      tags: ['api','v2','Agenda'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v2/agendas/grupo',
    handler: (request, reply) => {
      const crud = new CRUD('public.agenda_grupos');
      crud.create(request, reply);
    },
    config: {
      description: 'Cria um novo grupo de agenda',
      tags: ['api','v2','Agenda'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      response: {
        schema: CreateReturnSchema
      },
      validate: {
        payload: Joi.object({          
          nome: Joi.string().required().example('Clientes VIP').description('Nome'),
          descricao: Joi.string().allow(null).allow('').example('Clientes com prioridade de atendimento').description('Descrição'),
          contexto: Joi.string().allow(null).allow('').example('rota_prioritaria').description('Grupo de rota'),    
        })
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v2/agenda/grupo/{ID}',
    handler: (request, reply) => {
      const crud = new CRUD('public.agenda_grupos');
      crud.update(request, reply);
    },
    config: {
      description: 'Atualiza um grupo de rota',
      tags: ['api','v2','Agenda'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            ID: Joi.number().integer().required()
        },
        payload: Joi.object({
          nome: Joi.string().example('Clientes VIP').description('Nome'),
          descricao: Joi.string().allow(null).allow('').example('Clientes com prioridade de atendimento').description('Descrição'),
          contexto: Joi.string().allow(null).allow('').example('rota_prioritaria').description('Grupo de rota'),     
        })
      }
    }
  },
  {
    method: 'DELETE',
    path: '/api/v2/agenda/grupo/{ID}',
    handler: (request, reply) => {
      const crud = new CRUD('public.agenda_grupos');
      crud.delete(request, reply);
    },
    config: {
      description: 'Apaga um grupo de agenda',
      tags: ['api','v2','Agenda'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            ID: Joi.number().integer().required()
        }
      }
    }
  },
];
