'use strict'
const Joi = require('joi');
const CRUD = require('../../util/CRUD');

module.exports = [
  {
    method: 'GET',
    path: '/api/v2/agendamentos',
    handler: (request, reply) => {
      const crud = new CRUD('public.agendamentos');      
      crud.filtros = [];
      if(request.query.agente_agendador) crud.filtros.push({campo: 'LOWER(agente_agendador)', valor: decodeURIComponent(request.query.agente_agendador).toLowerCase()});
      if(request.query.ramal_agendador) crud.filtros.push({campo: 'ramal_agendador', valor: decodeURIComponent(request.query.ramal_agendador)});
      crud.getAll(request, reply);
    },
    config: {
      description: 'Obtem as agendamentos cadastrados',
      tags: ['api','v2','Agendamentos'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
                campo: Joi.string().description('Nome da coluna'),
                ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          agente_agendador: Joi.string().description('Filtra por agente_agendador'),
          ramal_agendador: Joi.string().description('Filtra por ramal_agendador')
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/agendamento/{ID}',
    handler: (request, reply) => {
      const crud = new CRUD('public.agendamentos');
      crud.id = 'uid';
      crud.getOne(request, reply);
    },
    config: {
      description: 'Obtem os dados do agendamentos pelo ID',
      tags: ['api','v2','Agendamentos'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            ID: Joi.string().required()
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v2/agendamento',
    handler: (request, reply) => {
      const crud = new CRUD('public.agendamentos');
      crud.id = 'uid';
      crud.create(request, reply);
    },
    config: {
      description: 'Cria um novo agendamento',
      tags: ['api','v2','Agendamentos'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        payload: Joi.object({
          uid: Joi.string().required().example('5655aadsd6688aasssa').description('ID do agendamento'),
          data_inicial: Joi.date().iso().example('2018-01-01 08:00:00').description('Data inicio'),
          data_final: Joi.date().iso().example('2018-01-01 08:00:00').description('Data fim'),
          numero: Joi.string().required().example('123456').description('Número para ligar'),
          nome: Joi.string().required().example('João').description('Nome'),
          ramal_agendador: Joi.string().required().example('5501').description('Numero do Ramal'),
          agente_agendador: Joi.string().required().example('Agente 1').description('Nome do agente'),
          efetuado: Joi.boolean().example(false).description('Efetuado?'), 
          descricao:Joi.string().allow(null).example('teste de eco').description('descricao')
        })
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v2/agendamento/{ID}',
    handler: (request, reply) => {
      const crud = new CRUD('public.agendamentos');
      crud.id = 'uid';
      crud.update(request, reply);
    },
    config: {
      description: 'Atualiza um agendamento',
      tags: ['api','v2','Agendamentos'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            ID: Joi.string().required()
        },
        payload: Joi.object({
          data_inicial: Joi.date().iso().example('2018-01-01 08:00:00').description('Data inicio'),
          data_final: Joi.date().iso().example('2018-01-01 08:00:00').description('Data fim'),
          numero: Joi.string().example('123456').description('Número para ligar'),
          nome: Joi.string().example('João').description('Nome'),
          ramal_agendador: Joi.string().example('5501').description('Numero do Ramal'),
          agente_agendador: Joi.string().example('Agente 1').description('Nome do agente'),
          efetuado: Joi.boolean().example(false).description('Efetuado?'),
          descricao:Joi.string().allow(null).example('teste de eco').description('descricao')
        })
      }
    }
  },
  {
    method: 'DELETE',
    path: '/api/v2/agendamento/{ID}',
    handler: (request, reply) => {
      const crud = new CRUD('public.agendamentos');
      crud.id = 'uid';
      crud.delete(request, reply);
    },
    config: {
      description: 'Apaga um agendamento',
      tags: ['api','v2','Agendamentos'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            ID: Joi.string().required()
        }
      }
    }
  },
];
