'use strict'

const Joi = require('joi');
const handlers = require('./handlers');

module.exports = [
  {
    method: 'GET',
    path: '/api/v2/agentes',
    handler: handlers.getall,
    config: {
      description: 'Obtem todos os agentes',
      tags: ['api','v2','Agentes'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
                campo: Joi.string().description('Nome da coluna'),
                ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          nome: Joi.string().description('Filtra por nome'),
          ramal: Joi.string().description('Filtra por ramal')
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/agente/{AgenteID}',
    handler: handlers.getAgente,
    config: {
      description: 'Obtem os dados do agente pelo ID',
      tags: ['api','v2','Agentes'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            AgenteID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/agente/ramal/{AgenteRamal}',
    handler: handlers.getAgenteByRamal,
    config: {
      description: 'Obtem os dados do agente pelo ramal. IMPORTANTE: Observe como o Swagger faz a chamada correta, Use Encode para a / do ramal.',
      tags: ['api','v2','Agentes'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            AgenteRamal: Joi.string().required()
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v2/agentes',
    handler: handlers.create,
    config: {
      description: 'Cria um agente',
      tags: ['api','v2','Agentes'],
      validate: {
        payload: Joi.object({
          username: Joi.string().required().description('Nome para login'),
          password: Joi.string().description('Senha para login')
        })
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v2/agente/{AgenteID}/login',
    handler: handlers.AgenteLogin,
    config: {
      description: 'Loga um agente na fila informada',
      tags: ['api','v2','Agentes'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            AgenteID: Joi.number().integer().required()
        },
        payload: Joi.object({
          fila: Joi.string().required().description('Nome da fila'),
          pausado: Joi.boolean().default(false).description('Informa se o agente deve logar pausado.')
        })
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v2/agente/{AgenteID}/logoff',
    handler: handlers.AgenteLogoff,
    config: {
      description: 'Remove um agente de uma fila',
      tags: ['api','v2','Agentes'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            AgenteID: Joi.number().integer().required()
        },
        payload: Joi.object({
          fila: Joi.string().required().description('Nome da fila')
        })
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v2/agente/{AgenteID}/pausa',
    handler: handlers.AgentePausa,
    config: {
      description: 'Pausa um agente',
      tags: ['api','v2','Agentes'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            AgenteID: Joi.number().integer().required()
        },
        payload: Joi.object({
          fila: Joi.string().required().description('Nome da fila'),
          motivo: Joi.string().description('Motivo')
        })
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v2/agente/{AgenteID}/despausa',
    handler: handlers.AgenteTiraPausa,
    config: {
      description: 'Tira um agente da pausa',
      tags: ['api','v2','Agentes'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            AgenteID: Joi.number().integer().required()
        },
        payload: Joi.object({
          fila: Joi.string().required().description('Nome da fila')
        })
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v2/agente/ramal/{AgenteRamal}/login',
    handler: handlers.AgenteLoginRamal,
    config: {
      description: 'Loga um agente na fila informada pelo ramal',
      tags: ['api','v2','Agentes'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            AgenteRamal: Joi.string().required()
        },
        payload: Joi.object({
          fila: Joi.string().required().description('Nome da fila'),
          nome: Joi.string().required().description('Nome do agente'),
          pausado: Joi.boolean().default(false).description('Informa se o agente deve logar pausado.')
        })
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v2/agente/ramal/{AgenteRamal}/logoff',
    handler: handlers.AgenteLogoffRamal,
    config: {
      description: 'Remove um agente de uma fila pelo ramal',
      tags: ['api','v2','Agentes'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            AgenteRamal: Joi.string().required()
        },
        payload: Joi.object({
          fila: Joi.string().required().description('Nome da fila')
        })
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v2/agente/ramal/{AgenteRamal}/pausa',
    handler: handlers.AgentePausaRamal,
    config: {
      description: 'Pausa um agente pelo ramal',
      tags: ['api','v2','Agentes'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            AgenteRamal: Joi.string().required()
        },
        payload: Joi.object({
          fila: Joi.string().required().description('Nome da fila'),
          motivo: Joi.string().description('Motivo')
        })
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v2/agente/ramal/{AgenteRamal}/despausa',
    handler: handlers.AgenteTiraPausaRamal,
    config: {
      description: 'Tira um agente da pausa pelo ramal',
      tags: ['api','v2','Agentes'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            AgenteRamal: Joi.string().required()
        },
        payload: Joi.object({
          fila: Joi.string().required().description('Nome da fila')
        })
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/agentes/eventos',
    handler: handlers.getEvents,
    config: {
      description: 'Obtem os eventos dos agentes',
      tags: ['api','v2','Agentes'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
                campo: Joi.string().description('Nome da coluna'),
                ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          data_ini: Joi.date().description('Filtro por data inicial'),
          data_fim: Joi.date().description('Filtro por data Final'),
          filas: Joi.array().items(Joi.string()).example(['FilaA','FilaB']).description('Filtra por uma ou mais filas'),
          agentes: Joi.array().items(Joi.string()).example(['AgenteA','AgenteB']).description('Filtra por um ou mais agentes'),
          eventos: Joi.array().items(Joi.string().valid('Login', 'Pause', 'Answer', 'NoAnswer')).example(['Login','Pause','Answer','NoAnswer']).description('Filtra por um ou mais eventos'),
          grupos: Joi.array().items(Joi.number().integer()).example(['1','2']).description('Filtra por um ou mais grupos de eventos'),
        }
      }
    }
  },
];
