'use strict'

const Paginacao = require('../../util/Paginacao');
const Boom = require('boom');
const db = require('../../util/db');
//const squel = require("squel");
const ws = require('../../ws');
const discador = require('../../discador');
const config = require('../../config.json');
const TextoUtil = require("../../util/texto");
const moment = require('moment-timezone');
moment.tz.setDefault(config.servidor.timezone);
const csv = require("fast-csv");
const uuid = require('uuid/v1');
const fs = require("fs");
const MimeType = require("mime");

const log = require('simple-node-logger').createSimpleLogger({
    logFilePath: '/var/log/xcontact.log',
    timestampFormat: 'DD-MM-YYYY HH:mm:ss.SSS'
});
const log_level = config.log ? config.log : 'error';
log.setLevel(log_level);

exports.create = (request, reply) => {
    const knex = db.knex('public.campanhas');
    knex.insert(request.payload);
    knex.returning('id')
        .then(function (id) {
            const ret = {};
            ret.error = false;
            ret.id = id[0];
            ret.message = 'Campanha id = ' + id[0] + ' criada com sucesso.';
            try {
                const campanha = request.payload;
                campanha.id = id[0];
                discador.addCampanha(campanha);
            }
            catch (ex) {
                console.error('Erro interno ao adicionar a campanha no discador, erro: ' + ex);
            }

            reply(ret);
        })
        .catch(function (error) {
            console.error(error);
            reply(Boom.badData(error));
        });
};

exports.update = (request, reply) => {
    const knex = db.knex('public.campanhas');
    knex.update(request.payload);

    knex.where('id', request.params.CampanhaID)
        .then(function (rows) {
            const ret = {};
            ret.error = false;
            ret.total = rows;
            ret.message = 'Campanha id = ' + request.params.CampanhaID + ' atualizada com sucesso.';

            try {
                discador.reloadCampanha(request.params.CampanhaID);
            }
            catch (ex) {
                console.log('Erro interno ao recarregar uma campanha no discador, erro: ' + ex);
            }

            reply(ret);
        })
        .catch(function (error) {
            console.error(error);
            reply(Boom.badData(error));
        });
};

/* exports.updateTurnos = (request, reply) => {
    const knex = db.knex;
    let idCampanha = request.params.CampanhaID;
    let turnos = request.payload.map(x => ({
        id_campanha: idCampanha,
        id_turno: x
    }));
    console.log(turnos);

    knex.transaction(t => {
        return knex("campanha_turno")
            .transacting(t)
            .where("id_campanha", idCampanha)
            .del()
            .then(() => {
                return knex.batchInsert("campanha_turno", turnos, 100)
                    .transacting(t)
                    .then()
                    .catch(error => {
                        console.log('Falha ao inserir os turnos da campanha. Erro = ' + error);
                        t.rollback();
                        throw error;
                    });
            })
            .then(t.commit)
            .catch(error => {
                console.error(error);
                t.rollback();
                throw error;
            });
    }).then(x => {
        console.log (x);
        console.log('Turnos da campanha atualizados com sucesso!');
        reply({
            error: false,
            message: "Turnos da campanha atualizados com sucesso!"
        });
    })
    .catch(error => {
        console.log('Falha ao atualizar os turnos da campanha. Erro = ' + error);
        reply(Boom.badData(error));
    });
}; */

exports.uploadArquivoCampanha = async (request, reply) => {
    let extractFileExtension = function (file) {
        try {
            let contentType = file.hapi.headers["content-type"];
            return MimeType.getExtension(contentType);
        } catch (e) { }

        return "";
    };

    let reportarErro = (e) => {
        log.error("[REST-API] [CAMPANHAS] uploadArquivoCampanha -> " + e);
        console.error(e);
        console.trace(e);
        reply(Boom.badData(e));
    };

    try {
        let data = request.payload;
        if (data && data.file) {
            let rows = await db.knex("public.campanhas")
                .where("id", request.params.CampanhaID)
                .select("fila", "caminho_arquivo_campanha");

            if (rows && rows.length > 0) {
                let campanha = rows.find(x => x);

                // Verifica se a campanha já possui um arquivo!
                if (campanha.caminho_arquivo_campanha && fs.existsSync(campanha.caminho_arquivo_campanha)) {
                    // Deleta o arquivo
                    fs.unlinkSync(campanha.caminho_arquivo_campanha);
                }

                // Gera caminho a salvar o arquivo
                let date = moment();
                let extension = extractFileExtension(data.file);
                let filename = `${uuid()}.${extension}`;
                let path = `/gravacoes/${campanha.fila}/${date.year()}/${date.month()}/${date.date()}/`;
                let filepath = path + filename;

                // Cria a estrutura de pastas, caso não exista
                if (!fs.existsSync(path)) fs.mkdirSync(path, { recursive: true });

                // Cria a stream do arquivo local
                let localFileStream = fs.createWriteStream(filepath);
                localFileStream.on("error", e => {// Em caso de erro ao escrever no arquivo
                    // Deleta o arquivo local
                    fs.unlinkSync(filepath);
                    reportarErro(e);
                });

                // Redireciona todos os chunks do arquivo pra stream do arquivo local
                data.file.pipe(localFileStream);

                // Quando receber todos os pedaços do arquivo
                data.file.on("end", async e => {
                    if (e) {
                        // Deleta o arquivo local
                        fs.unlinkSync(filepath);
                        reportarErro(e);
                    } else {
                        try {
                            // Altera a tabela de campanhas, salvando o caminho do arquivo
                            let rows = await db.knex("public.campanhas")
                                .update({ caminho_arquivo_campanha: filepath })
                                .where("id", request.params.CampanhaID);

                            console.log(rows);
                            reply({
                                msg: "OK",
                                extensao: extension,
                                arquivo: filename,
                                caminho: filepath
                            });
                        } catch (e) {
                            // Deleta o arquivo local
                            fs.unlinkSync(filepath);
                            reportarErro(e);
                        }
                    }
                });

            } else {
                reply(Boom.badData("Campanha especificada inválida!"));
            }
        } else {
            reply(Boom.badData("Arquivo inválido!"));
        }
    } catch (e) {
        reportarErro(e);
    }
};

exports.uploadImagemCampanha = async (request, reply) => {
    let extractFileExtension = function (file) {
        try {
            let contentType = file.hapi.headers["content-type"];
            if (contentType && contentType.startsWith("image/")) {
                return contentType.substring("image/".length)
            }
        } catch (e) { }

        return ".jpg";
    };

    let reportarErro = (e) => {
        log.error("[REST-API] [CAMPANHAS] uploadImagemCampanha -> " + e);
        console.error(e);
        console.trace(e);
        reply(Boom.badData(e));
    };

    try {
        let data = request.payload;
        if (data && data.file) {
            let rows = await db.knex("public.campanhas")
                .where("id", request.params.CampanhaID)
                .select("fila", "caminho_imagem_campanha");

            if (rows && rows.length > 0) {
                let campanha = rows.find(x => x);

                // Verifica se a campanha já possui uma imagem!
                if (campanha.caminho_imagem_campanha && fs.existsSync(campanha.caminho_imagem_campanha)) {
                    // Deleta o arquivo
                    fs.unlinkSync(campanha.caminho_imagem_campanha);
                }

                // Gera caminho a salvar o arquivo
                let date = moment();
                let extension = extractFileExtension(data.file);
                let filename = `${uuid()}.${extension}`;
                let path = `/gravacoes/${campanha.fila}/${date.year()}/${date.month()}/${date.date()}/`;
                let filepath = path + filename;

                // Cria a estrutura de pastas, caso não exista
                if (!fs.existsSync(path)) fs.mkdirSync(path, { recursive: true });

                // Cria a stream do arquivo local
                let localFileStream = fs.createWriteStream(filepath);
                localFileStream.on("error", e => {// Em caso de erro ao escrever no arquivo
                    // Deleta o arquivo local
                    fs.unlinkSync(filepath);
                    reportarErro(e);
                });

                // Redireciona todos os chunks do arquivo pra stream do arquivo local
                data.file.pipe(localFileStream);

                // Quando receber todos os pedaços do arquivo
                data.file.on("end", async e => {
                    if (e) {
                        // Deleta o arquivo local
                        fs.unlinkSync(filepath);
                        reportarErro(e);
                    } else {
                        try {
                            // Altera a tabela de campanhas, salvando o caminho do arquivo
                            let rows = await db.knex("public.campanhas")
                                .update({ caminho_imagem_campanha: filepath })
                                .where("id", request.params.CampanhaID);
                            reply({
                                msg: "OK",
                                extensao: extension,
                                arquivo: filename,
                                caminho: filepath
                            });
                        } catch (e) {
                            // Deleta o arquivo local
                            fs.unlinkSync(filepath);
                            reportarErro(e);
                        }
                    }
                });

            } else {
                reply(Boom.badData("Campanha especificada inválida!"));
            }
        } else {
            reply(Boom.badData("Arquivo inválido!"));
        }
    } catch (e) {
        reportarErro(e);
    }
};

exports.getTurnos = (request, reply) => {
    const campanha = request.params.CampanhaID;

    const max = request.query.max ? request.query.max : 15;
    const page = request.query.page ? request.query.page : 1;

    if (max === -1) {
        db.knex.select("horarios.*")
            .from("horarios")
            .innerJoin("campanhas", "campanhas.id", campanha)
            .where("horario.id = ANY(campanha.turnos)")
            .then(rows => reply({
                error: false,
                full: true,
                total: rows.length,
                message: "Lista dos turnos da campanha",
                dados: rows
            }))
            .catch(error => {
                console.error(error);
                reply(Boom.badData(error));
            });
    } else {
        const offset = max * (page - 1);
        db.knex.select("COUNT(horarios.*) as total")
            .from("horarios")
            .innerJoin("campanhas", "campanhas.id", campanha)
            .where("horario.id = ANY(campanha.turnos)")
            .then(total => {
                db.knex.select("horarios.*")
                    .from("horarios")
                    .innerJoin("campanhas", "campanhas.id", campanha)
                    .where("horario.id = ANY(campanha.turnos)")
                    .limit(max)
                    .offset(offset)
                    .then(res => {
                        const pages = Math.ceil(parseInt(total.rows[0].total) / max);

                        reply({
                            error: false,
                            full: pages === 1 ? true : false,
                            page: page,
                            max: max,
                            offset: offset,
                            total: parseInt(total.rows[0].total),
                            pages: pages,
                            message: 'Lista de campanhas_classificacoes paginada',
                            dados: res.rows
                        });
                    })
                    .catch(error => {
                        console.error(error);
                        reply(Boom.badData(error));
                    });
            })
            .catch(error => {
                console.error(error);
                reply(Boom.badData(error));
            });
    }
};

exports.ReplaceClassificacoes = (request, reply) => {
    const knex = db.knex;
    const classificacoes = [];

    for (let i = 0; i < request.payload.length; ++i) {
        const cla = {
            id_campanha: request.params.CampanhaID,
            id_classificacao: request.payload[i].id,
            acao: request.payload[i].acao
        };
        classificacoes.push(cla);
    }

    knex.transaction(function (t) {
        return knex('campanhas_classificacoes')
            .transacting(t)
            .where('id_campanha', request.params.CampanhaID)
            .del()
            .then(function () {
                return knex.batchInsert('campanhas_classificacoes', classificacoes, 100)
                    .transacting(t)
                    .then()
                    .catch(function (error) {
                        t.rollback();
                        throw error;
                    });
            })
            .then(t.commit)
            .catch(function (e) {
                t.rollback();
                throw e;
            })
    })
        .then(function () {
            const ret = {};
            ret.error = false;
            ret.message = 'Classificações da campanha atualizadas com sucesso!';
            reply(ret);
        })
        .catch(function (e) {
            reply(Boom.badData(e));
        });
};

exports.ClearClassificacoes = (request, reply) => {
    const knex = db.knex;

    knex.transaction(function (t) {
        return knex('campanhas_classificacoes')
            .transacting(t)
            .where('id_campanha', request.params.CampanhaID)
            .del()
            .then(t.commit)
            .catch(function (e) {
                t.rollback();
                throw e;
            })
    })
        .then(function () {
            const ret = {};
            ret.error = false;
            ret.message = 'Classificações da campanha limpas com sucesso!';
            reply(ret);
        })
        .catch(function (e) {
            reply(Boom.badData(e));
        });
};

exports.delete = (request, reply) => {
    const knex = db.knex;
    let transaction = knex.transaction(function (t) {
        return knex('campanhas')
            .transacting(t)
            .where('id', request.params.CampanhaID)
            .del()
            .then(function () {
                return knex('campanhas_contato')
                    .where('id_campanha', request.params.CampanhaID)
                    .del()
                    .then(function () {
                        return knex('campanhas_contato_numeros')
                            .where('id_campanha', request.params.CampanhaID)
                            .del()
                            .then(function () {
                                return knex('campanhas_contato_campos')
                                    .where('id_campanha', request.params.CampanhaID)
                                    .del();
                            })
                    })
            })
            .then(t.commit)
            .catch(function (e) {
                t.rollback();
                throw e;
            })
    });

    transaction.then(
        () => {
            const ret = {};
            ret.error = false;
            ret.message = 'Campanha id = ' + request.params.CampanhaID + ' apagada com sucesso.';

            // Notifica o discador que a campanha foi deletada
            discador.delCampanha(request.params.CampanhaID);

            reply(ret);
        },
        error => {
            reply(Boom.badData(error));
        }
    );
};

exports.zerar = (request, reply) => {
    const knex = db.knex;
    knex.transaction(function (t) {
        return knex('campanhas_contato_numeros')
            .transacting(t)
            .where('id_campanha', request.params.CampanhaID)
            .update({ sucesso: false, tentativas: 0, retentativas: 0, prioridade: 0 })
            .then(function () {
                return knex('campanhas_contato')
                    .where('id_campanha', request.params.CampanhaID)
                    .update({ sucesso: false })
                    .then(() => {
                        return knex('campanhas')
                            .where('id', request.params.CampanhaID)
                            .update({ total_discados_atual: 0 })
                    })
            })
            .then(t.commit)
            .catch(function (e) {
                t.rollback();
                throw e;
            })
    })
        .then(function () {
            const ret = {};
            ret.error = false;
            ret.message = 'Campanha id = ' + request.params.CampanhaID + ' zerada com sucesso.';
            reply(ret);
        })
        .catch(function (e) {
            reply(Boom.badData(e));
        });
};

exports.getall = (request, reply) => {
    const max = request.query.max ? request.query.max : 15;
    const page = request.query.page ? request.query.page : 1;
    //const knex = db.knex('public.v_campanhas');
    const knex = db.knex('public.campanhas');

    if (request.query.status && request.query.status.length > 0) knex.where('status', request.query.status);
    if (request.query.nome) knex.whereRaw('LOWER(nome) LIKE ?', '%' + decodeURIComponent(request.query.nome).toLowerCase() + '%');

    if (request.query.data_status_ini) knex.where('data_status', '>=', request.query.data_status_ini);
    if (request.query.data_status_fim) {
        knex.where(function () {
            this.where('data_status', '<=', request.query.data_status_fim).orWhereNull('data_status')
        })
    }

    if (request.query.data_start_ini) knex.where('data_start', '>=', request.query.data_start_ini);
    if (request.query.data_start_fim) {
        knex.where(function () {
            this.where('data_start', '<=', request.query.data_start_fim).orWhereNull('data_start')
        })
    }

    if (request.query.data_create_ini) knex.where('data_create', '>=', request.query.data_create_ini);
    if (request.query.data_create_fim) {
        knex.where(function () {
            this.where('data_create', '<=', request.query.data_create_fim).orWhereNull('data_create')
        })
    }

    if (request.query.data_agendamento_ini) knex.where('data_ini', '>=', request.query.data_agendamento_ini);
    if (request.query.data_agendamento_fim) {
        knex.where(function () {
            this.where('data_fim', '<=', request.query.data_agendamento_fim).orWhereNull('data_fim')
        })
    }

    if (max === -1) {
        //console.log('SQL = '+ knex.toString());
        knex.clearSelect();
        // Parametros de ordenação, fora do count
        if (request.query.order) {
            for (let p = 0; p < request.query.order.length; ++p) {
                if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                    knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
                }
                else if (request.query.order[p] && request.query.order[p].campo) {
                    knex.orderBy(request.query.order[p].campo, 'ASC');
                }
            }
        }
        knex.select('campanhas.*',
            db.knex.raw("(SELECT count(*) FROM campanhas_contato co WHERE co.id_campanha = campanhas.id) as total_contatos"),
            db.knex.raw("(SELECT count(*) FROM campanhas_contato_numeros ccn WHERE ccn.id_campanha = campanhas.id AND (ccn.sucesso IS TRUE OR ccn.tentativas >= campanhas.tentativas OR ccn.retentativas >= campanhas.tentativas)) as total_discados"),
            db.knex.raw("(SELECT count(*) FROM campanhas_contato_numeros ccn WHERE ccn.id_campanha = campanhas.id) as total_numeros"),
            db.knex.raw("(SELECT count(*) FROM campanhas_contato co WHERE co.id_campanha = campanhas.id AND co.sucesso IS true) as total_sucessos")
        ).then(function (rows) {
            const ret = {};
            ret.error = false;
            ret.full = true;
            ret.total = rows.length;
            ret.message = 'Lista de todas as campanhas';
            ret.dados = rows;
            if (request.query.return === 'array') reply(rows);
            else reply(ret);
        })
            .catch(function (error) {
                console.error(error);
                reply(Boom.badData(error));
            });
    }
    else {
        const offset = max * (page - 1);
        //console.log('SQL = '+knex.count('* as total').toString());
        knex.count('* as total')
            .then(function (total) {
                if (total[0].total > 0) {
                    knex.clearSelect();
                    // Parametros de ordenação, fora do count
                    if (request.query.order) {
                        for (let p = 0; p < request.query.order.length; ++p) {
                            if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                                knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
                            }
                            else if (request.query.order[p] && request.query.order[p].campo) {
                                knex.orderBy(request.query.order[p].campo, 'ASC');
                            }
                        }
                    }
                    //console.log('SQL = '+knex.select('*').limit(max).offset(offset).toString());
                    knex.select('campanhas.*',
                        db.knex.raw("(SELECT count(*) FROM campanhas_contato co WHERE co.id_campanha = campanhas.id) as total_contatos"),
                        db.knex.raw("(SELECT count(*) FROM campanhas_contato_numeros ccn WHERE ccn.id_campanha = campanhas.id AND (ccn.sucesso IS TRUE OR ccn.tentativas >= campanhas.tentativas OR ccn.retentativas >= campanhas.tentativas)) as total_discados"),
                        db.knex.raw("(SELECT count(*) FROM campanhas_contato_numeros ccn WHERE ccn.id_campanha = campanhas.id) as total_numeros"),
                        db.knex.raw("(SELECT count(*) FROM campanhas_contato co WHERE co.id_campanha = campanhas.id AND co.sucesso IS true) as total_sucessos")
                    ).limit(max).offset(offset).then(function (rows) {
                        const pages = Math.ceil(parseInt(total[0].total) / max);
                        const ret = {};
                        ret.error = false;
                        ret.full = pages === 1 ? true : false;
                        ret.page = page;
                        ret.max = max;
                        ret.offset = offset;
                        ret.total = parseInt(total[0].total);
                        ret.pages = pages;
                        ret.message = 'Lista de campanhas paginada';
                        ret.dados = rows;
                        if (request.query.return === 'array') reply(rows);
                        else reply(ret);
                    })
                        .catch(function (error) {
                            console.error(error);
                            reply(Boom.badData(error));
                        });
                }
                else {
                    const ret = {};
                    ret.error = true;
                    ret.total = 0;
                    ret.message = 'Nenhum registro encontrado';
                    reply(ret);
                }
            })
            .catch(function (error) {
                console.error(error);
                reply(Boom.badData(error));
            });
    }
};

exports.returnAll = async (request) => {
    const max = request.max ? request.max : 15;
    const page = request.page ? request.page : 1;
    //const knex = db.knex('public.v_campanhas');
    const knex = db.knex('public.campanhas');

    if (request.status && request.status.length > 0) knex.where('status', request.status);
    if (request.nome) knex.whereRaw('LOWER(nome) LIKE ?', '%' + decodeURIComponent(request.nome).toLowerCase() + '%');

    if (request.data_status_ini) knex.where('data_status', '>=', request.data_status_ini);
    if (request.data_status_fim) {
        knex.where(function () {
            this.where('data_status', '<=', request.data_status_fim).orWhereNull('data_status')
        })
    }

    if (request.data_start_ini) knex.where('data_start', '>=', request.data_start_ini);
    if (request.data_start_fim) {
        knex.where(function () {
            this.where('data_start', '<=', request.data_start_fim).orWhereNull('data_start')
        })
    }

    if (request.data_create_ini) knex.where('data_create', '>=', request.data_create_ini);
    if (request.data_create_fim) {
        knex.where(function () {
            this.where('data_create', '<=', request.data_create_fim).orWhereNull('data_create')
        })
    }

    if (request.data_agendamento_ini) knex.where('data_ini', '>=', request.data_agendamento_ini);
    if (request.data_agendamento_fim) {
        knex.where(function () {
            this.where('data_fim', '<=', request.data_agendamento_fim).orWhereNull('data_fim')
        })
    }

    const ret = {};
    knex.clearSelect();
    // Parametros de ordenação, fora do count
    if (request.order) {
        for (let p = 0; p < request.order.length; ++p) {
            if (request.order[p] && request.order[p].campo && request.order[p].ordem) {
                knex.orderBy(request.order[p].campo, request.order[p].ordem);
            }
            else if (request.order[p] && request.order[p].campo) {
                knex.orderBy(request.order[p].campo, 'ASC');
            }
        }
    }
    await knex.select('campanhas.*',
        db.knex.raw("(SELECT count(*) FROM campanhas_contato co WHERE co.id_campanha = campanhas.id) as total_contatos"),
        db.knex.raw("(SELECT count(*) FROM campanhas_contato_numeros ccn WHERE ccn.id_campanha = campanhas.id AND (ccn.sucesso IS TRUE OR ccn.tentativas >= campanhas.tentativas OR ccn.retentativas >= campanhas.tentativas)) as total_discados"),
        db.knex.raw("(SELECT count(*) FROM campanhas_contato_numeros ccn WHERE ccn.id_campanha = campanhas.id) as total_numeros"),
        db.knex.raw("(SELECT count(*) FROM campanhas_contato co WHERE co.id_campanha = campanhas.id AND co.sucesso IS true) as total_sucessos")
    ).then(function (rows) {
        ret.error = false;
        ret.full = true;
        ret.total = rows.length;
        ret.message = 'Lista de todas as campanhas';
        ret.dados = rows;
        if (request.return === 'array') return (rows);
        else return (ret);
    })
        .catch(function (error) {
            console.error(error);
            return (Boom.badData(error));
        });

    return ret
};

exports.getCampanha = (request, reply) => {
    const knex = db.knex('public.campanhas');
    knex.where('id', request.params.CampanhaID);
    knex.select('*')
        .then(function (rows) {
            const ret = {};
            ret.error = false;
            ret.full = true;
            ret.total = rows.length;
            ret.message = 'Dados da campanha id = ' + request.params.CampanhaID;
            ret.campanha = rows[0];
            reply(ret);
        })
        .catch(function (error) {
            console.error(error);
            reply(Boom.badData(error));
        });
};

exports.getclassificacoes = (request, reply) => {
    const campanha = request.params.CampanhaID;

    const max = request.query.max ? request.query.max : 15;
    const page = request.query.page ? request.query.page : 1;

    if (max === -1) {
        db.knex.select('classificacoes.*', 'campanhas_classificacoes.acao')
            .from('campanhas_classificacoes')
            .innerJoin('classificacoes', 'campanhas_classificacoes.id_classificacao', 'classificacoes.id')
            .where('campanhas_classificacoes.id_campanha', request.params.CampanhaID)
            .then(function (rows) {
                const ret = {};
                ret.error = false;
                ret.full = true;
                ret.total = rows.length;
                ret.message = 'Lista das classificações da campanha';
                ret.dados = rows;

                reply(ret);
            })
            .catch(function (error) {
                console.error(error);
                reply(Boom.badData(error));
            });
    }
    else {
        const offset = max * (page - 1);
        db.query("SELECT count(*) as total FROM public.campanhas_classificacoes WHERE id_campanha = $1", [campanha])
            .then(res1 => {
                db.query("SELECT * FROM public.campanhas_classificacoes WHERE id_campanha = $1 LIMIT $2 OFFSET $3", [campanha, max, offset])
                    .then(res => {
                        const pages = Math.ceil(parseInt(res1.rows[0].total) / max);
                        const ret = {};
                        ret.error = false;
                        ret.full = pages === 1 ? true : false;
                        ret.page = page;
                        ret.max = max;
                        ret.offset = offset;
                        ret.total = parseInt(res1.rows[0].total);
                        ret.pages = pages;
                        ret.message = 'Lista de campanhas_classificacoes paginada';
                        ret.dados = res.rows;
                        reply(ret);
                    })
                    .catch(e => {
                        reply(Boom.badData(e));
                    });
            })
            .catch(e => {
                reply(Boom.badData(e));
            });
    }
};

exports.getContatos = (request, reply) => {
    const max = request.query.max ? request.query.max : 15;
    const page = request.query.page ? request.query.page : 1;
    const knex = db.knex('public.campanhas_contatos_full');
    knex.where('id_campanha', request.params.CampanhaID);

    if (max === -1) {
        //console.log('SQL = '+ knex.toString());
        knex.clearSelect();
        // Parametros de ordenação, fora do count
        if (request.query.order) {
            for (let p = 0; p < request.query.order.length; ++p) {
                if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                    knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
                }
                else if (request.query.order[p] && request.query.order[p].campo) {
                    knex.orderBy(request.query.order[p].campo, 'ASC');
                }
            }
        }
        if (request.query.campos) {
            knex.whereRaw("campos @> ANY (ARRAY ['" + JSON.stringify(request.query.campos) + "']::jsonb[])");
        }
        knex.select('*')
            .then(function (rows) {
                const ret = {};
                ret.error = false;
                ret.full = true;
                ret.total = rows.length;
                ret.message = 'Lista de todos os cantatos  da campanha';
                ret.dados = rows;
                if (request.query.return === 'array') reply(rows);
                else reply(ret);
            })
            .catch(function (error) {
                console.error(error);
                reply(Boom.badData(error));
            });
    }
    else {
        const offset = max * (page - 1);
        //console.log('SQL = '+knex.count('* as total').toString());
        knex.count('* as total')
            .then(function (total) {
                if (total[0].total > 0) {
                    knex.clearSelect();
                    // Parametros de ordenação, fora do count
                    if (request.query.order) {
                        for (let p = 0; p < request.query.order.length; ++p) {
                            if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                                knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
                            }
                            else if (request.query.order[p] && request.query.order[p].campo) {
                                knex.orderBy(request.query.order[p].campo, 'ASC');
                            }
                        }
                    }
                    if (request.query.campos) {
                        knex.whereRaw("campos @> ANY (ARRAY ['" + JSON.stringify(request.query.campos) + "']::jsonb[])");
                    }
                    //console.log('SQL = '+knex.select('*').limit(max).offset(offset).toString());
                    knex.select('*').limit(max).offset(offset).then(function (rows) {
                        const pages = Math.ceil(parseInt(total[0].total) / max);
                        const ret = {};
                        ret.error = false;
                        ret.full = pages === 1 ? true : false;
                        ret.page = page;
                        ret.max = max;
                        ret.offset = offset;
                        ret.total = parseInt(total[0].total);
                        ret.pages = pages;
                        ret.message = 'Lista de contatos da campanha paginada';
                        ret.dados = rows;
                        if (request.query.return === 'array') reply(rows);
                        else reply(ret);
                    })
                        .catch(function (error) {
                            console.error(error);
                            reply(Boom.badData(error));
                        });
                }
                else {
                    const ret = {};
                    ret.error = true;
                    ret.total = 0;
                    ret.message = 'Nenhum registro encontrado';
                    reply(ret);
                }
            })
            .catch(function (error) {
                console.error(error);
                reply(Boom.badData(error));
            });
    }
};

exports.getCampanhasRegistros = (request, reply) => {
    const max = request.query.max ? request.query.max : 15;
    const page = request.query.page ? request.query.page : 1;
    const knex = db.knex.select('campanhas_registro.*',
        'status_ligacao.nome as status',
        'campanhas.nome as campanha',
        'campanhas.fila as fila',
        'classificacoes.nome as classificacao',
        'cdr_classificacoes.obs as observacao');

    if (request.query.campanhas) knex.whereIn('campanhas_registro.id_campanha', request.query.campanhas);
    if (request.query.status && request.query.status.length > 0) knex.whereIn('campanhas_registro.id_status', request.query.status);
    if (request.query.data_ini) knex.where('campanhas_registro.data_status', '>=', request.query.data_ini);
    if (request.query.data_fim) knex.where('campanhas_registro.data_status', '<=', request.query.data_fim);
    if (request.query.numero_operador && request.query.numero) {
        switch (request.query.numero_operador) {
            case 1:
                knex.where('campanhas_registro.numero', '=', request.query.numero);
                break;
            case 2:
                knex.where('campanhas_registro.numero', 'like', request.query.numero + '%');
                break;
            case 3:
                knex.where('campanhas_registro.numero', 'like', '%' + request.query.numero);
                break;
            case 4:
                knex.where('campanhas_registro.numero', 'not like', '%' + request.query.numero + '%');
                break;
            case 5:
                knex.where('campanhas_registro.numero', 'not like', request.query.numero);
                break;
        }
    }
    else if (request.query.numero) {
        knex.where('campanhas_registro.numero', 'like', '%' + request.query.numero + '%');
    }

    if (max === -1) {
        // Parametros de ordenação, fora do count
        if (request.query.order) {
            for (let p = 0; p < request.query.order.length; ++p) {
                if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                    knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
                }
                else if (request.query.order[p] && request.query.order[p].campo) {
                    knex.orderBy(request.query.order[p].campo, 'ASC');
                }
            }
        }
        knex.from('campanhas_registro')
            .leftJoin('status_ligacao', 'campanhas_registro.id_status', 'status_ligacao.id')
            .leftJoin('campanhas', 'campanhas_registro.id_campanha', 'campanhas.id')
            .leftJoin('cdr_classificacoes', function () {
                this.on('campanhas_registro.callid', '=', 'cdr_classificacoes.uniqueid').orOn('campanhas_registro.actionid', '=', 'cdr_classificacoes.uniqueid')
            })
            .leftJoin('classificacoes', 'cdr_classificacoes.id_classificacao', 'classificacoes.id')
            .then(function (rows) {
                const ret = {};
                ret.error = false;
                ret.full = true;
                ret.total = rows.length;
                ret.message = 'Registros das campanhas';
                ret.dados = rows;

                reply(ret);
            })
            .catch(function (error) {
                console.error(error);
                reply(Boom.badData(error));
            });
    }
    else {
        const offset = max * (page - 1);
        //console.log('SQL = '+knex.count('* as total').toString());
        knex.clearSelect();
        knex.count('* as total')
            .from('campanhas_registro')
            .then(function (total) {
                if (total[0].total > 0) {
                    knex.clearSelect();
                    // Parametros de ordenação, fora do count
                    if (request.query.order) {
                        for (let p = 0; p < request.query.order.length; ++p) {
                            if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                                knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
                            }
                            else if (request.query.order[p] && request.query.order[p].campo) {
                                knex.orderBy(request.query.order[p].campo, 'ASC');
                            }
                        }
                    }
                    //console.log('SQL = '+knex.select('*').limit(max).offset(offset).toString());
                    knex.select('campanhas_registro.*', 'status_ligacao.nome as status', 'campanhas.nome as campanha', 'campanhas.fila as fila', 'classificacoes.nome as classificacao', 'cdr_classificacoes.obs as observacao')
                        .from('campanhas_registro')
                        .leftJoin('status_ligacao', 'campanhas_registro.id_status', 'status_ligacao.id')
                        .leftJoin('campanhas', 'campanhas_registro.id_campanha', 'campanhas.id')
                        .leftJoin('cdr_classificacoes', function () {
                            this.on('campanhas_registro.callid', '=', 'cdr_classificacoes.uniqueid').orOn('campanhas_registro.actionid', '=', 'cdr_classificacoes.uniqueid')
                        })
                        .leftJoin('classificacoes', 'cdr_classificacoes.id_classificacao', 'classificacoes.id')
                        .limit(max)
                        .offset(offset)
                        .then(function (rows) {
                            const pages = Math.ceil(parseInt(total[0].total) / max);
                            const ret = {};
                            ret.error = false;
                            ret.full = pages === 1 ? true : false;
                            ret.page = page;
                            ret.max = max;
                            ret.offset = offset;
                            ret.total = parseInt(total[0].total);
                            ret.pages = pages;
                            ret.message = 'Lista de campanhas paginada';
                            ret.dados = rows;
                            reply(ret);
                        })
                        .catch(function (error) {
                            console.error(error);
                            reply(Boom.badData(error));
                        });
                }
                else {
                    const ret = {};
                    ret.error = true;
                    ret.total = 0;
                    ret.message = 'Nenhum registro encontrado';
                    reply(ret);
                }
            })
            .catch(function (error) {
                console.error(error);
                reply(Boom.badData(error));
            });
    }
};

exports.returnCampanhasRegistros = async (request) => {
    const max = request.max ? request.max : 15;
    const page = request.page ? request.page : 1;
    const knex = db.knex.select('campanhas_registro.*',
        'status_ligacao.nome as status',
        'campanhas.nome as campanha',
        'campanhas.fila as fila',
        'classificacoes.nome as classificacao',
        'cdr_classificacoes.obs as observacao');
    if (request.campanhas) knex.whereIn('campanhas_registro.id_campanha', request.campanhas);
    if (request.status && request.status.length > 0) knex.whereIn('campanhas_registro.id_status', request.status);
    if (request.data_ini) knex.where('campanhas_registro.data_status', '>=', request.data_ini);
    if (request.data_fim) knex.where('campanhas_registro.data_status', '<=', request.data_fim);
    if (request.numero_operador && request.numero) {
        switch (request.numero_operador) {
            case 1:
                knex.where('campanhas_registro.numero', '=', request.numero);
                break;
            case 2:
                knex.where('campanhas_registro.numero', 'like', request.numero + '%');
                break;
            case 3:
                knex.where('campanhas_registro.numero', 'like', '%' + request.numero);
                break;
            case 4:
                knex.where('campanhas_registro.numero', 'not like', '%' + request.numero + '%');
                break;
            case 5:
                knex.where('campanhas_registro.numero', 'not like', request.numero);
                break;
        }
    }
    else if (request.numero) {
        knex.where('campanhas_registro.numero', 'like', '%' + request.numero + '%');
    }

    const ret = {};
    if (request.order) {
        for (let p = 0; p < request.order.length; ++p) {
            if (request.order[p] && request.order[p].campo && request.order[p].ordem) {
                knex.orderBy(request.order[p].campo, request.order[p].ordem);
            }
            else if (request.order[p] && request.order[p].campo) {
                knex.orderBy(request.order[p].campo, 'ASC');
            }
        }
    }
    await knex.from('campanhas_registro')
        .leftJoin('status_ligacao', 'campanhas_registro.id_status', 'status_ligacao.id')
        .leftJoin('campanhas', 'campanhas_registro.id_campanha', 'campanhas.id')
        .leftJoin('cdr_classificacoes', function () {
            this.on('campanhas_registro.callid', '=', 'cdr_classificacoes.uniqueid').orOn('campanhas_registro.actionid', '=', 'cdr_classificacoes.uniqueid')
        })
        .leftJoin('classificacoes', 'cdr_classificacoes.id_classificacao', 'classificacoes.id')
        .then(function (rows) {
            ret.error = false;
            ret.full = true;
            ret.total = rows.length;
            ret.message = 'Registros das campanhas';
            ret.dados = rows;
        })
        .catch(function (error) {
            console.error(error);
            return (Boom.badData(error));
        });

    return ret
};

exports.getRegistros = (request, reply) => {
    const campanha = request.params.CampanhaID;

    const max = request.query.max ? request.query.max : 15;
    const page = request.query.page ? request.query.page : 1;

    if (max === -1) {
        db.query("SELECT a.*,b.nome as status FROM public.campanhas_registro a LEFT JOIN status_ligacao b on a.id_status = b.id WHERE a.id_campanha = $1", [campanha])
            .then(res => {
                const ret = {};
                ret.error = false;
                ret.full = true;
                ret.total = res.rowCount;
                ret.message = 'Registros da Campanha';
                ret.dados = res.rows;
                reply(ret);
            })
            .catch(e => {
                reply(Boom.badData(e));
            });
    }
    else {
        const offset = max * (page - 1);
        db.query("SELECT count(*) as total FROM public.campanhas_registro WHERE id_campanha = $1", [campanha])
            .then(res1 => {
                db.query("SELECT a.*,b.nome as status FROM public.campanhas_registro a LEFT JOIN status_ligacao b on a.id_status = b.id WHERE a.id_campanha = $1 LIMIT $2 OFFSET $3", [campanha, max, offset])
                    .then(res => {
                        const pages = Math.ceil(parseInt(res1.rows[0].total) / max);
                        const ret = {};
                        ret.error = false;
                        ret.full = pages === 1 ? true : false;
                        ret.page = page;
                        ret.max = max;
                        ret.offset = offset;
                        ret.total = parseInt(res1.rows[0].total);
                        ret.pages = pages;
                        ret.message = 'Registros da campanha paginados';
                        ret.dados = res.rows;
                        reply(ret);
                    })
                    .catch(e => {
                        reply(Boom.badData(e));
                    });
            })
            .catch(e => {
                reply(Boom.badData(e));
            });
    }
};

exports.uploadContatos2 = async (request, reply) => {
    try {
        let processarCSV = function (dadosDoArquivo, delimitador) {
            return new Promise((resolve, reject) => {
                console.log(`Processando CSV...`);
                let registros = [];
                csv.fromString(dadosDoArquivo, { headers: false, delimiter: delimitador })
                    .on("data", data => registros.push(data))
                    .on("error", error => reject(`Erro ao processar o CSV: ${error}`))
                    .on("end", () => resolve(registros));
            });
        };

        // Puxa as propriedades providas para a rota
        const idCampanha = request.params.CampanhaID;
        const importacaoPrioritaria = request.query.prioridade;

        console.log(`Processando upload de contatos '${idCampanha}' ${importacaoPrioritaria ? 'como prioridade' : ''}`);

        // Seleciona o layout vinculado a campanha especificada
        let layout = await db.knex("public.layouts")
            .leftJoin("public.campanhas", "campanhas.id_layout", "layouts.id")
            .where("campanhas.id", idCampanha)
            .select("layouts.*");
        if (!layout || layout.length <= 0 || !layout[0]) return reply(Boom.badData(`Layout não encontrado, verifique se a campanha especificada existe e possui um layout vinculada a ela.`));
        layout = layout.find(x => x);

        console.log(`Layout -> `, layout);

        // Se o layout possuir cabeçalho, deve-se considerar apenas da linha 1 pra baixo
        let indiceInicialRegistros = layout.cabecalho ? 1 : 0;

        // Seleciona os campos do layout
        let camposDoLayout = await db.knex("public.layouts_campos")
            .where("id_layout", layout.id)
            .select("*");
        if (!camposDoLayout || camposDoLayout.length <= 0) return reply(Boom.badData(`O Layout '${layout.id}' não possui nenhum campo!`));

        console.log(`Campos do layout -> `, camposDoLayout);

        // Separa os campos do layout em telefones e campos extras
        let layoutTelefones = camposDoLayout.filter(x => x && x.xcontact && x.xcontact.startsWith("telefone"));
        let layoutCamposExtras = camposDoLayout.filter(x => layoutTelefones.indexOf(x) === -1);


        // Verifica se existem telefones no layout
        if (layoutTelefones && layoutTelefones.length > 0) {
            // Lê o conteúdo do arquivo recebido
            let dadosDoArquivo = request.payload.file.toString("latin1", 0, request.payload.file.length);
            // Da parse no conteúdo do arquivo
            let registros = await processarCSV(dadosDoArquivo, layout.delimitador);

            console.log(`Registros do csv -> `, registros);

            // Insere os contatos
            let contatos = Array(registros.length - indiceInicialRegistros).fill({ id_campanha: idCampanha });
            let idsInseridos = await db.knex.batchInsert("campanhas_contato", contatos, contatos.length).returning("id");
            console.log(`Foram inseridos -> `, idsInseridos);

            // Se for especificado uma inserção prioritária, busca no banco a última prioridade inserida para a campanha especificada e incrementa-a em 1
            let prioridade = 0;
            if (importacaoPrioritaria) {
                // Busca a prioridade atual
                let prioridadeAtual = await db.knex("public.campanhas_contato_numeros")
                    .where("id_campanha", idCampanha)
                    .max("prioridade");
                console.log("Prioridade atual->", prioridadeAtual);

                if (prioridadeAtual && prioridadeAtual.length > 0 && "max" in prioridadeAtual[0]) {
                    // Incrementa a prioridade
                    prioridadeAtual = parseInt(prioridadeAtual[0].max);
                    prioridade += (prioridadeAtual || 0) + 1;
                }

                console.log(`Prioridade a ser inserida -> `, prioridade);
            }

            console.log(`Populando contatos e números...`);
            // Prepara os campos para inserir no banco
            let listaNumeros = [];
            let listaCampos = [];
            for (let i = indiceInicialRegistros; i < registros.length; i++) {

                // Popula a lista de números
                for (let telefone of layoutTelefones) {
                    let registro = registros[i][telefone.posicao_csv];

                    if (registro && registro.trim().length > 0) {
                        listaNumeros.push({
                            id_contato: idsInseridos[i - indiceInicialRegistros],
                            id_campanha: idCampanha,
                            numero: TextoUtil.limparTextoNumerico(registro),
                            fone_sequencia: telefone.xcontact,
                            nome: telefone.nome,
                            prioridade
                        });
                    }
                }

                // Popula a lista de campos
                for (let campo of layoutCamposExtras) {
                    let registro = registros[i][campo.posicao_csv];

                    if (registro && registro.trim().length > 0) {
                        listaCampos.push({
                            id_contato: idsInseridos[i - indiceInicialRegistros],
                            id_campanha: idCampanha,
                            nome: campo.xcontact === "nome" ? TextoUtil.limparTexto(campo.nome) : campo.nome,
                            valor: registro
                        });
                    }
                }

            }

            // Insere os números
            console.log(`Inserindo números...`);
            await db.knex.batchInsert("campanhas_contato_numeros", listaNumeros);

            // Insere os campos
            console.log(`Inserindo campos...`);
            await db.knex.batchInsert("campanhas_contato_campos", listaCampos);

            console.log(`Done`);
            reply({
                error: false,
                rows: registros.length,
                message: 'Arquivo importado com sucesso.'
            });
        } else {
            reply(Boom.badRequest(`O layout associado a CampanhaID '${idCampanha}' não possui campos de telefone definidos.`));
        }
    } catch (error) {
        console.error(error);
        reply(Boom.badData(error));
    }
};

exports.uploadContatos = (request, reply) => {
    // Obter a campanha
    db.knex('public.campanhas')
        .where('id', request.params.CampanhaID)
        .select('id_layout')
        .then(function (rows) {
            if (rows.length > 0) {
                if (rows[0].id_layout && rows[0].id_layout !== null) {
                    const id_layout = rows[0].id_layout;
                    // Obter os dados do layout
                    db.knex('public.layouts')
                        .where('id', id_layout)
                        .select('*')
                        .then(function (layouts) {
                            const delimitador = layouts[0].delimitador;
                            const reg_ini = layouts[0].cabecalho ? 1 : 0;
                            // Obter campos
                            db.knex('public.layouts_campos')
                                .where('id_layout', id_layout)
                                .select('*')
                                .then(function (layouts_campos) {
                                    if (layouts_campos.length > 0) {
                                        const fones = [];
                                        const campos = [];
                                        for (let t = 0; t < layouts_campos.length; ++t) {
                                            if (layouts_campos[t].xcontact && layouts_campos[t].xcontact !== null && layouts_campos[t].xcontact.startsWith("telefone")) {
                                                fones.push(layouts_campos[t]);
                                            } else {
                                                campos.push(layouts_campos[t]);
                                            }
                                        }

                                        // Tem telefone vamos ler os registros
                                        if (fones.length > 0) {
                                            const dados = request.payload.file.toString('latin1', 0, request.payload.file.length);
                                            const registros = [];
                                            csv.fromString(dados, { headers: false, delimiter: delimitador })
                                                .on("data", data => registros.push(data))
                                                .on("end", function () {
                                                    const id_campanha = request.params.CampanhaID;
                                                    // Organiza os registros para insert do contato, para cada registro do arquivo insert 1 contato
                                                    const contatos = [];
                                                    for (let r1 = reg_ini; r1 < registros.length; ++r1) {
                                                        const contato = { id_campanha: id_campanha };
                                                        contatos.push(contato);
                                                    }

                                                    db.knex.batchInsert('campanhas_contato', contatos, registros.length)
                                                        .returning('id')
                                                        .then(function (ids) {
                                                            const array_campos = [];
                                                            const array_numeros = [];
                                                            for (let r = reg_ini; r < registros.length; ++r) {
                                                                // Numeros
                                                                for (let f = 0; f < fones.length; ++f) {
                                                                    if (registros[r][fones[f].posicao_csv] && registros[r][fones[f].posicao_csv].trim().length > 0) {
                                                                        const id_contato = reg_ini === 0 ? ids[r] : ids[r - 1];
                                                                        const cn = {
                                                                            id_contato: id_contato,
                                                                            id_campanha: id_campanha,
                                                                            numero: TextoUtil.limparTextoNumerico(registros[r][fones[f].posicao_csv]),// Remove qualquer caractere que não seja um dígito do número do contato
                                                                            fone_sequencia: fones[f].xcontact,
                                                                            nome: fones[f].nome
                                                                        };
                                                                        array_numeros.push(cn);
                                                                    }
                                                                }
                                                                // Campos
                                                                for (let c = 0; c < campos.length; ++c) {
                                                                    if (registros[r][campos[c].posicao_csv] && registros[r][campos[c].posicao_csv].trim().length > 0) {
                                                                        const id_contato = reg_ini === 0 ? ids[r] : ids[r - 1];
                                                                        const cc = {
                                                                            id_contato: id_contato,
                                                                            id_campanha: id_campanha,
                                                                            nome: campos[c].nome,
                                                                            valor: registros[r][campos[c].posicao_csv],
                                                                        };

                                                                        // Remove acentos do nome do contato
                                                                        if (campos[c].xcontact === "nome") cc.nome = TextoUtil.limparTexto(cc.nome);

                                                                        array_campos.push(cc);
                                                                    }
                                                                }
                                                            }
                                                            // Insert Numeros
                                                            db.knex.batchInsert('campanhas_contato_numeros', array_numeros)
                                                                .returning('id')
                                                                .then(function (ids_numeros) {
                                                                    // OK para os numeros insert campos
                                                                    db.knex.batchInsert('campanhas_contato_campos', array_campos)
                                                                        .returning('id')
                                                                        .then(function (ids_campos) {
                                                                            // OK, tudo inserido....
                                                                            reply({ error: false, rows: registros.length, message: 'Arquivo importado com sucesso.' });
                                                                        })
                                                                        .catch(function (error) {
                                                                            reply(Boom.badData(error));
                                                                        });
                                                                })
                                                                .catch(function (error) {
                                                                    reply(Boom.badData(error));
                                                                });
                                                        })
                                                        .catch(function (error) {
                                                            reply(Boom.badData(error));
                                                        });
                                                });
                                        }
                                        else {
                                            reply(Boom.badRequest('O layout associado a CampanhaID = ' + request.params.CampanhaID + ' não possui campos de telefone definidos.'));
                                        }
                                    }
                                    else {
                                        reply(Boom.badRequest('O layout associado a CampanhaID = ' + request.params.CampanhaID + ' não possui campos definidos.'));
                                    }
                                })
                                .catch(function (error) {
                                    console.error(error);
                                    reply(Boom.badData(error));
                                });
                        })
                        .catch(function (error) {
                            console.error(error);
                            reply(Boom.badData(error));
                        });


                } else {
                    reply(Boom.badRequest('A CampanhaID = ' + request.params.CampanhaID + ' não possui layout cadastrado.'));
                }
            } else {
                reply(Boom.badRequest('Nenhuma campanha encontrada para CampanhaID = ' + request.params.CampanhaID));
            }
        })
        .catch(function (error) {
            console.error(error);
            reply(Boom.badData(error));
        });
};

exports.uploadContatosJSON = (request, reply) => {
    // Obter a campanha
    db.knex('public.campanhas')
        .where('id', request.params.CampanhaID)
        .select('id_layout')
        .then(function (rows) {
            if (rows.length > 0) {
                if (rows[0].id_layout && rows[0].id_layout !== null) {
                    const id_layout = rows[0].id_layout;
                    // Obter os dados do layout
                    db.knex('public.layouts')
                        .where('id', id_layout)
                        .select('*')
                        .then(function (layouts) {
                            // Obter campos
                            db.knex('public.layouts_campos')
                                .where('id_layout', id_layout)
                                .select('*')
                                .then(function (layouts_campos) {
                                    if (layouts_campos.length > 0) {
                                        const fones = [];
                                        const campos = [];
                                        for (let t = 0; t < layouts_campos.length; ++t) {
                                            if (layouts_campos[t].xcontact && layouts_campos[t].xcontact !== null && layouts_campos[t].xcontact.startsWith("telefone")) {
                                                fones.push(layouts_campos[t]);
                                            }
                                            else {
                                                campos.push(layouts_campos[t]);
                                            }
                                        }
                                        // Tem telefone vamos ler os registros
                                        if (fones.length > 0) {
                                            const registros = request.payload;
                                            const id_campanha = request.params.CampanhaID;
                                            // Organiza os registros para insert do contato, para cada registro do arquivo insert 1 contato
                                            const contatos = [];
                                            for (let r1 = 0; r1 < registros.length; ++r1) {
                                                const contato = { id_campanha: id_campanha };
                                                contatos.push(contato);
                                            }
                                            db.knex.batchInsert('campanhas_contato', contatos, registros.length)
                                                .returning('id')
                                                .then(function (ids) {
                                                    const array_campos = [];
                                                    const array_numeros = [];
                                                    for (let r = 0; r < registros.length; ++r) {
                                                        // Numeros
                                                        for (let f = 0; f < fones.length; ++f) {
                                                            if (registros[r][fones[f].nome] && registros[r][fones[f].nome].trim().length > 0) {
                                                                const id_contato = ids[r];
                                                                const cn = {
                                                                    id_contato: id_contato,
                                                                    id_campanha: id_campanha,
                                                                    numero: registros[r][fones[f].nome],
                                                                    fone_sequencia: fones[f].xcontact,
                                                                    nome: fones[f].nome
                                                                };
                                                                array_numeros.push(cn);
                                                            }
                                                        }
                                                        // Campos
                                                        for (let c = 0; c < campos.length; ++c) {
                                                            if (registros[r][campos[c].nome] && registros[r][campos[c].nome].trim().length > 0) {
                                                                const id_contato = ids[r];
                                                                const cc = {
                                                                    id_contato: id_contato,
                                                                    id_campanha: id_campanha,
                                                                    nome: campos[c].nome,
                                                                    valor: registros[r][campos[c].nome],
                                                                };
                                                                array_campos.push(cc);
                                                            }
                                                        }
                                                    }
                                                    // Insert Numeros
                                                    db.knex.batchInsert('campanhas_contato_numeros', array_numeros)
                                                        .returning('id')
                                                        .then(function (ids_numeros) {
                                                            // OK para os numeros insert campos
                                                            db.knex.batchInsert('campanhas_contato_campos', array_campos)
                                                                .returning('id')
                                                                .then(function (ids_campos) {
                                                                    // OK, tudo inserido....
                                                                    reply({ error: false, rows: registros.length, message: 'Registros importados com sucesso.' });
                                                                })
                                                                .catch(function (error) {
                                                                    reply(Boom.badData(error));
                                                                });
                                                        })
                                                        .catch(function (error) {
                                                            reply(Boom.badData(error));
                                                        });
                                                })
                                                .catch(function (error) {
                                                    reply(Boom.badData(error));
                                                });
                                        }
                                        else {
                                            reply(Boom.badRequest('O layout associado a CampanhaID = ' + request.params.CampanhaID + ' não possui campos de telefone definidos.'));
                                        }
                                    }
                                    else {
                                        reply(Boom.badRequest('O layout associado a CampanhaID = ' + request.params.CampanhaID + ' não possui campos definidos.'));
                                    }
                                })
                                .catch(function (error) {
                                    console.error(error);
                                    reply(Boom.badData(error));
                                });
                        })
                        .catch(function (error) {
                            console.error(error);
                            reply(Boom.badData(error));
                        });


                }
                else {
                    reply(Boom.badRequest('A CampanhaID = ' + request.params.CampanhaID + ' não possui layout cadastrado.'));
                }
            }
            else {
                reply(Boom.badRequest('Nenhuma campanha encontrada para CampanhaID = ' + request.params.CampanhaID));
            }
        })
        .catch(function (error) {
            console.error(error);
            reply(Boom.badData(error));
        });
};

exports.remove_contato = (request, reply) => {
    const knex = db.knex;
    knex.transaction(function (t) {
        return knex('campanhas_contato')
            .transacting(t)
            .where('id', request.params.ContatoID)
            .where('id_campanha', request.params.CampanhaID)
            .del()
            .then(function () {
                return knex('campanhas_contato_numeros')
                    .where('id_contato', request.params.ContatoID)
                    .where('id_campanha', request.params.CampanhaID)
                    .del()
                    .then(function () {
                        return knex('campanhas_contato_campos')
                            .where('id_contato', request.params.ContatoID)
                            .where('id_campanha', request.params.CampanhaID)
                            .del();
                    })
            })
            .then(t.commit)
            .catch(function (e) {
                t.rollback();
                throw e;
            })
    })
        .then(function () {
            const ret = {};
            ret.error = false;
            ret.message = 'Contato id = ' + request.params.ContatoID + ' apagado com sucesso.';
            reply(ret);
        })
        .catch(function (e) {
            reply(Boom.badData("Nenhum resgistro encontrado para ID = " + request.params.ContatoID));
        });
};

exports.remove_contato_por_campo = (request, reply) => {
    const knex = db.knex;
    knex.transaction(function (t) {
        return knex.select('id_contato')
            .from('campanhas_contato_campos')
            .where('nome', request.params.CampoNome)
            .where('valor', request.params.CampoValor)
            .then(function (rows) {
                if (rows.length > 0) {
                    return knex('campanhas_contato')
                        .transacting(t)
                        .where('id', rows[0].id_contato)
                        .where('id_campanha', request.params.CampanhaID)
                        .del()
                        .then(function () {
                            return knex('campanhas_contato_numeros')
                                .where('id_contato', rows[0].id_contato)
                                .where('id_campanha', request.params.CampanhaID)
                                .del()
                                .then(function () {
                                    return knex('campanhas_contato_campos')
                                        .where('id_contato', rows[0].id_contato)
                                        .where('id_campanha', request.params.CampanhaID)
                                        .del();
                                })
                        })
                        .then(t.commit)
                        .catch(function (e) {
                            t.rollback();
                            throw e;
                        })
                }
                else {
                    t.rollback();
                    throw new Error("Nenhum resgistro encontrado para " + request.params.CampoNome + " = " + request.params.CampoValor);
                }
            })
            .catch(function (error) {
                throw e;
            });
    })
        .then(function () {
            const ret = {};
            ret.error = false;
            ret.message = 'Contato removido com sucesso.';
            reply(ret);
        })
        .catch(function (e) {
            reply(Boom.badData("Nenhum resgistro encontrado para " + request.params.CampoNome + " = " + request.params.CampoValor));
        });
};

exports.remove_todos_contatos = (request, reply) => {
    const knex = db.knex;
    knex.transaction(function (t) {
        return knex('campanhas_contato')
            .transacting(t)
            .where('id_campanha', request.params.CampanhaID)
            .del()
            .then(function () {
                return knex('campanhas_contato_numeros')
                    .where('id_campanha', request.params.CampanhaID)
                    .del()
                    .then(function () {
                        return knex('campanhas_contato_campos')
                            .where('id_campanha', request.params.CampanhaID)
                            .del();
                    })
            })
            .then(t.commit)
            .catch(function (e) {
                t.rollback();
                console.log(e);
                throw e;
            })
    })
        .then(function () {
            const ret = {};
            ret.error = false;
            ret.message = 'Contatos pagados da campanha id = ' + request.params.CampanhaID;
            reply(ret);
        })
        .catch(function (e) {
            reply(Boom.badData(e));
        });
};

exports.getFilaDiscagem = (request, reply) => {
    const max = request.query.max ? request.query.max : 15;
    const page = request.query.page ? request.query.page : 1;
    const knex = db.knex('campanhas_contato_numeros');
    const pag = new Paginacao(max, page);

    knex.where('campanhas_contato_numeros.id_campanha', request.params.CampanhaID);

    knex.where('campanhas_contato_numeros.sucesso', false);
    knex.whereRaw("campanhas_contato_numeros.tentativas < campanhas.tentativas");
    knex.whereRaw('campanhas_contato_numeros.retentativas < campanhas.tentativas');

    knex.innerJoin('campanhas', 'campanhas_contato_numeros.id_campanha', 'campanhas.id');

    pag.select = [
        'campanhas_contato_numeros.*',
        db.knex.raw("(SELECT jsonb_agg(jsonb_build_object(nome, valor)) AS campos FROM campanhas_contato_campos WHERE campanhas_contato_campos.id_contato = campanhas_contato_numeros.id_contato) AS campos"),
        'campanhas.nome as campanha'
    ];

    pag.order = [
        { "campo": "campanhas_contato_numeros.prioridade", "ordem": "DESC" },
        { "campo": "campanhas_contato_numeros.fone_sequencia" },
        { "campo": "campanhas_contato_numeros.tentativas" },
        { "campo": "campanhas_contato_numeros.retentativas" },
        { "campo": "campanhas_contato_numeros.id_contato" }
    ];

    pag.knex = knex;

    pag.getRet((ret, err) => {
        if (err) {
            reply(Boom.badData(err));
        }
        else {
            reply(ret);
        }
    });
};