'use strict'
const Joi = require('joi');
const CRUD = require('../../util/CRUD');
const md5 = require('md5');
const db = require('../../util/db');
const Boom = require('boom');
const handlers = require('./handlers');
const util = require('../../util/restbase');
const Ws = require('../../ws/index');
const ClienteAtendimento = require('../../models/ClienteAtendimento');
const AtendimentoMsg = require('../../models/AtendimentoMensagem');
const Anexos = require('../../models/MensagemAnexos');
const Cliente = require('../../models/Cliente');

module.exports = [
  {
    method: 'POST',
    path: '/api/v2/chat',
    handler: async (request, reply) => {
      try {
        let dados = JSON.parse(request.payload.dados);
        dados.idCliente = dados.numero;
        let cliente = await Cliente.getClienteFromcall(dados);
        let atendimento = await new ClienteAtendimento({ fromID: dados.numero, fromName: cliente.nome, queue: dados.queue, idCliente: cliente.id, ativo: true, status: 'Entrou', queueTypeID: 1, action: 'Tocando', dadosAdicionais: dados.dadosAdicionais }).save();
        reply(atendimento);
      } catch (error) {
        reply(Boom.badData(error));
      }
    },
    config: {
      description: 'Gera um novo atendimento',
      tags: ['api', 'v2', 'Chat'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        payload: Joi.object({
          dados: Joi.string().required().description('Dados adicionais'),
        })
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/atendimentos/{idCliente}',
    handler: async (request, reply) => {
      try {
        let atendimentos = await ClienteAtendimento.query
          .require(false)
          .where({ idCliente: request.params.idCliente })
          .fetch();

        reply(atendimentos);
      } catch (error) {
        reply(error);
      }
    },
    config: {
      description: 'Obtem os chats',
      tags: ['api', 'v2', 'Atendimentos'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          idCliente: Joi.number().integer().required().description('id do cliente')
        }
      }
    }
  },

  {
    method: 'GET',
    path: '/api/v2/chats',
    handler: (request, reply) => {
      const crud = new CRUD('queueChat AS c');
      // Ativa/Desativa Knexnest
      crud.knexnest_op = request.query.FullMsg;

      crud.filtrosraw = [];
      crud.filtros = [];

      if (request.query.id) crud.filtros.push({ campo: 'c.id', operador: '=', valor: request.query.id });
      if (request.query.fromName) crud.filtrosraw.push("LOWER(\"c\".\"fromName\") LIKE '%" + decodeURIComponent(request.query.fromName).toLowerCase() + "%'");
      if (request.query.assunto) crud.filtrosraw.push("LOWER(c.assunto) LIKE LIKE '%" + decodeURIComponent(request.query.assunto).toLowerCase() + "%'");
      if (request.query.action) crud.filtros.push({ campo: 'c.action', operador: '=', valor: request.query.action });
      if (request.query.ativo !== undefined) crud.filtros.push({ campo: 'c.ativo', operador: '=', valor: request.query.ativo });
      if (request.query.filas) crud.filtrosraw.push('c.queue = ANY(ARRAY' + JSON.stringify(request.query.filas).replace(/"/g, "'") + ')');
      if (request.query.agentes) crud.filtrosraw.push('"c"."ultimoAgente" = ANY(ARRAY' + JSON.stringify(request.query.agentes).replace(/"/g, "'") + ')');
      if (request.query.dataIni) crud.filtros.push({ campo: '"c"."dataIni"', operador: '>=', valor: request.query.dataIni });
      if (request.query.dataFim) crud.filtros.push({ campo: '"c"."dataIni"', operador: '<=', valor: request.query.dataFim });
      if (request.query.sentido) crud.filtrosraw.push('c.sentido = ANY(ARRAY' + JSON.stringify(request.query.sentido).replace(/"/g, "'") + ')');
      if (request.query.queueTypeID) crud.filtrosraw.push('"c"."queueTypeID" = ANY(ARRAY' + JSON.stringify(request.query.queueTypeID).replace(/"/g, "'") + ')');
      if (request.query.idCliente) crud.filtrosraw.push('"c"."idCliente" = ANY(ARRAY[' + request.query.idCliente + '])');

      // Filtro Aguardando não existe, é o status = Entrou com ultimoAgente null      
      if (request.query.status && request.query.status.includes('Aguardando')) {
        request.query.status = request.query.status.map(x => x.replace('Aguardando', 'Entrou'));

        crud.filtrosraw.push('c.status = ANY(ARRAY' + JSON.stringify(request.query.status).replace(/"/g, "'") + ')');
        crud.filtrosraw.push('"c"."ultimoAgente" is null');
      }
      else if (request.query.status && request.query.status.includes('Entrou') && !request.query.agentes) {
        crud.filtrosraw.push('c.status = ANY(ARRAY' + JSON.stringify(request.query.status).replace(/"/g, "'") + ')');
        crud.filtrosraw.push('"c"."ultimoAgente" is not null');
      }
      else if (request.query.status) crud.filtrosraw.push('c.status = ANY(ARRAY' + JSON.stringify(request.query.status).replace(/"/g, "'") + ')');

      // if (request.query.classificacoes) crud.filtrosraw.push(/*sql*/`"c"."classificacoes" = ANY(ARRAY[${request.query.classificacoes.map(x => "'" + x + "'").join(", ")}])`);
      if (request.query.classificacoes) crud.filtrosraw.push(/*sql*/`"qcc"."classificacaoID" = ANY(ARRAY[${request.query.classificacoes.join(", ")}])`);

      if (request.query.order) crud.order = request.query.order;


      crud.leftjoin = [
        { tabela: "queueChatClassificacao AS qcc", campos: ["qcc.queueChatID", "c.id", "qcc.agente", "c.ultimoAgente"] },
        { tabela: "classificacoes AS cla", campo1: "qcc.classificacaoID", campo2: "cla.id" },
        { tabela: "subclassificacao AS subcla", campo1: "qcc.subClassificacaoID", campo2: "subcla.id" },
        { tabela: "cliente AS cli", campo1: "cli.id", campo2: "c.idCliente" },
      ];

      crud.select = [
        'c.*',
        'cla.nome as classificacao',
        'subcla.nome as subclassificacao',
        'qcc.obs as obs',
        'cli.profileImage',
        'cli.nome as clienteNome',
        'cli.id as idCliente',
        'cli.numero as numero',

      ];

      if (crud.knexnest_op) {
        crud.innerjoin = [
          { tabela: "queueChatMessages AS m", campo1: "m.idQueueChat", campo2: "c.id" }
        ];
        crud.leftjoin = [
          { tabela: "queueChatClassificacao AS qcc", campos: ["qcc.queueChatID", "c.id", "qcc.agente", "c.ultimoAgente"] },
          { tabela: "classificacoes AS cla", campo1: "qcc.classificacaoID", campo2: "cla.id" },
          { tabela: "subclassificacao AS subcla", campo1: "qcc.subClassificacaoID", campo2: "subcla.id" },
          { tabela: "cliente AS cli", campo1: "cli.id", campo2: "c.idCliente" },
          { tabela: "queueAnexos AS anexo", campo1: "m.id", campo2: "anexo.id_message" }
        ];
        crud.select = [
          'c.id as _id',
          'c.sentido as _sentido',
          'c.fromID as _fromID',
          'c.fromName as _fromName',
          'c.assunto as _assunto',
          'c.action as _action',
          'c.ativo as _ativo',
          'c.queue as _queue',
          'c.ultimoAgente as _ultimoAgente',
          'c.dataIni as _dataIni',
          'c.dataFim as _dataFim',
          'c.status as _status',
          'c.queueTypeID as _queuetype',
          'c.linkedid as _linkedid',
          'c.idCliente as _idCliente',
          'cli.numero as _numero',
          'cla.nome as _classificacao',
          'subcla.nome as _subclassificacao',
          'qcc.obs as _obs',
          'm.id as _queueChatMessages__id',
          'm.dataHora as _queueChatMessages__dataHora',
          'm.inOut as _queueChatMessages__inOut',
          'm.agente as _queueChatMessages__agente',
          'm.texto as _queueChatMessages__texto',
          'm.status as _queueChatMessages__status ',
          'm.repliedIdMsg as _queueChatMessages__repliedIdMsg',
          'cli.profileImage as _image'

        ];
      }
      crud.getAll(request, reply);
    },
    config: {
      description: 'Obtem os chats',
      tags: ['api', 'v2', 'Chat'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          FullMsg: Joi.bool().default(false).description('Se TRUE traz os dados do chat e TODAS as mensagens relacionadas a ele, não suporta paginação serverside'),
          order: Joi.array().items(
            Joi.object().keys({
              campo: Joi.string().description('Nome da coluna'),
              ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          id: Joi.number().integer().allow(null).description('Filtra por ID (Protocolo)'),
          fromID: Joi.string().allow(null).description('Filtra por numero do cliente'),
          fromName: Joi.string().allow(null).description('Filtra por nome do cliente'),
          assunto: Joi.string().allow(null).description('Filtra por assunto'),
          action: Joi.string().allow(null).description('Filtra por action'),
          idCliente: Joi.array().allow(null).items(Joi.number().integer()).description('Filtra por ID de Clientes'),
          ativo: Joi.boolean().allow(null).description('Filtra por ativo sim ou não'),
          classificacoes: Joi.array().allow(null).items(Joi.number()).description('Filtra por classificações'),
          filas: Joi.array().allow(null).items(Joi.string()).description('Filtra por filas'),
          agentes: Joi.array().allow(null).items(Joi.string()).description('Filtra por agentes'),
          status: Joi.array().allow(null).items(Joi.string()).description('Filtra por status'),
          queuetype: Joi.array().allow(null).items(Joi.number().integer()).description('Filtra por tipo, verifique os tipos disponíveis em GET fila/tipos'),
          dataIni: Joi.date().iso().example('2018-01-01 08:00:00').description('Data inicial'),
          dataFim: Joi.date().iso().example('2018-01-01 23:59:59').description('Data final'),
          sentido: Joi.array().allow(null).items(Joi.string()).description('Filtra por sentido'),
          queueTypeID: Joi.array().allow(null).items(Joi.number().integer()).description('Filtra por tipo de fila')
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/message/{IDWHATS}',
    handler: (request, reply) => {
      const crud = new CRUD('queueChatMessages AS q');
      crud.knexnest_op = true;
      crud.filtros = [];
      crud.filtrosraw = [];
      crud.filtros.push({ campo: '"repliedIdMsg"', operador: '=', valor: request.params.IDWHATS });
      crud.leftjoin = [
        { tabela: "queueAnexos AS b", campo1: "q.id", campo2: "b.id_message" }
      ];
      crud.select = [
        'q.id as _id',
        'q.idQueueChat as _idQueueChat',
        'q.dataHora as _dataHora',
        'q.inOut as _inOut',
        'q.agente as _agente',
        'q.texto as _texto',
        'q.repliedIdMsg as _repliedIdMsg',
        'q.status as _status',
        'q.mailMessageID as _mailMessageID',
        'b.id as _queueAnexos__id',
        'b.caminho as _queueAnexos__caminho',
        'b.mimeType as _queueAnexos__mimeType'
      ];
      crud.getAll(request, reply);
    }
  },
  {
    method: 'GET',
    path: '/api/v2/chat/{ID}/mensagens',
    handler: handlers.getMensagens,
    config: {
      description: 'Obtem as mensagens de um Chat',
      tags: ['api', 'v2', 'Chat'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        },
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
              campo: Joi.string().description('Nome da coluna'),
              ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          texto: Joi.string().allow(null).description('Filtra por texto'),
          inOut: Joi.number().integer().valid(1, 2).description('1 - Entrante, 2 - Sainte'),
          agentes: Joi.array().allow(null).items(Joi.string()).description('Filtra por agentes'),
          dataIni: Joi.date().allow(null).iso().example('2018-01-01 08:00:00').description('Data inicial'),
          dataFim: Joi.date().allow(null).iso().example('2018-01-01 23:59:59').description('Data final'),
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v2/chat/{ID}/classificacao',
    handler: handlers.classifica,
    config: {
      description: 'Classifica um chat pelo ID (Protocolo) e Agente. Faz REPLACE, remove a classificação anterior feita pelo usuário e substitui pela enviada, ou cria uma nova.',
      tags: ['api', 'v2', 'Chat'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required().description('ID do Chat(Protocolo)')
        },
        payload: Joi.object({
          agente: Joi.string().required().example('Agente01').description('Nome do Agente'),
          classificacaoID: Joi.number().integer().allow(null).description('ID da classificação').example(1),
          subClassificacaoID: Joi.number().allow(null).integer().description('ID da Subclassificacao').example(1),
          obs: Joi.string().allow(null).description('Observação').example('Cliente desligou'),
        })
      }
    }
  },

  {
    method: 'GET',
    path: '/api/v2/chat/{fromID}/protocolosByClienteID',
    handler: (request, reply) => {
      const crud = new CRUD('queueChat');
      crud.filtros = [];
      crud.filtros.push({ campo: '"idCliente"', operador: '=', valor: request.params.fromID });
      if (request.query.order) crud.order = request.query.order;
      crud.getAll(request, reply);
      //crud.select = ['id'];
    },
    config: {
      description: 'Obtem os protocolos ja abertos pelo cliente',
      tags: ['api', 'v2', 'Chat'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          fromID: Joi.string().required().description('fromID do Chat (Numero ou email)')
        },
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
              campo: Joi.string().description('Nome da coluna'),
              ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          agentes: Joi.array().allow(null).items(Joi.string()).description('Filtra por agentes'),
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/chat/{ID}/classificacoes',
    handler: (request, reply) => {
      const crud = new CRUD('queueChatClassificacao AS a');
      crud.filtros = [];
      crud.filtrosraw = [];
      crud.filtros.push({ campo: 'a."queueChatID"', operador: '=', valor: request.params.ID });
      if (request.query.agentes) crud.filtrosraw.push('a.agente = ANY(ARRAY' + JSON.stringify(request.query.agentes).replace(/"/g, "'") + ')');
      if (request.query.order) crud.order = request.query.order;
      crud.leftjoin = [
        { tabela: "classificacoes AS c", campo1: "a.classificacaoID", campo2: "c.id" },
        { tabela: "subclassificacao AS s", campo1: "a.subClassificacaoID", campo2: "s.id" }
      ];
      crud.select = [
        'a.dataRegistro',
        'a.queueChatID as protocolo',
        'a.agente',
        'a.obs',
        'a.classificacaoID',
        'c.nome as classificacao',
        'a.subClassificacaoID',
        's.nome as subclassificacao'
      ];

      crud.getAll(request, reply);
    },
    config: {
      description: 'Obtem as classificações de um Chat',
      tags: ['api', 'v2', 'Chat'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required().description('ID do Chat (Protocolo)')
        },
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
              campo: Joi.string().description('Nome da coluna'),
              ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          agentes: Joi.array().allow(null).items(Joi.string()).description('Filtra por agentes'),
        }
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v2/chat/{ID}',
    handler: (request, reply) => {
      const crud = new CRUD('queueChat');
      crud.update(request, reply);
    },
    config: {
      description: 'Atualiza um chat pelo ID',
      tags: ['api', 'v2', 'Chat'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required().description('ID do Chat(Protocolo)')
        },
        payload: Joi.object({
          fromID: Joi.string().allow(null).description('fromID'),
          fromName: Joi.string().allow(null).description('fromName'),
          assunto: Joi.string().allow(null).description('Assunto'),
          action: Joi.string().allow(null).description('Action'),
          ativo: Joi.boolean().allow(null).description('Ativo'),
          queue: Joi.string().allow(null).description('Fila'),
          dataIni: Joi.date().allow(null).iso().example('2018-01-01 08:00:00').description('Data inicial'),
          dataFim: Joi.date().allow(null).iso().example('2018-01-01 23:59:59').description('Data final'),
          status: Joi.string().allow(null).description('Status'),
          ultimoAgente: Joi.string().allow(null).description('Nome do último agente que atendeu'),
          clienteConectado: Joi.boolean().allow(null).description('Cliente conectado'),
          dadosAdicionais: Joi.string().allow(null).description('Dados adicionais'),
          email: Joi.string().allow(null).description('Email'),
          queueTypeID: Joi.number().integer().allow(null).description('1 - Entrante, 2 - Sainte'),
        })
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v2/chat/encerrar/{ID}',
    handler: (request, reply) => {
      Ws.wsEvento.emit('AtendimentoEncerrado', { protocolo: request.params.ID })
      reply();
    },
    config: {
      description: 'Encerra um chat pelo ID',
      tags: ['api', 'v2', 'Chat'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required().description('ID do Chat(Protocolo)')
        }
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v2/chat/encerrarlote',
    handler: (request, reply) => {
      for (let i = 0; i < request.payload.lote.length; i++) {
        Ws.wsEvento.emit('AtendimentoEncerrado', { protocolo: request.payload.lote[i] })
      }
      reply(true);
    },
    config: {
      description: 'Encerra um chat pelo ID',
      tags: ['api', 'v2', 'Chat'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        payload: Joi.object({
          lote: Joi.array().items(Joi.number().integer()).description('Lote de atendimentos para encerrar')
        })
      }
    }
  },
  {
    method: "POST",
    path: "/api/v2/chat/transferir/{agente}",
    handler: handlers.transferir,
    config: {
      description: "Tranfere um ou mais atendimentos para o agente especificado",
      tags: ["api", "v2", "Chat"],
      auth: {
        strategy: "bearer",
        scope: "user"
      },
      validate: {
        params: {
          agente: Joi.string().min(1).required().description("Nome do agente a ser transferido!")
        },
        payload: {
          atendimentosId: Joi.array().items(Joi.number().integer()).required().description("Array que indica os id's dos atendimentos a serem transferidos"),
          fila: Joi.string().required().description("Nome da fila destino")
        }
      }
    }
  },
  {
    method: "POST",
    path: "/api/v2/chat/transferirParaFila",
    handler: handlers.transferirParaFila,
    config: {
      description: "Tranfere um ou mais atendimentos para o agente especificado",
      tags: ["api", "v2", "Chat"],
      auth: {
        strategy: "bearer",
        scope: "user"
      },
      validate: {
        payload: {
          atendimentosId: Joi.array().items(Joi.number().integer()).required().description("Array que indica os id's dos atendimentos a serem transferidos"),
          fila: Joi.string().required().description("Nome da fila destino")
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/chat/fluxo',
    handler: handlers.fluxo,
    config: {
      description: 'Retorna o Fluxo de utilização do Chat',
      tags: ['api', 'v2', 'Chat'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          data_ini: Joi.date().iso().description('Filtro por data inicial'),
          data_fim: Joi.date().iso().description('Filtro por data Final'),
          id_client: Joi.object().keys(util.SearchExpressionObject()).allow(null).description('Filtro pela Identificação do Cliente (recurso disponível mediante solcitiação).'),
          opcao: Joi.object().keys(util.SearchExpressionObject()).allow(null).description('Filtro da Opção.'),
          menu: Joi.object().keys(util.SearchExpressionObject()).allow(null).description('Filtro Menu da Opção.'),
          resposta: Joi.object().keys(util.SearchExpressionObject()).allow(null).description('Filtro de Resposta Obtida.'),
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/chat/classificacoes',
    handler: handlers.getAllClassificacoes,
    config: {
      description: 'Retorna todas as classificações realizadas',
      tags: ['api', 'v2', 'Chat'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          data_ini: Joi.date().raw().optional().allow(null).description('Data inicial'),
          data_fim: Joi.date().raw().optional().allow(null).description('Data Final'),
          hora_ini: Joi.string().optional().allow(null).description('Hora inicial'),
          hora_fim: Joi.string().optional().allow(null).description('Hora final'),
          filas: Joi.array().allow(null).items(Joi.string()).description('Filtra por filas'),
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/chat/{ID}',
    handler: handlers.getChat,
    config: {
      description: 'Obtem os dados de um Chat',
      tags: ['api', 'v2', 'Chat'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/chat/agentes_eventos',
    handler: handlers.AgentesEventos,
    config: {
      description: 'Retorna os eventos realizados pelos agentes em um chat',
      tags: ['api', 'v2', 'Chat'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          data_ini: Joi.date().iso().description('Filtro por data inicial'),
          data_fim: Joi.date().iso().description('Filtro por data Final'),
          filas: Joi.array().allow(null).items(Joi.string()).description('Filtra por filas'),
          agentes: Joi.array().allow(null).items(Joi.string()).description('Filtra por agentes'),
          queueTypeID: Joi.array().allow(null).items(Joi.number().integer()).description('Filtra por tipo de fila'),
        }
      }
    }
  },
];
