'use strict'

const Joi = require('joi');
const handlers = require('./handlers');

const CreateReturnSchema = {
  error: Joi.boolean().example(false).description('Indica se o processo gerou algum erro.'),
  name: Joi.string().example('fila1').description('ID da fila'),
  message: Joi.string().example('Registro criado com aucesso.')
};

const ListaReturnSchema = {
  error: Joi.boolean().example(false).description('Indica se o processo gerou algum erro.'),
  full: Joi.boolean().example(false).description('Indica se o retorno é completo, caso seja false será paginado.'),
  total: Joi.number().integer().example('10').description('Quantidade de registros retornados'),
  page: Joi.number().integer().example('1').description('Pagina atual dos registros'),
  offset: Joi.number().integer().example('1').description('Offset'),
  pages: Joi.number().integer().example('1').description('total de página'),
  max: Joi.number().integer().example('15').description('Máximo de registros por página'),
  pages: Joi.number().integer().example('1').description('total de páginas'),
  message: Joi.string().example('Lista de registros').description('Descrição da lista'),
  dados: Joi.array().items().example([{ id: 1, nome: "Registro1" }, { id: 2, nome: "Registro2" }]).description('Lista dos registros')
};

module.exports = [
  {
    method: 'GET',
    path: '/api/v2/classificacoes',
    handler: handlers.getall,
    config: {
      description: 'Obtem as classificações',
      tags: ['api', 'v2', 'Classificações'],
      auth: {
        strategy: 'bearer',
        scope: ['user', 'app.classificacoes'] // or [ 'user', 'admin' ]
      },
      response: {
        schema: ListaReturnSchema
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
              campo: Joi.string().description('Nome da coluna'),
              ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          nome: Joi.string().description('Filtra por nome'),
          filas: Joi.array().allow(null).items(Joi.string()).description('Filtra por filas')
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/classificacao/{ID}',
    handler: handlers.getOne,
    config: {
      description: 'Obtem os dados da classificação',
      tags: ['api', 'v2', 'Classificações'],
      auth: {
        strategy: 'bearer',
        scope: ['user', 'app.classificacoes'] // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'DELETE',
    path: '/api/v2/classificacao/{ID}',
    handler: handlers.delete,
    config: {
      description: 'exclui pelo ID',
      tags: ['api', 'v2', 'Classificações'],
      auth: {
        strategy: 'bearer',
        scope: ['app.classificacoes', 'admin']
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v2/classificacao',
    handler: handlers.create,
    config: {
      description: 'Cria',
      tags: ['api', 'v2', 'Classificações'],
      auth: {
        strategy: 'bearer',
        scope: ['app.classificacoes', 'admin']
      },
      validate: {
        payload: Joi.object({
          nome: Joi.string().description('Nome'),
          fila: Joi.string().allow(null).allow('').optional().description('Fila'),
        })
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v2/classificacao/{ID}',
    handler: handlers.update,
    config: {
      description: 'Atualiza',
      tags: ['api', 'v2', 'Classificações'],
      auth: {
        strategy: 'bearer',
        scope: ['app.classificacoes', 'admin']
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        },
        payload: Joi.object({
          nome: Joi.string().description('Nome'),
          fila: Joi.string().allow(null).allow('').optional().description('Fila'),
        })
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/classificacao/{ID}/subclassificacoes',
    handler: handlers.getSubcla,
    config: {
      description: 'Obtem As subclassificações da classificação',
      tags: ['api', 'v2', 'Classificações'],
      auth: {
        strategy: 'bearer',
        scope: ['user', 'app.classificacoes']
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v2/classificacao/{ID}/subclassificacoes',
    handler: handlers.updateSubcla,
    config: {
      description: 'Atualiza subclassificações',
      tags: ['api', 'v2', 'Classificações'],
      auth: {
        strategy: 'bearer',
        scope: ['app.classificacoes', 'admin', 'user']
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        },
        payload: Joi.object({
          apagar: Joi.array().items(Joi.number().integer()).optional().allow(null),
          update: Joi.array().items({
            id: Joi.number().integer().optional().allow(null),
            idClassificacao: Joi.number().integer().required(),
            nome: Joi.string().required()
          }).description('Array de subclassificações')
        })
      }
    }
  },
];
