'use strict'

const Joi = require('joi');
const handlers = require('./handlers');

const updata = Joi.object({
  nome: Joi.string().required(),
  numero: Joi.string().allow(null).allow(""),
  email: Joi.string().allow(null).allow(""),
  facebookID: Joi.string().allow(null).allow(""),
  documento: Joi.string().allow(null).allow(""),
  isBusiness: Joi.boolean().allow(null).allow(""),
  profileImage: Joi.string().allow(null).allow(""),
  idGrupo: Joi.number().integer().allow(null).allow(""),
  NumerosAdicionais: Joi.array().allow(null).items(
    Joi.object().keys({
      nome: Joi.string().description('Nome do número'),
      numero: Joi.string().description('Número'),
    })
  ).description('Array de números adicionais'),
  CamposAdicionais: Joi.array().allow(null).items(
    Joi.object().keys({
      nome: Joi.string().description('Nome para o campo'),
      valor: Joi.string().description('Valor para o campo'),
    })
  ).description('Array de campos adicionais'),
  url: Joi.string().allow(null, "").optional(),
});

module.exports = [
  {
    method: 'GET',
    path: '/api/v2/clientes',
    handler: handlers.getClientes,
    config: {
      description: 'Obtem todos os clientes',
      tags: ['api', 'v2', 'Clientes'],
      auth: {
        strategy: 'bearer',
        scope: ['app.clientes', 'admin', 'app.webchat']
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
              campo: Joi.string().description('Nome da coluna'),
              ordem: Joi.string().valid('asc', 'desc').description('asc ou desc')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"asc ou desc"}'),
          nome: Joi.string().allow('').allow(null).description('Filtra por nome'),
          numero: Joi.string().allow('').allow(null).description('Filtra por numero'),
          documento: Joi.string().allow('').allow(null).description('Filtra por documento'),
          idGrupo: Joi.number().integer().allow('').allow(null).description('Filtra por grupo'),
          searchText: Joi.string().allow('').allow(null).description('Filtra por nome, numero, documento e email'),
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/clientesfull',
    handler: handlers.getClientesFull,
    config: {
      description: 'Obtem os clientes com todos os campos',
      tags: ['api', 'v2', 'Clientes'],
      auth: {
        strategy: 'bearer',
        scope: ['app.clientes', 'admin']
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
              campo: Joi.string().description('Nome da coluna'),
              ordem: Joi.string().valid('asc', 'desc').description('asc ou desc')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"asc ou desc"}'),
          nome: Joi.string().allow('').allow(null).description('Filtra por nome'),
          numero: Joi.string().allow('').allow(null).description('Filtra por numero'),
          documento: Joi.string().allow('').allow(null).description('Filtra por documento'),
          idGrupo: Joi.number().integer().allow('').allow(null).description('Filtra por grupo'),
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/clientesGrupos',
    handler: handlers.getClientesGrupos,
    config: {
      description: 'Obtem todos os grupos de clientes',
      tags: ['api', 'v2', 'Clientes'],
      auth: {
        strategy: 'bearer',
        scope: ['app.clientes', 'admin']
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
              campo: Joi.string().description('Nome da coluna'),
              ordem: Joi.string().valid('asc', 'desc').description('asc ou desc')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),

        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/clientes/{ID}',
    handler: handlers.getCliente,
    config: {
      description: 'Obtem os dados do cliente pelo ID',
      tags: ['api', 'v2', 'Clientes'],
      auth: {
        strategy: 'bearer',
        scope: ['app.clientes', 'admin']
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/clientesByNumero/{NUMERO}',
    handler: handlers.getClienteByNumero,
    config: {
      description: 'Obtem os dados do cliente pelo Número',
      tags: ['api', 'v2', 'Clientes'],
      auth: {
        strategy: 'bearer',
        scope: ['app.clientes', 'admin']
      },
      validate: {
        params: {
          NUMERO: Joi.string().required()
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/clientesByDocumento/{DOCUMENTO}',
    handler: handlers.getClienteByDocumento,
    config: {
      description: 'Obtem os dados do cliente documento',
      tags: ['api', 'v2', 'Clientes'],
      auth: {
        strategy: 'bearer',
        scope: ['app.clientes', 'admin']
      },
      validate: {
        params: {
          DOCUMENTO: Joi.string().required()
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v2/clientes',
    handler: handlers.createCliente,
    config: {
      description: 'Cria um cliente',
      tags: ['api', 'v2', 'Clientes'],
      auth: {
        strategy: 'bearer',
        scope: ['app.clientes', 'admin']
      },
      validate: {
        payload: updata
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v2/clientes/{ID}',
    handler: handlers.updateCliente,
    config: {
      description: 'Atualiza um cliente',
      tags: ['api', 'v2', 'Clientes'],
      auth: {
        strategy: 'bearer',
        scope: ['app.clientes', 'admin']
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        },
        payload: updata
      }
    }
  },
  {
    method: 'DELETE',
    path: '/api/v2/clientes/{ID}',
    handler: handlers.deleteCliente,
    config: {
      description: 'exclui um cliente pelo ID',
      tags: ['api', 'v2', 'Clientes'],
      auth: {
        strategy: 'bearer',
        scope: ['app.clientes', 'admin']
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'DELETE',
    path: '/api/v2/clientes_all',
    handler: handlers.deleteAll,
    config: {
      description: 'exclui TODOS os clientes',
      tags: ['api', 'v2', 'Clientes'],
      auth: {
        strategy: 'bearer',
        scope: ['app.clientes', 'admin']
      }
    }
  },
  {
    method: 'DELETE',
    path: '/api/v2/clientesgroup_all',
    handler: handlers.deleteGroupAll,
    config: {
      description: 'exclui o grupo TODOS os clientes',
      tags: ['api', 'v2', 'Clientes'],
      auth: {
        strategy: 'bearer',
        scope: ['app.clientes', 'admin']
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/clientesGrupos/{ID}',
    handler: handlers.getClienteGrupo,
    config: {
      description: 'Obtem os dados do grupo de clientes pelo ID',
      tags: ['api', 'v2', 'Clientes'],
      auth: {
        strategy: 'bearer',
        scope: ['app.clientes', 'admin']
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v2/clientesGrupos',
    handler: handlers.createClientesGrupos,
    config: {
      description: 'Cria um grupo de clientes',
      tags: ['api', 'v2', 'Clientes'],
      auth: {
        strategy: 'bearer',
        scope: ['app.clientes', 'admin']
      },
      validate: {
        payload: {
          nome: Joi.string().required(),
          rota: Joi.string().allow(null).allow(""),
          roteia: Joi.boolean().allow(null).allow("")
        }
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v2/clientesGrupos/{ID}',
    handler: handlers.updateClientesGrupos,
    config: {
      description: 'Edita um grupo de clientes',
      tags: ['api', 'v2', 'Clientes'],
      auth: {
        strategy: 'bearer',
        scope: ['app.clientes', 'admin']
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        },
        payload: {
          nome: Joi.string().allow(null).allow(""),
          rota: Joi.string().allow(null).allow(""),
          roteia: Joi.boolean().allow(null).allow("")
        }
      }
    }
  },
  {
    method: 'DELETE',
    path: '/api/v2/clientesGrupos/{ID}',
    handler: handlers.deleteClientesGrupos,
    config: {
      description: 'exclui um grupo de cliente pelo ID',
      tags: ['api', 'v2', 'Clientes'],
      auth: {
        strategy: 'bearer',
        scope: ['app.clientes', 'admin']
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v2/clientesimport',
    handler: handlers.importClientes,
    config: {
      description: 'Importa um arquivo CSV de clientes',
      notes: 'Espera um arquivo CSV com qualquer delimitador com os seguintes campos em sequência: "Número telefone","Nome do cliente","Email" (Não é obrigatório),"Documento" (Não é obrigatório)',
      tags: ['api', 'v2', 'Clientes'],
      auth: {
        strategy: 'bearer',
        scope: ['app.clientes', 'admin']
      },
      plugins: {
        'hapi-swagger': {
          payloadType: 'form'
        }
      },
      payload: {
        output: 'stream',
        allow: 'multipart/form-data',
        maxBytes: 104857600, // 100mb
        parse: true,
      },
      validate: {
        payload: {
          idGrupo: Joi.number().integer().allow(null).description('Informa se os clientes contidos no arquivo devem ser importados em um grupo'),
          layout: Joi.number().integer().required().description('ID do layout de clientes'),
          file: Joi.any().meta({ swaggerType: 'file' }).required()
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/clientesfind',
    handler: handlers.FindClient,
    config: {
      description: 'Busca clientes',
      tags: ['api', 'v2', 'Clientes'],
      auth: {
        strategy: 'bearer',
        scope: ['app.clientes', 'admin']
      },
      validate: {
        query: {
          q: Joi.string().required()
        }
      }
    }
  },
];
