'use strict'

const Boom = require('boom');
const db = require('../../util/db');
const fs = require('fs');
const namiLib = require("nami");
const Ami = require('../../ami');

const fsPromises = fs.promises;
const config_path = '/etc/asterisk/extensions_custom.conf';

const callNami = (action) => {
  return new Promise((resolve, reject) => {
    Ami.nami.send(action, function (result) {
      setTimeout(() => {
        if (result.response == "Success") {
          //Retorna os dados
          resolve(result);
        }
        else {
          reject(result.message);
        }
      }, 500);
    });
  });
};

exports.getall = (request, reply) => {
  const max = request.query.max ? request.query.max : 15;
  const page = request.query.page ? request.query.page : 1;
  const knex = db.knex('public.contexto');

  if (request.query.context) knex.where('context', request.query.context);
  if (request.query.descricao) knex.where('descricao', request.query.descricao);
  if (request.query.editavel === true || request.query.editavel === false) knex.where('editavel', request.query.editavel);
  if (request.query.sentido) knex.whereIn('sentido', request.query.sentido);

  if (max === -1) {
    knex.clearSelect();
    // Parametros de ordenação, fora do count
    if (request.query.order) {
      for (let p = 0; p < request.query.order.length; ++p) {
        if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
          knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
        }
        else if (request.query.order[p] && request.query.order[p].campo) {
          knex.orderBy(request.query.order[p].campo, 'ASC');
        }
      }
    }
    knex.select('*')
      .then(function (rows) {
        const ret = {};
        ret.error = false;
        ret.full = true;
        ret.total = rows.length;
        ret.message = 'Lista de todos os grupos de rotas';
        ret.dados = rows;
        reply(ret);
      })
      .catch(function (error) {
        console.error(error);
        reply(Boom.badData(error));
      });
  }
  else {
    const offset = max * (page - 1);
    knex.count('* as total')
      .then(function (total) {
        if (total[0].total > 0) {
          knex.clearSelect();
          // Parametros de ordenação, fora do count
          if (request.query.order) {
            for (let p = 0; p < request.query.order.length; ++p) {
              if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
              }
              else if (request.query.order[p] && request.query.order[p].campo) {
                knex.orderBy(request.query.order[p].campo, 'ASC');
              }
            }
          }
          knex.select('*').limit(max).offset(offset).then(function (rows) {
            const pages = Math.ceil(parseInt(total[0].total) / max);
            const ret = {};
            ret.error = false;
            ret.full = pages === 1 ? true : false;
            ret.page = page;
            ret.max = max;
            ret.offset = offset;
            ret.total = parseInt(total[0].total);
            ret.pages = pages;
            ret.message = 'Lista de grupos de rotas, paginada';
            ret.dados = rows;
            reply(ret);
          })
            .catch(function (error) {
              console.error(error);
              reply(Boom.badData(error));
            });
        }
        else {
          const ret = {};
          ret.error = true;
          ret.total = 0;
          ret.message = 'Nenhum registro encontrado';
          reply(ret);
        }
      })
      .catch(function (error) {
        console.error(error);
        reply(Boom.badData(error));
      });
  }
};

exports.create = (request, reply) => {
  let contextID = null;
  const knex = db.knex;
  knex.transaction(function (t) {
    return db.knex('public.contexto')
      .insert(request.payload)
      .transacting(t)
      .returning("id")
      .then(async function (id) {
        contextID = id;
        return await registerContext(request.payload)
          .then(async () => {
            let action = new namiLib.Actions.Reload();
            return await callNami(action)
              .then(t.commit)
              .catch((e) => { throw e; });
          })
          .catch((e) => { throw e; });
      })
      .catch((e) => { throw e; });
  })
    .then(function () {
      const ret = {};
      ret.error = false;
      ret.id = contextID;
      ret.message = 'Contexto id = ' + contextID + ', criado com sucesso.';
      reply(ret);
    })
    .catch(function (error) {
      console.error(error);
      reply(Boom.badData(error));
    });
};

exports.update = (request, reply) => {
  const knex = db.knex;
  knex.transaction(function (t) {
    return db.knex('public.contexto')
      .update(request.payload)
      .where('id', request.params.ID)
      .then(async function () {
        return await registerContext(request.payload)
          .then(async () => {
            let action = new namiLib.Actions.Reload();
            return await callNami(action)
              .then(t.commit)
              .catch((e) => { throw e; });
          })
          .catch((e) => { throw e; });
      })
      .catch((e) => { throw e; });
  })
    .then(function () {
      const ret = {};
      ret.error = false;
      ret.id = request.params.ID;
      ret.message = 'Contexto id = ' + request.params.ID + ', alterado com sucesso.';
      reply(ret);
    })
    .catch(function (error) {
      console.error(error);
      reply(Boom.badData(error));
    });
};

exports.delete = (request, reply) => {
  const knex = db.knex;
  knex.transaction(function (t) {
    return knex('public.contexto')
      .transacting(t)
      .where(request.params.name ? 'context' : 'id', request.params.name ? request.params.name : request.params.ID)
      .select('*')
      .then(function (rows) {
        return knex('public.contexto')
          .transacting(t)
          .where(request.params.name ? 'context' : 'id', request.params.name ? request.params.name : request.params.ID)
          .del()
          .then(async () => {
            var data = await cfgFromFile(config_path, rows[0]);
            return await deleteFromFile(config_path, data)
              .then(async () => {
                let action = new namiLib.Actions.Reload();
                return await callNami(action)
                  .then(t.commit)
                  .catch((e) => { throw e; });
              })
              .catch((e) => { throw e; });
          })
          .catch((e) => { throw e; });
      })
      .catch((e) => { throw e; });
  })
    .then(function () {
      const ret = {};
      ret.error = false;
      ret.message = 'Contexto ' + request.params.name ? '' : 'id' + ' = ' + request.params.name ? request.params.name : request.params.ID + ' apagado com sucesso.';
      reply(ret);
    })
    .catch(function (e) {
      reply(Boom.badData(e));
    });
};

//Efetua o registro de um contexto na Asterisk
async function registerContext(tdata) {
  var filehandle = null;
  var content = "";
  tdata.conf = "switch = realtime";
  var newFile = false;
  try {
    try {
      //Leitura e escrita
      filehandle = await fsPromises.open(config_path, 'r+');
    }
    catch (err) {
      //Não existindo
      filehandle = await fsPromises.open(config_path, 'a+');
      newFile = true;
    }
    content = await filehandle.readFile({ encoding: 'utf8', flag: 'r' });

    //Config  
    const config = stringfyConfig(tdata.context, [tdata.conf]);
    const oldConfig = stringfyConfig(tdata.context, extractConfig(content, tdata.context));
    if (oldConfig) {
      content = content.replace(oldConfig, config);
    }
    else
      content = content + config;


    //Grava tudo
    if (!newFile) {
      await filehandle.truncate(0);
      await filehandle.sync();
    }
    await filehandle.write(content, 0, 'utf8');

  } finally {
    if (filehandle) {
      //Fecha o arquivo
      await filehandle.close();
    }
  }
};

function stringfyConfig(name, data) {
  if (data && Array.isArray(data) && data.length > 0)
    return `[${name}]\n${data.join('\n').replace('\n\n', '')}${data.indexOf('\n') == data.length ? '\n' : '\n\n'}`;
  else
    return "";
}

async function deleteFromFile(file, data) {
  let filehandle = null;
  var content = "";
  data.conf = "switch = realtime";
  try {
    //Leitura e escrita
    filehandle = await fsPromises.open(file, 'r+');
    content = await filehandle.readFile({ encoding: 'utf8', flag: 'r' });
    if (data.conf) content = content.replace(stringfyConfig(data.context, [data.conf]), '');
    await filehandle.truncate(0);
    await filehandle.sync();
    await filehandle.write(content, 0, 'utf8');
  } finally {
    if (filehandle) {
      //Fecha o arquivo
      await filehandle.close();
    }
  }
};

async function cfgFromFile(file, data) {
  let filehandle = null;
  var content = "";
  try {
    if (data && data.context) {
      //Leitura e escrita
      filehandle = await fsPromises.open(file, 'r+');
      content = await filehandle.readFile({ encoding: 'utf8', flag: 'r' });
      await filehandle.close();
      filehandle = null;
      data.conf = extractConfig(content, data.context);
    }
    return data;
  } finally {
    if (filehandle) {
      //Fecha o arquivo
      await filehandle.close();
    }
  }
};

function extractConfig(text, cfgName) {
  if (text && cfgName) {
    var start = text.indexOf("[" + cfgName + "]")
    var cfgRaw = text.substr(start, text.indexOf("\n\n", start) - start);
    var cfg = null
    if (cfgRaw && cfgRaw[0] == '[') {
      cfg = cfgRaw.split('\n');
      //O primeiro elemento é o identificador, e ele não existe na vida real
      cfg.splice(0, 1);
    }

    return cfg;
  }
  else
    return [];
}