'use strict'

const Joi = require('joi');
const handlers = require('./handlers');

const CreateReturnSchema = {
    error: Joi.boolean().example(false).description('Indica se o processo gerou algum erro.'),
    id: Joi.number().integer().example('1').description('ID do registro criado'),
    message: Joi.string().example('Registro criado com aucesso.')
};

module.exports = [
    {
        method: 'GET',
        path: '/api/v2/horarios',
        handler: handlers.getall,
        config: {
            description: 'Obtem os horarios cadastrados',
            tags: ['api', 'v2', 'Horários'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                query: {
                    max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
                    page: Joi.number().integer().default(1).description('Página'),
                    order: Joi.array().items(
                        Joi.object().keys({
                            campo: Joi.string().description('Nome da coluna'),
                            ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
                        })
                    ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
                    nome: Joi.string().description('Filtra por nome'),
                    tipo: Joi.string().valid('F', 'T').description('Filtra por tipo (F=Feriado, T=Turno)'),
                    dias: Joi.array().items(Joi.number().integer()).description('Filtra por dias'),
                    messes: Joi.array().allow(null).items(Joi.number().integer().min(1).max(12)).description('Filtra por Meses 0-11 (Começando por Janeiro)'),
                    dias_semana: Joi.array().allow(null).items(Joi.number().integer().min(0).max(6)).description('Filtra por Dias da Semana 0-6 (Começando pelo Domingo)')
                }
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v2/horario/{HorarioID}',
        handler: handlers.getOne,
        config: {
            description: 'Obtem os dados do horario',
            tags: ['api', 'v2', 'Horários'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    HorarioID: Joi.number().integer().required()
                }
            }
        }
    },
    {
        method: 'POST',
        path: '/api/v2/horario',
        handler: handlers.create,
        config: {
            description: 'Cria um novo horario',
            tags: ['api', 'v2', 'Horários'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            response: {
                schema: CreateReturnSchema
            },
            validate: {
                payload: Joi.object({
                    nome: Joi.string().required().description('Nome'),
                    hora_ini: Joi.string().allow(null).allow('').optional().description('Hora inicial no formato HH:mm:ss'),
                    hora_fim: Joi.string().allow(null).allow('').optional().description('Hora final no formato HH:mm:ss'),
                    descricao: Joi.string().allow(null).allow('').optional().description('Descrição'),
                    tipo: Joi.string().valid('F', 'T').description('Tipo (F=Feriado, T=Turno)'),
                    dias: Joi.array().allow(null).items(Joi.number().integer()).description('Dias'),
                    messes: Joi.array().allow(null).items(Joi.number().integer().min(1).max(12)).description('Meses 0-11 (Começando por Janeiro)'),
                    dias_semana: Joi.array().allow(null).items(Joi.number().integer().min(0).max(6)).description('Dias da Semana 0-6 (Começando pelo Domingo)')
                })
            }
        }
    },
    {
        method: 'PUT',
        path: '/api/v2/horario/{HorarioID}',
        handler: handlers.update,
        config: {
            description: 'Atualiza um Horario',
            tags: ['api', 'v2', 'Horários'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    HorarioID: Joi.number().integer().required()
                },
                payload: Joi.object({
                    hora_ini: Joi.string().allow(null).allow('').optional().description('Hora inicial no formato HH:mm:ss'),
                    hora_fim: Joi.string().allow(null).allow('').optional().description('Hora final no formato HH:mm:ss'),
                    nome: Joi.string().description('Nome'),
                    descricao: Joi.string().allow(null).allow('').optional().description('Descrição'),
                    tipo: Joi.string().valid('F', 'T').description('Tipo (F=Feriado, T=Turno)'),
                    dias: Joi.array().allow(null).items(Joi.number().integer()).description('Dias'),
                    messes: Joi.array().allow(null).items(Joi.number().integer().min(0).max(11)).description('Meses 0-11 (Começando por Janeiro)'),
                    dias_semana: Joi.array().allow(null).items(Joi.number().integer().min(0).max(6)).description('Dias da Semana 0-6 (Começando pelo Domingo)')
                })
            }
        }
    },
    {
        method: 'DELETE',
        path: '/api/v2/horario/{HorarioID}',
        handler: handlers.delete,
        config: {
            description: 'Apaga um horario',
            tags: ['api', 'v2', 'Horários'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    HorarioID: Joi.number().integer().required()
                }
            }
        }
    },
];
