'use strict'

const Joi = require('joi');
const handlers = require('./handlers');

const CreateReturnSchema = {
    error: Joi.boolean().example(false).description('Indica se o processo gerou algum erro.'),
    id: Joi.number().integer().example('1').description('ID do registro criado'),
    message: Joi.string().example('Registro criado com aucesso.')
};

module.exports = [
    {
        method: 'GET',
        path: '/api/v2/layouts',
        handler: handlers.getall,
        config: {
            description: 'Obtem os layouts cadastrados',
            tags: ['api', 'v2', 'Campanhas'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                query: {
                    max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
                    page: Joi.number().integer().default(1).description('Página'),
                    order: Joi.array().items(
                        Joi.object().keys({
                            campo: Joi.string().description('Nome da coluna'),
                            ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
                        })
                    ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
                    nome: Joi.string().description('Filtra por nome'),
                    modulo: Joi.string().description('Filtra por modulo')
                }
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v2/layout/{LayoutID}',
        handler: handlers.getLayout,
        config: {
            description: 'Obtem os dados do layout',
            tags: ['api', 'v2', 'Campanhas'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    LayoutID: Joi.number().integer().required()
                }
            }
        }
    },
    {
        method: 'POST',
        path: '/api/v2/layout',
        handler: handlers.create,
        config: {
            description: 'Cria um novo layout',
            tags: ['api', 'v2', 'Campanhas'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            response: {
                schema: CreateReturnSchema
            },
            validate: {
                payload: Joi.object({
                    nome: Joi.string().required().example('Layout padrão').description('Nome do layout'),
                    delimitador: Joi.string().required().default(';').example(';').description('Delimitador de colunas do arquivo'),
                    cabecalho: Joi.boolean().required().default(false).example(false).description('Define se o arquivo vai possuir cabeçalho, se essa opção for definida a primeira linha do arquivo será ignorada'),
                    modulo: Joi.string().optional().allow(null).allow('').description('Modulo do layout')
                })
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v2/layout/{LayoutID}/campos',
        handler: handlers.getLayoutCampos,
        config: {
            description: 'Obtem os campos do layout',
            tags: ['api', 'v2', 'Campanhas'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    LayoutID: Joi.number().integer().required()
                },
                query: {
                    max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
                    page: Joi.number().integer().default(1).description('Página'),
                    order: Joi.array().items(
                        Joi.object().keys({
                            campo: Joi.string().description('Nome da coluna'),
                            ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
                        })
                    ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
                    nome: Joi.string().description('Filtra por nome')
                }
            }
        }
    },
    {
        method: 'PUT',
        path: '/api/v2/layout/{LayoutID}/campos',
        handler: handlers.ReplaceCampos,
        config: {
            description: 'REPLACE dos campos do layout (Apaga todos os registros e substitui pelos enviadas)',
            tags: ['api', 'v2', 'Campanhas'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    LayoutID: Joi.number().integer().required()
                },
                payload: Joi.array().items(
                    Joi.object().keys({
                        nome: Joi.string().required().example('campo1').description('Nome para o campo'),
                        posicao_csv: Joi.number().integer().required().example(0).description('Posição do campo no arquivo, começa em zero'),
                        xcontact: Joi.string().allow('', null).description('Campo reservado para o Xcontact')
                    })
                ).description('Lista de classificações da campanha')
            }
        }
    },
    {
        method: 'PUT',
        path: '/api/v2/layout/{LayoutID}',
        handler: handlers.update,
        config: {
            description: 'Atualiza um layout',
            tags: ['api', 'v2', 'Campanhas'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    LayoutID: Joi.number().integer().required()
                },
                payload: Joi.object({
                    nome: Joi.string().required().example('Layout padrão').description('Nome do layout'),
                    delimitador: Joi.string().required().default(';').example(';').description('Delimitador de colunas do arquivo'),
                    cabecalho: Joi.boolean().required().default(false).example(false).description('Define se o arquivo vai possuir cabeçalho, se essa opção for definida a primeira linha do arquivo será ignorada'),
                    modulo: Joi.string().optional().allow(null).allow('').description('Modulo do layout')
                })
            }
        }
    },
    {
        method: 'DELETE',
        path: '/api/v2/layout/{LayoutID}',
        handler: handlers.delete,
        config: {
            description: 'Apaga um layout',
            tags: ['api', 'v2', 'Campanhas'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    LayoutID: Joi.number().integer().required()
                }
            }
        }
    },
];
