'use strict'

const Joi = require('joi');
const handlers = require('./handlers');

module.exports = [
  {
    method: 'GET',
    path: '/api/v2/ligacoes',
    handler: handlers.getall,
    config: {
      description: 'Obtem as ligações',
      tags: ['api', 'v2', 'Ligações'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
              campo: Joi.string().description('Nome da coluna'),
              ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          data_ini: Joi.date().iso().description('Filtro por data inicial'),
          data_fim: Joi.date().iso().description('Filtro por data Final'),
          origem: Joi.string().description('Filtro por origem (src)'),
          origem_operador: Joi.string().valid('contendo', 'igual', 'iniciando', 'finalizando', 'nao_contendo', 'diferente').description('Operador para origem'),
          destino: Joi.string().description('Filtro por destino (dst)'),
          destino_operador: Joi.string().valid('contendo', 'igual', 'iniciando', 'finalizando', 'nao_contendo', 'diferente').description('Operador para destino'),
          callid: Joi.string().description('Filtro por callid'),
          sentido: Joi.array().items(Joi.string()).example(['Entrante', 'Sainte']).description('Filtra por um ou mais sentidos'),
          status: Joi.array().items(Joi.string()).description('Filtra por um ou mais status'),
          protocolo: Joi.string().description('Filtro por protocolo'),
          canal: Joi.string().description('Filtro por canal'),
          pin_id: Joi.string().description('Filtro por pin_id'),
          pin_usuario: Joi.string().description('Filtro por pin_usuario')
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/ligacao/{CallID}',
    handler: handlers.getOne,
    config: {
      description: 'Obtem os dados de uma ligação pelo callid',
      tags: ['api', 'v2', 'Ligações'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          CallID: Joi.string().required().description('Callid da ligação')
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/ligacoes/status',
    handler: handlers.getStatus,
    config: {
      description: 'Obtem a lista de status das ligações',
      tags: ['api', 'v2', 'Ligações'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
              campo: Joi.string().description('Nome da coluna'),
              ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          nome: Joi.string().description('Filtra por nome'),
          return: Joi.string().valid('array').description('Muda o retorno para um array puro de registros'),
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v2/ligacao/classifica',
    handler: handlers.classifica,
    config: {
      description: 'Classifica uma ligação',
      tags: ['api', 'v2', 'Ligações'],
      validate: {
        payload: Joi.object({
          uniqueid: Joi.string().required().example('huy7654efss.9087').description('Callid da ligação'),
          id_classificacao: Joi.number().integer().description('ID da classificação').example(1),
          id_subclassificacao: Joi.number().integer().description('ID da Subclassificacao').example(1),
          obs: Joi.string().description('Observação').example('Cliente desligou'),
          protocolo: Joi.string().description('Código vinculado a ligação, pode ser um número de chamado por exemplo.').example('123456'),
          luc: Joi.string().description('LUC, campos texto utilizado na Brmalls').example('ADC02514445'),
          id_origem: Joi.number().integer().description('ID da origem, campo usado na Brmals').example(1),
        })
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v2/ligacao/efetuar',
    handler: handlers.discar,
    config: {
      description: 'Efetua uma ligação',
      tags: ['api', 'v2', 'Ligações'],
      validate: {
        payload: Joi.object({
          origem: Joi.string().required().example('SIP/1000').description('Ramal de origem.'),
          destino: Joi.string().required().description('Destino').example('05130305500'),
          destino_nome: Joi.string().description('Opcional. Nome para o destino').example('João da Silva'),
          rota: Joi.string().required().description('Rota de saída. Grupo de rota informado do cadastro do Ramal.').example('Ramais'),
          max_chamar: Joi.number().integer().description('Tempo máximo chamando').default(30).example(30),
          campos: Joi.object().keys().unknown().example('{"CPF":"123456"}').description('Campos associados a ligação para fins de integração com o Cliente.')
        })
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v2/ligacao/conferencia',
    handler: handlers.addConferencia,
    config: {
      description: 'Efetua uma ligação',
      tags: ['api', 'v2', 'Ligações'],
      validate: {
        payload: Joi.object({
          destino: Joi.string().required().description('Destino').example('05130305500'),
          destino_nome: Joi.string().description('Opcional. Nome para o destino').example('João da Silva'),
          sala: Joi.string().required().description('ID da sala').example('213'),
          rota: Joi.string().required().description('Rota de saída. Grupo de rota informado do cadastro do Ramal.').example('Ramais'),
          max_chamar: Joi.number().integer().description('Tempo máximo chamando').default(30).example(30),
          campos: Joi.object().keys().unknown().example('{"CPF":"123456"}').description('Campos associados a ligação para fins de integração com o Cliente.')
        })
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v2/ligacao/desligar',
    handler: handlers.desligar,
    config: {
      description: 'Desliga uma ligação pelo ramal ou pelo destino',
      tags: ['api', 'v2', 'Ligações'],
      validate: {
        payload: Joi.object({
          destino: Joi.string().required().description('SIP/Ramal').example('SIP/9001'),
          causa: Joi.number().integer().description('Causa da desconexão').default(16).example(16)
        })
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/ligacoestotais',
    handler: handlers.getTotais,
    config: {
      description: 'Obtem o total de ligações realizadas/recebidas pelos ramais informados dentro do período (Default últimos 30 dias)',
      tags: ['api', 'v2', 'Ligações'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          dataIni: Joi.date().optional().allow(null).iso().example('2018-01-01 00:00:00').description('Data inicial (Não é obrigatório informar)'),
          dataFim: Joi.date().optional().allow(null).iso().example('2018-01-30 23:59:59').description('Data final (Não é obrigatório informar)'),          
          ramais: Joi.array().optional().allow(null).items(Joi.string()).example(['1000', '1001', '1002']).description('Lista de ramais (Não é obrigatório informar)')
        }
      }
    }
  },
];
