'use strict'
const Joi = require('joi');
const CRUD = require('../../util/CRUD');

const CreateReturnSchema = {
    error: Joi.boolean().example(false).description('Indica se o processo gerou algum erro.'),
    id: Joi.number().integer().example('1').description('ID do registro criado'),
    message: Joi.string().example('Registro criado com aucesso.')
};

module.exports = [
  {
    method: 'GET',
    path: '/api/v2/nivelservico',
    handler: (request, reply) => {
      const crud = new CRUD('public.calculos_nivelservico');      
      crud.filtros = [];
      if(request.query.nome) crud.filtros.push({campo: 'LOWER(nome)', operador: 'LIKE', valor: '%'+decodeURIComponent(request.query.nome).toLowerCase()+'%'});      
      crud.getAll(request, reply);
    },
    config: {
      description: 'Obtem os níveis de serviço cadastrados',
      tags: ['api','Nível de Serviço'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
                campo: Joi.string().description('Nome da coluna'),
                ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          nome: Joi.string().description('Filtra por nome')
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/nivelservico/{ID}',
    handler: (request, reply) => {
      const crud = new CRUD('public.calculos_nivelservico');
      crud.getOne(request, reply);
    },
    config: {
      description: 'Obtem os dados pelo ID',
      tags: ['api','Nível de Serviço'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v2/nivelservico',
    handler: (request, reply) => {
      const crud = new CRUD('public.calculos_nivelservico');
      crud.create(request, reply);
    },
    config: {
      description: 'Cria um novo registro',
      tags: ['api','Nível de Serviço'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      response: {
        schema: CreateReturnSchema
      },
      validate: {
        payload: Joi.object({          
          nome: Joi.string().required().example('Calculo padrão').description('Nome'),
          formula: Joi.string().required().example('TA/TB').description('Formula'),
          descricao: Joi.string().example('Texto').description('Descrição'),
        })
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v2/nivelservico/{ID}',
    handler: (request, reply) => {
      const crud = new CRUD('public.calculos_nivelservico');
      crud.update(request, reply);
    },
    config: {
      description: 'Atualiza um registro',
      tags: ['api','Nível de Serviço'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            ID: Joi.number().integer().required()
        },
        payload: Joi.object({
          nome: Joi.string().example('Calculo padrão').description('Nome'),
          formula: Joi.string().example('TA/TB').description('Formula'),
          descricao: Joi.string().allow(null).allow('').example('Texto').description('Descrição'),   
        })
      }
    }
  },
  {
    method: 'DELETE',
    path: '/api/v2/nivelservico/{ID}',
    handler: (request, reply) => {
      const crud = new CRUD('public.calculos_nivelservico');
      crud.delete(request, reply);
    },
    config: {
      description: 'Apaga uma registro',
      tags: ['api','Nível de Serviço'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/nivelservico/variaveis',
    handler: (request, reply) => {
      const crud = new CRUD('public.calculos_nivelservico_variaveis');      
      crud.filtros = [];
      if(request.query.nome) crud.filtros.push({campo: 'LOWER(nome)', operador: 'LIKE', valor: '%'+decodeURIComponent(request.query.nome).toLowerCase()+'%'});      
      crud.getAll(request, reply);
    },
    config: {
      description: 'Obtem as variáveis para os níveis de serviço cadastradas',
      tags: ['api','Nível de Serviço'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
                campo: Joi.string().description('Nome da coluna'),
                ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          nome: Joi.string().description('Filtra por nome')
        }
      }
    }
  },
];
