'use strict'
const Joi = require('joi');
const Boom = require('boom');
const Pesquisa = require('../../models/Pesquisa');

module.exports = [
  {
    method: 'GET',
    path: '/api/v2/pesquisas',
    handler: async (request, reply) => {
      try {
        // Para paginção server side
        var pag = undefined;
        if(request.query.max && request.query.max > 0){
          pag = { page: request.query.page, perPage: request.query.max };
        }

        const query = Pesquisa.query;
        var max = request.query.max > 0 ? request.query.max : undefined;
        if(request.query.nome) query.where(Pesquisa.where.like({ nome: `%${decodeURIComponent(request.query.nome)}%` }));

        var pesquisas = await query.fetch(pag);
        reply(pesquisas);
      } catch (error) {
        reply(Boom.notFound(error.message))
      }
    },
    config: {
      description: 'Lista as pesquisas',
      notes: 'Traz um array de pesquisas, você pode usar filtros de buscas',
      tags: ['api','v2','Pesquisas'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max <= 0 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          nome: Joi.string().allow(null).description('Filtra pelo nome')
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v2/pesquisa/{id}',
    handler: async (request, reply) => {
      try {
        var pesquisa = await Pesquisa.fetchById(request.params.id);
        reply(pesquisa);
      } catch (error) {
        reply(Boom.notFound(error.message))
      }
    },
    config: {
      description: 'Obtem uma pesquisa pelo ID',
      tags: ['api','v2','Pesquisas'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          id: Joi.number().integer().required().description('ID da pesquisa')
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v2/pesquisa',
    handler: async (request, reply) => {
      try {
        var pesquisa = await new Pesquisa(request.payload).save();
        reply(pesquisa);
      } catch (error) {
        reply(Boom.badData(error.message))
      }
    },
    config: {
      description: 'Cria uma pesquisa',
      tags: ['api','v2','Pesquisas'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        payload: Joi.object({
          nome: Joi.string().required().example('teste').description('Nome da pesquisa'),
          descricao: Joi.string().description('Descrição da pesquisa, no caso de pesquisas de texto esse é o Texto que será enviado.').example('Alguma coisa'),
        })
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v2/pesquisa/{id}',
    handler: async (request, reply) => {
      try {
        request.payload.id = request.params.id;
        var pesquisa = await new Pesquisa(request.payload).save();
        reply(pesquisa);
      } catch (error) {
        reply(Boom.badData(error.message))
      }
    },
    config: {
      description: 'Atualiza uma pesquisa',
      tags: ['api','v2','Pesquisas'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          id: Joi.number().integer().required().description('ID da pesquisa')
        },
        payload: Joi.object({
          nome: Joi.string().allow(null).example('teste').description('Nome da pesquisa'),
          descricao: Joi.string().allow(null).description('Descrição da pesquisa, no caso de pesquisas de texto esse é o Texto que será enviado.').example('Alguma coisa'),
        })
      }
    }
  },
]