'use strict'
const Joi = require('joi');
const CRUD = require('../../util/CRUD');
const md5 = require('md5');
const db = require('../../util/db');
const Boom = require('boom');
const Agentes = require('../../models/Agente');
const AgentesFilas = require('../../models/AgenteFilas');
const Filas = require('../../models/Fila');
const timerExpediente = require('../../timer_expediente');
var crypto = require('crypto');
const handlers = require('./handlers');

function ordered_menu(array, parent_id) {
  const temp_array = [];
  for (let i = 0; i < array.length; ++i) {
    if (array[i].parent == parent_id) {
      const modulo = { nome: array[i].descricao, icon: array[i].icon, scope: array[i].nome, order: array[i].ordem, extra: array[i].extra_usuario, modulos: [] };
      modulo.modulos = ordered_menu(array, array[i].id);
      temp_array.push(modulo);
    }
  }
  return temp_array;
}

module.exports = [
  {
    method: 'GET',
    path: '/api/v2/usuarios/log',
    handler: (request, reply) => {
      const crud = new CRUD('public.supervisores_log AS log');
      crud.filtrosraw = [];
      crud.filtros = [];
      if (request.query.id_supervisor) crud.filtrosraw.push('log.id_supervisor = ANY(ARRAY[' + request.query.id_supervisor + '])');
      if (request.query.id_user) crud.filtrosraw.push('log.id_user = ANY(ARRAY[' + request.query.id_user + '])');
      if (request.query.modulo) crud.filtrosraw.push('log.modulo = ANY(ARRAY[' + request.query.modulo + '])');
      if (request.query.acao) crud.filtrosraw.push('log.acao = ANY(ARRAY' + JSON.stringify(request.query.acao).replace(/"/g, "'") + ')');
      if (request.query.data_ini) crud.filtros.push({ campo: 'log.data_hora', operador: '>=', valor: request.query.data_ini });
      if (request.query.data_fim) crud.filtros.push({ campo: 'log.data_hora', operador: '<=', valor: request.query.data_fim });
      if (request.query.order) crud.order = request.query.order;

      crud.leftjoin = [
        { tabela: "supervisores AS adm", campo1: "log.id_supervisor", campo2: "adm.id" },
        { tabela: "supervisores AS user", campo1: "log.id_user", campo2: "user.id" },
        { tabela: "modulos", campo1: "log.modulo", campo2: "modulos.id" }
      ];

      crud.select = [
        'log.*',
        'adm.fullname as supervisor',
        'user.fullname as usuario',
        'modulos.descricao as modulo_nome'
      ];

      crud.getAll(request, reply);
    },
    config: {
      description: 'Obtem os logs dos usuários',
      tags: ['api', 'v2', 'Usuários'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
              campo: Joi.string().description('Nome da coluna'),
              ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          id_supervisor: Joi.array().items(Joi.number().integer()).description('Filtra por ids do usuários que práticou a ação'),
          data_ini: Joi.date().iso().example('2018-01-01 08:00:00').description('Data inicial'),
          data_fim: Joi.date().iso().example('2018-01-01 23:59:59').description('Data final'),
          acao: Joi.array().items(Joi.string().valid('V', 'C', 'A', 'R', 'U')).description('Filtra por ação: V = Visualizou, C = Criou, A = Adicionou, R = Removeu, U = Atualizou'),
          modulo: Joi.array().items(Joi.number().integer()).description('Filtra por ids de módulos'),
          id_user: Joi.array().items(Joi.number().integer()).description('Filtra por ids de usuários afetados pela ação'),
          gravacao: Joi.array().items(Joi.string().valid('A', 'R')).description('Filtra por ações de acesso a gravações: A = Acesso concedido, R = Acesso removido'),
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v2/usuarios/supervisores/login',
    handler: (request, reply) => {
      const knex = db.knex('public.supervisores');
      knex.where('supervisores.nome', request.payload.nome);
      knex.where('supervisores.senha', md5(request.payload.senha));
      knex.leftJoin('supervisores_modulos', 'supervisores_modulos.id_supervisor', 'supervisores.id');
      knex.leftJoin('modulos', 'modulos.id', 'supervisores_modulos.id_modulo');
      knex.orderBy('modulos.ordem', 'ASC');
      knex.select(['modulos.id', 'modulos.parent', 'modulos.nome', 'modulos.descricao', 'modulos.icon', 'supervisores.fullname', 'supervisores.token', 'supervisores_modulos.extra_usuario', 'modulos.ordem'])
        .then(function (rows) {
          const ret = {};
          if (rows.length > 0) {
            ret.error = false;
            ret.message = 'Credenciais de login';
            ret.user = {};
            ret.user.nome = rows[0].fullname;
            ret.user.token = rows[0].token;
            ret.user.modulos = [];
            for (let i = 0; i < rows.length; ++i) {
              let modulo = { nome: rows[i].descricao, icon: rows[i].icon, scope: rows[i].nome, order: rows[i].ordem, extra: rows[i].extra_usuario };
              ret.user.modulos.push(modulo);
            }
            ret.user.modulos_tree = ordered_menu(rows, null);
          }
          else {
            ret.error = true;
            ret.total = 0;
            ret.message = 'Nenhum registro encontrado';
          }
          reply(ret);
        })
        .catch(function (error) {
          reply(Boom.badData(error));
        });
    },
    config: {
      description: 'Efetua login de supervisores',
      tags: ['api', 'v2', 'Usuários'],
      validate: {
        payload: Joi.object({
          nome: Joi.string().required().example('admin').description('Nome para login'),
          senha: Joi.string().example('123456').description('Senha para login'),
        })
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v2/usuarios/agentes/login',
    handler: handlers.AgenteLogin,
    config: {
      description: 'Efetua login de agentes',
      tags: ['api', 'v2', 'Usuários'],
      validate: {
        payload: Joi.object({
          nome: Joi.string().required().example('admin').description('Nome para login'),
          senha: Joi.string().example('123456').description('Senha para login'),
        })
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v1/usuarios/agentes/login',
    handler: async (request, reply) => {
      let ret = {};
      try {
        let agente = await Agentes.query.require(true).first()
          .where({ nome: request.payload.nome })
          .where({
            senha: crypto.createHash('md5').update(request.payload.senha).digest("hex")
          }).fetch();
        //reply(agente);
        if (agente) {
          let auxLocal = agente.ramal.split(`@`);
          let ramal = auxLocal[0].split('/');
          agente.ramal_numero = ramal[1];
          if (ramal[0] === `SIP`) {
            const knex = db.knex('public.ramais');
            knex.where('ramais.name', agente.ramal_numero);
            knex.select(['ramais.secret', 'ramais.context'])
              .then(async (rows) => {
                if (rows.length > 0) {
                  agente.ramal_senha = rows[0].secret;
                  agente.contexto = rows[0].context;
                  agente.filas = await AgentesFilas.query.require(true)
                    .where({ id_agente: agente.id }).fetch();
                  for (let fila of agente.filas) {
                    let f = await Filas.fetchByName(fila.fila);
                    fila.queue_type = f.queue_type;
                  }
                  let terminou = await timerExpediente.verficaTimerAgente(agente, `login`);
                  if (terminou) throw 'O seu expediente ja acabou!';
                  reply(agente);
                } else {
                  throw 'Senha incorreta';
                }
              }).catch(err => {
                ret.error = true;
                ret.msg = err
                console.error(ret);
                reply(ret);
              })
          } else {
            agente.ramal_senha = null;
            agente.contexto = ramal[0];
            agente.filas = await AgentesFilas.query.require(true)
              .where({ id_agente: agente.id }).fetch();
            for (let fila of agente.filas) {
              let f = await Filas.fetchByName(fila.fila);
              fila.queue_type = f.queue_type;
            }
            reply(agente);
          }

        }


      } catch (error) {
        ret.error = true;
        ret.msg = error.message && error.message === `Agente: no rows fetched` ? `Senha incorreta` : error.message;
        console.error(ret);
        reply(ret);
      }
    },
    config: {
      description: 'Efetua login de agentes',
      tags: ['api', 'v2', 'Usuários'],
      validate: {
        payload: Joi.object({
          nome: Joi.string().required().example('admin').description('Nome para login'),
          senha: Joi.string().example('123456').description('Senha para login'),
        })
      }
    }
  }
];
