'use strict'

const Boom = require('boom');
const db = require('../../../util/db');
const config = require('../../../config.json');
const moment = require('moment-timezone');
moment.tz.setDefault(config.servidor.timezone);
const Paginacao = require('../../../util/Paginacao');
const csv = require("fast-csv");

exports.getAll = (request, reply) => {
  const max = request.query.max ? request.query.max : 15;
  const page = request.query.page ? request.query.page : 1;
  const knex = db.knex({ a: 'agenda' })
    .leftOuterJoin({ g: 'agenda_grupos' }, 'a.id_grupo', 'g.id');

  if (request.query.nome) knex.whereRaw('LOWER(a.nome) LIKE ?', '%' + decodeURIComponent(request.query.nome).toLowerCase() + '%');
  if (request.query.numero) knex.whereRaw('CAST(a.numero as VARCHAR) LIKE ?', '%' + decodeURIComponent(request.query.numero) + '%');

  const pag = new Paginacao(max, page);
  pag.knex = knex;
  pag.select = ['a.id', 'a.numero', 'a.nome', 'a.id_grupo', 'g.nome as grupo'];
  pag.order = request.query.order;
  pag.getRet((ret, err) => {
    if (err) {
      console.error(err);
      reply(Boom.badData(err));
    }
    else {
      reply(ret);
    }
  });
};

exports.getOne = (request, reply) => {
  const knex = db.knex();
  knex.select('a.numero', 'a.nome', 'a.id_grupo', 'g.nome AS grupo')
    .from({ a: 'public.agenda' })
  knex.leftOuterJoin({ g: 'public.agenda_grupos' }, 'a.id_grupo', 'g.id')
  knex.where('id', request.params.ID)
    .then(function (rows) {
      const ret = {};
      if (rows.length > 0) {
        ret.error = false;
        ret.full = true;
        ret.total = rows.length;
        ret.message = 'Dados da agenda id = ' + request.params.ID;
        ret.dados = rows[0];
      }
      else {
        ret.error = true;
        ret.total = rows.length;
        ret.message = 'Nenhum registro encontrado para id = ' + request.params.ID;
        ret.dados = rows[0];
      }
      reply(ret);
    })
    .catch(function (error) {
      console.error(error);
      reply(Boom.badData(error));
    });
};

exports.create = (request, reply) => {
  if (request.payload.grupo) delete request.payload.grupo;
  const knex = db.knex('public.agenda');
  knex.insert(request.payload);
  knex.returning('id')
    .then(function (id) {
      const ret = {};
      ret.error = false;
      ret.id = id[0];
      ret.message = 'Agenda id = ' + id[0] + ' criada com sucesso.';
      reply(ret);
    })
    .catch(function (error) {
      console.error(error);
      reply(Boom.badData(error));
    });
};

exports.update = (request, reply) => {
  const knex = db.knex('public.agenda');
  knex.update(request.payload);

  knex.where('id', request.params.ID)
    .then(function (rows) {
      const ret = {};
      ret.error = false;
      ret.total = rows;
      ret.message = 'Agenda id = ' + request.params.ID + ' atualizada com sucesso.';
      reply(ret);
    })
    .catch(function (error) {
      console.error(error);
      reply(Boom.badData(error));
    });
};

exports.delete = (request, reply) => {
  const knex = db.knex('public.agenda');
  knex.where('id', request.params.ID)
    .del()
    .then(function (rows) {
      if (rows > 0) {
        const ret = {};
        ret.error = false;
        ret.total = rows;
        ret.message = 'Agenda id = ' + request.params.ID + ' apagada com sucesso.';
        reply(ret);
      }
      else {
        reply(Boom.badData("Nenhum registro encontrado para ID = " + request.params.ID));
      }
    })
    .catch(function (error) {
      console.error(error);
      reply(Boom.badData(error));
    });
};

exports.upload = (request, reply) => {
  const dados = request.payload.file.toString('latin1', 0, request.payload.file.length);
  const registros = [];
  csv.fromString(dados, { headers: false, delimiter: ';' })
    .on("data", function (data) {
      registros.push(data);
    })
    .on("end", function () {
      const id_grupo = request.params.id_grupo;
      // Organiza os registros para insert do contato, para cada registro do arquivo insert 1 contato
      const itens = [];
      for (let i = 0; i < registros.length; ++i) {
        const item = {
          id_grupo: id_grupo,
          numero: registros[i][0],
          nome: registros[i][1],
        };
        itens.push(item);
      };
      db.knex.transaction(function (t) {
        return db.knex.batchInsert('agenda', itens, registros.length)
          .transacting(t);
      })
        .then(function () {
          const ret = {};
          ret.error = false;
          ret.total = registros.length;
          ret.message = `${registros.length} registros adicionados na Agenda.`;
          reply(ret);
        })
        .catch(function (error) {
          console.error(error);
          reply(Boom.badData(error));
        });
    });
};

exports.getGrupos = (request, reply) => {
  const max = request.query.max ? request.query.max : 15;
  const page = request.query.page ? request.query.page : 1;
  const knex = db.knex('public.agenda_grupos');

  if (request.query.nome) knex.whereRaw('LOWER(nome) LIKE ?', '%' + decodeURIComponent(request.query.nome).toLowerCase() + '%');

  const pag = new Paginacao(max, page);
  pag.knex = knex;
  pag.getRet((ret, err) => {
    if (err) {
      console.error(err);
      reply(Boom.badData(err));
    }
    else {
      reply(ret);
    }
  });
};

exports.limparGrupo = (request, reply) => {
  const knex = db.knex;
  knex.transaction(function (t) {
    return knex('public.agenda')
      .transacting(t)
      .where('id_grupo', request.params.ID)
      .del()
      .then(t.commit)
      .catch(t.rollback)
  })
    .then(function () {
      const ret = {};
      ret.error = false;
      ret.message = 'Números do grupo id = ' + request.params.ID + ' apagados com sucesso.';
      reply(ret);
    })
    .catch(function (e) {
      reply(Boom.badData(e));
    });
};