'use strict'
const Joi = require('joi');
const CRUD = require('../../../util/CRUD');
const handlers = require('./handlers');
const util = require('../../../util/restbase');

const ListaReturnSchema = util.CreateReturnSchema(null);

const operationData = Joi.object().keys({
  numero: Joi.number().integer().example('123456').description('Número'),
  nome: Joi.string().example('João').description('Nome'),
  id_grupo: Joi.number().integer().example('1').description('ID do grupo de agenda'),  
  grupo:  Joi.string().optional(),
});

const operationDataG = Joi.object().keys({          
  nome: Joi.string().required().example('Clientes VIP').description('Nome'),
  descricao: Joi.string().allow(null).allow('').example('Clientes com prioridade de atendimento').description('Descrição'),
  contexto: Joi.alternatives(
    Joi.string().example('rota_prioritaria').description('Grupo de rota').required(),    
    Joi.object().keys({
      context: Joi.string().required(),
      descricao: Joi.string().optional(),
      editavel: Joi.bool(),
      sentido: Joi.string(),
      id: Joi.number().integer()
    })
    )
});

module.exports = [
  {
    method: 'GET',
    path: '/api/v3/agenda',
    handler: handlers.getAll,
    config: {
      description: 'Pesquisa as agendas cadastradas. Informe os filtros para obter uma consulta personalizada ou chame o método sem dados para obter a primeira página de resultados.',
      tags: ['api','Agenda'],
      auth: {
          strategy: 'bearer',
          scope: [ 'user', 'admin' ]
      },
      response: {
        schema: ListaReturnSchema
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
                campo: Joi.string().description('Nome da coluna'),
                ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          nome: Joi.string().description('Filtra por nome'),
          numero: Joi.string().description('Filtra por numero')
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/agenda/{ID}',
    handler: handlers.getOne,
    config: {
      description: 'Obtém os dados da agenda pelo ID',
      tags: ['api','Agenda'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v3/agenda',
    handler: handlers.create,
    config: {
      description: 'Cria uma nova agenda',
      tags: ['api','Agenda'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        payload: operationData
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v3/agenda/{ID}',
    handler: (request, reply) => {
      if(request.payload.grupo) delete request.payload.grupo;
      const crud = new CRUD('public.agenda');
      crud.update(request, reply);
    },
    config: {
      description: 'Atualiza uma agenda',
      tags: ['api','Agenda'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            ID: Joi.number().integer().required()
        },
        payload: operationData
      }
    }
  },
  {
    method: 'DELETE',
    path: '/api/v3/agenda/{ID}',
    handler: (request, reply) => {
      const crud = new CRUD('public.agenda');
      crud.delete(request, reply);
    },
    config: {
      description: 'Apaga uma agenda',
      tags: ['api','Agenda'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v3/agenda/upload/{id_grupo}',
    handler: handlers.upload,
    config: {
      description: 'Recebe arquivos com conteúdo para importar com agenda',
      tags: ['api','Agenda'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      payload: {
        output: 'data',
        allow: 'multipart/form-data',
        maxBytes: 104857600, // 100mb
        parse: true,
    },
    validate: {
        params: {
            id_grupo: Joi.number().integer().optional()
        },
        payload: {
          file: Joi.object().unknown().description('Arquivo')
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/agenda/grupo',
    handler: (request, reply) => {
      const crud = new CRUD('public.agenda_grupos'); 
      crud.filtros = [];
      if(request.query.nome) crud.filtros.push({campo: 'LOWER(nome)', operador: 'LIKE', valor: '%'+decodeURIComponent(request.query.nome).toLowerCase()+'%'});     
      crud.getAll(request, reply);
    },
    config: {
      description: 'Pesquisa os grupos de agendas cadastradas. Informe os filtros para obter uma consulta personalizada ou chame o método sem dados para obter a primeira página de resultados.',
      tags: ['api','Agenda'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
                campo: Joi.string().description('Nome da coluna'),
                ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          nome: Joi.string().description('Filtra por nome'),
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/agenda/grupo/{ID}',
    handler: (request, reply) => {
      const crud = new CRUD('public.agenda_grupos');
      crud.getOne(request, reply);
    },
    config: {
      description: 'Obtém os dados grupo da agenda pelo ID',
      tags: ['api','Agenda'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v3/agenda/grupo',
    handler: (request, reply) => {
      const crud = new CRUD('public.agenda_grupos');
      crud.create(request, reply);
    },
    config: {
      description: 'Cria um novo grupo de agenda',
      tags: ['api','Agenda'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        payload: operationDataG
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v3/agenda/grupo/{ID}',
    handler: (request, reply) => {
      const crud = new CRUD('public.agenda_grupos');
      crud.update(request, reply);
    },
    config: {
      description: 'Atualiza um grupo de rota',
      tags: ['api','Agenda'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            ID: Joi.number().integer().required()
        },
        payload: operationDataG
      }
    }
  },
  {
    method: 'DELETE',
    path: '/api/v3/agenda/grupo/{ID}',
    handler: (request, reply) => {
      const crud = new CRUD('public.agenda_grupos');
      crud.delete(request, reply);
    },
    config: {
      description: 'Apaga um grupo de agenda',
      tags: ['api','Agenda'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'DELETE',
    path: '/api/v3/agenda/grupo/limpar/{ID}',
    handler: handlers.limparGrupo,
    config: {
      description: 'Apaga todos os números (registros da agenda) pertencentes a um grupo de agenda',
      tags: ['api','Agenda'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            ID: Joi.number().integer().required()
        }
      }
    }
  },
];
