'use strict'

const Joi = require('joi');
const handlers = require('./handlers');

const operationData = Joi.object({
    nome: Joi.string(),
    fullname: Joi.string(),
    ramal: Joi.string(),
    status: Joi.object({
        codigo: Joi.number().integer(),
        StatusNome: Joi.string(),
        DataStatus: Joi.date().iso(),
        pausa: Joi.number().integer(),
    }),
    datastatus: Joi.date().iso(),
    max_chat_atende: Joi.number().integer().min(0),
    visualiza_todos_clientes: Joi.boolean(),
    edita_rota: Joi.boolean(),
    assinatura: Joi.string().allow("", null),
    senha: Joi.string(),
    token: Joi.string().allow(null, "").optional(),
    extra1: Joi.string().allow(null, "").optional(),
    extra2: Joi.string().allow(null, "").optional(),
    token_create: Joi.string(),
    modo: Joi.string().length(1),
    mudasenha: Joi.bool(),
    tempo_maximo_sessao: Joi.number().allow(null).optional(),
    tempo_auto_pausa: Joi.number().allow(null).optional(),
    filas: Joi.array().items(Joi.any()),
    atalho_url: Joi.string().allow(null, "").optional(),
    abrir_url: Joi.boolean().allow(null, "").optional()
});

module.exports = [
    {
        method: 'GET',
        path: '/api/v3/agente',
        handler: handlers.getall,
        config: {
            description: 'Pesquisa os agentes. Informe os filtros para obter uma consulta personalizada ou chame o método sem dados para obter a primeira página de resultados.',
            tags: ['api', 'Agente'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                query: {
                    max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
                    page: Joi.number().integer().default(1).description('Página'),
                    order: Joi.array().items(
                        Joi.object().keys({
                            campo: Joi.string().description('Nome da coluna'),
                            ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
                        })
                    ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
                    nome: Joi.string().description('Filtra por nome'),
                    ramal: Joi.string().description('Filtra por ramal'),
                    fila: Joi.string().description('Filtra por fila')
                }
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v3/agente/{ID}',
        handler: handlers.getAgente,
        config: {
            description: 'Obtém os dados do agente pelo ID',
            tags: ['api', 'Agente'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    ID: Joi.number().integer().required()
                }
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v3/agente/ramal/{AgenteRamal}',
        handler: handlers.getAgenteByRamal,
        config: {
            description: 'Obtém os dados do agente pelo ramal. IMPORTANTE: Observe como o Swagger faz a chamada correta, Use Encode para a / do ramal.',
            tags: ['api', 'Agente'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    AgenteRamal: Joi.string().required()
                }
            }
        }
    },
    {
        method: 'POST',
        path: '/api/v3/agente',
        handler: handlers.create,
        config: {
            description: 'Cria um agente',
            tags: ['api', 'Agente'],
            validate: {
                payload: operationData
            }
        }
    },
    {
        method: 'PUT',
        path: '/api/v3/agente/{ID}',
        handler: handlers.update,
        config: {
            description: 'Atualiza um agente',
            tags: ['api', 'Agente'],
            validate: {
                params: {
                    ID: Joi.number().integer().required()
                },
                payload: operationData
            }
        }
    },
    {
        method: 'DELETE',
        path: '/api/v3/agente/{ID}',
        handler: handlers.delete,
        config: {
            description: 'Atualiza um agente',
            tags: ['api', 'Agente'],
            validate: {
                params: {
                    ID: Joi.number().integer().required()
                }
            }
        }
    },
    {
        method: 'PUT',
        path: '/api/v3/agente/{ID}/login',
        handler: handlers.AgenteLogin,
        config: {
            description: 'Registra um agente na fila informada. A partir deste momento o agente passará a receber ligações da fila, dependendo de sua situação.',
            tags: ['api', 'Agente'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    ID: Joi.number().integer().required()
                },
                payload: Joi.object({
                    fila: Joi.string().required().description('Nome da fila'),
                    pausado: Joi.boolean().default(false).description('Informa se o agente deve logar pausado.')
                })
            }
        }
    },
    {
        method: 'PUT',
        path: '/api/v3/agente/{ID}/logoff',
        handler: handlers.AgenteLogoff,
        config: {
            description: 'Desregistra um agente na fila informada. A partir deste momento o agente não mais receberá ligações da fila.',
            tags: ['api', 'Agente'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    ID: Joi.number().integer().required()
                },
                payload: Joi.object({
                    filas: Joi.array().items(Joi.string()).description('Array com as filas que se deseja efetuar logoff. Não informe para efetuar loggof de todas as filas que o agente estiver logado.')
                }).allow(null)
            }
        }
    },
    {
        method: 'PUT',
        path: '/api/v3/agente/{ID}/pausa',
        handler: handlers.AgentePausa,
        config: {
            description: 'Pausa um agente. Agentes em pausa não recebem novas ligações das filas em que estão registrados.',
            tags: ['api', 'Agente'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    ID: Joi.number().integer().required()
                },
                payload: Joi.object({
                    fila: Joi.string().required().description('Nome da fila'),
                    motivo: Joi.string().description('Motivo')
                })
            }
        }
    },
    {
        method: 'PUT',
        path: '/api/v3/agente/{ID}/despausa',
        handler: handlers.AgenteTiraPausa,
        config: {
            description: 'Tira um agente da pausa. Permite que o agente fique apto a receber licações das filas em que está registrado.',
            tags: ['api', 'Agente'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    ID: Joi.number().integer().required()
                },
                payload: Joi.object({
                    fila: Joi.string().required().description('Nome da fila')
                })
            }
        }
    },
    {
        method: 'PUT',
        path: '/api/v3/agente/{ID}/alterasenha',
        handler: handlers.AgenteAlteraSenha,
        config: {
            description: 'Permite alterar a senha de um agente.',
            tags: ['api', 'Agente'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    ID: Joi.number().integer().required().description('Identificador do agente')
                },
                payload: Joi.object({
                    oldPassword: Joi.string().required().description('Senha atual do agente'),
                    newPassword: Joi.string().required().description('Nova senha do agente'),
                    newConfirm: Joi.string().required().description('Confirmação da nova senha do agente')
                })
            }
        }
    },
    {
        method: 'PUT',
        path: '/api/v3/agente/ramal/{AgenteRamal}/login',
        handler: handlers.AgenteLoginRamal,
        config: {
            description: 'Registra um agente, através de seu ramal, na fila indicada.',
            tags: ['api', 'Agente'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    AgenteRamal: Joi.string().required()
                },
                payload: Joi.object({
                    fila: Joi.string().required().description('Nome da fila'),
                    nome: Joi.string().required().description('Nome do agente'),
                    pausado: Joi.boolean().default(false).description('Informa se o agente deve logar pausado.')
                })
            }
        }
    },
    {
        method: 'PUT',
        path: '/api/v3/agente/ramal/{AgenteRamal}/logoff',
        handler: handlers.AgenteLogoffRamal,
        config: {
            description: 'Desregistra um agente , através de seu ramal, na fila indicada.',
            tags: ['api', 'Agente'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    AgenteRamal: Joi.string().required()
                },
                payload: Joi.object({
                    fila: Joi.string().required().description('Nome da fila')
                })
            }
        }
    },
    {
        method: 'PUT',
        path: '/api/v3/agente/ramal/{AgenteRamal}/pausa',
        handler: handlers.AgentePausaRamal,
        config: {
            description: 'Pausa um agente pelo ramal',
            tags: ['api', 'Agente'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    AgenteRamal: Joi.string().required()
                },
                payload: Joi.object({
                    fila: Joi.string().required().description('Nome da fila'),
                    motivo: Joi.string().description('Motivo')
                })
            }
        }
    },
    {
        method: 'PUT',
        path: '/api/v3/agente/ramal/{AgenteRamal}/despausa',
        handler: handlers.AgenteTiraPausaRamal,
        config: {
            description: 'Tira um agente da pausa pelo ramal',
            tags: ['api', 'Agente'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    AgenteRamal: Joi.string().required()
                },
                payload: Joi.object({
                    fila: Joi.string().required().description('Nome da fila')
                })
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v3/agente/evento',
        handler: handlers.getEvents,
        config: {
            description: 'Pesquisa os eventos dos agentes. Informe os filtros para obter uma consulta personalizada ou chame o método sem dados para obter a primeira página de resultados.',
            tags: ['api', 'Agente'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                query: {
                    max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
                    page: Joi.number().integer().default(1).description('Página'),
                    order: Joi.array().items(
                        Joi.object().keys({
                            campo: Joi.string().description('Nome da coluna'),
                            ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
                        })
                    ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
                    data_ini: Joi.date().iso().description('Filtro por data inicial'),
                    data_fim: Joi.date().iso().description('Filtro por data Final'),
                    filas: Joi.array().items(Joi.string()).example(['FilaA', 'FilaB']).description('Filtra por uma ou mais filas'),
                    agentes: Joi.array().items(Joi.string()).example(['AgenteA', 'AgenteB']).description('Filtra por um ou mais agentes'),
                    eventos: Joi.array().items(Joi.string().valid('Login', 'Pause', 'Answer', 'NoAnswer')).example(['Login', 'Pause', 'Answer', 'NoAnswer']).description('Filtra por um ou mais eventos'),
                    grupos: Joi.array().items(Joi.number().integer()).example(['1', '2']).description('Filtra por um ou mais grupos de eventos'),
                }
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v3/agente/all',
        handler: handlers.all,
        config: {
            description: 'Pesquisa todos os agentes de todos os tipos.',
            tags: ['api', 'Agente'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                query: {
                    filas: Joi.array().items(Joi.string()).example(['FilaA', 'FilaB']).description('Filtra por uma ou mais filas'),
                }
            }
        },
    },
];
