// 'use strict';

const Paginacao = require('../../../util/Paginacao');
const Boom = require('boom');
const db = require('../../../util/db');
//const squel = require("squel");
const ws = require('../../../ws');
const discador = require('../../../discador');
const config = require('../../../config.json');
const TextoUtil = require("../../../util/texto");
const moment = require('moment-timezone');
moment.tz.setDefault(config.servidor.timezone);
const csv = require("fast-csv");
const fs = require("fs");
const _ = require('lodash');
const { requirements } = require('vision');
const SimpleNodeLogger = require('simple-node-logger');
const Cliente = require('../../../models/Cliente');
const ClienteCamposAdicionais = require('../../../models/ClienteCamposAdicionais');
const ClienteAtendimento = require('../../../models/ClienteAtendimento');
const ClienteGrupo = require('../../../models/ClienteGrupo');
const ClienteNumerosAdicionais = require('../../../models/ClienteNumerosAdicionais');

//#region Simple node logger setup
const log = SimpleNodeLogger.createSimpleLogger({
    logFilePath: '/var/log/xcontact.log',
    timestampFormat: 'DD-MM-YYYY HH:mm:ss.SSS'
});
log.setLevel(config.log || 'error');
//#endregion

exports.create = (request, reply) => {
    const knex = db.knex('public.campanhas');
    knex.insert(request.payload);
    knex.returning('id')
        .then(function (id) {
            const ret = {};
            ret.error = false;
            ret.id = id[0];
            ret.message = 'Campanha id = ' + id[0] + ' criada com sucesso.';
            try {
                const campanha = request.payload;
                campanha.id = id[0];
                discador.addCampanha(campanha);
            }
            catch (ex) {
                console.log('Erro interno ao adicionar a campanha no discador, erro: ' + ex);
            }

            reply(ret);
        })
        .catch(function (error) {
            console.error(error);
            reply(Boom.badData(error));
        });
};

exports.update = (request, reply) => {
    const knex = db.knex('public.campanhas');
    knex.update(request.payload);

    knex.where('id', request.params.CampanhaID)
        .then(function (rows) {
            const ret = {};
            ret.error = false;
            ret.total = rows;
            ret.message = 'Campanha id = ' + request.params.CampanhaID + ' atualizada com sucesso.';

            try {
                discador.reloadCampanha(request.params.CampanhaID);
            }
            catch (ex) {
                console.log('Erro interno ao recarregar uma campanha no discador, erro: ' + ex);
            }

            reply(ret);
        })
        .catch(function (error) {
            console.error(error);
            reply(Boom.badData(error));
        });
};

exports.ReplaceClassificacoes = (request, reply) => {
    const knex = db.knex;
    const classificacoes = [];

    for (let i = 0; i < request.payload.length; ++i) {
        const cla = {
            id_campanha: request.params.CampanhaID,
            id_classificacao: request.payload[i].id,
            acao: request.payload[i].acao
        };
        classificacoes.push(cla);
    }

    knex.transaction(function (t) {
        return knex('campanhas_classificacoes')
            .transacting(t)
            .where('id_campanha', request.params.CampanhaID)
            .del()
            .then(function () {
                return knex.batchInsert('campanhas_classificacoes', classificacoes, 100)
                    .transacting(t)
                    .then()
                    .catch(function (error) {
                        console.log('Falha ao inserir as classificações da campanha. Erro = ' + error);
                        t.rollback();
                        throw error;
                    });
            })
            .then(t.commit)
            .catch(function (e) {
                t.rollback();
                console.log(e);
                throw e;
            })
    })
        .then(function () {
            console.log('Classificações da campanha atualizadas com sucesso!');
            const ret = {};
            ret.error = false;
            ret.message = 'Classificações da campanha atualizadas com sucesso!';
            reply(ret);
        })
        .catch(function (e) {
            console.log('Falha ao atualizar as classificações da campanha. Erro = ' + e);
            reply(Boom.badData(e));
        });
};

exports.delete = (request, reply) => {
    const knex = db.knex;
    knex.transaction(function (t) {
        return knex('campanhas')
            .transacting(t)
            .where('id', request.params.CampanhaID)
            .del()
            .then(function () {
                return knex('campanhas_contato')
                    .where('id_campanha', request.params.CampanhaID)
                    .del()
                    .then(function () {
                        return knex('campanhas_contato_numeros')
                            .where('id_campanha', request.params.CampanhaID)
                            .del()
                            .then(function () {
                                return knex('campanhas_contato_campos')
                                    .where('id_campanha', request.params.CampanhaID)
                                    .del();
                            })
                    })
            })
            .then(t.commit)
            .catch(function (e) {
                t.rollback();
                console.log(e);
                throw e;
            })
    })
        .then(function () {
            console.log('Campanha apagada com sucesso!');
            const ret = {};
            ret.error = false;
            ret.message = 'Campanha id = ' + request.params.CampanhaID + ' apagada com sucesso.';
            reply(ret);
        })
        .catch(function (e) {
            console.log('Falha ao apagar campanha. Erro = ' + e);
            reply(Boom.badData(e));
        });
};

exports.zerar = (request, reply) => {
    const knex = db.knex;
    knex.transaction(function (t) {
        return knex('campanhas_contato_numeros')
            .transacting(t)
            .where('id_campanha', request.params.CampanhaID)
            .update({ sucesso: false, tentativas: 0, retentativas: 0 })
            .then(function () {
                return knex('campanhas_contato')
                    .where('id_campanha', request.params.CampanhaID)
                    .update({ sucesso: false });
            })
            .then(t.commit)
            .catch(function (e) {
                t.rollback();
                console.log(e);
                throw e;
            })
    })
        .then(function () {
            console.log('Campanha zerada com sucesso!');
            const ret = {};
            ret.error = false;
            ret.message = 'Campanha id = ' + request.params.CampanhaID + ' zerada com sucesso.';
            reply(ret);
        })
        .catch(function (e) {
            console.log('Falha ao zerar a campanha. Erro = ' + e);
            reply(Boom.badData(e));
        });
};

exports.getall = (request, reply) => {
    const max = request.query.max ? request.query.max : 15;
    const page = request.query.page ? request.query.page : 1;
    const knex = db.knex('public.v_campanhas');

    //Filtro de filas que são permitidas ao supervisor (Usuário)
    if (request.query.filas && Array.isArray(request.query.filas) && request.query.filas.length > 0) {
        knex.whereIn('fila', request.query.filas);
    }

    if (request.query.status && request.query.status.length > 0) knex.where('status', request.query.status);
    if (request.query.nome) knex.whereRaw('LOWER(nome) LIKE ?', '%' + decodeURIComponent(request.query.nome).toLowerCase() + '%');

    if (request.query.data_status_ini) knex.where('data_status', '>=', request.query.data_status_ini);
    if (request.query.data_status_fim) knex.where('data_status', '<=', request.query.data_status_fim);

    if (request.query.data_start_ini) knex.where('data_start', '>=', request.query.data_start_ini);
    if (request.query.data_start_fim) knex.where('data_start', '<=', request.query.data_start_fim);

    if (request.query.data_create_ini) knex.where('data_create', '>=', request.query.data_create_ini);
    if (request.query.data_create_fim) knex.where('data_create', '<=', request.query.data_create_fim);


    if (request.query.data_agendamento_ini) knex.where('data_ini', '>=', request.query.data_agendamento_ini);
    if (request.query.data_agendamento_fim) knex.where('data_fim', '<=', request.query.data_agendamento_fim);

    if (max === -1) {
        //console.log('SQL = '+ knex.toString());
        knex.clearSelect();
        // Parametros de ordenação, fora do count
        if (request.query.order) {
            for (let p = 0; p < request.query.order.length; ++p) {
                if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                    knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
                }
                else if (request.query.order[p] && request.query.order[p].campo) {
                    knex.orderBy(request.query.order[p].campo, 'ASC');
                }
            }
        }
        knex.select('*')
            .then(function (rows) {
                const ret = {};
                ret.error = false;
                ret.full = true;
                ret.total = rows.length;
                ret.message = 'Lista de todas as campanhas';
                ret.dados = rows;
                if (request.query.return === 'array') reply(rows);
                else reply(ret);
            })
            .catch(function (error) {
                console.error(error);
                reply(Boom.badData(error));
            });
    }
    else {
        const offset = max * (page - 1);
        //console.log('SQL = '+knex.count('* as total').toString());
        knex.count('* as total')
            .then(function (total) {
                if (total[0].total > 0) {
                    knex.clearSelect();
                    // Parametros de ordenação, fora do count
                    if (request.query.order) {
                        for (let p = 0; p < request.query.order.length; ++p) {
                            if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                                knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
                            }
                            else if (request.query.order[p] && request.query.order[p].campo) {
                                knex.orderBy(request.query.order[p].campo, 'ASC');
                            }
                        }
                    }
                    //console.log('SQL = '+knex.select('*').limit(max).offset(offset).toString());
                    knex.select('*').limit(max).offset(offset).then(function (rows) {
                        const pages = Math.ceil(parseInt(total[0].total) / max);
                        const ret = {};
                        ret.error = false;
                        ret.full = pages === 1 ? true : false;
                        ret.page = page;
                        ret.max = max;
                        ret.offset = offset;
                        ret.total = parseInt(total[0].total);
                        ret.pages = pages;
                        ret.message = 'Lista de campanhas paginada';
                        ret.dados = rows;
                        if (request.query.return === 'array') reply(rows);
                        else reply(ret);
                    })
                        .catch(function (error) {
                            console.error(error);
                            reply(Boom.badData(error));
                        });
                }
                else {
                    let ret = {};
                    ret.error = true;
                    ret.total = 0;
                    ret.message = 'Nenhum registro encontrado';
                    reply(ret);
                }
            })
            .catch(function (error) {
                console.error(error);
                reply(Boom.badData(error));
            });
    }
};

exports.getclassificacoes = (request, reply) => {
    const campanha = request.params.CampanhaID;

    const max = request.query.max ? request.query.max : 15;
    const page = request.query.page ? request.query.page : 1;

    if (max === -1) {
        db.knex.select('classificacoes.*', 'campanhas_classificacoes.acao')
            .from('campanhas_classificacoes')
            .innerJoin('classificacoes', 'campanhas_classificacoes.id_classificacao', 'classificacoes.id')
            .where('campanhas_classificacoes.id_campanha', request.params.CampanhaID)
            .then(function (rows) {
                const ret = {};
                ret.error = false;
                ret.full = true;
                ret.total = rows.length;
                ret.message = 'Lista das classificações da campanha';
                ret.dados = rows;

                reply(ret);
            })
            .catch(function (error) {
                console.error(error);
                reply(Boom.badData(error));
            });
    }
    else {
        const offset = max * (page - 1);
        db.query("SELECT count(*) as total FROM public.campanhas_classificacoes WHERE id_campanha = $1", [campanha])
            .then(res1 => {
                db.query("SELECT * FROM public.campanhas_classificacoes WHERE id_campanha = $1 LIMIT $2 OFFSET $3", [campanha, max, offset])
                    .then(res => {
                        const pages = Math.ceil(parseInt(res1.rows[0].total) / max);
                        const ret = {};
                        ret.error = false;
                        ret.full = pages === 1 ? true : false;
                        ret.page = page;
                        ret.max = max;
                        ret.offset = offset;
                        ret.total = parseInt(res1.rows[0].total);
                        ret.pages = pages;
                        ret.message = 'Lista de campanhas_classificacoes paginada';
                        ret.dados = res.rows;
                        reply(ret);
                    })
                    .catch(e => {
                        console.log(e);
                        reply(Boom.badData(e));
                    });
            })
            .catch(e => {
                console.log(e);
                reply(Boom.badData(e));
            });
    }
};

exports.getContatos = (request, reply) => {
    const max = request.query.max ? request.query.max : 15;
    const page = request.query.page ? request.query.page : 1;
    const knex = db.knex('public.campanhas_contatos_full');
    knex.where('id_campanha', request.params.CampanhaID);

    if (max === -1) {
        //console.log('SQL = '+ knex.toString());
        knex.clearSelect();
        // Parametros de ordenação, fora do count
        if (request.query.order) {
            for (let p = 0; p < request.query.order.length; ++p) {
                if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                    knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
                }
                else if (request.query.order[p] && request.query.order[p].campo) {
                    knex.orderBy(request.query.order[p].campo, 'ASC');
                }
            }
        }
        if (request.query.campos) {
            knex.whereRaw("campos @> ANY (ARRAY ['" + JSON.stringify(request.query.campos) + "']::jsonb[])");
        }
        knex.select('*')
            .then(function (rows) {
                const ret = {};
                ret.error = false;
                ret.full = true;
                ret.total = rows.length;
                ret.message = 'Lista de todos os cantatos  da campanha';
                ret.dados = rows;
                if (request.query.return === 'array') reply(rows);
                else reply(ret);
            })
            .catch(function (error) {
                console.error(error);
                reply(Boom.badData(error));
            });
    }
    else {
        const offset = max * (page - 1);
        //console.log('SQL = '+knex.count('* as total').toString());
        knex.count('* as total')
            .then(function (total) {
                if (total[0].total > 0) {
                    knex.clearSelect();
                    // Parametros de ordenação, fora do count
                    if (request.query.order) {
                        for (let p = 0; p < request.query.order.length; ++p) {
                            if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                                knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
                            }
                            else if (request.query.order[p] && request.query.order[p].campo) {
                                knex.orderBy(request.query.order[p].campo, 'ASC');
                            }
                        }
                    }
                    if (request.query.campos) {
                        knex.whereRaw("campos @> ANY (ARRAY ['" + JSON.stringify(request.query.campos) + "']::jsonb[])");
                    }
                    //console.log('SQL = '+knex.select('*').limit(max).offset(offset).toString());
                    knex.select('*').limit(max).offset(offset).then(function (rows) {
                        const pages = Math.ceil(parseInt(total[0].total) / max);
                        const ret = {};
                        ret.error = false;
                        ret.full = pages === 1 ? true : false;
                        ret.page = page;
                        ret.max = max;
                        ret.offset = offset;
                        ret.total = parseInt(total[0].total);
                        ret.pages = pages;
                        ret.message = 'Lista de contatos da campanha paginada';
                        ret.dados = rows;
                        if (request.query.return === 'array') reply(rows);
                        else reply(ret);
                    })
                        .catch(function (error) {
                            console.error(error);
                            reply(Boom.badData(error));
                        });
                }
                else {
                    const ret = {};
                    ret.error = true;
                    ret.total = 0;
                    ret.message = 'Nenhum registro encontrado';
                    reply(ret);
                }
            })
            .catch(function (error) {
                console.error(error);
                reply(Boom.badData(error));
            });
    }
};

exports.getCampanhasRegistros = (request, reply) => {
    const max = request.query.max ? request.query.max : 15;
    const page = request.query.page ? request.query.page : 1;
    const knex = db.knex.select('campanhas_registro.*', 'status_ligacao.nome as status', 'campanhas.nome as campanha', 'classificacoes.nome as classificacao', 'cdr_classificacoes.obs as observacao');

    if (request.query.campanhas) knex.whereIn('campanhas_registro.id_campanha', request.query.campanhas);
    if (request.query.status && request.query.status.length > 0) knex.whereIn('campanhas_registro.id_status', request.query.status);
    if (request.query.data_ini) knex.where('campanhas_registro.data_status', '>=', request.query.data_ini);
    if (request.query.data_fim) knex.where('campanhas_registro.data_status', '<=', request.query.data_fim);
    if (request.query.numero_operador && request.query.numero) {
        switch (request.query.numero_operador) {
            case 1:
                knex.where('campanhas_registro.numero', '=', request.query.numero);
                break;
            case 2:
                knex.where('campanhas_registro.numero', 'like', request.query.numero + '%');
                break;
            case 3:
                knex.where('campanhas_registro.numero', 'like', '%' + request.query.numero);
                break;
            case 4:
                knex.where('campanhas_registro.numero', 'not like', '%' + request.query.numero + '%');
                break;
            case 5:
                knex.where('campanhas_registro.numero', 'not like', request.query.numero);
                break;
        }
    }
    else if (request.query.numero) {
        knex.where('campanhas_registro.numero', 'like', '%' + request.query.numero + '%');
    }

    if (max === -1) {
        // Parametros de ordenação, fora do count
        if (request.query.order) {
            for (let p = 0; p < request.query.order.length; ++p) {
                if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                    knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
                }
                else if (request.query.order[p] && request.query.order[p].campo) {
                    knex.orderBy(request.query.order[p].campo, 'ASC');
                }
            }
        }
        knex.from('campanhas_registro')
            .leftJoin('status_ligacao', 'campanhas_registro.id_status', 'status_ligacao.id')
            .leftJoin('campanhas', 'campanhas_registro.id_campanha', 'campanhas.id')
            .leftJoin('cdr_classificacoes', 'campanhas_registro.callid', 'cdr_classificacoes.uniqueid')
            .leftJoin('classificacoes', 'cdr_classificacoes.id_classificacao', 'classificacoes.id')
            .then(function (rows) {
                const ret = {};
                ret.error = false;
                ret.full = true;
                ret.total = rows.length;
                ret.message = 'Registros das campanhas';
                ret.dados = rows;

                reply(ret);
            })
            .catch(function (error) {
                console.error(error);
                reply(Boom.badData(error));
            });
    }
    else {
        const offset = max * (page - 1);
        //console.log('SQL = '+knex.count('* as total').toString());
        knex.clearSelect();
        knex.count('* as total')
            .from('campanhas_registro')
            .then(function (total) {
                if (total[0].total > 0) {
                    knex.clearSelect();
                    // Parametros de ordenação, fora do count
                    if (request.query.order) {
                        for (let p = 0; p < request.query.order.length; ++p) {
                            if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                                knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
                            }
                            else if (request.query.order[p] && request.query.order[p].campo) {
                                knex.orderBy(request.query.order[p].campo, 'ASC');
                            }
                        }
                    }
                    //console.log('SQL = '+knex.select('*').limit(max).offset(offset).toString());
                    knex.select('campanhas_registro.*', 'status_ligacao.nome as status', 'campanhas.nome as campanha', 'classificacoes.nome as classificacao', 'cdr_classificacoes.obs as observacao')
                        .from('campanhas_registro')
                        .leftJoin('status_ligacao', 'campanhas_registro.id_status', 'status_ligacao.id')
                        .leftJoin('campanhas', 'campanhas_registro.id_campanha', 'campanhas.id')
                        .leftJoin('cdr_classificacoes', 'campanhas_registro.callid', 'cdr_classificacoes.uniqueid')
                        .leftJoin('classificacoes', 'cdr_classificacoes.id_classificacao', 'classificacoes.id')
                        .limit(max)
                        .offset(offset)
                        .then(function (rows) {
                            const pages = Math.ceil(parseInt(total[0].total) / max);
                            const ret = {};
                            ret.error = false;
                            ret.full = pages === 1 ? true : false;
                            ret.page = page;
                            ret.max = max;
                            ret.offset = offset;
                            ret.total = parseInt(total[0].total);
                            ret.pages = pages;
                            ret.message = 'Lista de campanhas paginada';
                            ret.dados = rows;
                            reply(ret);
                        })
                        .catch(function (error) {
                            console.error(error);
                            reply(Boom.badData(error));
                        });
                }
                else {
                    const ret = {};
                    ret.error = true;
                    ret.total = 0;
                    ret.message = 'Nenhum registro encontrado';
                    reply(ret);
                }
            })
            .catch(function (error) {
                console.error(error);
                reply(Boom.badData(error));
            });
    }
};

exports.getRegistros = (request, reply) => {
    const campanha = request.params.CampanhaID;

    const max = request.query.max ? request.query.max : 15;
    const page = request.query.page ? request.query.page : 1;

    if (max === -1) {
        db.query("SELECT a.*,b.nome as status FROM public.campanhas_registro a LEFT JOIN status_ligacao b on a.id_status = b.id WHERE a.id_campanha = $1", [campanha])
            .then(res => {
                const ret = {};
                ret.error = false;
                ret.full = true;
                ret.total = res.rowCount;
                ret.message = 'Registros da Campanha';
                ret.dados = res.rows;
                reply(ret);
            })
            .catch(e => {
                console.log(e);
                reply(Boom.badData(e));
            });
    }
    else {
        const offset = max * (page - 1);
        db.query("SELECT count(*) as total FROM public.campanhas_registro WHERE id_campanha = $1", [campanha])
            .then(res1 => {
                db.query("SELECT a.*,b.nome as status FROM public.campanhas_registro a LEFT JOIN status_ligacao b on a.id_status = b.id WHERE a.id_campanha = $1 LIMIT $2 OFFSET $3", [campanha, max, offset])
                    .then(res => {
                        const pages = Math.ceil(parseInt(res1.rows[0].total) / max);
                        const ret = {};
                        ret.error = false;
                        ret.full = pages === 1 ? true : false;
                        ret.page = page;
                        ret.max = max;
                        ret.offset = offset;
                        ret.total = parseInt(res1.rows[0].total);
                        ret.pages = pages;
                        ret.message = 'Registros da campanha paginados';
                        ret.dados = res.rows;
                        reply(ret);
                    })
                    .catch(e => {
                        console.log(e);
                        reply(Boom.badData(e));
                    });
            })
            .catch(e => {
                console.log(e);
                reply(Boom.badData(e));
            });
    }
};

exports.uploadContatos = (request, reply) => {
    const registros = [];
    // Obter a campanha
    db.knex.transaction(function (t) {
        return db.knex('public.campanhas')
            .transacting(t)
            .where('id', request.params.CampanhaID)
            .select('id_layout')
            .then(function (rows) {
                if (rows.length > 0) {
                    if (rows[0].id_layout && rows[0].id_layout !== null) {
                        const id_layout = rows[0].id_layout;
                        console.log('layout encontrado = ' + id_layout);
                        // Obter os dados do layout
                        return db.knex('public.layouts')
                            .transacting(t)
                            .where('id', id_layout)
                            .select('*')
                            .then(function (layouts) {
                                const delimitador = layouts[0].delimitador;
                                const reg_ini = layouts[0].cabecalho ? 1 : 0;
                                console.log('Delimitador: ' + delimitador + ', Cabeçalho: ' + reg_ini);
                                // Obter campos
                                return db.knex('public.layouts_campos')
                                    .transacting(t)
                                    .where('id_layout', id_layout)
                                    .select('*')
                                    .then(async function (layouts_campos) {
                                        if (layouts_campos.length > 0) {
                                            const fones = [];
                                            const campos = [];
                                            for (let t = 0; t < layouts_campos.length; ++t) {
                                                if (layouts_campos[t].xcontact && layouts_campos[t].xcontact !== null && layouts_campos[t].xcontact.startsWith("telefone")) {
                                                    fones.push(layouts_campos[t]);
                                                }
                                                else {
                                                    campos.push(layouts_campos[t]);
                                                }
                                            }
                                            // Tem telefone vamos ler os registros
                                            if (fones.length > 0) {
                                                const dados = request.payload.file.toString('latin1', 0, request.payload.file.length);
                                                const id_campanha = request.params.CampanhaID;
                                                const contatos = [];

                                                //Processa o CSV de forma síncrona para manter a transação
                                                let parse = new Promise((resolve, reject) => {
                                                    csv.fromString(dados, { headers: false, delimiter: delimitador })
                                                        .on("data", function (data) {
                                                            registros.push(data);
                                                        })
                                                        .on("end", function () {
                                                            console.log("CSV recebido, lendo registros...");

                                                            // Organiza os registros para insert do contato, para cada registro do arquivo insert 1 contato
                                                            for (let r1 = reg_ini; r1 < registros.length; ++r1) {
                                                                const contato = { id_campanha: id_campanha };
                                                                contatos.push(contato);
                                                            }
                                                            console.log('CSV - Fim da Leitura dos registros');
                                                            resolve(true);
                                                        });
                                                });
                                                await parse;

                                                console.log(contatos);
                                                return db.knex.batchInsert('campanhas_contato', contatos, 10000)
                                                    .returning('id')
                                                    .transacting(t)
                                                    .then(function (ids) {
                                                        console.log('Contatos inseridos, colocando ids dos contatos nos registros');
                                                        var array_campos = [];
                                                        var array_numeros = [];
                                                        if (!ids || !Array.isArray(ids)) throw 'Não houveram resultados válidos na criação dos contatos da campanha.';
                                                        console.log(ids);
                                                        for (let r = reg_ini; r < registros.length; ++r) {
                                                            // Números
                                                            for (let f = 0; f < fones.length; ++f) {
                                                                if (registros[r][fones[f].posicao_csv] && registros[r][fones[f].posicao_csv].trim().length > 0) {
                                                                    const id_contato = reg_ini === 0 ? ids[r] : ids[r - 1];
                                                                    const cn = {
                                                                        id_contato: id_contato,
                                                                        id_campanha: id_campanha,
                                                                        numero: registros[r][fones[f].posicao_csv],
                                                                        fone_sequencia: fones[f].xcontact,
                                                                        nome: fones[f].nome
                                                                    };
                                                                    array_numeros.push(cn);
                                                                }
                                                            }
                                                            // Campos
                                                            for (let c = 0; c < campos.length; ++c) {
                                                                if (registros[r][campos[c].posicao_csv] && registros[r][campos[c].posicao_csv].trim().length > 0) {
                                                                    const id_contato = reg_ini === 0 ? ids[r] : ids[r - 1];
                                                                    const cc = {
                                                                        id_contato: id_contato,
                                                                        id_campanha: id_campanha,
                                                                        nome: campos[c].nome,
                                                                        valor: registros[r][campos[c].posicao_csv],
                                                                    };
                                                                    array_campos.push(cc);
                                                                }
                                                            }
                                                        }
                                                        // Insert Numeros
                                                        return db.knex.batchInsert('campanhas_contato_numeros', array_numeros, 5000)
                                                            .returning('id')
                                                            .transacting(t)
                                                            .then(function (ids_numeros) {
                                                                // OK para os numeros insert campos
                                                                return db.knex.batchInsert('campanhas_contato_campos', array_campos, 5000)
                                                                    .returning('id')
                                                                    .transacting(t)
                                                                    .then(t.commit)
                                                                    .catch(function (error) {
                                                                        throw 'Erro no insert dos campos. ' + error;
                                                                    });
                                                            })
                                                            .catch(function (error) {
                                                                throw 'Erro no insert dos campos. ' + error;
                                                            });
                                                    })
                                                    .catch(function (error) {
                                                        throw 'Erro no insert dos campos. ' + error;
                                                    });

                                            }
                                            else {
                                                throw 'O layout associado a CampanhaID = ' + request.params.CampanhaID + ' não possui campos de telefone definidos.';
                                            }
                                        }
                                        else {
                                            throw 'O layout associado a CampanhaID = ' + request.params.CampanhaID + ' não possui campos definidos.';
                                        }
                                    })
                                    .catch(function (error) {
                                        throw error;
                                    });
                            })
                            .catch(function (error) {
                                throw error;
                            });
                    }
                    else {
                        throw 'A CampanhaID = ' + request.params.CampanhaID + ' não possui layout cadastrado.';
                    }
                }
                else {
                    throw 'Nenhuma campanha encontrada para CampanhaID = ' + request.params.CampanhaID;
                }
            })
            .catch(function (error) {
                console.error(error);
                t.rollback();
                throw error;
            });
    })
        .then(function () {
            // OK, tudo inserido....
            reply({ error: false, rows: registros.length, message: 'Arquivo importado com sucesso.' });
        })
        .catch(function (error) {
            console.error(error);
            reply(Boom.badData(error));
        });
};

exports.uploadContatosJSON = async (request, reply) => {
    try {
        // Obter a campanha
        const campanha = await getCampanha(request.params.CampanhaID);

        if (!campanha) reply(Boom.badData(`A campanha ${request.params.CampanhaID} não existe.`));

        await reply(insertContatos(campanha, request.payload, true, request.query.unique));
    }
    catch (error) {
        reply(Boom.badData(error));
    }
};

async function insertContatos(campanha, registros, byName, unique) {
    if (byName === null || byName === undefined) byName = true;
    // Tem telefone vamos ler os registros
    if (campanha.fones.length > 0) {
        console.log("Contatos recebidos, lendo registros...");

        // Organiza os registros para insert do contato, para cada registro do arquivo insere 1 contato
        let contatos = registros.map((x) => { return { id_campanha: campanha.id }; });
        let array_campos = [];
        let array_numeros = [];

        let parse = new Promise((resolve, reject) => {
            for (let r = 0; r < registros.length; ++r) {
                // Numeros
                for (let f = 0; f < campanha.fones.length; ++f) {
                    if (registros[r]) {
                        let layoutValue = byName ? campanha.fones[f].nome : campanha.fones[f].posicao_csv;
                        let regVal = byName ? registros[r][layoutValue] : registros[r][Object.keys(registros[r])[layoutValue]];
                        if (regVal && regVal.trim().length > 0) {
                            let cn = {
                                contato: TextoUtil.limparTexto(contatos[r]),
                                id_campanha: campanha.id,
                                numero: TextoUtil.limparTextoNumerico(regVal),
                                fone_sequencia: campanha.fones[f].xcontact,
                                nome: TextoUtil.limparTexto(campanha.fones[f].nome)
                            };
                            array_numeros.push(cn);
                        }
                    }
                }
                // Campos
                for (let c = 0; c < campanha.campos.length; ++c) {
                    if (registros[r]) {
                        let layoutValue = byName ? campanha.campos[c].nome : campanha.campos[c].posicao_csv;
                        let regVal = byName ? registros[r][layoutValue] : registros[r][Object.keys(registros[r])[layoutValue]];
                        if (regVal && regVal.trim().length > 0) {
                            const cc = {
                                contato: contatos[r],
                                id_campanha: campanha.id,
                                nome: TextoUtil.limparTexto(campanha.campos[c].nome),
                                valor: TextoUtil.limparTexto(regVal),
                            };
                            array_campos.push(cc);
                        }
                    }
                }
            }

            //Para controle de unicidade, utilizado por hora apenas na API ou para o serviço de CallBack
            if (unique === true) {
                //Teste de unicidade coletivo
                db.knex('public.campanhas_contato_numeros')
                    .where('id_campanha', campanha.id)
                    .whereIn('numero', array_numeros.map((x) => { return x.numero; }))
                    .distinct('numero')
                    .then(function (rows) {
                        console.log("Registros duplicados: " + rows.length);
                        if (rows.length > 0) {
                            //Ajustar arrays
                            var i = array_numeros.length;
                            while (i--) {
                                if (rows.find((x) => { return x.numero == array_numeros[i].numero; }))
                                    array_numeros.splice(i, 1);
                            }

                            if (array_numeros.length > 0) {
                                //Remove os campos e contatos, pois não temos o que fazer com contatos e campos sem números
                                //Utilizamos aqui uma referência de objeto
                                array_campos = array_campos.filter((y) => {
                                    return array_numeros.find((x) => {
                                        return x.contato == y.contato;
                                    });
                                });
                                contatos = contatos.filter((y) => {
                                    return array_numeros.find((x) => {
                                        return x.contato == y;
                                    });
                                });
                            }
                            else {
                                array_campos = [];
                                contatos = [];
                            }
                            resolve(true);
                        }
                        else
                            resolve(true);
                    })
                    .catch((e) => {
                        console.log("Erro na verificação de duplicidade " + e);
                        throw e;
                    });
            }
            else
                resolve(true);
        });
        await parse;

        if (contatos.length > 0) {
            //Inserção
            await db.knex.transaction(function (t) {
                return db.knex.batchInsert('campanhas_contato', contatos, 10000)
                    .returning('id')
                    .transacting(t)
                    .then(function (ids) {
                        console.log(`${ids.length}, contatos inseridos, colocando ids dos contatos nos registros...`);
                        //Ajusta os ids dos contatos para inserção
                        for (let i = 0; i < ids.length; i++) {
                            contatos[i].id_contato = ids[i];
                        }
                        // Insert Numeros
                        return db.knex.batchInsert('campanhas_contato_numeros', array_numeros.map((e) => {
                            e.id_contato = e.contato.id_contato;
                            delete e.contato;
                            return e;
                        }), 5000)
                            .returning('id')
                            .transacting(t)
                            .then(function (ids_numeros) {
                                console.log(`${ids_numeros.length}, números de contatos inseridos, inserindo campos...`);
                                return db.knex.batchInsert('campanhas_contato_campos', array_campos.map((e) => {
                                    e.id_contato = e.contato.id_contato;
                                    delete e.contato;
                                    return e;
                                }), 5000)
                                    .returning('id')
                                    .transacting(t)
                                    .then(function (ids) {
                                        console.log(`${ids.length}, campos de contatos inseridos.`);
                                        if (contatos.length > 0) {
                                            const ret = {};
                                            ret.error = false;
                                            ret.total = contatos.length;
                                            ret.message = `${contatos.length}, contatos criados com sucesso.`;
                                            return (ret);
                                        }
                                        else {
                                            throw `Nenhum contato registrado para a campanha ${campanha.id}`;
                                        }
                                    })
                                    .catch((error) => {
                                        throw 'Erro no insert dos campos. ' + error;
                                    });
                            })
                            .catch((error) => {
                                throw 'Erro no insert dos números. ' + error;
                            });
                    })
                    .catch((error) => {
                        throw 'Erro na criação dos contatos. ' + error;
                    });
            })
                .catch(function (error) {
                    console.error(error);
                    throw error;
                });
        }
        else
            throw `Não haviam dados a inserir.`;
    }
    else
        throw `A campanha ${campanha.id} não contém telefones no layout.`;
};

// Obter a campanha
function getCampanha(id, full) {
    let campanha = {
        fones: [],
        campos: []
    };

    return new Promise((resolve, reject) => {
        db.knex('public.campanhas')
            .where('id', id)
            .select('*')
            .then(function (rows) {
                if (rows.length > 0) {
                    if (rows[0].id_layout && rows[0].id_layout !== null) {
                        //Redefine o objeto
                        campanha.id = id;
                        if (full === true) {
                            campanha = rows[0];
                            campanha.fones = [];
                            campanha.campos = [];
                        }
                        const id_layout = rows[0].id_layout;
                        console.log('layout encontrado = ' + id_layout);
                        // Obter os dados do layout
                        db.knex('public.layouts')
                            .where('id', id_layout)
                            .select('*')
                            .then(function (layouts) {
                                // Obter campos
                                db.knex('public.layouts_campos')
                                    .where('id_layout', id_layout)
                                    .select('*')
                                    .then(function (layouts_campos) {
                                        if (layouts_campos.length > 0) {
                                            for (let t = 0; t < layouts_campos.length; ++t) {
                                                if (layouts_campos[t].xcontact && layouts_campos[t].xcontact !== null && layouts_campos[t].xcontact.startsWith("telefone")) {
                                                    campanha.fones.push(layouts_campos[t]);
                                                }
                                                else {
                                                    campanha.campos.push(layouts_campos[t]);
                                                }
                                            }
                                            resolve(campanha);
                                        }
                                        else
                                            resolve(null);
                                    })
                                    .catch((e) => { throw e; });
                            })
                            .catch((e) => { throw e; });
                    }
                }
            })
            .catch(function (error) {
                console.error(error);
                reject(error);
            });
    });
};

exports.remove_contato = (request, reply) => {
    db.knex('public.campanhas_contato')
        .where('id', request.params.ContatoID)
        .where('id_campanha', request.params.CampanhaID)
        .del()
        .then(function (rows) {
            if (rows > 0) {
                const ret = {};
                ret.error = false;
                ret.total = rows;
                ret.message = 'Contato id = ' + request.params.ContatoID + ' apagada com sucesso.';
                reply(ret);
            }
            else {
                reply(Boom.badData("Nenhum registro encontrado para ID = " + request.params.ContatoID));
            }
        })
        .catch(function (error) {
            console.error(error);
            reply(Boom.badData(error));
        });
};

exports.remove_contato_por_campo = (request, reply) => {
    db.knex.transaction(function (t) {
        return db.knex.select('id_contato')
            .from('campanhas_contato_campos')
            .where('nome', request.params.CampoNome)
            .where('valor', request.params.CampoValor)
            .then(function (rows) {
                if (rows.length > 0) {
                    console.log(`AQUI +++++++++++++++++++++++++++++++++`, rows[0].id_contato);
                    console.log(`id`, request.params.CampanhaID);
                    return db.knex('campanhas_contato_campos')
                        .transacting(t)
                        .where('id_contato', rows[0].id_contato)
                        .where('id_campanha', request.params.CampanhaID)
                        .del()
                        .then(function () {
                            return db.knex('campanhas_contato_numeros')
                                .transacting(t)
                                .where('id_contato', rows[0].id_contato)
                                .where('id_campanha', request.params.CampanhaID)
                                .del()
                                .then(function () {
                                    return db.knex('campanhas_contato')
                                        .transacting(t)
                                        .where('id', rows[0].id_contato)
                                        .where('id_campanha', request.params.CampanhaID)
                                        .del()
                                        .then(t.commit)
                                        .catch((e) => {
                                            throw e;
                                        });
                                })
                                .catch((e) => {
                                    console.error(e);
                                    throw e;
                                });
                        })
                        .catch((e) => {
                            console.error(e);
                            throw e;
                        });
                }
                else {
                    throw new Error("Nenhum resgistro encontrado para " + request.params.CampoNome + " = " + request.params.CampoValor);
                }
            })
            .catch((e) => {
                console.error(e);
                t.rollback();
                throw e;
            });
    })
        .then(function () {
            const ret = {};
            ret.error = false;
            ret.message = 'Contato removido com sucesso.';
            reply(ret);
        })
        .catch(function (e) {
            reply(Boom.badData("Nenhum resgistro encontrado para " + request.params.CampoNome + " = " + request.params.CampoValor));
        });
};

exports.remove_todos_contatos = (request, reply) => {
    const knex = db.knex;
    knex.transaction(function (t) {
        return knex('campanhas_contato')
            .transacting(t)
            .where('id_campanha', request.params.CampanhaID)
            .del()
            .then(function () {
                return knex('campanhas_contato_numeros')
                    .where('id_campanha', request.params.CampanhaID)
                    .del()
                    .then(function () {
                        return knex('campanhas_contato_campos')
                            .where('id_campanha', request.params.CampanhaID)
                            .del();
                    })
            })
            .then(t.commit)
            .catch(function (e) {
                t.rollback();
                console.log(e);
                throw e;
            })
    })
        .then(function () {
            console.log('Contatos da campanha apagados com sucesso!');
            const ret = {};
            ret.error = false;
            ret.message = 'Contatos pagados da campanha id = ' + request.params.CampanhaID;
            reply(ret);
        })
        .catch(function (e) {
            console.log('Falha ao apagar os contatos da campanha. Erro = ' + e);
            reply(Boom.badData(e));
        });
};

exports.remove_contato_por_numero = (request, reply) => {
    db.knex.transaction(function (t) {
        return db.knex.select('id_contato')
            .transacting(t)
            .from('campanhas_contato_numeros')
            .where('id_campanha', request.params.CampanhaID)
            .where('numero', request.params.numero)
            .then(function (rows) {
                if (rows.length > 0) {
                    return db.knex('campanhas_contato_numeros')
                        .transacting(t)
                        .where('id_contato', rows[0].id_contato)
                        .where('id_campanha', request.params.CampanhaID)
                        .del()
                        .then(function () {
                            return db.knex('campanhas_contato_campos')
                                .transacting(t)
                                .where('id_contato', rows[0].id_contato)
                                .where('id_campanha', request.params.CampanhaID)
                                .del()
                                .then(function () {
                                    return db.knex('campanhas_contato')
                                        .transacting(t)
                                        .where('id', rows[0].id_contato)
                                        .where('id_campanha', request.params.CampanhaID)
                                        .del()
                                        .then(t.commit)
                                        .catch((e) => {
                                            throw e;
                                        });
                                })
                                .catch((e) => {
                                    throw e;
                                });
                        })
                        .catch((e) => {
                            throw e;
                        });
                }
                else {
                    throw new Error(`Nenhum registro encontrado na campanha ${request.params.CampanhaID} com número ${request.params.numero}`);
                }
            })
            .catch((e) => {
                console.log(e);
                t.rollback();
                throw e;
            });
    })
        .then(function () {
            console.log('Contato removido com sucesso.');
            const ret = {};
            ret.error = false;
            ret.message = 'Contato removido com sucesso.';
            reply(ret);
        })
        .catch(function (e) {
            console.log(e);
            reply(Boom.badData(e));
        });
};

exports.getFilaDiscagem = (request, reply) => {
    const max = request.query.max ? request.query.max : 15;
    const page = request.query.page ? request.query.page : 1;
    const knex = db.knex('campanhas_contato_numeros');
    const pag = new Paginacao(max, page);

    knex.where('campanhas_contato_numeros.id_campanha', request.params.CampanhaID);

    knex.where('campanhas_contato_numeros.sucesso', false);
    knex.whereRaw("campanhas_contato_numeros.tentativas < campanhas.tentativas");
    knex.whereRaw('campanhas_contato_numeros.retentativas < campanhas.tentativas');

    knex.innerJoin('campanhas', 'campanhas_contato_numeros.id_campanha', 'campanhas.id');

    pag.select = [
        'campanhas_contato_numeros.*',
        db.knex.raw("(SELECT jsonb_agg(jsonb_build_object(nome, valor)) AS campos FROM campanhas_contato_campos WHERE campanhas_contato_campos.id_contato = campanhas_contato_numeros.id_contato) AS campos"),
        'campanhas.nome as campanha'
    ];

    pag.order = [
        { "campo": "campanhas_contato_numeros.fone_sequencia" },
        { "campo": "campanhas_contato_numeros.tentativas" },
        { "campo": "campanhas_contato_numeros.retentativas" },
        { "campo": "campanhas_contato_numeros.id_contato" }
    ];

    pag.knex = knex;

    pag.getRet((ret, err) => {
        if (err) {
            console.log('Opa, tivemos um erro. Erro = ' + err);
            reply(Boom.badData(err));
        }
        else {
            reply(ret);
        }
    });
};

/*
    Importação de campanhas baseadas num arquivo CSV que contem agente (Fullname) e uma Data.
    Além destas informações o arquivo deverá conter os dados que satisfaçam o layout de uma campanha MODELO (Sample) a ser utilizada sampleCampainID.
    A função da campanha modelo é suprir as várias informações necessárias para a criação de uma campanha.
    A Fila da Campanha será a primeira fila na qual o Agente estiver vinculado.
    As campanhas serão criadas pelo agrupamento entre Agente + Data. Sendo os contatos que satisfazem a este critério importados para sua respectiva campanha.
    Caso a campanha já exista os dados dos contatos serão importados com unicidade na campanha existente (merge).
*/

async function cadastroClientesCampanha(camp) {
    for (var i = 0; i < camp.registros.length; i++)
        try {
            let cliente = await Cliente.BuscaClientePorNumero(camp.registros[i].TELEFONEPRINCIPAL);
            let grupo = await ClienteGrupo.query.require(true).first().fields(['id']).where({ nome: camp.registros[i].Vendedor }).fetch()
            console.log(camp.registros[i].Vendedor)
            if (cliente && cliente.length > 0) {
                let clienteAtualizado = await new Cliente({
                    id: cliente[0].id,
                    numero: '55' + camp.registros[i].TELEFONEPRINCIPAL,
                    nome: `${camp.registros[i]["CODIGO CLIENTE"]} - ${camp.registros[i]["NOME DO CLIENTE"]}`,
                    isBusiness: false,
                    idGrupo: grupo.id
                }).save()
                cadastroNumerosAdicionais(camp.registros[i], clienteAtualizado.id)
                cadastroDadosAdicionais(camp.registros[i], clienteAtualizado.id)
            } else {
                let clienteNovo = await new Cliente({
                    numero: '55' + camp.registros[i].TELEFONEPRINCIPAL,
                    nome: `${camp.registros[i]["CODIGO CLIENTE"]} - ${camp.registros[i]["NOME DO CLIENTE"]}`,
                    isBusiness: false,
                    idGrupo: grupo.id
                }).save()
                cadastroNumerosAdicionais(camp.registros[i], clienteNovo.id)
                cadastroDadosAdicionais(camp.registros[i], clienteNovo.id)
            }
        }
        catch (err) { console.log(err) }
};

async function cadastroDadosAdicionais(camp, idCliente) {
    try {
        if (camp.FILA != '') {
            let fila = await ClienteCamposAdicionais.query.require(false).where({ clienteID: idCliente, nome: 'FILA' }).fetch()
            if (fila.length > 0) {
                let filaAtualizada = new ClienteCamposAdicionais({
                    id: fila[0].id,
                    clienteID: idCliente,
                    nome: 'FILA',
                    valor: camp.FILA
                }).save()
            } else {
                let filaAtualizada = new ClienteCamposAdicionais({
                    clienteID: idCliente,
                    nome: 'FILA',
                    valor: camp.FILA
                }).save()
            }
        }
        if (camp.FILAVOZ != '') {
            let filaVoz = await ClienteCamposAdicionais.query.require(false).where({ clienteID: idCliente, nome: 'FILAVOZ' }).fetch()
            if (filaVoz.length > 0) {
                let filaVozAtualizada = new ClienteCamposAdicionais({
                    id: filaVoz[0].id,
                    clienteID: idCliente,
                    nome: 'FILAVOZ',
                    valor: camp.FILAVOZ
                }).save()
            } else {
                let filaVozAtualizada = new ClienteCamposAdicionais({
                    clienteID: idCliente,
                    nome: 'FILAVOZ',
                    valor: camp.FILAVOZ
                }).save()
            }
        }
    } catch (err) { console.log(err) }
}

async function cadastroNumerosAdicionais(camp, idCliente) {
    try {
        if (camp.TELEFONE1 != '') {
            let tel1 = await ClienteNumerosAdicionais.query.require(false).where({ clienteID: idCliente, nome: 'TELEFONE1' }).fetch()
            if (tel1.length > 0) {
                let telefone1Atualizado = new ClienteNumerosAdicionais({
                    id: tel1[0].id,
                    clienteID: idCliente,
                    nome: 'TELEFONE1',
                    numero: '55' + camp.TELEFONE1
                }).save()
            } else {
                let telefone1Novo = new ClienteNumerosAdicionais({
                    clienteID: idCliente,
                    nome: 'TELEFONE1',
                    numero: '55' + camp.TELEFONE1
                }).save()
            }
        }

        if (camp.TELEFONE2 != '') {
            let tel2 = await ClienteNumerosAdicionais.query.require(false).where({ clienteID: idCliente, nome: 'TELEFONE2' }).fetch()
            if (tel2.length > 0) {
                let telefone2Atualizado = new ClienteNumerosAdicionais({
                    id: tel2[0].id,
                    clienteID: idCliente,
                    nome: 'TELEFONE2',
                    numero: '55' + camp.TELEFONE2
                }).save()
            } else {
                let telefone2Novo = new ClienteNumerosAdicionais({
                    clienteID: idCliente,
                    nome: 'TELEFONE2',
                    numero: '55' + camp.TELEFONE2
                }).save()
            }
        }

        if (camp.TELEFONE3 != '') {
            let tel3 = await ClienteNumerosAdicionais.query.require(false).where({ clienteID: idCliente, nome: 'TELEFONE3' }).fetch()
            if (tel3.length > 0) {
                let telefone3Atualizado = new ClienteNumerosAdicionais({
                    id: tel3[0].id,
                    clienteID: idCliente,
                    nome: 'TELEFONE3',
                    numero: '55' + camp.TELEFONE3
                }).save()
            } else {
                let telefone3Novo = new ClienteNumerosAdicionais({
                    clienteID: idCliente,
                    nome: 'TELEFONE3',
                    numero: '55' + camp.TELEFONE3
                }).save()
            }
        }

        if (camp.TELEFONE4 != '') {
            let tel4 = await ClienteNumerosAdicionais.query.require(false).where({ clienteID: idCliente, nome: 'TELEFONE4' }).fetch()
            if (tel4.length > 0) {
                let telefone4Atualizado = new ClienteNumerosAdicionais({
                    id: tel4[0].id,
                    clienteID: idCliente,
                    nome: 'TELEFONE4',
                    numero: '55' + camp.TELEFONE4
                }).save()
            } else {
                let telefone4Novo = new ClienteNumerosAdicionais({
                    clienteID: idCliente,
                    nome: 'TELEFONE4',
                    numero: '55' + camp.TELEFONE4
                }).save()
            }
        }
    } catch (err) { console.log(err) }
};

exports.importJBS = async (request, reply) => {
    if (request.payload && request.payload.file) {
        let buffer = "";

        request.payload.file.on("data", chunk => {
            console.log(`[Rest/v3/campanhas/importJBS] Recebendo pedaços do arquivo!`);
            buffer += chunk;
        });

        request.payload.file.on("end", async () => {
            try {
                console.log(`[Rest/v3/campanhas/importJBS] Arquivo recebido! Processando...`);

                buffer = buffer.toString("latin1", 0, buffer.length);
                let dados = [];

                let parse = new Promise((resolve, reject) => {
                    csv.fromString(buffer, { headers: true, delimiter: ';' })
                        .on('data', row => dados.push(row))
                        .on('end', () => resolve(true));
                });
                await parse;

                console.log(`[Rest/v3/campanhas/importJBS] CSV processado! Inserindo dados...`);

                const campanhas = agruparCampanhas(dados);
                const sample = await getCampanha(request.query.sampleCampainID, true);
                const errorLog = [];

                let promises = [];
                campanhas.forEach(async c => {
                    //Localiza o Agente
                    let p = getAgente(c.vendedor).then(

                        async agente => {
                            c.agente = agente;
                            c.datas.forEach(async d => {
                                //Prepara os demais dados da campanha (lidos de uma campanha exemplo)
                                let campanha = Object.assign({}, sample);

                                //Ajustes do objeto
                                delete campanha.id;
                                delete campanha.data_create;
                                campanha.nome = `${c.agente.nome}-${d.data.replace(/\//g, '')}`;
                                campanha.fila = c.agente.filas[0];
                                campanha.data_ini = `${d.data} 08:00:00`;
                                campanha.data_fim = `${d.data} 18:00:00`;

                                //Cria ou carrega a campanha retornando o ID e se a campanha é nova.
                                return await createOrLoadCampanha(campanha)
                                    .then(async (camp) => {
                                        //Insere os Registros com unicidade se a campanha não for nova
                                        await insertContatos(camp, d.registros, false, !campanha.nova)
                                            .then(retorno => { return; })
                                            .catch(error => {
                                                errorLog.push(`Falha ao inserir contatos na campanha para ${c.vendedor}: ${campanha.nome}. ${error}`);
                                            });
                                    })
                                    .catch((e) => {
                                        errorLog.push(`Falha ao criar a campanha para ${c.vendedor}: ${campanha.nome}. ${e}`);
                                    });
                            });
                            for (var i = 0; i < c.datas.length; i++)
                                await cadastroClientesCampanha(c.datas[i]);

                        },

                        error => {
                            errorLog.push(`Erro ao localizar o agente para o vendedor: ${c.vendedor}. ${error}`);
                        }

                    );
                    promises.push(p);
                });

                await Promise.all(promises);

                console.log(`[Rest/v3/campanhas/importJBS] Fim!`);

                if (errorLog.length > 0)
                    reply(Boom.badData(errorLog.join('\l\n')));
                else
                    reply('Importação concluída com sucesso!');

            } catch (error) {
                log.error(`[Rest/v3/campanhas/importJBS] Erro ao processar arquivo -> ` + error);
                reply(Boom.badData(`Erro ao processar arquivo!\n` + error));
            }
        });

        request.payload.file.on("error", error => {
            log.error(`[Rest/v3/campanhas/importJBS] Erro ao receber arquivo -> ` + error);
            reply(Boom.badData(`Erro ao receber arquivo!\n` + error));
        });
    }
};

//Agrupa os dados do arquivo em campanhas a serem inseridas
function agruparCampanhas(data) {
    const rt = _.chain(data)
        .sortBy('Vendedor')
        .groupBy('Vendedor')
        .map(function (values, key) {
            let c = {
                vendedor: key,
            };

            c.datas = _.chain(values)
                .sortBy('Data')
                .groupBy('Data')
                .map(function (v) {
                    return {
                        data: v[0].Data,
                        registros: v
                    };
                }).value();

            return c;
        }).value();

    return rt;
}

// Obter a campanha
function getAgente(nome) {
    const agente = {
        filas: []
    };

    return new Promise((resolve, reject) => {
        db.knex('public.agentes as a')
            .innerJoin('public.agentes_filas as f', 'a.id', 'f.id_agente')
            .whereRaw('LOWER(a.fullname) LIKE ?', '%' + nome.toLowerCase() + '%')
            .select('a.nome', 'f.fila')
            .then(function (rows) {
                if (rows && rows.length > 0) {
                    agente.nome = rows[0].nome,
                        agente.filas.push(rows[0].fila);
                    resolve(agente);
                }
                else
                    reject(`Nenhum agente encontrado com o Nome Completo: ${nome}`);
            })
            .catch(function (error) {
                console.error(error);
                reject(error);
            });
    });
};

function createOrLoadCampanha(campanha) {
    return new Promise((resolve, reject) => {
        const knex = db.knex('public.campanhas');
        knex.where("nome", campanha.nome)
            .where("fila", campanha.fila)
            .select("id")
            .then(function (rows) {
                if (rows && Array.isArray(rows) && rows.length > 0) {
                    campanha.id = rows[0].id;
                    campanha.nova = false;

                    try {
                        discador.addCampanha(campanha);
                    } catch (ex) {
                        console.log('Erro interno ao adicionar a campanha no discador, erro: ' + ex);
                    }

                    resolve(campanha);
                }
                else {
                    //Ajuste do modelo para persistência
                    let xc = Object.assign({}, campanha);
                    delete xc.fones;
                    delete xc.campos;

                    db.knex('public.campanhas');
                    knex.insert(xc);
                    knex.returning('id')
                        .then(function (id) {
                            campanha.id = id[0];
                            campanha.nova = true;

                            try {
                                discador.addCampanha(campanha);
                            } catch (ex) {
                                console.log('Erro interno ao adicionar a campanha no discador, erro: ' + ex);
                            }

                            resolve(campanha);
                        })
                        .catch(function (error) {
                            console.error(error);
                            reject(error);
                        });
                }
            })
            .catch(function (error) {
                console.error(error);
                reject(error);
            });
    });
};