'use strict'

const Joi = require('joi');
const handlers = require('./handlers');
const CRUD = require('../../../util/CRUD');
const util = require('../../../util/restbase');

//Conjunto de dados padrão + específico
const ListaReturnSchema = util.CreateReturnSchema(null);


const CreateReturnSchema = {
  error: Joi.boolean().example(false).description('Indica se o processo gerou algum erro.'),
  id: Joi.number().integer().example('1').description('ID do registro criado'),
  message: Joi.string().example('Registro criado com aucesso.')
};

module.exports = [
  {
    method: 'GET',
    path: '/api/v3/campanha',
    handler: handlers.getall,
    config: {
      description: 'Pesquisa as campanhas. Informe os filtros para obter uma consulta personalizada ou chame o método sem dados para obter a primeira página de resultados.',
      tags: ['api', 'Campanha'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
              campo: Joi.string().description('Nome da coluna'),
              ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          status: Joi.string().valid('A', 'P', 'F').description('Status da campanha, A=Ativa,P=Pausa e F=Finalizada'),
          data_status_ini: Joi.date().iso().example('2018-01-01 08:00:00').description('Data status inicial'),
          data_status_fim: Joi.date().iso().example('2018-01-01 23:59:59').description('Data status final'),
          data_start_ini: Joi.date().iso().example('2018-01-01 08:00:00').description('Data start inicial'),
          data_start_fim: Joi.date().iso().example('2018-01-01 23:59:59').description('Data start final'),
          data_create_ini: Joi.date().iso().example('2018-01-01 08:00:00').description('Data create inicial'),
          data_create_fim: Joi.date().iso().example('2018-01-01 23:59:59').description('Data create final'),
          data_agendamento_ini: Joi.date().iso().example('2018-01-01 08:00:00').description('Data agendamento inicial'),
          data_agendamento_fim: Joi.date().iso().example('2018-01-01 23:59:59').description('Data agendamento final'),
          nome: Joi.string().description('Filtra por nome'),
          return: Joi.string().valid('array').description('Muda o retorno para um array puro de registros'),
          filas: Joi.array().items(Joi.string()).description('Filtro de Filas')
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/campanha/{ID}',
    handler: (request, reply) => {
      const crud = new CRUD('public.v_campanhas');
      crud.id = 'id';
      crud.getOne(request, reply);
    },
    config: {
      description: 'Obtém os dados da campanha',
      tags: ['api', 'Campanha'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/campanha/{CampanhaID}/classificacao',
    handler: handlers.getclassificacoes,
    config: {
      description: 'Pesquisa as classificações vinculadas à campanha. Informe os filtros para obter uma consulta personalizada ou chame o método sem dados para obter a primeira página de resultados.',
      tags: ['api', 'Campanha'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          CampanhaID: Joi.number().integer().required()
        },
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página')
        }
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v3/campanha/{CampanhaID}/classificacao',
    handler: handlers.ReplaceClassificacoes,
    config: {
      description: 'Substitui as classificações vinculadas à campanha (Apaga todas classificações e substitui pelas enviadas)',
      tags: ['api', 'Campanha'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          CampanhaID: Joi.number().integer().required()
        },
        payload: Joi.array().items(
          Joi.object().keys({
            id: Joi.number().integer().required().example(1).description('ID da classificação.'),
            fila: Joi.string().example('Fila01').description('Nome da fila vinculada a classificacao, não é obrigatório'),
            acao: Joi.string().valid('AN', 'RN', 'RC').description('Ação da classificação, AN=Redisca número,RN=Remove número e RC=Remove contato'),
            nome: Joi.string().example('classificacao01').description('Nome da classificação')
          })
        ).description('Lista de classificações da campanha')
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/campanha/{CampanhaID}/contato',
    handler: handlers.getContatos,
    config: {
      description: 'Pesquisa os contatos vinculadas à campanha. Informe os filtros para obter uma consulta personalizada ou chame o método sem dados para obter a primeira página de resultados.',
      tags: ['api', 'Campanha'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          CampanhaID: Joi.number().integer().required()
        },
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
              campo: Joi.string().description('Nome da coluna'),
              ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          campos: Joi.array().items().example([{ "CPF": "123456" }, { "Cidade": "POA" }]).description('Filtra pelos campos do contato, você pode filtrar por um ou mais campos no formato de um array de objetos json. Exemplo: [{"CPF": "123456"},{"Cidade":"POA"}]')
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v3/campanha/{CampanhaID}/contato_upload',
    handler: handlers.uploadContatos,
    config: {
      description: 'Envia um arquivo de contatos para ser importado na campanha.',
      tags: ['api', 'Campanha'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      payload: {
        output: 'data',
        allow: 'multipart/form-data',
        maxBytes: 104857600, // 100mb
        parse: true,
      },
      validate: {
        params: {
          CampanhaID: Joi.number().integer().required()
        },
        payload: {
          file: Joi.object().unknown().description('Arquivo')
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v3/campanha/{CampanhaID}/contato_import',
    handler: handlers.uploadContatosJSON,
    config: {
      description: 'Importa uma lista de contatos no formato JSON para uma campanha, os nomes dos campos devem estar de acordo com o layout associado à campanha.',
      tags: ['api', 'Campanha'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          CampanhaID: Joi.number().integer().required()
        },
        query: {
          unique: Joi.bool().optional()
        },
        payload: Joi.array().items(Joi.object().unknown()).example('[{"Nome":"João da sila","CPF":"123456","Fixo":"5506","Celular":"999999"}]').description('Array de contatos')
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/campanha/{CampanhaID}/registro',
    handler: handlers.getRegistros,
    config: {
      description: 'Pesquisa os registros da campanha (log). Informe os filtros para obter uma consulta personalizada ou chame o método sem dados para obter a primeira página de resultados.',
      tags: ['api', 'Campanha'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          CampanhaID: Joi.number().integer().required()
        },
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          data_ini: Joi.date().iso().example('2018-01-01 08:00:00').description('Data inicial'),
          data_final: Joi.date().iso().example('2018-01-01 23:59:59').description('Data final'),
          numero: Joi.string().description('Filtra pelo número'),
          numero_operador: Joi.number().integer().valid(1, 2, 3, 4, 5).description('Operador para filtro do número, Quando não informado é aplicado like %%, 1=Igual,2=Iniciando,3=Finalizando,4=Não contendo e 5=Diferente')
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/campanha/{CampanhaID}/fila_discagem',
    handler: handlers.getFilaDiscagem,
    config: {
      description: 'Pesquisa a lista dos números que serão discados na campanha em sequencia. Informe os filtros para obter uma consulta personalizada ou chame o método sem dados para obter a primeira página de resultados.',
      tags: ['api', 'Campanha'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          CampanhaID: Joi.number().integer().required()
        },
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página')
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/campanha/registro',
    handler: handlers.getCampanhasRegistros,
    config: {
      description: 'Pesquisa os registros de TODAS as campanhas (log). Informe os filtros para obter uma consulta personalizada ou chame o método sem dados para obter a primeira página de resultados.',
      tags: ['api', 'Campanha'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
              campo: Joi.string().description('Nome da coluna'),
              ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          data_ini: Joi.date().iso().example('2018-01-01 08:00:00').description('Data inicial'),
          data_fim: Joi.date().iso().example('2018-01-01 23:59:59').description('Data final'),
          campanhas: Joi.array().items(Joi.number().integer()).example([1, 2, 3]).description('Array de IDs de Campanhas'),
          status: Joi.array().items(Joi.number().integer()).example([1, 2, 3]).description('Array de IDs de status'),
          numero: Joi.string().description('Filtra pelo número'),
          numero_operador: Joi.number().integer().valid(1, 2, 3, 4, 5).description('Operador para filtro do número, Quando não informado é aplicado like %%, 1=Igual,2=Iniciando,3=Finalizando,4=Não contendo e 5=Diferente')
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v3/campanha',
    handler: handlers.create,
    config: {
      description: 'Cria uma nova campanha',
      tags: ['api', 'Campanha'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      response: {
        schema: CreateReturnSchema
      },
      validate: {
        payload: Joi.object({
          nome: Joi.string().required().example('Campanha 1').description('Nome da campanha'),
          fila: Joi.string().example('Fila01').description('Nome da fila associada a campanha'),
          macro: Joi.string().example('fila').description('Nome da macro que o discador deve executar ao entregar a ligação'),
          requer_agente_online: Joi.boolean().example(true).description('Define se o discador deve aguardar pela disponibilidade dos agente para iniciar uma discagem.'),
          modo: Joi.string().valid('A', 'M').description('Modo da campanha, A=Power (Automática),M=Preview (Manual)'),
          status: Joi.string().valid('A', 'P', 'F').description('Status da campanha, A=Ativa,P=Pausa e F=Finalizada'),
          tentativas: Joi.number().integer().required().example('3').description('Número de tentativas'),
          max_por_agente: Joi.number().integer().required().example(1).description('ligações simultâneas por agente'),
          max_chamar: Joi.number().integer().required().example(30).description('Tempo máximo em segundos que a chamada vai ficar tocando no cliente.'),
          tempo_minimo_completar: Joi.number().integer().example(0).description('Tempo em segundos para considerar uma ligação válida, se 0 não aplica.'),
          tempo_autopausa_agente: Joi.number().integer().example(0).description('Tempo em segundos que o agente ficará em pausa ao termino de uma ligação, se 0 não aplica.'),
          autofim: Joi.boolean().required().example(true).description('Define se o discador pode finaliar a campanha se não houver mais números para discar.'),
          max_simultaneas_campanha: Joi.number().integer().required().example(20).description('Número máximo de ligações simultâneas na campanha'),
          desconecta_multiplas: Joi.boolean().example(true).description('Define se o discador deve desconecar ligações simultâneas caso uma delas complete.'),
          rota: Joi.string().example('discador').description('Rota que o discador irá usar para efetuar as ligações'),
          data_ini: Joi.date().iso().example('2018-01-01 08:00:00').description('Data que a campanha deve iniciar'),
          data_fim: Joi.date().iso().example('2018-01-01 18:00:00').description('Data que a campanha deve terminar'),
          data_status: Joi.date().iso().example('2018-01-01 15:00:00').description('Data do ultimo status'),
          descricao: Joi.string().example('Campanha teste').description('Descrição para campanha'),
          id_layout: Joi.number().integer().required().example(1).description('ID do layout.'),
          notificar_final: Joi.boolean().example(true).description('Define se o discador deve notificar o final da campanha.'),
          notificar_email: Joi.string().example('meuemail@meudominio.com,meuoutroemail@meudominio.com...').description('Define uma lista de email para receber a notificação.'),
          exec_contexto: Joi.string().allow(null).allow('').example('Ramais').description('Caso definido este parametro o discador entrega a ligação para o contexto aqui informado e parametro macro é usado como rota.'),
        })
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v3/campanha/{CampanhaID}',
    handler: handlers.update,
    config: {
      description: 'Atualiza uma campanha',
      tags: ['api', 'Campanha'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          CampanhaID: Joi.number().integer().required()
        },
        payload: Joi.object({
          nome: Joi.string().example('Campanha 1').description('Nome da campanha'),
          fila: Joi.string().example('Fila01').description('Nome da fila associada a campanha'),
          macro: Joi.string().example('fila').description('Nome da macro que o discador deve executar ao entregar a ligação'),
          requer_agente_online: Joi.boolean().example(true).description('Define se o discador deve aguardar pela disponibilidade dos agente para iniciar uma discagem.'),
          modo: Joi.string().valid('A', 'M').description('Modo da campanha, A=Power (Automática),M=Preview (Manual)'),
          status: Joi.string().valid('A', 'P', 'F').description('Status da campanha, A=Ativa,P=Pausa e F=Finalizada'),
          tentativas: Joi.number().integer().example('3').description('Número de tentativas'),
          max_por_agente: Joi.number().integer().example(1).description('ligações simultâneas por agente'),
          max_chamar: Joi.number().integer().example(30).description('Tempo máximo em segundos que a chamada vai ficar tocando no cliente.'),
          tempo_minimo_completar: Joi.number().integer().example(0).description('Tempo em segundos para considerar uma ligação válida, se 0 não aplica.'),
          tempo_autopausa_agente: Joi.number().integer().example(0).description('Tempo em segundos que o agente ficará em pausa ao termino de uma ligação, se 0 não aplica.'),
          autofim: Joi.boolean().example(true).description('Define se o discador pode finaliar a campanha se não houver mais números para discar.'),
          max_simultaneas_campanha: Joi.number().integer().example(20).description('Número máximo de ligações simultâneas na campanha'),
          desconecta_multiplas: Joi.boolean().example(true).description('Define se o discador deve desconecar ligações simultâneas caso uma delas complete.'),
          rota: Joi.string().example('discador').description('Rota que o discador irá usar para efetuar as ligações'),
          data_ini: Joi.date().iso().example('2018-01-01 08:00:00').description('Data que a campanha deve iniciar'),
          data_fim: Joi.date().iso().example('2018-01-01 18:00:00').description('Data que a campanha deve terminar'),
          data_status: Joi.date().iso().example('2018-01-01 15:00:00').description('Data do ultimo status'),
          descricao: Joi.string().example('Campanha teste').description('Descrição para campanha'),
          id_layout: Joi.number().integer().example(1).description('ID do layout.'),
          notificar_final: Joi.boolean().example(true).description('Define se o discador deve notificar o final da campanha.'),
          notificar_email: Joi.string().example('meuemail@meudominio.com,meuoutroemail@meudominio.com...').description('Define uma lista de email para receber a notificação.'),
          exec_contexto: Joi.string().allow(null).allow('').example('Ramais').description('Caso definido este parametro o discador entrega a ligação para o contexto aqui informado e parametro macro é usado como rota.'),
        })
      }
    }
  },
  {
    method: 'DELETE',
    path: '/api/v3/campanha/{CampanhaID}',
    handler: handlers.delete,
    config: {
      description: 'Apaga uma campanha',
      tags: ['api', 'Campanha'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          CampanhaID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'DELETE',
    path: '/api/v3/campanha/{CampanhaID}/zerar',
    handler: handlers.zerar,
    config: {
      description: 'Zera os contadores de discagem de uma campanha',
      tags: ['api', 'Campanha'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          CampanhaID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'DELETE',
    path: '/api/v3/campanha/{CampanhaID}/contato/{ContatoID}',
    handler: handlers.remove_contato,
    config: {
      description: 'Remove um contato da campanha',
      tags: ['api', 'Campanha'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          CampanhaID: Joi.number().integer().required().description('ID da campanha'),
          ContatoID: Joi.number().integer().required().description('ID do contato')
        }
      }
    }
  },
  {
    method: 'DELETE',
    path: '/api/v3/campanha/{CampanhaID}/contato/{CampoNome}/{CampoValor}',
    handler: handlers.remove_contato_por_campo,
    config: {
      description: 'Remove um contato por um campo e valor',
      tags: ['api', 'Campanha'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          CampanhaID: Joi.number().integer().required().description('ID da campanha'),
          CampoNome: Joi.string().required().description('Nome do campo'),
          CampoValor: Joi.string().required().description('Valor do campo')
        }
      }
    }
  },
  {
    method: 'DELETE',
    path: '/api/v3/campanha/{CampanhaID}/contato/numero/{numero}',
    handler: handlers.remove_contato_por_numero,
    config: {
      description: 'Remove um contato por um numero',
      tags: ['api', 'Campanha'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          CampanhaID: Joi.number().integer().required().description('ID da campanha'),
          numero: Joi.string().required().description('Nome do campo'),
        }
      }
    }
  },
  {
    method: 'DELETE',
    path: '/api/v3/campanha/{CampanhaID}/contato',
    handler: handlers.remove_todos_contatos,
    config: {
      description: 'Remove TODOS os contatos da campanha',
      tags: ['api', 'Campanha'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          CampanhaID: Joi.number().integer().required().description('ID da campanha')
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v3/campanha/importJBS',
    handler: handlers.importJBS,
    config: {
      description: 'Importa um arquivo CSV de formato específico para gerar múltiplas campanhas agrupadas por agente e data.',
      tags: ['api', 'Campanha'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      plugins: {
        'hapi-swagger': {
          payloadType: 'form'
        }
      },
      payload: {
        output: 'stream',
        allow: 'multipart/form-data',
        maxBytes: 104857600, // 100mb
        parse: true
      },
      validate: {
        query: {
          sampleCampainID: Joi.number().integer().required().description('ID da Campanha de Exemplo para as campanhas a serem criadas que terão os seus dados clonados da campanha exemplo.')
        },
        payload: {
          file: Joi.any().meta({ swaggerType: 'file' }).required()
        }
      }
    }
  },
];
