'use strict'

const Joi = require('joi');
const CRUD = require('../../../util/CRUD');
const util = require('../../../util/restbase');
const handlers = require('./handlers');

//Conjunto de dados padrão + específico
const ListaReturnSchema = util.CreateReturnSchema({dados: Joi.array().items().example([{id: 1, nome: "Registro1"},{id: 2, nome: "Registro2"}]).description('Lista dos registros')});
const operationData = {
      descricao: Joi.string().allow(null).default(null).description("Descrição"),
      canal: Joi.string().required().description("Canal"),
      id_operadora: Joi.number().integer().required().description("Operadora"),
      id_grupo: Joi.number().integer().required().description("Grupo de canais GSM"),
};

module.exports = [
    {
      method: 'GET',
      path: '/api/v3/canal_gsm',
      handler: handlers.getall,
      config: {
        description: 'Pesqusia os canais GSM cadastrados. Informe os filtros para obter uma consulta personalizada ou chame o método sem dados para obter a primeira página de resultados.',
        tags: ['api','Canais GSM'],
        auth: {
            strategy: 'bearer',
            scope: 'user' // or [ 'user', 'admin' ]
        },
        response: {
          schema: ListaReturnSchema
        },
        validate: {
          query: {
            max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
            page: Joi.number().integer().default(1).description('Página'),
            order: Joi.array().items(
              Joi.object().keys({
                  campo: Joi.string().description('Nome da coluna'),
                  ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
              })
            ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
            descricao: Joi.string().description('Filtra por descrição'),
            canal: Joi.string().description('Filtra por canal'),
          }
        }
      }
    },
    {
      method: 'GET',
      path: '/api/v3/canal_gsm/{ID}',
      handler: (request, reply) => {
        const crud = new CRUD('public.canais_gsm');
        crud.id = 'id';
        crud.getOne(request, reply);
      },
      config: {
        description: 'Obtém os dados do canal GSM',
        tags: ['api','Canais GSM'],
        auth: {
            strategy: 'bearer',
            scope: 'user' // or [ 'user', 'admin' ]
        },
        validate: {
          params: {
            ID: Joi.number().integer().required()
          }
        }
      }
    },
    {
        method: 'POST',
        path: '/api/v3/canal_gsm',
        handler: (request, reply) => {
            const crud = new CRUD('public.canais_gsm');
            crud.id = 'id';
            crud.create(request, reply);
        },
        config: {
          description: 'Cria um novo canal GSM',
          tags: ['api','Canais GSM'],
          auth: {
              strategy: 'bearer',
              scope: 'user' // or [ 'user', 'admin' ]
          },
          validate: {
            payload: operationData
          }
        }
    },
    {
        method: 'PUT',
        path: '/api/v3/canal_gsm/{ID}',
        handler: (request, reply) => {
            const crud = new CRUD('public.canais_gsm');
            crud.id = 'id';
            crud.update(request, reply);
          },
        config: {
          description: 'Altera os dados do canal GSM',
          tags: ['api','Canais GSM'],
          auth: {
              strategy: 'bearer',
              scope: 'user' // or [ 'user', 'admin' ]
          },
          validate: {
            params: {
              ID: Joi.number().integer().required()
            },
            payload: operationData
          }
        }
    },
    {
        method: 'DELETE',
        path: '/api/v3/canal_gsm/{ID}',
        handler: (request, reply) => {
            const crud = new CRUD('public.canais_gsm');
            crud.id = 'id';
            crud.delete(request, reply);
          },
        config: {
          description: 'Remove o canal GSM, indicado',
          tags: ['api','Canais GSM'],
          auth: {
              strategy: 'bearer',
              scope: 'user' // or [ 'user', 'admin' ]
          },
          validate: {
            params: {
              ID: Joi.number().integer().required()
            }
          }
        }
    },
    {
      method: 'DELETE',
      path: '/api/v3/canal_gsm/zeraContadores/{ID}',
      handler: handlers.zeraContadores,
      config: {
        description: 'Limpa os contadores do canal GSM, indicado',
        tags: ['api','Canais GSM'],
        auth: {
            strategy: 'bearer',
            scope: 'user' // or [ 'user', 'admin' ]
        },
        validate: {
          params: {
            ID: Joi.number().integer().required()
          }
        }
      }
  },
];