'use strict'

const Joi = require('joi');
const handlers = require('./handlers');
const CRUD = require('../../../util/CRUD');
const util = require('../../../util/restbase');

//Conjunto de dados padrão + específico
const ListaReturnSchema = util.CreateReturnSchema(null);
const optSentido = ['Entrada', 'Saida', 'URA', 'URA_MENSAGEM'];
const operationData = {
    context: Joi.string().required(),
    descricao: Joi.string().allow(null).description("Descrição, para facilitar a referência ao contexto"),
    editavel: Joi.boolean().allow(null).default(true),
    sentido: Joi.string().valid(optSentido).required(),
};

module.exports = [
    {
        method: 'GET',
        path: '/api/v3/contexto',
        handler: handlers.getall,
        config: {
            description: 'Pesqusia os grupos de rotas. Informe os filtros para obter uma consulta personalizada ou chame o método sem dados para obter a primeira página de resultados.',
            tags: ['api', 'Grupo de Rotas'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            response: {
                schema: ListaReturnSchema
            },
            validate: {
                query: {
                    max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
                    page: Joi.number().integer().default(1).description('Página'),
                    order: Joi.array().items(
                        Joi.object().keys({
                            campo: Joi.string().description('Nome da coluna'),
                            ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
                        })
                    ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
                    context: Joi.string().description('Filtra por identificação do contexto (contexto)'),
                    descricao: Joi.string().description('Filtra por descrição'),
                    editavel: Joi.bool().description('Filtra pela opção de ser editavel'),
                    sentido: Joi.array().items(Joi.string().valid(optSentido).description('Filtra pelo sentido da rota')),
                }
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v3/contexto/{ID}',
        handler: (request, reply) => {
            const crud = new CRUD('public.contexto');
            crud.id = 'id';
            crud.getOne(request, reply);
        },
        config: {
            description: 'Obtém os dados do grupo de rotas',
            tags: ['api', 'Grupo de Rotas'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    ID: Joi.number().integer().required()
                }
            }
        }
    },
    {
        method: 'POST',
        path: '/api/v3/contexto',
        handler: handlers.create,
        config: {
            description: 'Cria um novo grupo de rotas',
            tags: ['api', 'Grupo de Rotas'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                payload: operationData
            }
        }
    },
    {
        method: 'PUT',
        path: '/api/v3/contexto/{ID}',
        handler: handlers.update,
        config: {
            description: 'Altera os dados do grupo de rotas',
            tags: ['api', 'Grupo de Rotas'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    ID: Joi.number().integer().required()
                },
                payload: operationData
            }
        }
    },
    {
        method: 'DELETE',
        path: '/api/v3/contexto/{ID}',
        handler: handlers.delete,
        config: {
            description: 'Remove o grupo de rotas indicado',
            tags: ['api', 'Grupo de Rotas'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    ID: Joi.number().integer().required()
                }
            }
        }
    },
    {
        method: 'DELETE',
        path: '/api/v3/contexto/byname/{name}',
        handler: handlers.delete,
        config: {
            description: 'Remove o grupo de rotas indicado',
            tags: ['api', 'Grupo de Rotas'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    name: Joi.string().required()
                }
            }
        }
    }
];