'use strict'

const Joi = require('joi');
const handlers = require('./handlers');
const CRUD = require('../../../util/CRUD');
const util = require('../../../util/restbase');

//Conjunto de dados padrão + específico
const ListaReturnSchema = util.CreateReturnSchema({ dados: Joi.array().items().example([{ id: 1, nome: "Registro1" }, { id: 2, nome: "Registro2" }]).description('Lista dos registros') });

module.exports = [
  {
    method: 'GET',
    path: '/api/v3/grupo',
    handler: handlers.getall,
    config: {
      description: 'Pesqusia os grupos de ramais. Informe os filtros para obter uma consulta personalizada ou chame o método sem dados para obter a primeira página de resultados.',
      tags: ['api', 'Grupo de Ramais'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      response: {
        schema: ListaReturnSchema
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
              campo: Joi.string().description('Nome da coluna'),
              ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          nome: Joi.string().description('Filtra por nome'),
          context: Joi.string().description('Filtra por contexto (Grupo de Rotas)'),
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/grupo/{ID}',
    handler: (request, reply) => {
      const crud = new CRUD('public.grupos');
      crud.id = 'id';
      crud.getOne(request, reply);
    },
    config: {
      description: 'Obtém os dados do grupo de gamais',
      tags: ['api', 'Grupo de Ramais'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v3/grupo',
    handler: (request, reply) => {
      const crud = new CRUD('public.grupos');
      crud.id = 'id';
      crud.create(request, reply);
    },
    config: {
      description: 'Cria um novo grupo de gamais',
      tags: ['api', 'Grupo de Ramais'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        payload: {
          nome: Joi.string().required(),
          context: Joi.string().allow(null),
        }
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v3/grupo/{ID}',
    handler: (request, reply) => {
      const crud = new CRUD('public.grupos');
      crud.id = "id";
      crud.update(request, reply);
    },
    config: {
      description: 'Altera os dados do grupo de gamais',
      tags: ['api', 'Grupo de Ramais'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        },
        payload: {
          nome: Joi.string().required(),
          context: Joi.string().allow(null),
        }
      }
    }
  },
  {
    method: 'DELETE',
    path: '/api/v3/grupo/{ID}',
    handler: (request, reply) => {
      const crud = new CRUD('public.grupos');
      crud.id = "id";
      crud.delete(request, reply);
    },
    config: {
      description: 'Remove o grupo de gamais indicado',
      tags: ['api', 'Grupo de Ramais'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        }
      }
    }
  }
];