'use strict'

const Joi = require('joi');
const handlers = require('./handlers');
const CRUD = require('../../../util/CRUD');
const util = require('../../../util/restbase');

//Conjunto de dados padrão + específico
const ListaReturnSchema = util.CreateReturnSchema(null);

const CreateReturnSchema = {
    error: Joi.boolean().example(false).description('Indica se o processo gerou algum erro.'),
    id: Joi.number().integer().example('1').description('ID do registro criado'),
    message: Joi.string().example('Registro criado com aucesso.')
};

const operationData = Joi.object({
  hora_ini : Joi.string().allow(null).allow('').optional().description('Hora inicial no formato HH:mm:ss'),
  hora_fim : Joi.string().allow(null).allow('').optional().description('Hora final no formato HH:mm:ss'),
  nome: Joi.string().description('Nome'),
  descricao: Joi.string().allow(null).allow('').optional().description('Descrição'),
  tipo: Joi.string().valid('F','T').description('Tipo (F=Feriado, T=Turno)'),
  dias: Joi.array().allow(null).items(Joi.number().integer()).description('Dias'),
  messes: Joi.array().allow(null).items(Joi.string().valid('Jan', 'Fev', 'Mar', 'Abr', 'Mai', 'Jun', 'Jul', 'Ago', 'Set', 'Out', 'Nov', 'Dez')).description('Messes (Jan,Fev,Mar...)'),
  dias_semana: Joi.array().allow(null).items(Joi.string().valid('Dom', 'Seg', 'Ter', 'Qua', 'Qui', 'Sex', 'Sáb')).description('Dias da Semana (Seg,Ter,Qua...)')
});

module.exports = [
  {
    method: 'GET',
    path: '/api/v3/horario',
    handler: handlers.getall,
    config: {
      description: 'Pesquisa os horários cadastrados. Informe os filtros para obter uma consulta personalizada ou chame o método sem dados para obter a primeira página de resultados.',
      tags: ['api','Horário'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      response: {
        schema: ListaReturnSchema
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
                campo: Joi.string().description('Nome da coluna'),
                ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          nome: Joi.string().description('Filtra por nome'),
          tipo: Joi.string().valid('F','T').description('Filtra por tipo (F=Feriado, T=Turno)'),
          dias: Joi.array().items(Joi.number().integer()).description('Filtra por dias'),
          messes: Joi.array().items(Joi.string().valid('Jan', 'Fev', 'Mar', 'Abr', 'Mai', 'Jun', 'Jul', 'Ago', 'Set', 'Out', 'Nov', 'Dez')).description('Filtra por messes (Jan,Fev,Mar...)'),
          dias_semana: Joi.array().items(Joi.string().valid('Dom', 'Seg', 'Ter', 'Qua', 'Qui', 'Sex', 'Sáb')).description('Filtra por Dias da Semana (Seg,Ter,Qua...)'),
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/horario/{HorarioID}',
    handler: (request, reply) => {
      const crud = new CRUD('public.');
      crud.id = 'id';
      crud.getOne(request, reply);
    },
    config: {
      description: 'Obtém os dados do horario',
      tags: ['api','Horário'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            HorarioID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v3/horario',
    handler: handlers.create,
    config: {
      description: 'Cria um novo horario',
      tags: ['api','Horário'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      response: {
        schema: CreateReturnSchema
      },
      validate: {
        payload: operationData
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v3/horario/{HorarioID}',
    handler: handlers.update,
    config: {
      description: 'Atualiza um Horario',
      tags: ['api','Horário'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            HorarioID: Joi.number().integer().required()
        },
        payload: operationData
      }
    }
  },
  {
    method: 'DELETE',
    path: '/api/v3/horario/{HorarioID}',
    handler: handlers.delete,
    config: {
      description: 'Apaga um horario',
      tags: ['api','Horário'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            HorarioID: Joi.number().integer().required()
        }
      }
    }
  },
];
