'use strict'

const Joi = require('joi');
const handlers = require('./handlers');

module.exports = [
  {
    method: 'GET',
    path: '/api/v3/ligacao',
    handler: handlers.getall,
    config: {
      description: 'Pesquisa as ligações. Informe os filtros para obter uma consulta personalizada ou chame o método sem dados para obter a primeira página de resultados.',
      tags: ['api', 'Ligação'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
              campo: Joi.string().description('Nome da coluna'),
              ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          data_ini: Joi.date().iso().description('Filtro por data inicial'),
          data_fim: Joi.date().iso().description('Filtro por data Final'),
          origem: Joi.string().description('Filtro por origem (src)'),
          origemc: Joi.string().allow("").description('Condição do filtro de origem (src)'),
          destino: Joi.string().description('Filtro por destino (dst)'),
          destinoc: Joi.string().allow("").description('Condição do filtro de destino (dst)'),
          callid: Joi.string().description('Filtro por callid'),
          status: Joi.string().description('Filtro por status'),
          protocolo: Joi.string().description('Filtro por protocolo'),
          sentido: Joi.string().description('Filtro por sentido'),
          canal: Joi.string().description('Filtro por canal'),
          pin_ids: Joi.array().items(Joi.string()).description('Filtro por pin_id'),
          pin_usuario: Joi.string().description('Filtro por pin_usuario'),
          idcliente: Joi.string().description('Filtro pela Identificação do Cliente (recurso disponível mediante solcitiação).'),
          idclientec: Joi.string().allow("").description('Condição do filtro de identificação do cliente'),
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/ligacao/{CallID}',
    handler: handlers.getOne,
    config: {
      description: 'Obtém os dados de uma ligação pelo callid',
      tags: ['api', 'Ligação'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          CallID: Joi.string().required().description('Callid da ligação')
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/ligacao/status',
    handler: handlers.getStatus,
    config: {
      description: 'Pesquisa a lista de status das ligações. Informe os filtros para obter uma consulta personalizada ou chame o método sem dados para obter a primeira página de resultados.',
      tags: ['api', 'Ligação'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
              campo: Joi.string().description('Nome da coluna'),
              ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          nome: Joi.string().description('Filtra por nome'),
          return: Joi.string().valid('array').description('Muda o retorno para um array puro de registros'),
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v3/ligacao/classifica',
    handler: handlers.classifica,
    config: {
      description: 'Classifica uma ligação',
      tags: ['api', 'Ligação'],
      validate: {
        payload: Joi.object({
          uniqueid: Joi.string().required().example('huy7654efss.9087').description('Callid da ligação'),
          id_classificacao: Joi.number().integer().description('ID da classificação').example(1),
          id_subclassificacao: Joi.number().integer().description('ID da Subclassificacao').example(1),
          obs: Joi.string().description('Observação').example('Cliente desligou'),
          protocolo: Joi.string().description('Código vinculado a ligação, pode ser um número de chamado por exemplo.').example('123456'),
          luc: Joi.string().description('LUC, campos texto utilizado na Brmalls').example('ADC02514445'),
          id_origem: Joi.number().integer().description('ID da origem, campo usado na Brmals').example(1),
        })
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v3/ligacao/efetuar',
    handler: handlers.discar,
    config: {
      description: 'Efetua uma ligação.',
      tags: ['api', 'Ligação'],
      validate: {
        payload: Joi.object({
          origem: Joi.string().required().example('SIP/1000').description('Ramal de origem.'),
          destino: Joi.string().required().description('Destino').example('05130305500'),
          destino_nome: Joi.string().description('Opcional. Nome para o destino').example('João da Silva'),
          rota: Joi.string().required().description('Rota de saída. Grupo de rota informado do cadastro do Ramal.').example('Ramais'),
          max_chamar: Joi.number().integer().description('Tempo máximo chamando').default(30).example(30),
          campos: Joi.object().keys().unknown().example('{"CPF":"123456"}').description('Campos associados a ligação para fins de integração com o Cliente.')
        })
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v3/ligacao/desligar',
    handler: handlers.desligar,
    config: {
      description: 'Desliga uma ligação pelo destino',
      tags: ['api', 'Ligação'],
      validate: {
        payload: Joi.object({
          destino: Joi.string().required().description('Destino').example('05130305500@Ramais'),
          causa: Joi.number().integer().description('Causa da desconexão').default(16).example(16)
        })
      }
    }
  }
];
