'use strict'
const Joi = require('joi');
const CRUD = require('../../../util/CRUD');
const md5 = require('md5');
const db = require('../../../util/db');
const Boom = require('boom');
const handlers = require('./handlers');
const util = require('../../../util/restbase');

//Conjunto de dados padrão + específico
const ListaReturnSchema = util.CreateReturnSchema(null);

module.exports = [
  {
    method: 'GET',
    path: '/api/v3/modulo',
    handler: handlers.getall,
    config: {
      description: 'Pesqusia as permissões de acesso. Informe os filtros para obter uma consulta personalizada ou chame o método sem dados para obter a primeira página de resultados.',
      tags: ['api','Permissões'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      response: {
        schema: ListaReturnSchema
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
                campo: Joi.string().description('Nome da coluna'),
                ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          nome: Joi.string().description('Filtra por nome'),
          descricao: Joi.string().description('Filtra pela descrição'),
          menu: Joi.string().allow('Campanhas','Relatórios','Configurações','Dashboards').description('Filtra por menu'),
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/modulo/{ID}',
    handler: (request, reply) => {
      const crud = new CRUD('public.modulos');
      crud.id = 'id';
      crud.getOne(request, reply);
    },
    config: {
      description: 'Obtém os dados do módulo',
      tags: ['api','Permissões'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/modulo/modulosusuario/{ID}',
    handler: handlers.getModulosUsuario,
    config: {
      description: 'Obtém as permissões de acesso de um determinado usuário',
      tags: ['api','Permissões'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/modulo/filasusuario/{ID}',
    handler: handlers.getFilasUsuario,
    config: {
      description: 'Obtém as filas que um determinado usuário tem acesso',
      tags: ['api','Permissões'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v3/modulo/{ID}',
    handler: handlers.setAcessos,
    config: {
      description: 'Altera as permissões de acesso do usuário',
      tags: ['api','Permissões'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        },
        payload: {
          filas: Joi.array().items({
            id: Joi.number().integer().required(),
            name: Joi.string().required(),
            tem: Joi.bool().required(),
          }),
          modulos: Joi.array().items({
            id: Joi.number().integer().required(),
            parent: Joi.number().integer().required().allow(null),
            descricao: Joi.string().required(),
            extra: Joi.string().required().allow('G',null), 
            tem: Joi.bool().required(),
            extra_usuario: Joi.bool().required()
          })
        }
      }
    }
  },
];
