'use strict'

const Joi = require('joi');
const CRUD = require('../../../util/CRUD');
const util = require('../../../util/restbase');

//Conjunto de dados padrão + específico
const ListaReturnSchema = util.CreateReturnSchema({dados: Joi.array().items().example([{id: 1, nome: "Registro1"},{id: 2, nome: "Registro2"}]).description('Lista dos registros')});
const operationData = {
      nome: Joi.string().required().description("Nome"),
      tipo: Joi.string().required().default("Movel").allow("Movel").description("Tipo"),
};

module.exports = [
    {
      method: 'GET',
      path: '/api/v3/operadora',
      handler: (request, reply) => {
        const crud = new CRUD('public.operadoras');      
        crud.filtros = [];
        if(request.query.nome) crud.filtros.push({campo: 'LOWER(nome)', operador: 'LIKE', valor: '%'+decodeURIComponent(request.query.nome).toLowerCase()+'%'});
        crud.getAll(request, reply);
      },
      config: {
        description: 'Pesqusia as operadoras cadastradas. Informe os filtros para obter uma consulta personalizada ou chame o método sem dados para obter a primeira página de resultados.',
        tags: ['api','Operadora'],
        auth: {
            strategy: 'bearer',
            scope: 'user' // or [ 'user', 'admin' ]
        },
        response: {
          schema: ListaReturnSchema
        },
        validate: {
          query: {
            max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
            page: Joi.number().integer().default(1).description('Página'),
            order: Joi.array().items(
              Joi.object().keys({
                  campo: Joi.string().description('Nome da coluna'),
                  ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
              })
            ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
            nome: Joi.string().description('Filtra por nome'),
          }
        }
      }
    },
    {
      method: 'GET',
      path: '/api/v3/operadora/{ID}',
      handler: (request, reply) => {
        const crud = new CRUD('public.operadoras');
        crud.id = 'id';
        crud.getOne(request, reply);
      },
      config: {
        description: 'Obtém os dados da operadora',
        tags: ['api','Operadora'],
        auth: {
            strategy: 'bearer',
            scope: 'user' // or [ 'user', 'admin' ]
        },
        validate: {
          params: {
            ID: Joi.number().integer().required()
          }
        }
      }
    },
    {
        method: 'POST',
        path: '/api/v3/operadora',
        handler: (request, reply) => {
            const crud = new CRUD('public.operadoras');
            crud.id = 'id';
            crud.create(request, reply);
        },
        config: {
          description: 'Cria uma nova operadora',
          tags: ['api','Operadora'],
          auth: {
              strategy: 'bearer',
              scope: 'user' // or [ 'user', 'admin' ]
          },
          validate: {
            payload: operationData
          }
        }
    },
    {
        method: 'PUT',
        path: '/api/v3/operadora/{ID}',
        handler: (request, reply) => {
            const crud = new CRUD('public.operadoras');
            crud.id = 'id';
            crud.update(request, reply);
          },
        config: {
          description: 'Altera os dados da operadora',
          tags: ['api','Operadora'],
          auth: {
              strategy: 'bearer',
              scope: 'user' // or [ 'user', 'admin' ]
          },
          validate: {
            params: {
              ID: Joi.number().integer().required()
            },
            payload: operationData
          }
        }
    },
    {
        method: 'DELETE',
        path: '/api/v3/operadora/{ID}',
        handler: (request, reply) => {
            const crud = new CRUD('public.operadoras');
            crud.id = 'id';
            crud.delete(request, reply);
          },
        config: {
          description: 'Remove a operadora indicada',
          tags: ['api','Operadora'],
          auth: {
              strategy: 'bearer',
              scope: 'user' // or [ 'user', 'admin' ]
          },
          validate: {
            params: {
              ID: Joi.number().integer().required()
            }
          }
        }
    },
];