'use strict'

const Joi = require('joi');
const handlers = require('./handlers');
const CRUD = require('../../../util/CRUD');
const util = require('../../../util/restbase');

//Dados das operações de escrita
const operationData =
{
    nome: Joi.string().required().description("Nome da Clasificação"),
    comando: Joi.string().required().default("Pausa").allow("Pausa").description("Indica se a pausa poderá ser editada"),
    editavel: Joi.bool().required().description("Indica se a pausa será editável"),
    fila: Joi.string().optional().allow(null).description("Fila"),
    produtiva: Joi.bool().required().description("Indica se a pausa poderá ser editada"),
};

//Conjunto de dados padrão + específico
const ListaReturnSchema = util.CreateReturnSchema(null);

module.exports = [
  {
    method: 'GET',
    path: '/api/v3/pausa',
    handler: handlers.getall,
    config: {
      description: 'Pesquisa as pausa. Informe os filtros para obter uma consulta personalizada ou chame o método sem dados para obter a primeira página de resultados.',
      tags: ['api','Pausa'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      response: {
        schema: ListaReturnSchema
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
                campo: Joi.string().description('Nome da coluna'),
                ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          nome: Joi.string().description('Filtra por nome'),
          fila: Joi.string().description('Filtra por fila'),
          produtiva: Joi.bool().description('Filtra por fila'),
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/pausa/{ID}',
    handler: handlers.getOne,
    config: {
      description: 'Obtém os dados da pausa',
      tags: ['api','Pausa'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v3/pausa',
    handler: (request, reply) => {
        const crud = new CRUD('public.agentes_status');
        crud.id = 'id';
        crud.create(request, reply);
    },
    config: {
      description: 'Cria uma nova pausa',
      tags: ['api','Pausa'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        payload: operationData
      }
    }
},
{
    method: 'PUT',
    path: '/api/v3/pausa/{ID}',
    handler: (request, reply) => {
        const crud = new CRUD('public.agentes_status');
        crud.id = "id";
        crud.update(request, reply);
      },
    config: {
      description: 'Altera os dados da pausa',
      tags: ['api','Pausa'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        },
        payload: operationData
      }
    }
},
{
    method: 'DELETE',
    path: '/api/v3/pausa/{ID}',
    handler: (request, reply) => {
        const crud = new CRUD('public.agentes_status');
        crud.id = "id";
        crud.delete(request, reply);
      },
    config: {
      description: 'Remove a pausa indicada',
      tags: ['api','Pausa'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        }
      }
    }
  } 
];
