'use strict'

const Boom = require('boom');
const db = require('../../../util/db');
const config = require('../../../config.json');
const Paginacao = require('../../../util/Paginacao');
const moment = require('moment-timezone');
moment.tz.setDefault(config.servidor.timezone);
var knexnest = require('knexnest');
const PesquisaNota = require('../../../models/PesquisaNota');

exports.getOne = (request, reply) => {
    const knex = db.knex('public.pesquisa_satisfacao as p')
        .leftJoin('public.pesquisa_satisfacao_respostas as pv', 'p.id', 'pv.id_pesquisa');
    knex.where('p.id', request.params.ID);
    knex.select('p.id as _id', 'p.nome as _nome', 'p.descricao as _descricao', 'pv.id as _respostas__id', 'pv.nota as _respostas__nota', 'pv.significado as _respostas__significado');
    knexnest(knex)
        .then(function (rows) {
            const ret = {};
            ret.error = false;
            ret.full = true;
            ret.total = rows.length;
            ret.message = 'Dados da Pesquisa de Satisfação id = ' + request.params.ID;
            ret.dados = rows[0];
            reply(ret);
        })
        .catch(function (error) {
            console.error(error);
            reply(Boom.badData(error));
        });
};

exports.create = (request, reply) => {
    //Tratamento dos valores recebidos, pois são opcionais
    const respostas = request.payload.respostas;
    if (request.payload.respostas) delete request.payload.respostas;
    let newId = null;
    let valInserts = 0;

    const knex = db.knex;
    knex.transaction(function (t) {
        return db.knex("public.pesquisa_satisfacao")
            .transacting(t)
            .insert(request.payload)
            .returning("id")
            .then((idc) => {
                newId = idc[0];
                if (respostas && Array.isArray(respostas) && respostas.length > 0) {
                    return knex.batchInsert("public.pesquisa_satisfacao_respostas", respostas.map((x) => {
                        delete x.id;
                        x.id_pesquisa = newId;
                        return x;
                    }), 50)
                        .transacting(t)
                        .then(() => {
                            valInserts = respostas.length;
                        })
                        .catch((e) => { throw e; });
                }
            })
            .catch((e) => { throw e; });
    }).then(function () {
        //Retorno padrão
        const ret = {};
        ret.error = false;
        ret.id = newId;
        ret.message = `ID = ${newId} criado com sucesso, com ${valInserts} respostas criadas.`;
        reply(ret);
    })
        .catch(function (error) {
            console.error(error);
            reply(Boom.badData(error));
        });
};

exports.update = (request, reply) => {
    //Tratamento dos valores recebidos, pois são opcionais
    const respostas = request.payload.respostas;
    if (request.payload.respostas) delete request.payload.respostas;
    let valInserts = 0;

    const knex = db.knex;
    knex.transaction(function (t) {
        return db.knex("public.pesquisa_satisfacao")
            .transacting(t)
            .where('id', request.params.ID)
            .update(request.payload)
            .then(() => {
                return db.knex("public.pesquisa_satisfacao_respostas")
                    .where('id_pesquisa', request.params.ID)
                    .delete()
                    .then(() => {
                        if (respostas && Array.isArray(respostas) && respostas.length > 0) {
                            return knex.batchInsert("public.pesquisa_satisfacao_respostas", respostas.map((x) => {
                                delete x.id;
                                x.id_pesquisa = request.params.ID;
                                return x;
                            }), 50)
                                .transacting(t)
                                .then(() => {
                                    valInserts = respostas.length;
                                })
                                .catch((e) => { throw e; });
                        }
                    })
                    .catch((e) => { throw e; });
            })
            .catch((e) => { throw e; });
    })
        .then(function () {
            //Retorno padrão
            const ret = {};
            ret.error = false;
            ret.message = `ID = ${request.params.ID} atualizado com sucesso, com ${valInserts} respostas criadas/atualizadas.`;
            reply(ret);
        })
        .catch(function (error) {
            console.error(error);
            reply(Boom.badData(error));
        });
};

exports.getPesquisaRegistros = (request, reply) => {
    const max = request.query.max ? request.query.max : 15;
    const page = request.query.page ? request.query.page : 1;
    const pag = new Paginacao(max, page);

    const knex = db.knex('public.pesquisa_satisfacao_registros AS psr')
        .innerJoin('public.pesquisa_satisfacao AS b', 'psr.id_pesquisa', 'b.id')
        .leftOuterJoin('public.pesquisa_satisfacao_respostas AS r', function () {
            this.on("psr.nota", "r.nota")
                .on("r.id_pesquisa", "b.id");
        });

    // Filtros
    if (request.query.dataini && !request.query.horaini) knex.where('psr.data_registro', '>=', moment(request.query.dataini, 'DD/MM/YYYY').format('YYYY/MM/DD') + ' 00:00:00');
    if (request.query.datafim && !request.query.horafim) knex.where('psr.data_registro', '<=', moment(request.query.datafim, 'DD/MM/YYYY').format('YYYY/MM/DD') + ' 23:59:59');

    if (request.query.dataini && request.query.horaini) knex.where('psr.data_registro', '>=', moment(request.query.dataini, 'DD/MM/YYYY').format('YYYY/MM/DD') + ` ` + request.query.horaini);
    if (request.query.datafim && request.query.horafim) knex.where('psr.data_registro', '<=', moment(request.query.datafim, 'DD/MM/YYYY').format('YYYY/MM/DD') + ` ` + request.query.horafim);

    if (request.query.fila) knex.whereIn('psr.fila', request.query.fila);
    if (request.query.agente) knex.whereIn('psr.agente', request.query.agente);
    if (request.query.pesquisa) knex.whereIn('psr.id_pesquisa', request.query.pesquisa);
    if (request.query.resposta) knex.whereIn('psr.nota', request.query.resposta);

    pag.select = [
        'psr.*',
        db.knex.raw("CONCAT(TRIM(psr.nota), ' - ', r.significado) as notaFullname"),
        'b.nome as pesquisa',
        db.knex.raw('COALESCE((SELECT b.nome FROM cdr_classificacoes a JOIN classificacoes b on a.id_classificacao = b.id WHERE uniqueid = psr.uniqueid limit 1),(SELECT b.nome FROM "queueChatClassificacao" a JOIN classificacoes b on a."classificacaoID" = b.id WHERE a."queueChatID"::TEXT = psr.uniqueid ORDER by a."dataRegistro" DESC LIMIT 1) ) as classificacao'),
        db.knex.raw('(SELECT a.link FROM cdr_gravacao a WHERE a.uniqueid = psr.uniqueid limit 1) as gravacao'),
        db.knex.raw('COALESCE((SELECT a.protocolo FROM cdr_classificacoes a WHERE uniqueid = psr.uniqueid limit 1), psr.uniqueid) as protocolo')
    ];

    pag.order = request.query.order;
    pag.knex = knex;

    pag.getRet((ret, err) => {
        if (err) {
            console.log('Erro listando Registros de Pesquisa. Erro = ' + err);
            reply(Boom.badData(err));
        } else {
            reply(ret);
        }
    });
};

exports.setNota = async (request, reply) => {
    try {
        const dados = request.payload;
        dados.idPesquisa = request.params.ID;
        reply(await new PesquisaNota(dados).save());
    } catch (error) {
        reply(Boom.badData(error))
    }
};
