'use strict'

const Joi = require('joi');
const handlers = require('./handlers');
const CRUD = require('../../../util/CRUD');
const util = require('../../../util/restbase');

//Conjunto de dados padrão + específico
const ListaReturnSchema = util.CreateReturnSchema({ dados: Joi.array().items().example([{ id: 1, nome: "Registro1" }, { id: 2, nome: "Registro2" }]).description('Lista dos registros') });
const operationData = {
  nome: Joi.string().required(),
  descricao: Joi.string().required(),
  respostas: Joi.array().items({
    id: Joi.number().integer().optional().allow(null),
    id_pesquisa: Joi.number().integer().optional().allow(null),
    nota: Joi.string().max(5).required().description("Resposta a ser informada pelo cliente pesquisado"),
    significado: Joi.string().max(50).required().description("Semântica do valor. Ex: 1 significa Não. Este campo deverá conter: Não"),
  }).optional().allow(null)
};
const operationDataValor = {
  id_pesquisa: Joi.number().integer().required(),
  nota: Joi.string().length(5).required().description("Resposta a ser informada pelo cliente pesquisado"),
  significado: Joi.string().length(50).required().description("Semântica do valor. Ex: 1 significa Não. Este campo deverá conter: Não"),
};

module.exports = [
  {
    method: 'GET',
    path: '/api/v3/pesquisa',
    handler: (request, reply) => {
      const crud = new CRUD('public.pesquisa_satisfacao');
      crud.filtros = [];
      if (request.query.nome) crud.filtros.push({ campo: 'LOWER(nome)', operador: 'LIKE', valor: '%' + decodeURIComponent(request.query.nome).toLowerCase() + '%' });
      crud.getAll(request, reply);
    },
    config: {
      description: 'Pesqusia as pesquisas de satisfação cadastradas. Informe os filtros para obter uma consulta personalizada ou chame o método sem dados para obter a primeira página de resultados.',
      tags: ['api', 'Pesquisa de Satisfação'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      response: {
        schema: ListaReturnSchema
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
              campo: Joi.string().description('Nome da coluna'),
              ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          nome: Joi.string().description('Filtra por nome'),
          horaini: Joi.string().description(`Filtra por hora inicial`).optional().allow(null).allow(``),
          horafim: Joi.string().description(`Filtra por hora final`).optional().allow(null).allow(``)
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/pesquisa/{ID}',
    handler: handlers.getOne,
    config: {
      description: 'Obtém os dados da pesquisa de satisfação',
      tags: ['api', 'Pesquisa de Satisfação'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v3/pesquisa',
    handler: handlers.create,
    config: {
      description: 'Cria uma nova pesquisa de satisfação',
      tags: ['api', 'Pesquisa de Satisfação'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        payload: operationData
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v3/pesquisa/{ID}',
    handler: handlers.update,
    config: {
      description: 'Altera os dados da pesquisa de satisfação',
      tags: ['api', 'Pesquisa de Satisfação'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        },
        payload: operationData
      }
    }
  },
  {
    method: 'DELETE',
    path: '/api/v3/pesquisa/{ID}',
    handler: (request, reply) => {
      const crud = new CRUD('public.pesquisa_satisfacao');
      crud.id = "id";
      crud.delete(request, reply);
    },
    config: {
      description: 'Remove a pesquisa de satisfação indicada',
      tags: ['api', 'Pesquisa de Satisfação'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/pesquisa/registro',
    handler: handlers.getPesquisaRegistros,
    config: {
      description: 'Pesqusia os registros das pesquisas de satisfação, gerados. Informe os filtros para obter uma consulta personalizada ou chame o método sem dados para obter a primeira página de resultados.',
      tags: ['api', 'Pesquisa de Satisfação'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      response: {
        schema: ListaReturnSchema
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
              campo: Joi.string().description('Nome da coluna'),
              ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          dataini: Joi.string().description('Filtra por data (início do período)'),
          datafim: Joi.string().description('Filtra por data (fim do período)'),
          fila: Joi.array().items(Joi.string()).optional().allow(null).description('Filtra por fila'),
          agente: Joi.array().items(Joi.string()).optional().allow(null).description('Filtra por agente'),
          pesquisa: Joi.array().items(Joi.number().integer()).optional().allow(null).description('Filtra por pesquisa'),
          resposta: Joi.array().items(Joi.number().integer()).optional().allow(null).description('Filtra respostas de pesquisas'),
          classificacao: Joi.array().items(Joi.number().integer()).optional().allow(null).description('Filtra as classificações da ligação das pesquisas'),
          horaini: Joi.string().description(`Filtra por hora inicial`).optional().allow(null).allow(``),
          horafim: Joi.string().description(`Filtra por hora final`).optional().allow(null).allow(``)
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v3/pesquisa/{ID}/registro',
    handler: handlers.setNota,
    config: {
      description: 'Inclui um registro na pesquisa',
      tags: ['api', 'Pesquisa de Satisfação'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        },
        payload: {
          nota: Joi.string().description(`Nota`),
          agente: Joi.string().description(`Nome do agente`).optional().allow(null).allow(``),
          fila: Joi.string().description(`Nome da fila`).optional().allow(null).allow(``),
          uniqueid: Joi.string().description(`ID da ligação`).optional().allow(null).allow(``),
          src: Joi.string().description(`Numero de origem`).optional().allow(null).allow(``),
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/pesquisa/resposta',
    handler: (request, reply) => {
      const crud = new CRUD('public.pesquisa_satisfacao_respostas');
      crud.id = 'id';
      crud.filtros = [];
      if (request.query.nota) crud.filtros.push({ campo: 'nota', operador: 'like', valor: request.query.nota + '%' });
      if (request.query.significado) crud.filtros.push({ campo: 'significado', operador: 'like', valor: request.query.significado + '%' });
      if (request.query.pesquisas && Array.isArray(request.query.pesquisas)) crud.filtros.push({ campo: 'id_pesquisa', valor: request.query.pesquisas });

      crud.getAll(request, reply);
    },
    config: {
      description: 'Obtém as respostas a pesquisas de satisfação',
      tags: ['api', 'Pesquisa de Satisfação - Valores'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      response: {
        schema: ListaReturnSchema
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
              campo: Joi.string().description('Nome da coluna'),
              ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          nota: Joi.string().description("Nota ou Valor de Resposta desejado"),
          significado: Joi.string().description("Significado de uma Resposta, desejado"),
          pesquisas: Joi.array().items(Joi.number().integer()).description('Pesquisas das quais se desejam as respostas.')
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/pesquisa/resposta/{ID}',
    handler: (request, reply) => {
      const crud = new CRUD('public.pesquisa_satisfacao_respostas');
      crud.id = 'id';
      crud.getOne(request, reply);
    },
    config: {
      description: 'Obtém os dados da resposta à pesquisa de satisfação',
      tags: ['api', 'Pesquisa de Satisfação - Valores'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v3/pesquisa/resposta',
    handler: (request, reply) => {
      const crud = new CRUD('public.pesquisa_satisfacao_respostas');
      crud.id = 'id';
      crud.create(request, reply);
    },
    config: {
      description: 'Cria um novo valor de resposta para uma pesquisa de satisfação',
      tags: ['api', 'Pesquisa de Satisfação - Valores'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        payload: operationDataValor
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v3/pesquisa/resposta/{ID}',
    handler: (request, reply) => {
      const crud = new CRUD('public.pesquisa_satisfacao_respostas');
      crud.id = "id";
      crud.update(request, reply);
    },
    config: {
      description: 'Altera os dados da resposta à pesquisa de satisfação',
      tags: ['api', 'Pesquisa de Satisfação - Valores'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        },
        payload: operationDataValor
      }
    }
  },
  {
    method: 'DELETE',
    path: '/api/v3/pesquisa/resposta/{ID}',
    handler: (request, reply) => {
      const crud = new CRUD('public.pesquisa_satisfacao_respostas');
      crud.id = "id";
      crud.delete(request, reply);
    },
    config: {
      description: 'Remove uma resposta a uma pesquisa de satisfação indicada',
      tags: ['api', 'Pesquisa de Satisfação - Valores'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        }
      }
    }
  },
];