'use strict'

const md5 = require('md5');
const Boom = require('boom');
const db = require('../../../util/db');
const config = require('../../../config.json');
const moment = require('moment-timezone');
moment.tz.setDefault(config.servidor.timezone);

exports.getall = (request, reply) => {
  const max = request.query.max ? request.query.max : 15;
  const page = request.query.page ? request.query.page : 1;
  const knex = db.knex('public.pins as p')
  .select(db.knex.raw(`p.id, p.nome, p.ativo, ARRAY( SELECT g.nome
           FROM pin_grupos b
          INNER JOIN pin_grupo g ON b.id_grupo=g.id
          WHERE b.id_pin = p.id) AS grupos`));
  //TODO: Aqui se utiliza uma View para reduzir e aninhar a tablea de filhos (n-n) pin_grupos. Talvez possamos usar Objection ou outro ORM no futuro.
  if(request.query.nome) knex.whereRaw('LOWER(p.nome) LIKE ?', '%'+decodeURIComponent(request.query.nome).toLowerCase()+'%');
  if(typeof request.query.ativo != 'undefined') knex.where('p.ativo', request.query.ativo);
  
  if(max === -1){
    //console.log('SQL = '+ knex.toString());
    knex.clearSelect();
    // Parametros de ordenação, fora do count
    if(request.query.order){
      for(let p=0;p<request.query.order.length;++p){
        if(request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
            knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
        }
        else if(request.query.order[p] && request.query.order[p].campo) {
          knex.orderBy(request.query.order[p].campo, 'ASC');
        }
      }
    }
    knex.select('*')
    .then(function(rows) {
      const ret = {};
      ret.error = false;
      ret.full = true;
      ret.total = rows.length;
      ret.message = 'Lista de todos os PINs';
      ret.dados = rows;
      reply(ret);
    })
    .catch(function(error) {
      console.error(error);
      reply(Boom.badData(error));
    });
  }
  else {
    const offset = max * (page - 1);
    //console.log('SQL = '+knex.count('* as total').toString());
    knex.clearSelect();
    knex.count('* as total')
    .then(function(total) {
      if(total[0].total > 0){
        knex.clearSelect();
        // Parametros de ordenação, fora do count
        if(request.query.order){
          for(let p=0;p<request.query.order.length;++p){
            if(request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
            }
            else if(request.query.order[p] && request.query.order[p].campo) {
              knex.orderBy(request.query.order[p].campo, 'ASC');
            }
          }
        }
        //console.log('SQL = '+knex.select('*').limit(max).offset(offset).toString());
        knex.select(db.knex.raw(`p.id, p.nome, p.ativo, 
        ARRAY( SELECT g.nome
                 FROM pin_grupos b
                INNER JOIN pin_grupo g ON b.id_grupo=g.id
                WHERE b.id_pin = p.id) AS grupos`)).limit(max).offset(offset).then(function(rows) {
          const pages = Math.ceil(parseInt(total[0].total) / max);
          const ret = {};
          ret.error = false;
          ret.full = pages === 1 ? true : false;
          ret.page = page;
          ret.max = max;
          ret.offset = offset;
          ret.total = parseInt(total[0].total);
          ret.pages = pages;
          ret.dados = rows;
          ret.message = 'Lista de PINs, paginada';
          reply(ret);
        })
        .catch(function(error) {
          console.error(error);
          reply(Boom.badData(error));
        });
      }
      else {
        const ret = {};
        ret.error = true;
        ret.total = 0;
        ret.message = 'Nenhum registro encontrado';
        reply(ret);
      }
    })
    .catch(function(error) {
      console.error(error);
      reply(Boom.badData(error));
    });
  }
};

exports.getOne = (request, reply) => {
  const knex = db.knex('public.pins as p');
  knex.where('id', request.params.ID);
  knex.select(db.knex.raw(`p.id, p.nome, p.ativo, 
        ARRAY( SELECT g.id
                 FROM pin_grupos b
                INNER JOIN pin_grupo g ON b.id_grupo=g.id
                WHERE b.id_pin = p.id) AS grupos`))
  .then(function(rows) {
    const ret = {};
    ret.error = false;
    ret.full = true;
    ret.total = rows.length;
    ret.message = 'Dados do PIN id = '+request.params.ID;
    ret.dados = rows[0];
    reply(ret);
  })
  .catch(function(error) {
    console.error(error);
    reply(Boom.badData(error));
  });
};

exports.create = (request, reply) => {
  const grupos = request.payload.grupos;
  delete request.payload.grupos;
  let newID = 0;

  const knex = db.knex;
  knex.transaction(function(t) {
    return knex('public.pins')
     .transacting(t)
     .then(function() {
          if(!request.payload.senha){ 
            throw 'Para criar um PIN você deve informar uma senha.';}
          else{
            request.payload.senha = md5(request.payload.senha);}
          if(!request.payload.data_criacao) request.payload.data_criacao = moment(new Date().getTime()).format('YYYY-MM-DD HH:mm');
          return knex('public.pins')
             .transacting(t) 
             .insert(request.payload)
             .returning('id')
             .then(async function(id){
                //console.log(JSON.stringify(row));
                newID = parseInt(id);
                if(grupos && grupos.length > 0)
                {
                  let newRows = [];
                  grupos.forEach(g=>{
                      const pg =
                      {
                        id_pin: newID,
                        id_grupo: parseInt(g)
                      };
                      newRows.push(pg);
                  });
                  //console.log(JSON.stringify(newRows));
                  await knex.insert(newRows)
                  .into('public.pin_grupos')
                  .transacting(t)
                  .then(t.commit)
                  .catch(t.rollback);
                }
              })
              .catch(t.rollback)
     })
     .then(t.commit)
     .catch(t.rollback)
  })
  .then(function() {
    console.log('PIN cadastrado com sucesso!');
    const ret = {};
    ret.error = false;
    ret.message = 'PIN id = '+newID+' criado com sucesso.';
    reply(ret);
  })
  .catch(function(e) {
    console.log('Falha ao criar PIN. Erro = '+e);
    reply(Boom.badData(e));
  });
};

exports.update = (request, reply) => {
  const grupos = request.payload.grupos;
  delete request.payload.grupos;
  if(!request.payload.senha) 
    delete request.payload.senha;
  else
    request.payload.senha = md5(request.payload.senha);  
  
  request.payload.data_ativacao = request.payload.ativo ? moment(new Date().getTime()).format('YYYY-MM-DD HH:mm') : null,
  request.payload.data_desativacao = request.payload.ativo ? null : moment(new Date().getTime()).format('YYYY-MM-DD HH:mm');

  const knex = db.knex;
  knex.transaction(function(t) {
    return knex('public.pins')
     .transacting(t)
     .where('id', request.params.ID)
     .update(request.payload)
     .then(function() {
          return knex('public.pin_grupos')
             .where('id_pin', request.params.ID)
             .del()
             .then(async function(){
              //console.log(JSON.stringify(row));
              if(grupos && grupos.length > 0)
              {
                let newRows = [];
                grupos.forEach(g=>{
                    const pg =
                    {
                      id_pin: parseInt(request.params.ID),
                      id_grupo: parseInt(g)
                    };
                    newRows.push(pg);
                });
                //console.log(JSON.stringify(newRows));
                await knex.insert(newRows)
                .into("public.pin_grupos")
                .transacting(t);
              }
            })
            .catch(function(e) {
              throw e;
            });
     })
     .then(t.commit)
     .catch(function(e) {
        t.rollback();
        throw e;
    });
  })
  .then(function() {
    console.log('PIN atualizado com sucesso!');
    const ret = {};
    ret.error = false;
    ret.message = 'PIN id = '+request.params.ID+' atualizado com sucesso.';
    reply(ret);
  })
  .catch(function(e) {
    console.log('Falha ao apagar PIN. Erro = '+e);
    reply(Boom.badData(e));
  });
};

exports.delete = (request, reply) => {
  const knex = db.knex;
  knex.transaction(function(t) {
    return knex('public.pin_grupos')
     .transacting(t)
     .where('id_pin', request.params.ID)
     .del()
     .then(function() {
          return knex('public.pins')
             .where('id', request.params.ID)
             .del()
             .then(t.commit)
             .catch(t.rollback);
     })
     .catch(t.rollback)
  })
  .then(function() {
    console.log('PIN apagado com sucesso!');
    const ret = {};
    ret.error = false;
    ret.message = 'PIN id = '+request.params.ID+' apagado com sucesso.';
    reply(ret);
  })
  .catch(function(e) {
    console.log('Falha ao apagar PIN. Erro = '+e);
    reply(Boom.badData(e));
  });
};

exports.toggleActive = (request, reply, active) => {
  const ac = {
      ativo: active,
      data_ativacao: active ? moment(new Date().getTime()).format('YYYY-MM-DD HH:mm') : null,
      data_desativacao: active ? null : moment(new Date().getTime()).format('YYYY-MM-DD HH:mm')
  };
  
  const knex = db.knex;
  knex('public.pins')
  .where('id', request.params.ID)
  .update(ac)
  .then(function() {
    console.log(`PIN ${active?"ativado":"desativado"} com sucesso!`);
    const ret = {};
    ret.error = false;
    ret.message = 'PIN id = '+request.params.ID+' apagado com sucesso.';
    reply(ret);
  })
  .catch(function(e) {
    console.log('Falha ao alternar ativação do PIN. Erro = '+e);
    reply(Boom.badData(e));
  });
};