'use strict'

const Joi = require('joi');
const handlers = require('./handlers');
const CRUD = require('../../../util/CRUD');
const util = require('../../../util/restbase');

//Dados das operações de escrita
const operationData =
{
    nome: Joi.string().required(),
    senha: Joi.string().optional().allow('',null).description("Deixe vazio para não alterar a senha atual"),
    ativo: Joi.bool().required().description("Indica se o PIN está ativo"),
    grupos: Joi.array().optional().items(Joi.number().integer()).allow(null)
};

//Conjunto de dados padrão + específico
const ListaReturnSchema = util.CreateReturnSchema(null);

module.exports = [
  {
    method: 'GET',
    path: '/api/v3/pin',
    handler: handlers.getall,
    config: {
      description: 'Pesquisa um PIN. Informe os filtros para obter uma consulta personalizada ou chame o método sem dados para obter a primeira página de resultados.',
      tags: ['api','Pin'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      response: {
        schema: ListaReturnSchema
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
                campo: Joi.string().description('Nome da coluna'),
                ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          nome: Joi.string().description('Filtra por nome'),
          ativo: Joi.bool().description('Filtra pela situação de ativo'),
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/pin/{ID}',
    handler: handlers.getOne,
    config: {
      description: 'Obtém os dados do PIN',
      tags: ['api','Pin'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v3/pin',
    handler: handlers.create,
    config: {
      description: 'Cria um novo PIN',
      tags: ['api','Pin'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        payload: operationData
      }
    }
},
{
    method: 'PUT',
    path: '/api/v3/pin/{ID}',
    handler: handlers.update,
    config: {
      description: 'Altera os dados do PIN',
      tags: ['api','Pin'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        },
        payload: operationData
      }
    }
},
{
    method: 'DELETE',
    path: '/api/v3/pin/{ID}',
    handler: handlers.delete,
    config: {
      description: 'Remove o PIN indicado',
      tags: ['api','Pin'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v3/pin/ativar/{ID}',
    handler: (request, reply) => {handlers.toggleActive(request, reply, true);},
    config: {
      description: 'Altera os dados do PIN',
      tags: ['api','Pin'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        },
        payload: operationData
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v3/pin/desativar/{ID}',
    handler: (request, reply) => {handlers.toggleActive(request, reply, false);},
    config: {
      description: 'Altera os dados do PIN',
      tags: ['api','Pin'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        },
        payload: operationData
      }
    }
  }, 
];
