'use strict'

const Boom = require('boom');
const db = require('../../../util/db');
const config = require('../../../config.json');
const moment = require('moment-timezone');
const namiLib = require("nami");
const nami = require('../../../ami/index');
const csv = require("fast-csv");
const fs = require("fs");
moment.tz.setDefault(config.servidor.timezone);

exports.getall = (request, reply) => {
    const max = request.query.max ? request.query.max : 15;
    const page = request.query.page ? request.query.page : 1;
    const knex = db.knex('public.ramais');

    if (request.query.callgroup) knex.where('callgroup', request.query.callgroup);
    if (request.query.pickupgroup) knex.where('pickupgroup', request.query.pickupgroup);
    if (request.query.context) knex.where('context', request.query.context);
    if (request.query.name) knex.whereRaw('LOWER(name) LIKE ?', '%' + decodeURIComponent(request.query.name).toLowerCase() + '%');
    if (request.query.fullname) knex.whereRaw('LOWER(fullname) LIKE ?', '%' + decodeURIComponent(request.query.fullname).toLowerCase() + '%');

    if (max === -1) {
        knex.clearSelect();

        // Parametros de ordenação, fora do count
        if (request.query.order) {
            for (let p = 0; p < request.query.order.length; ++p) {
                if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                    knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
                }
                else if (request.query.order[p] && request.query.order[p].campo) {
                    knex.orderBy(request.query.order[p].campo, 'ASC');
                }
            }
        }
        knex.select('*')
            .then(function (rows) {
                const ret = {};
                ret.error = false;
                ret.full = true;
                ret.total = rows.length;
                ret.message = 'Lista de todos os ramais';
                ret.dados = rows;
                reply(ret);
            })
            .catch(function (error) {
                console.error(error);
                reply(Boom.badData(error));
            });
    }
    else {
        const offset = max * (page - 1);

        knex.count('* as total')
            .then(function (total) {
                if (total[0].total > 0) {
                    knex.clearSelect();
                    // Parametros de ordenação, fora do count
                    if (request.query.order) {
                        for (let p = 0; p < request.query.order.length; ++p) {
                            if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                                knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
                            }
                            else if (request.query.order[p] && request.query.order[p].campo) {
                                knex.orderBy(request.query.order[p].campo, 'ASC');
                            }
                        }
                    }

                    knex.select('*').limit(max).offset(offset).then(function (rows) {
                        const pages = Math.ceil(parseInt(total[0].total) / max);
                        const ret = {};
                        ret.error = false;
                        ret.full = pages === 1 ? true : false;
                        ret.page = page;
                        ret.max = max;
                        ret.offset = offset;
                        ret.total = parseInt(total[0].total);
                        ret.pages = pages;
                        ret.message = 'Lista de ramais paginada';
                        ret.dados = rows;
                        reply(ret);
                    })
                        .catch(function (error) {
                            console.error(error);
                            reply(Boom.badData(error));
                        });
                }
                else {
                    const ret = {};
                    ret.error = true;
                    ret.total = 0;
                    ret.message = 'Nenhum registro encontrado';
                    reply(ret);
                }
            })
            .catch(function (error) {
                console.error(error);
                reply(Boom.badData(error));
            });
    }
};

exports.getOne = (request, reply) => {
    const knex = db.knex('public.ramais');
    knex.where('id', request.params.ID);
    knex.select('id', 'name', 'fullname', 'secret', 'callgroup', 'context', 'allow', 'pickupgroup', 'nat', 'subscribecontext')
        .then(function (rows) {
            const ret = {};
            ret.error = false;
            ret.full = true;
            ret.total = rows.length;
            ret.message = 'Dados do Ramal id = ' + request.params.ID;
            ret.dados = rows[0];
            reply(ret);
        })
        .catch(function (error) {
            console.error(error);
            reply(Boom.badData(error));
        });
};

function changeFormData(ramal) {
    //Validações e transformações
    if (ramal.callgroup != null && Array.isArray(ramal.callgroup)) {
        ramal.context = ramal.callgroup.context;
        ramal.callgroup = ramal.callgroup.id;
    }

    if (!!ramal.nat2) {
        ramal.nat = ramal.nat2 ? 'force_rport,comedia' : 'no';
    }
    delete ramal["nat2"];

    if (!!ramal.callgroup && !ramal.pickupgroup) {
        ramal.pickupgroup = ramal.callgroup;
    }

    return ramal;
}

exports.create = (data, reply) => {
    const ramal = changeFormData(data.payload);

    //Definição do comando Asterisk
    let action = new namiLib.Actions.Command();

    //Finalmente prepara a escrita e gestão do ramal no Asterisk
    const knex = db.knex("public.ramais");

    //Insert
    knex.insert(ramal);
    knex.returning("id")
        .then(function (idc) {
            //Ami commands
            action.command = "sip show peer " + ramal.name;
            nami.send(action, null);

            //Retorno padrão
            const ret = {};
            ret.error = false;
            ret.id = idc[0];
            ret.message = 'ID = ' + idc[0] + ' criado com sucesso.';
            reply(ret);
        })
        .catch(function (error) {
            console.error(error);
            reply(Boom.badData(error));
        });
};

exports.update = (data, reply) => {
    const ramal = changeFormData(data.payload);

    //Definição do comando
    let action = new namiLib.Actions.Command();

    //Finalmente prepara a escrita e gestão do ramal no Asterisk
    const knex = db.knex("public.ramais");

    //Update
    knex.update(ramal);
    knex.where("id", data.params.ID)
        .then(function (idc) {
            //AMI Commands
            action.command = "sip prune realtime peer " + ramal.name;

            //Envio via AMI
            nami.send(action, null);
            action.command = "sip show peer " + ramal.name;
            nami.send(action, null);

            //Monta o Retorno Padrão
            const ret = {};
            ret.error = false;
            ret.id = idc[0];
            ret.message = 'Ramal ID = ' + idc[0] + ' atualizado com sucesso.';
            reply(ret);
        })
        .catch(function (error) {
            console.error(error);
            reply(Boom.badData(error));
        });
};

exports.uploadCSV = async (request, reply) => {
    const dados = request.payload.file.toString('latin1', 0, request.payload.file.length);
    const registros = [];
    const ramais = [];

    //Processa o CSV de forma síncrona
    let parse = new Promise((resolve, reject) => {
        csv.fromString(dados, { headers: false, delimiter: ";" })
            .on("data", function (data) {
                registros.push(data);
            })
            .on("end", function () {
                console.log(`CSV recebido, lendo ${registros.length} registros...`);
                for (let r = 0; r < registros.length; r++) {
                    const ramalOLD = {
                        name: registros[r][0],
                        secret: registros[r][1],
                        context: registros[r][2],
                        defaultuser: registros[r][3],
                        fullname: registros[r][4],
                        mac: registros[r][5] && registros[r][5].length > 0 ? registros[r][5] : null,
                        callgroup: registros[r][6]
                    };
                    const ramal = {};

                    if (registros[r][0] != "")
                        ramal.type = registros[r][0]
                    if (registros[r][1] != "")
                        ramal.name = registros[r][1]
                    if (registros[r][2] != "")
                        ramal.secret = registros[r][2]
                    if (registros[r][3] != "")
                        ramal.context = registros[r][3]
                    if (registros[r][4] != "")
                        ramal.host = registros[r][4]
                    if (registros[r][5] != "")
                        ramal.ipaddr = registros[r][5]
                    if (registros[r][6] != "")
                        ramal.port = registros[r][6]
                    if (registros[r][7] != "")
                        ramal.regseconds = registros[r][7]
                    if (registros[r][8] != "")
                        ramal.defaultuser = registros[r][8]
                    if (registros[r][9] != "")
                        ramal.fullcontact = registros[r][9]
                    if (registros[r][10] != "")
                        ramal.useragent = registros[r][10]
                    if (registros[r][11] != "")
                        ramal.lastms = registros[r][11]
                    if (registros[r][12] != "")
                        ramal.callbackextension = registros[r][12]
                    if (registros[r][13] != "")
                        ramal.fullname = registros[r][13]
                    if (registros[r][14] != "")
                        ramal.regserver = registros[r][14]
                    if (registros[r][15] != "")
                        ramal.allow = registros[r][15]
                    if (registros[r][16] != "")
                        ramal.insecure = registros[r][16]
                    if (registros[r][17] != "")
                        ramal.status = registros[r][17]
                    if (registros[r][18] != "")
                        ramal.qualify = registros[r][18]
                    if (registros[r][19] != "")
                        ramal.hassip = registros[r][19]
                    if (registros[r][20] != "")
                        ramal.qualifyfreq = registros[r][20]
                    if (registros[r][21] != "")
                        ramal.subscribecontext = registros[r][21]
                    if (registros[r][22] != "")
                        ramal.pickupgroup = registros[r][22]
                    if (registros[r][23] != "")
                        ramal.callgroup = registros[r][23]
                    if (registros[r][24] != "")
                        ramal.transport = registros[r][24]
                    if (registros[r][25] != "")
                        ramal["call-limit"] = registros[r][25]
                    if (registros[r][26] != "")
                        ramal.nat = registros[r][26]
                    if (registros[r][27] != "")
                        ramal.mac = registros[r][27]
                    if (registros[r][28] != "")
                        ramal.cc_agent_policy = registros[r][28]
                    if (registros[r][29] != "")
                        ramal.cc_monitor_policy = registros[r][29]
                    if (registros[r][30] != "")
                        ramal.accountcode = registros[r][30]
                    if (registros[r][31] != "")
                        ramal.amaflags = registros[r][31]
                    if (registros[r][32] != "")
                        ramal.dtmfmode = registros[r][32]
                    if (registros[r][33] != "")
                        ramal.fromuser = registros[r][33]
                    if (registros[r][34] != "")
                        ramal.fromdomain = registros[r][34]
                    if (registros[r][35] != "")
                        ramal.language = registros[r][35]
                    if (registros[r][36] != "")
                        ramal.mailbox = registros[r][36]
                    if (registros[r][37] != "")
                        ramal.permit = registros[r][37]
                    if (registros[r][38] != "")
                        ramal.deny = registros[r][38]
                    if (registros[r][39] != "")
                        ramal.mask = registros[r][39]
                    if (registros[r][40] != "")
                        ramal.restrictcid = registros[r][40]
                    if (registros[r][41] != "")
                        ramal.rtptimeout = registros[r][41]
                    if (registros[r][42] != "")
                        ramal.rtpholdtimeout = registros[r][42]
                    if (registros[r][43] != "")
                        ramal.musiconhold = registros[r][43]
                    if (registros[r][44] != "")
                        ramal.regexten = registros[r][44]
                    if (registros[r][45] != "")
                        ramal.cancallforward = registros[r][45]
                    if (registros[r][46] != "")
                        ramal.callerid = registros[r][46]
                    if (registros[r][47] != "")
                        ramal.hasvoicemail = registros[r][47]
                    if (registros[r][48] != "")
                        ramal.encryption = registros[r][48]
                    if (registros[r][49] != "")
                        ramal.avpf = registros[r][49]
                    if (registros[r][50] != "")
                        ramal.icesupport = registros[r][50]
                    if (registros[r][51] != "")
                        ramal.dtlsenable = registros[r][51]
                    if (registros[r][52] != "")
                        ramal.dtlsverify = registros[r][52]
                    if (registros[r][53] != "")
                        ramal.dtlscertfile = registros[r][53]
                    if (registros[r][54] != "")
                        ramal.dtlssetup = registros[r][54]
                    if (registros[r][55] != "")
                        ramal.rtcp_mux = registros[r][55]

                    console.log(`AQUIIII`, ramal);

                    ramais.push(ramal);
                }
                console.log('CSV - Fim da Leitura dos registros');
                resolve(true);
            });
    });
    await parse;
    if (ramais.length > 0) {
        //console.log(`${ramais.length} ramais prontos para inserção.`);
        db.knex.batchInsert('ramais', ramais, 500)
            .returning('id')
            .then(function (ids) {
                const ret = {};
                ret.error = false;
                ret.message = `${ids.length} ramais inseridos com sucesso!`;
                console.log(ret.message);
                reply(ret);
            }).catch(function (error) {
                console.log('Erro no insert dos ramais');
                console.log(error);
                reply(Boom.badData(error));
            });
    }
};

exports.deploy = (request, reply) => {
    const knex = db.knex('public.ramais');
    knex.whereRaw('mac is not null');
    if (request.params.ID) knex.where('id', request.params.ID);
    knex.select('name', 'secret', 'fullname', 'mac')
        .then(function (rows) {
            let f = 0;
            try {
                fs.mkdirSync('/var/www/html/deployment');
            } catch (error) {
                //Provavelmente já temos a pasta criada
                if (error.code != 'EEXIST') throw error;
            }

            rows.forEach(r => {
                const data = `#!version: 1.0.0.1\n` +
                    `account.1.enable=1\n` +
                    `account.1.label=${r.name}\n` +
                    `account.1.display_name=${r.fullname}\n` +
                    `account.1.auth_name=${r.name}\n` +
                    `account.1.user_name=${r.name}\n` +
                    `account.1.password=${r.secret}`;

                fs.writeFileSync(`/var/www/html/deployment/${r.mac}.cfg`, data);

                f++;
            });

            const ret = {};
            ret.error = false;
            ret.total = f;
            ret.message = `Gerados ${f}, arquivos de configuração.`;
            reply(ret);
        })
        .catch(function (error) {
            console.error(error);
            reply(Boom.badData(error));
        });
}