'use strict'

const Joi = require('joi');
const handlers = require('./handlers');
const CRUD = require('../../../util/CRUD');
const util = require('../../../util/restbase');

//Conjunto de dados padrão + específico
const ListaReturnSchema = util.CreateReturnSchema(null);
const operationData = {
  name: Joi.string(),
  fullname: Joi.string().allow(null),
  secret: Joi.string().allow(null),
  callgroup: Joi.any().description("Grupo de Chamadas").allow(null),
  context: Joi.any().description("Grupo de Rota").allow(null),
  allow: Joi.string().allow(null),
  pickupgroup: Joi.any().description("Grupo de Captura").allow(null),
  nat: Joi.string().description("Indicador do modo NAT").allow(null),
  nat2: Joi.bool(),
  subscribecontext: Joi.string().allow(null),
  mac: Joi.string().optional().allow(null).allow(""),
  hassip: Joi.string().valid("yes", "no").example("no"),
  transport: Joi.string().valid("wss", "udp").example("udp"),
  encryption: Joi.string().valid("yes", "no").example("no"),
  avpf: Joi.string().valid("yes", "no").example("no"),
  icesupport: Joi.string().valid("yes", "no").example("no"),
  dtlsenable: Joi.string().valid("yes", "no").example("no"),
  dtlsverify: Joi.string().valid("fingerprint", null).allow(null).example("no"),
  dtlscertfile: Joi.string().valid("/etc/asterisk/keys/asterisk.pem", null).allow(null).example("no"),
  dtlssetup: Joi.string().valid("actpass", null).allow(null).example("no"),
  rtcp_mux: Joi.string().valid("yes", "no").example("no"),
  xc_tipo: Joi.string().optional().allow(null).valid("ramal", "multi").example("ramal"),
  tem_multi: Joi.boolean().optional().allow(null),
  multi_regra: Joi.string().optional().allow(null).allow(""),
};

module.exports = [
  {
    method: 'GET',
    path: '/api/v3/ramal',
    handler: handlers.getall,
    config: {
      description: 'Pesqusia os ramais. Informe os filtros para obter uma consulta personalizada ou chame o método sem dados para obter a primeira página de resultados.',
      tags: ['api', 'Ramal'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      response: {
        schema: ListaReturnSchema
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
              campo: Joi.string().description('Nome da coluna'),
              ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          pickupgroup: Joi.string().description('Filtra por grupo de captura'),
          context: Joi.string().description('Filtra por contexto'),
          callgroup: Joi.string().description('Filtra por grupo de rota'),
          name: Joi.string().description('Filtra por grupo de rota'),
          fullname: Joi.string().description('Filtra por grupo de rota'),
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/ramal/{ID}',
    handler: handlers.getOne,
    config: {
      description: 'Obtém os dados do ramal',
      tags: ['api', 'Ramal'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v3/ramal',
    handler: handlers.create,
    config: {
      description: 'Cria um novo ramal',
      tags: ['api', 'Ramal'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        payload: operationData
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v3/ramal/{ID}',
    handler: handlers.update,
    config: {
      description: 'Altera os dados do ramal',
      tags: ['api', 'Ramal'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        },
        payload: operationData
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v3/ramal/upload',
    handler: handlers.uploadCSV,
    config: {
      description: 'Importa ramais indicados no CSV',
      tags: ['api', 'Ramal'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      }
    }
  },
  {
    method: 'DELETE',
    path: '/api/v3/ramal/{ID}',
    handler: (request, reply) => {
      const crud = new CRUD('public.ramais');
      crud.id = "id";
      crud.delete(request, reply);
    },
    config: {
      description: 'Remove o ramal indicado',
      tags: ['api', 'Ramal'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/ramal/deploy/{ID}',
    handler: handlers.deploy,
    config: {
      description: 'Gera arquivos de configuração para ramais Yealink.',
      tags: ['api', 'Ramal'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().optional().allow("", null)
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/ramal/deploy',
    handler: handlers.deploy,
    config: {
      description: 'Gera arquivos de configuração para ramais Yealink.',
      tags: ['api', 'Ramal'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      }
    }
  },
];