'use strict'

const Joi = require('joi');
const handlers = require('./handlers');
const CRUD = require('../../../util/CRUD');
const util = require('../../../util/restbase');

//Conjunto de dados padrão + específico
const ListaReturnSchema = util.CreateReturnSchema(null);

//Dados das operações de escrita
const baseData = {
  context: Joi.string().required().description("Grupo de Rotas"),
  exten: Joi.string().required().description("Prefixo"),
  priority: Joi.number().integer().required().description("Sequencia"),
  app: Joi.string().required().description("Aplicação"),
  appdata: Joi.string().optional().allow('',null).description("Argumentos da aplicação"),
  nome: Joi.string().optional().allow(null).description("Nome da rota"),
  rota_original: Joi.string().optional().allow(null).description("Exten antigo"),
}

const operationData = Joi.array().items(baseData);

const operationDataWithID = Object.assign({id: Joi.number().integer().optional()}, baseData);

module.exports = [
  {
    method: 'GET',
    path: '/api/v3/rota',
    handler: handlers.getall,
    config: {
      description: 'Pesqusia as rotas. Informe os filtros para obter uma consulta personalizada ou chame o método sem dados para obter a primeira página de resultados.',
      tags: ['api','Rota'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      response: {
        schema: ListaReturnSchema
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
                campo: Joi.string().description('Nome da coluna'),
                ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          nome: Joi.string().description('Filtra por nome'),
          context: Joi.string().description('Filtra por contexto (Grupo de Rotas)'),
          sentido: Joi.string().description('Filtra pelo sentido do contexto (Grupo de Rotas)'),
          exten: Joi.string().description('Filtra por prefixo'),
          priority: Joi.number().integer().description('Filtra por sequencia'),
          app: Joi.string().description('Filtra por Aplicação'),
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/rota/{RotaID}',
    handler: (request, reply) => {
      const crud = new CRUD('public.rotas');
      crud.id = 'id';
      crud.getOne(request, reply);
    },
    config: {
      description: 'Obtém os dados da rota',
      tags: ['api','Rota'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            RotaID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v3/rota',
    handler: (request, reply) => {
        const crud = new CRUD('public.rotas');
        crud.id = 'id';
        crud.create(request, reply);
    },
    config: {
      description: 'Cria uma nova rota',
      tags: ['api','Rota'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        payload: operationData
      }
    }
},
{
    method: 'PUT',
    path: '/api/v3/rota',
    handler: handlers.updateRota,
    config: {
      description: 'Altera os dados da rota',
      tags: ['api','Rota'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        payload: operationData
      }
    }
},
{
    method: 'DELETE',
    path: '/api/v3/rota/{ID}',
    handler: (request, reply) => {
        const crud = new CRUD('public.rotas');
        crud.id = "id";
        crud.delete(request, reply);
      },
    config: {
      description: 'Remove a rota',
      tags: ['api','Rota'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'DELETE',
    path: '/api/v3/rota/limparGrupo/{context}',
    handler: handlers.limpaGrupo,
    config: {
      description: 'Remove TODAS as rotas do Grupo de Rotas informado',
      tags: ['api','Rota'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          context: Joi.string().required()
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/rota/ext/{exten}/{context}',
    handler: handlers.getByExtenContext,
    config: {
      description: 'Obtém os dados da rota de mesmo prefixo e grupo',
      tags: ['api','Rota'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
            exten: Joi.string().required().description("Prefixo"),
            context: Joi.string().required().description("Grupo de Rotas"),
        }
      }
    }
  },
  {
    method: 'DELETE',
    path: '/api/v3/rota/ext/{exten}/{context}',
    handler: handlers.deleteByExtenContext,
    config: {
      description: 'Apaga TODAS as rotas com o prefíxo (extrensão/ramal) e grupo de rotas informados.',
      tags: ['api','Rota'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          exten: Joi.string().required(),
          context: Joi.string().required()
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v3/rota/clonaRota',
    handler: handlers.clonaRota,
    config: {
      description: 'Clona as rotas informadas',
      tags: ['api','Rota'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        payload: Joi.array().items(operationDataWithID).min(1).unique().required() 
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v3/rota/clonaGrupo/{origem}',
    handler: handlers.clonaGrupo,
    config: {
      description: 'Clona as rotas do grupo de origem nos grupos de destino',
      tags: ['api','Rota'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          origem: Joi.string().required()
        },
        payload: Joi.array().items(Joi.string()).min(1).unique().required() 
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v3/rota/full',
    handler: handlers.updateRotaFull,
    config: {
      description: 'Atualiza toda a rota',
      tags: ['api','Rota'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        payload: operationData
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/rota/URA/fluxo',
    handler: handlers.uraFluxo,
    config: {
      description: 'Retorna o Fluxo de utilização de uma ou mais URAs',
      tags: ['api','Rota'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          data_ini: Joi.date().iso().description('Filtro por data inicial'),
          data_fim: Joi.date().iso().description('Filtro por data Final'),
          contexts: Joi.array().items(Joi.string()).allow(null),
          extens: Joi.array().items(Joi.string()).allow(null),
          id_client: Joi.object().keys(util.SearchExpressionObject()).allow(null).description('Filtro pela Identificação do Cliente (recurso disponível mediante solcitiação).'),
          data_a: Joi.object().keys(util.SearchExpressionObject()).allow(null).description('Filtro pelo campo livre de uso configurável "data_a" (recurso disponível mediante solcitiação).'),
          data_b: Joi.object().keys(util.SearchExpressionObject()).allow(null).description('Filtro pelo campo livre de uso configurável "data_b" (recurso disponível mediante solcitiação).'),
          src: Joi.object().keys(util.SearchExpressionObject()).allow(null).description('Filtro Origem da ligação.'),
          src_menu: Joi.object().keys(util.SearchExpressionObject()).allow(null).description('Filtro Menu Origem.'),
          opcao: Joi.object().keys(util.SearchExpressionObject()).allow(null).description('Filtro da Opção.'),
          menu: Joi.object().keys(util.SearchExpressionObject()).allow(null).description('Filtro Menu da Opção.'),
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/rota/URA',
    handler: handlers.getUra,
    config: {
      description: 'Retorna as Rotas de uma URA',
      tags: ['api','Rota'],
      auth: {
          strategy: 'bearer',
          scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          context: Joi.string().required()
        }
      }
    }
  }
];
