'use strict'

const Joi = require('joi');
const handlers = require('./handlers');
const CRUD = require('../../../util/CRUD');
const util = require('../../../util/restbase');

//Conjunto de dados padrão + específico
const ListaReturnSchema = util.CreateReturnSchema({dados: Joi.array().items().example([{id: 1, nome: "Registro1"},{id: 2, nome: "Registro2"}]).description('Lista dos registros')});
const operationData = {
  nome: Joi.string().required(),
  rota: Joi.string().required(),
  registro: Joi.string().allow(null),
  conf: Joi.array().items(Joi.string()),
};

module.exports = [
    {
      method: 'GET',
      path: '/api/v3/tronco',
      handler: handlers.getall,
      config: {
        description: 'Pesqusia os troncos. Informe os filtros para obter uma consulta personalizada ou chame o método sem dados para obter a primeira página de resultados.',
        tags: ['api','Tronco'],
        auth: {
            strategy: 'bearer',
            scope: 'user' // or [ 'user', 'admin' ]
        },
        response: {
          schema: ListaReturnSchema
        },
        validate: {
          query: {
            max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
            page: Joi.number().integer().default(1).description('Página'),
            order: Joi.array().items(
              Joi.object().keys({
                  campo: Joi.string().description('Nome da coluna'),
                  ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
              })
            ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
            context: Joi.string().description('Filtra por contexto (Grupo de Rotas)'),
          }
        }
      }
    },
    {
      method: 'GET',
      path: '/api/v3/tronco/{ID}',
      handler: handlers.getOne,
      config: {
        description: 'Obtém os dados do tronco',
        tags: ['api','Tronco'],
        auth: {
            strategy: 'bearer',
            scope: 'user' // or [ 'user', 'admin' ]
        },
        validate: {
          params: {
            ID: Joi.number().integer().required()
          }
        }
      }
    },
    {
        method: 'POST',
        path: '/api/v3/tronco',
        handler: handlers.create,
        config: {
          description: 'Cria um novo tronco',
          tags: ['api','Tronco'],
          auth: {
              strategy: 'bearer',
              scope: 'user' // or [ 'user', 'admin' ]
          },
          validate: {
            payload: operationData
          }
        }
    },
    {
        method: 'PUT',
        path: '/api/v3/tronco/{ID}',
        handler: handlers.update,
        config: {
          description: 'Altera os dados do tronco',
          tags: ['api','Tronco'],
          auth: {
              strategy: 'bearer',
              scope: 'user' // or [ 'user', 'admin' ]
          },
          validate: {
            params: {
              ID: Joi.number().integer().required()
            },
            payload: operationData
          }
        }
    },
    {
      method: 'DELETE',
      path: '/api/v3/tronco/{ID}',
      handler: handlers.delete,
      config: {
        description: 'Remove o tronco indicado',
        tags: ['api','Tronco'],
        auth: {
            strategy: 'bearer',
            scope: 'user' // or [ 'user', 'admin' ]
        },
        validate: {
          params: {
            ID: Joi.number().integer().required()
          }
        }
      }
    },
    {
      method: 'POST',
      path: '/api/v3/tronco/state',
      handler: handlers.getState,
      config: {
        description: 'Obtém os dados de registro dos troncos',
        tags: ['api','Tronco'],
        auth: {
            strategy: 'bearer',
            scope: 'user' // or [ 'user', 'admin' ]
        },
        validate: {
          payload: {
            troncos: Joi.array().items(Joi.string()).required()
          }
        }
      }
    }, 
];