'use strict'

const Boom = require('boom');
const md5 = require('md5');
const db = require('../../../util/db');
const config = require('../../../config.json');
const util = require('../../../util/restbase');
const moment = require('moment-timezone');
moment.tz.setDefault(config.servidor.timezone);

exports.getall = (request, reply) => {
  const max = request.query.max ? request.query.max : 15;
  const page = request.query.page ? request.query.page : 1;
  const knex = db.knex('public.supervisores');

  if (request.query.nome && request.query.fullname) {
    knex.whereRaw('(LOWER(fullname) LIKE :vfn OR LOWER(nome) LIKE :vn)', { vfn: '%' + decodeURIComponent(request.query.fullname).toLowerCase() + '%', vn: '%' + decodeURIComponent(request.query.nome).toLowerCase() + '%' });
  }
  else {
    if (request.query.nome) knex.whereRaw('LOWER(nome) LIKE ?', '%' + decodeURIComponent(request.query.nome).toLowerCase() + '%');
    if (request.query.fullname) knex.whereRaw('LOWER(fullname) LIKE ?', '%' + decodeURIComponent(request.query.fullname).toLowerCase() + '%');
  }
  if (request.query.token) knex.where('token', request.query.token);
  if (request.query.criacao_token) {
    let date = moment(request.query.criacao_token).format('YYYY-MM-DD');
    knex.where('token_create', '>=', date + '00:00:00');
    knex.where('token_create', '<=', date + '23:59:59');
  }

  if (max === -1) {
    //console.log('SQL = '+ knex.toString());
    knex.clearSelect();
    // Parametros de ordenação, fora do count
    if (request.query.order) {
      for (let p = 0; p < request.query.order.length; ++p) {
        if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
          knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
        }
        else if (request.query.order[p] && request.query.order[p].campo) {
          knex.orderBy(request.query.order[p].campo, 'ASC');
        }
      }
    }
    knex.select(['id', 'nome', 'senha', 'fullname', 'status', 'autenticacao_fonte', 'usa_grupo_fila'])
      .then(function (rows) {
        const ret = {};
        ret.error = false;
        ret.full = true;
        ret.total = rows.length;
        ret.message = 'Lista de todos os usuários';
        ret.dados = rows;
        reply(ret);
      })
      .catch(function (error) {
        console.error(error);
        reply(Boom.badData(error));
      });
  }
  else {
    const offset = max * (page - 1);
    //console.log('SQL = '+knex.count('* as total').toString());
    knex.count('* as total')
      .then(function (total) {
        if (total[0].total > 0) {
          knex.clearSelect();
          // Parametros de ordenação, fora do count
          if (request.query.order) {
            for (let p = 0; p < request.query.order.length; ++p) {
              if (request.query.order[p] && request.query.order[p].campo && request.query.order[p].ordem) {
                knex.orderBy(request.query.order[p].campo, request.query.order[p].ordem);
              }
              else if (request.query.order[p] && request.query.order[p].campo) {
                knex.orderBy(request.query.order[p].campo, 'ASC');
              }
            }
          }
          //console.log('SQL = '+knex.select('*').limit(max).offset(offset).toString());
          knex.select(['id', 'nome', 'senha', 'fullname', 'status', 'autenticacao_fonte', 'usa_grupo_fila']).limit(max).offset(offset).then(function (rows) {
            const pages = Math.ceil(parseInt(total[0].total) / max);
            const ret = {};
            ret.error = false;
            ret.full = pages === 1 ? true : false;
            ret.page = page;
            ret.max = max;
            ret.offset = offset;
            ret.total = parseInt(total[0].total);
            ret.pages = pages;
            ret.message = 'Lista de usuários, paginada';
            ret.dados = rows;
            reply(ret);
          })
            .catch(function (error) {
              console.error(error);
              reply(Boom.badData(error));
            });
        }
        else {
          const ret = {};
          ret.error = true;
          ret.total = 0;
          ret.message = 'Nenhum registro encontrado';
          reply(ret);
        }
      })
      .catch(function (error) {
        console.error(error);
        reply(Boom.badData(error));
      });
  }
};

exports.create = (request, reply) => {
  //Se estamos alterando a senha, vamos criptografa-la
  if (request.payload.senha)
    request.payload.senha = md5(request.payload.senha);
  else
    throw 'Para criar um usuário você precisa informar uma senha.';

  // Gera o token do novo usuário
  request.payload.token = md5(util.GenerateUID());
  let newId = null;
  let userreq = getUserReq(request);
  //Operações com banco de dados
  const knex = db.knex;
  knex.transaction(function (t) {
    return knex('public.supervisores')
      .transacting(t)
      .insert(request.payload)
      .returning('id')
      .then(function (id) {
        newId = id[0];
        //Só loga quando está na UI e com alteração de senha
        if (userreq && request.payload.senha) {
          const log = {
            id_supervisor: userreq,
            acao: 'C',
            id_user: newId,
            descricao: 'Usuário de id = ' + newId + ' e login = ' + request.payload.nome + ' criado. IP: ' + util.GetRequestIP(request),
          };
          return knex('public.supervisores_log')
            .transacting(t)
            .insert(log)
            .then(t.commit)
            .catch((e) => { throw e; });
        }
      })
      .catch((e) => { throw e; });
  })
    .then(function () {
      console.log('Usuário criado com sucesso!');
      const ret = {};
      ret.error = false;
      ret.message = 'Usuário id = ' + newId + ' criado com sucesso.';
      reply(ret);
    })
    .catch(function (e) {
      console.log('Falha ao criar usuário. Erro = ' + e);
      reply(Boom.badData(e));
    });
};

exports.update = (request, reply) => {
  //Se estamos alterando a senha, vamos criptografa-la
  if (request.payload.senha)
    request.payload.senha = md5(request.payload.senha);
  else
    delete request.payload.senha;

  let userreq = getUserReq(request);
  // Qualquer mudança nos dados do usuário gera u novo token para invalidar o atual login
  request.payload.token = md5(util.GenerateUID());
  //Operações com banco de dados
  const knex = db.knex;
  knex.transaction(function (t) {
    return knex('public.supervisores')
      .transacting(t)
      .where('id', request.params.ID)
      .update(request.payload)
      .then(function () {
        //Só loga quando está na UI e com alteração de senha
        if (userreq && request.payload.senha) {
          const log = {
            id_supervisor: userreq,
            acao: 'U',
            id_user: request.params.ID,
            descricao: 'Senha alterada. IP: ' + util.GetRequestIP(request),
          };
          return knex('public.supervisores_log')
            .transacting(t)
            .insert(log)
            .then(t.commit)
            .catch((e) => { throw e; });
        }
      })
      .catch((e) => { throw e; });
  })
    .then(function () {
      console.log('Usuário alterado com sucesso!');
      const ret = {};
      ret.error = false;
      ret.message = 'Usuário id = ' + request.params.ID + ' alterado com sucesso.';
      reply(ret);
    })
    .catch(function (e) {
      console.log('Falha ao alterar usuário. Erro = ' + e);
      reply(Boom.badData(e));
    });
};

exports.delete = (request, reply) => {
  let userreq = getUserReq(request);
  const knex = db.knex;
  var usuario = null;
  knex.transaction(function (t) {
    return knex('public.supervisores')
      .where('id', request.params.ID)
      .transacting(t)
      .select()
      .then(function (rows) {
        if (rows.length == 0) throw `Usuário identificado por ${request.params.ID}, não foi encontrado. Verifique se ainda está disponível para esta operação.`;
        usuario = rows[0];
        return knex('public.supervisores_modulos')
          .transacting(t)
          .where('id_supervisor', request.params.ID)
          .del()
          .then(function () {
            return knex('public.supervisores_filas')
              .where('id_supervisor', request.params.ID)
              .transacting(t)
              .del()
              .then(function () {
                return knex('public.supervisores')
                  .where('id', request.params.ID)
                  .transacting(t)
                  .del()
                  .then(function () {
                    //Só loga quando está na UI
                    if (userreq) {
                      const log = {
                        id_supervisor: userreq,
                        acao: 'R',
                        id_user: request.params.ID,
                        descricao: `Usuário de id = ${request.params.ID}, ${usuario.fullname}, login = ${usuario.nome}, removido. IP: ${util.GetRequestIP(request)}`,
                      };
                      return knex('public.supervisores_log')
                        .transacting(t)
                        .insert(log)
                        .then(t.commit)
                        .catch((e) => { throw e; });
                    }
                  })
                  .catch((e) => { throw e; });
              })
              .catch((e) => { throw e; });
          })
          .catch((e) => { throw e; });
      })
      .catch((e) => { throw e; });
  })
    .then(function () {
      console.log('Usuário apagado com sucesso!');
      const ret = {};
      ret.error = false;
      ret.message = 'Usuário id = ' + request.params.ID + ' apagado com sucesso.';
      reply(ret);
    })
    .catch(function (e) {
      console.log('Falha ao apagar usuário. Erro = ' + e);
      reply(Boom.badData(e));
    });
};

exports.login = (request, reply) => {
  const knex = db.knex('public.supervisores');
  knex.where('supervisores.nome', request.payload.nome);
  knex.where('supervisores.senha', md5(request.payload.senha));
  knex.leftJoin('supervisores_modulos', 'supervisores_modulos.id_supervisor', 'supervisores.id');
  knex.leftJoin('modulos', 'modulos.id', 'supervisores_modulos.id_modulo');
  knex.orderBy('modulos.ordem', 'ASC');
  knex.select(['supervisores.id as idSuper', 'supervisores.fullname', 'supervisores.token', 'supervisores_modulos.extra_usuario', 'modulos.id', 'modulos.parent', 'modulos.nome', 'modulos.descricao', 'modulos.icon', 'modulos.ordem'])
    .then(async function (rows) {
      const ret = {};
      if (rows.length > 0) {
        ret.error = false;
        ret.message = 'Credenciais de login';
        ret.dados = {};
        ret.dados.id = rows[0].idSuper;
        ret.dados.nome = rows[0].fullname;
        ret.dados.token = rows[0].token;
        ret.dados.modulos = [];
        for (let i = 0; i < rows.length; ++i) {
          let modulo = { nome: rows[i].nome, icon: rows[i].icon, descricao: rows[i].descricao, order: rows[i].ordem, extra: rows[i].extra_usuario };
          ret.dados.modulos.push(modulo);
        }
        ret.dados.modulos_tree = ordered_menu(rows, null);
        await db.knex('public.supervisores_filas')
          .where("id_supervisor", ret.dados.id)
          .select("id_fila")
          .then(function (rows) {
            ret.dados.filas = rows.map((x) => { return x.id_fila; });
          });
      }
      else {
        ret.error = true;
        ret.total = 0;
        ret.message = 'Nenhum registro encontrado';
      }
      reply(ret);
    })
    .catch(function (error) {
      reply(Boom.badData(error));
    });
};

function getUserReq (request) {
  let userreq = null;
  if (request.payload && request.payload.userreq) {
    userreq = request.payload.userreq;
    delete request.payload.userreq;
  }
  else if (request.query && request.query.userreq) {
    userreq = request.query.userreq;
    delete request.query.userreq;
  }
  return userreq;
};

function ordered_menu (array, parent_id) {
  const temp_array = [];
  for (let i = 0; i < array.length; ++i) {
    if (array[i].parent == parent_id) {
      const modulo = { nome: array[i].nome, icon: array[i].icon, descricao: array[i].descricao, order: array[i].ordem, extra: array[i].extra_usuario, modulos: [] };
      modulo.modulos = ordered_menu(array, array[i].id);
      temp_array.push(modulo);
    }
  }
  return temp_array;
}