'use strict'
const Joi = require('joi');
const CRUD = require('../../../util/CRUD');
const md5 = require('md5');
const db = require('../../../util/db');
const Boom = require('boom');
const handlers = require('./handlers');
const util = require('../../../util/restbase');

//Conjunto de dados padrão + específico
const ListaReturnSchema = util.CreateReturnSchema(null);

//Dados das operações de escrita
const operationData =
{
  nome: Joi.string().required().description("Nome de usuário"),
  fullname: Joi.string().required().description("Nome completo"),
  senha: Joi.string().optional().allow(null).description("Deixe vazio para não alterar"),
  userreq: Joi.number().integer().optional().allow(null).description("Usuário que requisitou a operação")
};

module.exports = [
  {
    method: 'GET',
    path: '/api/v3/usuario',
    handler: handlers.getall,
    config: {
      description: 'Pesqusia os usuários. Informe os filtros para obter uma consulta personalizada ou chame o método sem dados para obter a primeira página de resultados.',
      tags: ['api', 'Usuário'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      response: {
        schema: ListaReturnSchema
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
              campo: Joi.string().description('Nome da coluna'),
              ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          nome: Joi.string().description('Filtra por nome'),
          fullname: Joi.string().description('Filtra pelo nome completo'),
          token: Joi.string().description('Filtra por token'),
          criacao_token: Joi.number().integer().description('Filtra por data de criação do token'),
          situacao: Joi.string().description('Filtra por situação'),
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/usuario/{ID}',
    handler: (request, reply) => {
      const crud = new CRUD('public.supervisores');
      crud.id = 'id';
      crud.select = ['id', 'nome', 'senha', 'fullname', 'status', 'autenticacao_fonte', 'usa_grupo_fila'];
      crud.getOne(request, reply);
    },
    config: {
      description: 'Obtém os dados do usuário',
      tags: ['api', 'Usuário'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v3/usuario',
    handler: handlers.create,
    config: {
      description: 'Cria um novo usuário',
      tags: ['api', 'Usuário'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        payload: operationData
      }
    }
  },
  {
    method: 'PUT',
    path: '/api/v3/usuario/{ID}',
    handler: handlers.update,
    config: {
      description: 'Altera os dados do usuário',
      tags: ['api', 'Usuário'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        },
        payload: operationData
      }
    }
  },
  {
    method: 'DELETE',
    path: '/api/v3/usuario/{ID}',
    handler: handlers.delete,
    config: {
      description: 'Remove o usuário',
      tags: ['api', 'Usuário'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        params: {
          ID: Joi.number().integer().required()
        }
      }
    }
  },
  {
    method: 'GET',
    path: '/api/v3/usuario/log',
    handler: (request, reply) => {
      const crud = new CRUD('public.supervisores_log AS log');
      crud.filtrosraw = [];
      crud.filtros = [];
      if (request.query.id_supervisor) crud.filtrosraw.push('log.id_supervisor = ANY(ARRAY[' + request.query.id_supervisor + '])');
      if (request.query.id_user) crud.filtrosraw.push('log.id_user = ANY(ARRAY[' + request.query.id_user + '])');
      if (request.query.modulo) crud.filtrosraw.push('log.modulo = ANY(ARRAY[' + request.query.modulo + '])');
      if (request.query.acao) crud.filtrosraw.push('log.acao = ANY(ARRAY' + JSON.stringify(request.query.acao).replace(/"/g, "'") + ')');
      if (request.query.data_ini) crud.filtros.push({ campo: 'log.data_hora', operador: '>=', valor: request.query.data_ini });
      if (request.query.data_fim) crud.filtros.push({ campo: 'log.data_hora', operador: '<=', valor: request.query.data_fim });
      if (request.query.order) crud.order = request.query.order;

      crud.leftjoin = [
        { tabela: "supervisores AS adm", campo1: "log.id_supervisor", campo2: "adm.id" },
        { tabela: "supervisores AS user", campo1: "log.id_user", campo2: "user.id" },
        { tabela: "modulos", campo1: "log.modulo", campo2: "modulos.id" }
      ];

      crud.select = [
        'log.*',
        'adm.fullname as supervisor',
        'user.fullname as usuario',
        'modulos.descricao as modulo_nome'
      ];

      crud.getAll(request, reply);
    },
    config: {
      description: 'Pesquisa os logs dos usuários. Informe os filtros para obter uma consulta personalizada ou chame o método sem dados para obter a primeira página de resultados.',
      tags: ['api', 'Usuário'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
              campo: Joi.string().description('Nome da coluna'),
              ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          id_supervisor: Joi.array().items(Joi.number().integer()).description('Filtra por ids do usuários que práticou a ação'),
          data_ini: Joi.date().iso().example('2018-01-01 08:00:00').description('Data inicial'),
          data_fim: Joi.date().iso().example('2018-01-01 23:59:59').description('Data final'),
          acao: Joi.array().items(Joi.string().valid('V', 'C', 'A', 'R', 'U')).description('Filtra por ação: V = Visualizou, C = Criou, A = Adicionou, R = Removeu, U = Atualizou'),
          modulo: Joi.array().items(Joi.number().integer()).description('Filtra por ids de módulos'),
          id_user: Joi.array().items(Joi.number().integer()).description('Filtra por ids de usuários afetados pela ação'),
          gravacao: Joi.array().items(Joi.string().valid('A', 'R')).description('Filtra por ações de acesso a gravações: A = Acesso concedido, R = Acesso removido'),
        }
      }
    }
  },
  {
    method: 'POST',
    path: '/api/v3/usuario/login',
    handler: handlers.login,
    config: {
      description: 'Efetua login como usuário. Um usuário é distinto de um agente. Pois sua função é interagir com o Xcontact gerindo ou supervisionando operações.',
      tags: ['api', 'Usuário'],
      validate: {
        payload: Joi.object({
          nome: Joi.string().required().example('admin').description('Nome para login'),
          senha: Joi.string().example('123456').description('Senha para login'),
        })
      }
    }
  }
];
