const Joi = require("joi");
const handlers = require("./handlers");

module.exports = [
  {
    method: "GET",
    path: "/api/v4/agente",
    handler: handlers.getAll,
    config: {
      description: "Obtem os agentes",
      tags: ["api", "Agente"],
      auth: {
        strategy: "bearer",
        scope: ["admin", "app.agentes", "app.agenteeventos", "app.webchat"],
      },
    },
  },
  {
    method: "GET",
    path: "/api/v4/agente/{AgenteID}",
    handler: handlers.getOne,
    config: {
      description: "Obtem os dados do agente pelo ID",
      tags: ["api", "Agente"],
      auth: {
        strategy: "bearer",
        scope: ["app.agentes", "admin"],
      },
      validate: {
        params: {
          AgenteID: Joi.number().integer().required(),
        },
      },
    },
  },
  {
    method: "POST",
    path: "/api/v4/agente",
    handler: handlers.create,
    config: {
      description: "Cria um agente",
      tags: ["api", "Agente"],
      auth: {
        strategy: "bearer",
        scope: ["app.agentes", "admin"],
      },
      validate: {
        payload: Joi.object({
          nome: Joi.string().description("Nome para login"),
          senha: Joi.string().description("Senha para login"),
          fullname: Joi.string().description("Nome do agente"),
          ramal: Joi.string().allow("", null).description("Ramal"),
          max_chat_atende: Joi.number()
            .integer()
            .min(0)
            .description("Número máximo de atendimento simultaneos no ommni"),
          assinatura: Joi.string()
            .allow("", null)
            .description("HTML para assinatura de email"),
          extra1: Joi.string()
            .allow(null, "")
            .optional()
            .description("Campo extra associado ao agente"),
          extra2: Joi.string()
            .allow(null, "")
            .optional()
            .description("Campo extra associado ao agente"),
          modo: Joi.string()
            .valid("F", "T")
            .description("F = Multifila ou T = Fila única"),
          tempo_maximo_sessao: Joi.number()
            .allow(null)
            .optional()
            .description("Tempo máximo que o agente pode ficar logado"),
          tempo_auto_pausa: Joi.number()
            .allow(null)
            .optional()
            .description("Tempo em segundos para autopausa"),
          filas: Joi.array()
            .items(Joi.string())
            .description("Filas que o agente pode atender"),
          grupo_filas: Joi.array()
            .items(Joi.string())
            .description("Filas que o agente pode atender"),
          usa_grupo_fila: Joi.boolean()
            .allow(null, "")
            .optional()
            .description("Indica se o agente usa grupo de filas"),
          atalho_url: Joi.string()
            .allow(null, "")
            .optional()
            .description(
              "Código da tecla de atalho para abrir url de integração"
            ),
          abrir_url: Joi.boolean()
            .allow(null, "")
            .optional()
            .description("Indica se é para abrir a URL de integração"),
          edita_config: Joi.boolean()
            .allow(null, "")
            .optional()
            .description("Indica se o agente pode abrir a config do client"),
          editaCliente: Joi.boolean()
            .allow(null, "")
            .optional()
            .description("Indica se o agente pode editar cliente"),
          editaClienteGrupo: Joi.boolean()
            .allow(null, "")
            .optional()
            .description("Indica se o agente pode editar o grupo do cliente"),
          visualiza_todos_clientes: Joi.boolean()
            .allow(null, "")
            .optional()
            .description(
              "Indica se o agente pode visializar todos os clientes no client"
            ),
          default_instance: Joi.string()
            .allow(null, "")
            .optional()
            .description("Instancia padrão do usuário"),
          pode_selecionar_instancias: Joi.boolean()
            .allow(null, "")
            .optional()
            .description(
              "Indica se o agente pode mudar a instancia do WhatsApp no client na hora de enviar mensagem."
            ),
          autenticacao_fonte: Joi.number().integer().optional().allow(null),
          timerClassificacao: Joi.number().integer().optional().allow(null),
          auto_atendimento: Joi.boolean()
            .allow(null, "")
            .optional()
            .description("Indica se tem auto atendimento"),
          auto_atendimento_omni: Joi.boolean()
            .allow(null, "")
            .optional()
            .description("Indica se tem auto atendimento"),
          assinatura_na_mensagem: Joi.boolean()
            .allow(null, "")
            .optional()
            .description("Indica se a assinatura esta ativa"),
          classificacoes_por_fila: Joi.boolean()
            .allow(null, "")
            .optional()
            .description(
              "Indica se vai mostrar apenas as classificações da fila do atendimento"
            ),
        }),
      },
    },
  },
  {
    method: "PUT",
    path: "/api/v4/agente/{AgenteID}",
    handler: handlers.update,
    config: {
      description: "Atualiza um agente",
      tags: ["api", "Agente"],
      auth: {
        strategy: "bearer",
        scope: ["app.agentes", "admin"],
      },
      validate: {
        params: {
          AgenteID: Joi.number().integer().required(),
        },
        payload: Joi.object({
          nome: Joi.string().description("Nome para login"),
          senha: Joi.string()
            .allow("", null)
            .description(
              "Senha para login, quando não informada permace a mesma"
            ),
          fullname: Joi.string().description("Nome do agente"),
          ramal: Joi.string().allow("", null).description("Ramal"),
          max_chat_atende: Joi.number()
            .integer()
            .min(0)
            .description("Número máximo de atendimento simultaneos no ommni"),
          assinatura: Joi.string()
            .allow("", null)
            .description("HTML para assinatura de email"),
          extra1: Joi.string()
            .allow(null, "")
            .optional()
            .description("Campo extra associado ao agente"),
          extra2: Joi.string()
            .allow(null, "")
            .optional()
            .description("Campo extra associado ao agente"),
          modo: Joi.string()
            .valid("F", "T")
            .description("F = Multifila ou T = Fila única"),
          tempo_maximo_sessao: Joi.number()
            .allow(null)
            .optional()
            .description("Tempo máximo que o agente pode ficar logado"),
          tempo_auto_pausa: Joi.number()
            .allow(null)
            .optional()
            .description("Tempo em segundos para autopausa"),
          filas: Joi.array()
            .items(Joi.string())
            .description("Filas que o agente pode atender"),
          grupo_filas: Joi.array()
            .items(Joi.string())
            .description("Filas que o agente pode atender"),
          usa_grupo_fila: Joi.boolean()
            .allow(null, "")
            .optional()
            .description("Indica se o agente usa grupo de filas"),
          atalho_url: Joi.string()
            .allow(null, "")
            .optional()
            .description(
              "Código da tecla de atalho para abrir url de integração"
            ),
          abrir_url: Joi.boolean()
            .allow(null, "")
            .optional()
            .description("Indica se é para abrir a URL de integração"),
          edita_config: Joi.boolean()
            .allow(null, "")
            .optional()
            .description("Indica se o agente pode abrir a config do client"),
          editaCliente: Joi.boolean()
            .allow(null, "")
            .optional()
            .description("Indica se o agente pode editar cliente"),
          editaClienteGrupo: Joi.boolean()
            .allow(null, "")
            .optional()
            .description("Indica se o agente pode editar o grupo do cliente"),
          visualiza_todos_clientes: Joi.boolean()
            .allow(null, "")
            .optional()
            .description(
              "Indica se o agente pode visializar todos os clientes no client"
            ),
          default_instance: Joi.string()
            .allow(null, "")
            .optional()
            .description("Instancia padrão do usuário"),
          pode_selecionar_instancias: Joi.boolean()
            .allow(null, "")
            .optional()
            .description(
              "Indica se o agente pode mudar a instancia do WhatsApp no client na hora de enviar mensagem."
            ),
          autenticacao_fonte: Joi.number().integer().optional().allow(null),
          timerClassificacao: Joi.number().integer().optional().allow(null),
          auto_atendimento: Joi.boolean()
            .allow(null, "")
            .optional()
            .description("Indica se tem auto atendimento"),
          assinatura_na_mensagem: Joi.boolean()
            .allow(null, "")
            .optional()
            .description("Indica se a assinatura esta ativa"),
          auto_atendimento_omni: Joi.boolean()
            .allow(null, "")
            .optional()
            .description("Indica se tem auto atendimento"),
          classificacoes_por_fila: Joi.boolean()
            .allow(null, "")
            .optional()
            .description(
              "Indica se vai mostrar apenas as classificações da fila do atendimento"
            ),
          token: Joi.string().allow(null, "").optional().description("token"),
          token_create: Joi.string()
            .allow(null, "")
            .optional()
            .description("token_create"),
        }),
      },
    },
  },
  {
    method: "DELETE",
    path: "/api/v4/agente/{AgenteID}",
    handler: handlers.delete,
    config: {
      description: "exclui um agente pelo ID",
      tags: ["api", "Agente"],
      auth: {
        strategy: "bearer",
        scope: ["app.agentes", "admin"],
      },
      validate: {
        params: {
          AgenteID: Joi.number().integer().required(),
        },
      },
    },
  },
  {
    method: "PUT",
    path: "/api/v4/agente/{AgenteID}/logoffall",
    handler: handlers.logoffall,
    config: {
      description: "Desloga o agentes de todas as suas filas",
      tags: ["api", "Agente"],
      auth: {
        strategy: "bearer",
        scope: ["app.agentes", "admin"],
      },
      validate: {
        params: {
          AgenteID: Joi.number().integer().required(),
        },
      },
    },
  },
  {
    method: "POST",
    path: "/api/v4/agente/{agenteID}/profileimage",
    handler: handlers.uploadImage,
    config: {
      description: "Envia uma imgagem para o profile do agente",
      tags: ["api", "Agente"],
      auth: {
        strategy: "bearer",
        scope: ["app.agentes", "admin"],
      },
      payload: {
        output: "stream",
        allow: "multipart/form-data",
        maxBytes: 50 * 1024 * 1024, // 50mb
        parse: true,
      },
      validate: {
        params: {
          agenteID: Joi.number().integer().required(),
        },
        payload: {
          file: Joi.object().unknown().description("Arquivo"),
        },
      },
    },
  },
  {
    method: "GET",
    path: "/api/v4/agente/{AgenteFullName}/profileimage",
    handler: handlers.getProfileImage,
    config: {
      description: "Obtem a imagem de profile do agente",
      tags: ["api", "Agente"],
      cache: {
        expiresIn: 10 * 60 * 1000, // 10 minutos em milissegundos
        privacy: "private",
      },
      validate: {
        params: {
          AgenteFullName: Joi.string().required().description("Nome do agente"),
        },
      },
    },
  },
  {
    method: "GET",
    path: "/api/v4/agente/{AgenteFullName}/clientes_atendidos",
    handler: handlers.getClientesAtendidos,
    config: {
      description: "Obtem os clientes atendidos pelo agente",
      tags: ["api", "Agente"],
      auth: {
        strategy: "bearer",
        scope: ["app.agentes", "admin"],
      },
      validate: {
        params: {
          AgenteFullName: Joi.string().required(),
        },
        query: {
          apenasabertos: Joi.boolean()
            .allow(null)
            .allow("")
            .optional()
            .description(
              "Traz apenas os clientes com atendimentos abertos (Padrão falso)"
            ),
        },
      },
    },
  },
  {
    method: "GET",
    path: "/api/v4/agente/{AgenteID}/{ClienteID}/mensagens",
    handler: handlers.getClienteAtendimentos,
    config: {
      description:
        "Obtem as mensagens trocadas com o cliente em todas as filas que o agente faz parte",
      tags: ["api", "Agente"],
      auth: {
        strategy: "bearer",
        scope: ["app.agentes", "admin"],
      },
      validate: {
        params: {
          AgenteID: Joi.number()
            .integer()
            .required()
            .description("Id do agente"),
          ClienteID: Joi.number()
            .integer()
            .required()
            .description("Id do cliente"),
        },
        query: {
          rowsPerPage: Joi.number()
            .integer()
            .default(40)
            .optional()
            .allow(null)
            .description(
              "Registros por página, rowsPerPage=0 traz todos os registros."
            ),
          page: Joi.number()
            .integer()
            .default(1)
            .optional()
            .allow(null)
            .description("Página"),
          dataHora: Joi.date()
            .iso()
            .allow(null)
            .allow("")
            .description(
              "Mensagens a partir da data informada (padrão TODAS, sem limite)"
            ),
        },
      },
    },
  },

  {
    method: 'GET',
    path: '/api/v4/agente/{AgenteID}/{ClienteID}/emails',
    handler: handlers.getClienteEmails,
    config: {
      description: 'Obtem os emails com o cliente em todas as filas que o agente faz parte',
      tags: ['api', 'Agente'],
      auth: {
        strategy: 'bearer',
        scope: ['app.agentes', 'admin']
      },
      validate: {
        params: {
          AgenteID: Joi.number().integer().required().description('Id do agente'),
          ClienteID: Joi.number().integer().required().description('Id do cliente'),
        },
        query: {
          rowsPerPage: Joi.number().integer().default(40).optional().allow(null).description('Registros por página, rowsPerPage=0 traz todos os registros.'),
          page: Joi.number().integer().default(1).optional().allow(null).description('Página'),
          dataHora: Joi.date().iso().allow(null).allow('').description('Mensagens a partir da data informada (padrão TODAS, sem limite)')
        }
      }
    }
  },

  {
    method: "GET",
    path: "/api/v4/agente/mensagensByProtocolo/{AgenteID}/{idQueueChat}",
    handler: handlers.getClienteAtendimentosByProtocolo,
    config: {
      description:
        "Obtem as mensagens trocadas com o cliente em todas as filas que o agente faz parte",
      tags: ["api", "Agente"],
      auth: {
        strategy: "bearer",
        scope: ["app.agentes", "admin"],
      },
      validate: {
        params: {
          AgenteID: Joi.number()
            .integer()
            .required()
            .description("Id do agente"),
          idQueueChat: Joi.number()
            .integer()
            .required()
            .description("Id do protocolo"),
        },
        query: {
          dataHora: Joi.date()
            .iso()
            .allow(null)
            .allow("")
            .description(
              "Mensagens a partir da data informada (padrão TODAS, sem limite)"
            ),
        },
      },
    },
  },
  {
    method: "GET",
    path: "/api/v4/agente/callsByCliente/{AgenteID}/{idCliente}",
    handler: handlers.getCallsByCliente,
    config: {
      description:
        "Obtem as calls do cliente em todas as filas que o agente faz parte",
      tags: ["api", "Agente"],
      auth: {
        strategy: "bearer",
        scope: ["app.agentes", "admin"],
      },
      validate: {
        params: {
          AgenteID: Joi.number()
            .integer()
            .required()
            .description("Id do agente"),
          idCliente: Joi.number()
            .integer()
            .required()
            .description("Id do cliente"),
        },
        query: {
          dataHora: Joi.date()
            .iso()
            .allow(null)
            .allow("")
            .description(
              "Mensagens a partir da data informada (padrão TODAS, sem limite)"
            ),
        },
      },
    },
  },
  {
    method: "GET",
    path: "/api/v4/agente/callsByAgente/{AgenteID}/{agente}",
    handler: handlers.getCallsByAgente,
    config: {
      description:
        "Obtem as calls do cliente em todas as filas que o agente faz parte",
      tags: ["api", "Agente"],
      auth: {
        strategy: "bearer",
        scope: ["app.agentes", "admin"],
      },
      validate: {
        params: {
          AgenteID: Joi.number()
            .integer()
            .required()
            .description("Id do agente"),
          agente: Joi.string().description("Nome do agente"),
        },
        query: {
          dataHora: Joi.date()
            .iso()
            .allow(null)
            .allow("")
            .description(
              "Mensagens a partir da data informada (padrão TODAS, sem limite)"
            ),
        },
      },
    },
  },
];
