'use strict'

const Joi = require('joi');
const handlers = require('./handlers');

module.exports = [
    {
        method: 'GET',
        path: '/api/v4/campanhas/temposMovimentos',
        handler: handlers.getTemposEMovimentos,
        config: {
            description: 'Obtem os dados do relatório tempos e movimentos',
            tags: ['api', 'v4', 'Campanhas'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                query: {
                    data_ini: Joi.date().iso().example('2018-01-01 08:00:00').description('Data inicial'),
                    data_fim: Joi.date().iso().example('2018-01-01 23:59:59').description('Data final')
                }
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/campanhas/registrosTags',
        handler: handlers.getCampanhasRegistrosTags,
        config: {
            description: 'Obtem os registros das campanhas (log)',
            tags: ['api', 'v4', 'Campanhas'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                query: {
                    max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
                    page: Joi.number().integer().default(1).description('Página'),
                    order: Joi.array().items(
                        Joi.object().keys({
                            campo: Joi.string().description('Nome da coluna'),
                            ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
                        })
                    ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
                    data_ini: Joi.date().iso().example('2018-01-01 08:00:00').description('Data inicial'),
                    data_fim: Joi.date().iso().example('2018-01-01 23:59:59').description('Data final'),
                    campanhas: Joi.array().items(Joi.number().integer()).example([1, 2, 3]).description('Array de IDs de Campanhas'),
                    status: Joi.array().items(Joi.number().integer()).example([1, 2, 3]).description('Array de IDs de status'),
                    numero: Joi.string().description('Filtra pelo número'),
                    numero_operador: Joi.number().integer().valid(1, 2, 3, 4, 5).description('Operador para filtro do número, Quando não informado é aplicado like %%, 1=Igual,2=Iniciando,3=Finalizando,4=Não contendo e 5=Diferente')
                }
            }
        }
    },

    {
        method: 'DELETE',
        path: '/api/v4/campanha/{CampanhaID}/contatos',
        handler: handlers.remove_todos_contatos,
        config: {
            description: 'Remove TODOS os contatos de uma ou mais campanhas',
            tags: ['api', 'v4', 'Campanhas'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    CampanhaID: Joi.array().items(Joi.number().integer()).required().description('ID da campanha')
                }
            }
        }
    },

    {
        method: 'DELETE',
        path: '/api/v4/campanha/{CampanhaID}',
        handler: handlers.delete,
        config: {
            description: 'Apaga uma ou mais campanhas',
            tags: ['api', 'v4', 'Campanhas'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    CampanhaID: Joi.array().items(Joi.number().integer()).required().description('ID da campanha')
                }
            },
        }
    },

    {
        method: 'PUT',
        path: '/api/v4/campanha/{Telefone}/sucesso',
        handler: handlers.sucesso_true,
        config: {
            description: 'Marca como sucesso a tentativa de contato de um numero em todas as campanhas',
            tags: ['api', 'v4', 'Campanhas'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    Telefone: Joi.string().required().description('Numero do cliente')
                }
            },
        }
    },

    {
        method: 'PUT',
        path: '/api/v4/campanha/status',
        handler: handlers.atualiza_status,
        config: {
            description: 'Atualiza o status de uma ou mais campanhas',
            tags: ['api', 'v4', 'Campanhas'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                payload: Joi.object({
                    CampanhaID: Joi.array().items(Joi.number().integer()).required().description('ID da campanha'),
                    status: Joi.string().valid('A', 'P', 'F').description('Status da campanha, A=Ativa,P=Pausa e F=Finalizada'),
                    data_status: Joi.date().iso().example('2018-01-01 15:00:00').description('Data do ultimo status'),
                })
            }
        }
    },

    {
        method: 'POST',
        path: '/api/v4/campanha/{CampanhaID}/contatos_import',
        handler: handlers.uploadContatosJSON,
        config: {
            description: 'Importa uma lista de contatos no formato JSON para uma campanha e cadastram os clientes, os nomes dos campos devem estar de acordo com o layout associado a campanha.',
            tags: ['api', 'v4', 'Campanhas'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    CampanhaID: Joi.number().integer().required()
                },
                payload: Joi.array().items(Joi.object().unknown()).example('[{"Nome":"João da sila","CPF":"123456","Fixo":"5506","Celular":"999999"}]').description('Array de contatos')
            }
        }
    },

    {
        method: 'POST',
        path: '/api/v4/campanha/{CampanhaID}/contatos_upload',
        handler: handlers.uploadContatos,
        config: {
            description: 'Envia um arquivo de contatos para ser importado da campanha.',
            tags: ['api', 'v4', 'Campanhas'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            payload: {
                output: 'data',
                allow: 'multipart/form-data',
                maxBytes: 104857600, // 100mb
                parse: true,
            },
            validate: {
                query: {
                    prioridade: Joi.boolean().default(false).description("Define se os contatos a serem importados devem entrar como prioridade sobre os já inseridos").optional()
                },
                params: {
                    CampanhaID: Joi.number().integer().required()
                },
                payload: {
                    file: Joi.any().description('Arquivo')
                }
            }
        }
    },
    {
        method: 'POST',
        path: '/api/v4/campanhas/importLotes',
        handler: handlers.importLotes,
        config: {
            description: 'Importa um arquivo CSV de formato específico para gerar múltiplas campanhas agrupadas por agente e data.',
            tags: ['api', 'Campanhas', 'v4'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            plugins: {
                'hapi-swagger': {
                    payloadType: 'form'
                }
            },
            payload: {
                output: 'stream',
                allow: 'multipart/form-data',
                maxBytes: 104857600, // 100mb
                parse: true
            },
            validate: {
                query: {
                    sampleCampainID: Joi.number().integer().required().description('ID da Campanha de Exemplo para as campanhas a serem criadas que terão os seus dados clonados da campanha exemplo.')
                },
                payload: {
                    file: Joi.any().meta({ swaggerType: 'file' }).required()
                }
            }
        }
    },
    {
        method: 'POST',
        path: '/api/v4/campanhas/importLotesJSON',
        handler: handlers.importLotesJSON,
        config: {
            description: 'Importa um JSON de formato específico para gerar múltiplas campanhas agrupadas por agente e data.',
            tags: ['api', 'Campanhas', 'v4'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            plugins: {
                'hapi-swagger': {
                    payloadType: 'form'
                }
            },
            validate: {
                query: {
                    sampleCampainID: Joi.number().integer().required().description('ID da Campanha de Exemplo para as campanhas a serem criadas que terão os seus dados clonados da campanha exemplo.')
                },
                payload: {
                    dados: Joi.array().items(Joi.object()).required()
                }
            }
        }
    },
]

const a =
    [
        {
            "Diretoria": "CD 4",
            "Filial": "DMG",
            "Gerente_Comercial": "Luiz Henrique Tavares De Souza Ruiz (3856006)",
            "Coordenador": "Ludimila Amorim Santos (3724266)",
            "Cod_vendedor_ERP": "3371238",
            "Vendedor": "andersonhipolito-dsp",
            "Nome_Vendedor": "ANDERSON HIPOLITO_DSP",
            "Codigo": "2474009",
            "Nome": "BELA VISTA ATACADO E VAREJO DE HORTIFRUTIGRANJEIRO ",
            "Endereço": "RUA ALVARES MACIEL  1164",
            "Cidade": "ITUIUTABA-INDEPENDENCIA",
            "Cod_Divisao_Comercial": "22",
            "Divisao_Comercial": "VENDA DISTRIB. DIRETA - VDD",
            "Tabela_Preco": "401",
            "Status": "CLIENTE OK",
            "Dia_para_discar": "SEGUNDA-FEIRA",
            "Data": "01/11/2023",
            "Comprador": "WINICUIUS",
            "TelefonePrincipal": "34996323103",
            "Telefone1": "23452345324",
            "Media_6_Meses": "260",
            "Volume_Atual": "0",
            "Lacuna_Atual": "-286",
            "Cliente_Positivado": "Não",
            "Flag_TLV": "Sim",
            "Data_Ultima_Compra": "29/09/2023",
            "Dias_Sem_Comprar": "56",
            "Estavel": "Não",
            "Flutuante": "Sim",
            "Friboi_Online": "Sim",
            "Tem_Cumpo_B2B": "Não",
            "Prazo_Pagamento": "21 Dias - 42",
            "Saldo_Credito": "10.000",
            "Grade_Entrega": "a",
            "FILA": "Whatsapp_andersonhipolito-dsp",
            "FILAVOZ": ""
        }
    ]