"use strict";
const Joi = require("joi");
const handlers = require("./handlers");

module.exports = [
  {
    method: "GET",
    path: "/api/v4/atendimentos",
    handler: handlers.getAtendimentos,
    config: {
      description: "Obtem os chats",
      tags: ["api", "Chat"],
      auth: {
        strategy: "bearer",
        scope: "user", // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          filas: Joi.array()
            .allow(null)
            .items(Joi.string())
            .description("Filtra por filas"),
          agentes: Joi.array()
            .allow(null)
            .items(Joi.string())
            .description("Filtra por agentes"),
          protocolo: Joi.array()
            .allow(null)
            .items(Joi.number().integer())
            .description("Filtra por protocolos"),
          cliente: Joi.array()
            .allow(null)
            .items(Joi.number().integer())
            .description("Filtra por clientes"),
          data_ini: Joi.date()
            .raw()
            .optional()
            .allow(null)
            .description("Data inicial"),
          data_fim: Joi.date()
            .raw()
            .optional()
            .allow(null)
            .description("Data Final"),
          hora_ini: Joi.string()
            .optional()
            .allow(null)
            .description("Hora inicial"),
          hora_fim: Joi.string()
            .optional()
            .allow(null)
            .description("Hora final"),
          status: Joi.array()
            .allow(null)
            .items(Joi.string())
            .description("STATUS"),
        },
      },
    },
  },
  {
    method: "GET",
    path: "/api/v4/chats",
    handler: handlers.getChats,
    config: {
      description: "Obtem os chats",
      tags: ["api", "Chat"],
      auth: {
        strategy: "bearer",
        scope: "user", // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          filas: Joi.array()
            .allow(null)
            .items(Joi.string())
            .description("Filtra por filas"),
          agentes: Joi.array()
            .allow(null)
            .items(Joi.string())
            .description("Filtra por agentes"),
          protocolo: Joi.array()
            .allow(null)
            .items(Joi.number().integer())
            .description("Filtra por protocolos"),
          cliente: Joi.array()
            .allow(null)
            .items(Joi.number().integer())
            .description("Filtra por clientes"),
          data_ini: Joi.date()
            .raw()
            .optional()
            .allow(null)
            .description("Data inicial"),
          data_fim: Joi.date()
            .raw()
            .optional()
            .allow(null)
            .description("Data Final"),
          hora_ini: Joi.string()
            .optional()
            .allow(null)
            .description("Hora inicial"),
          hora_fim: Joi.string()
            .optional()
            .allow(null)
            .description("Hora final"),
          status: Joi.array()
            .allow(null)
            .items(Joi.string())
            .description("STATUS"),
        },
      },
    },
  },
  {
    method: "GET",
    path: "/api/v4/chats/tags",
    handler: handlers.getAllTags,
    config: {
      description: "Retorna todas as tags colocadas",
      tags: ["api", "v4", "Chat"],
      auth: {
        strategy: "bearer",
        scope: "user", // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          data_ini: Joi.date()
            .raw()
            .optional()
            .allow(null)
            .description("Data inicial"),
          data_fim: Joi.date()
            .raw()
            .optional()
            .allow(null)
            .description("Data Final"),
          hora_ini: Joi.string()
            .optional()
            .allow(null)
            .description("Hora inicial"),
          hora_fim: Joi.string()
            .optional()
            .allow(null)
            .description("Hora final"),
          filas: Joi.array()
            .allow(null)
            .items(Joi.string())
            .description("Filtra por filas"),
          agentes: Joi.array().allow(null).items(Joi.string()).description('Filtra por agentes'),
          tags_id: Joi.array().items(Joi.number()).example([1, 2]).description('Filtra por um ou mais id de tags'),
          order: Joi.array().items(
            Joi.object().keys({
              campo: Joi.string().description('Nome da coluna'),
              ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
        },
      },
    },
  },
  {
    method: "GET",
    path: "/api/v4/chatsTags",
    handler: handlers.getChatsTags,
    config: {
      description: "Obtem os chats",
      tags: ["api", "Chat"],
      auth: {
        strategy: "bearer",
        scope: "user", // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(0).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
              campo: Joi.string().description('Nome da coluna'),
              ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          filas: Joi.array().allow(null).items(Joi.string()).description('Filtra por filas'),
          agentes: Joi.array().allow(null).items(Joi.string()).description('Filtra por agentes'),
          protocolo: Joi.array().allow(null).items(Joi.number().integer()).description('Filtra por protocolos'),
          cliente: Joi.array().allow(null).items(Joi.number().integer()).description('Filtra por clientes'),
          data_ini: Joi.date().raw().optional().allow(null).description('Data inicial'),
          data_fim: Joi.date().raw().optional().allow(null).description('Data Final'),
          hora_ini: Joi.string().optional().allow(null).description('Hora inicial'),
          hora_fim: Joi.string().optional().allow(null).description('Hora final'),
          status: Joi.array().allow(null).items(Joi.string()).description('STATUS'),
          tags_id: Joi.array().items(Joi.number()).example([1, 2]).description('Filtra por um ou mais id de tags'),
        }
      }
    }
  },
  {
    method: "POST",
    path: "/api/v4/chatsTagsIds",
    handler: handlers.getChatsTagsIds,
    config: {
      description: "Obtem os chats",
      tags: ["api", "v4", "Filas"],
      auth: {
        strategy: "bearer",
        scope: ["app.cdrfilas", "admin"],
      },
      validate: {
        payload: {
          protocolos: Joi.array().items(Joi.number()).example([451659, 76569]),
        },
      },
    },
  },
  {
    method: "GET",
    path: "/api/v4/getMessagesByCliente",
    handler: handlers.getMessagesByCliente,
    config: {
      description: "Obtem os chats",
      tags: ["api", "Chat"],
      auth: {
        strategy: "bearer",
        scope: "user", // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          idCliente: Joi.number().integer().description("id do cliente"),
          max: Joi.number()
            .max(100)
            .default(15)
            .description("Máximo de registros, max=-1 traz todos os registros.")
            .integer(),
          page: Joi.number().integer().default(1).description("Página"),
          agente: Joi.string().allow(null),
        },
      },
    },
  },
  {
    method: "PUT",
    path: "/api/v4/chats/{protocolo}/{documento}/updateDocumento",
    handler: handlers.updateDocumento,
    config: {
      description: "Atualiza o documento do atendimento",
      tags: ["api", "Chat"],
      auth: {
        strategy: "bearer",
        scope: ["user"],
      },
      validate: {
        params: {
          protocolo: Joi.string().required(),
          documento: Joi.string().required(),
        },
      },
    },
  },
  {
    method: "PUT",
    path: "/api/v4/chats/{agente}/{idCliente}/MensagemNaoLida",
    handler: handlers.marcaNaoLida,
    config: {
      description: "Marca mensagens como não lidas",
      tags: ["api", "Chat"],
      auth: {
        strategy: "bearer",
        scope: ["user"],
      },
      validate: {
        params: {
          idCliente: Joi.string().required(),
          agente: Joi.string().required(),
        },
      },
    },
  },
  {
    method: "GET",
    path: "/api/v4/chats/encerrarAllAtendimentosAguardando",
    handler: handlers.encerrarAllAtendimentosAguardando,
    config: {
      description: "Encerra os atendimentos em aguardando",
      tags: ["api", "Chat"],
      auth: {
        strategy: "bearer",
        scope: ["user"],
      },
    },
  },
  {
    method: "GET",
    path: "/api/v4/chats/anexo",
    handler: handlers.getAnexo,
    config: {
      description: "Faz download de um arquivo anexado no chat",
      tags: ["api", "Chat"],
      auth: {
        strategy: "bearer",
        scope: ["user", "app.webchat"],
      },
      validate: {
        query: {
          access_token: Joi.string().description("Token"),
          caminho: Joi.string().required().description("Caminho do arquivo"),
        },
      },
    },
  },
  {
    method: "GET",
    path: "/api/v4/chats/getMensagemAnexos/{idMensagem}",
    handler: handlers.getMensagemAnexos,
    config: {
      description: "Faz download de um arquivo anexado no chat",
      tags: ["api", "Chat"],
      auth: {
        strategy: "bearer",
        scope: ["user", "app.webchat"],
      },
      validate: {
        params: {
          idMensagem: Joi.number().required(),
        }
      },
    },
  },
  {
    method: "GET",
    path: "/api/v4/chats/alertaAtendimentosAguardando",
    handler: handlers.verificaAtendimentos,
    config: {
      description: "Faz download de um arquivo anexado no chat",
      tags: ["api", "Chat"],
      auth: {
        strategy: "bearer",
        scope: ["user", "app.webchat"],
      },
    },
  },
  {
    method: "POST",
    path: "/api/v4/chats/{agente}/{instancia}/{fila}/{atendimentoID}/anexos",
    handler: handlers.uploadAttatch,
    config: {
      description: "Envia um ou mais anexos",
      tags: ["api", "Chat"],
      auth: {
        strategy: "bearer",
        scope: ["app.webchat", "admin"],
      },
      payload: {
        output: "stream",
        allow: "multipart/form-data",
        maxBytes: 100 * 1024 * 1024, // 100mb
        parse: true,
      },
      validate: {
        params: {
          agente: Joi.string().required(),
          instancia: Joi.string().required(),
          fila: Joi.string().required(),
          atendimentoID: Joi.number().integer().required(),
        },
        payload: {
          files: Joi.any().required().description("Arquivos"),
          legendas: Joi.string().allow(null).description("Legendas"),
          isEmail: Joi.boolean().default(false).optional().description("Se é um email"),
          email: Joi.array().items(Joi.string()).optional().description("Emails"),
          encaminha: Joi.boolean().default(false).optional().description("Encaminhar email"),
          cc:Joi.array().items(Joi.string()).optional().description("CC"),
          cco:Joi.array().items(Joi.string()).optional().description("CCO"),
          template: Joi.object().optional().description("Template"),
        },
      },
    },
  },
];
