const Joi = require('joi');
const handlers = require('./handlers');

module.exports = [
    {
        method: 'GET',
        path: '/api/v4/filas/ligacoes',
        handler: handlers.getLigacoes,
        config: {
            description: 'Obtem as ligações das filas',
            tags: ['api', 'v4', 'Filas'],
            auth: {
                strategy: 'bearer',
                scope: ['app.cdrfilas', 'admin']
            },
            validate: {
                query: {
                    calldate: Joi.array().items(Joi.string()).optional().allow(null).example(['2020-01-01', '2020-01-02', '2020-01-03']).description('Filtra por um array de datas'),
                    status: Joi.array().items(Joi.string().valid('Atendida', 'Abandonada', 'Transbordou', 'Não atendida', 'Ocupado', 'Falha')).description('Filtra por um ou mais status'),
                    filas: Joi.array().items(Joi.string()).example(['FilaA', 'FilaB']).description('Filtra por uma ou mais filas'),
                    data_ini: Joi.string().optional().allow(null).description('Data inicial dos registros'),
                    data_fim: Joi.string().optional().allow(null).description('Data final dos registros'),
                    hora_ini: Joi.string().optional().allow(null).default(null).description('Hora inicial dos registros'),
                    hora_fim: Joi.string().optional().allow(null).default(null).description('Hora final dos registros')
                }
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/filas/ligacoesTags',
        handler: handlers.getLigacoesTags,
        config: {
            description: 'Obtem as ligações das filas',
            tags: ['api', 'v4', 'Filas'],
            auth: {
                strategy: 'bearer',
                scope: ['app.cdrfilas', 'admin']
            },
            validate: {
                query: {
                    max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
                    page: Joi.number().integer().default(1).description('Página'),
                    order: Joi.array().items(
                        Joi.object().keys({
                            campo: Joi.string().description('Nome da coluna'),
                            ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
                        })
                    ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
                    data_ini: Joi.date().description('Filtro por data inicial'),
                    data_fim: Joi.date().description('Filtro por data Final'),
                    numero: Joi.string().description('Filtro por numero (pode ser origem ou destino dependendo do sentido)'),
                    filas: Joi.array().items(Joi.string()).example(['FilaA', 'FilaB']).description('Filtra por uma ou mais filas'),
                    tags_id: Joi.array().items(Joi.number()).example([1, 2]).description('Filtra por um ou mais id de tags'),
                    agentes: Joi.array().items(Joi.string()).example(['AgenteA', 'AgenteB']).description('Filtra por um ou mais agentes'),
                    status: Joi.array().items(Joi.string().valid('Atendida', 'Abandonada', 'Transbordou', 'Não atendida', 'Ocupado', 'Falha')).description('Filtra por um ou mais status'),
                    protocolo: Joi.string().description('Filtro por protocolo'), sentido: Joi.number().integer().valid(1, 2).description('Filtro por sentido, 1 = Entrante, 2 = Sainte'),
                    sentido: Joi.number().integer().valid(1, 2).description('Filtro por sentido, 1 = Entrante, 2 = Sainte'),
                    finalizadapor: Joi.number().integer().valid(1, 2).description('Filtro por quem finalizou a ligação (Valida apenas para entrantes), 1 = Agente, 2 = Cliente'),
                    identclient: Joi.string().description('Filtra por Identificação do cliente, campo que pode ser alimentado por integrações'),
                    identclientc: Joi.string().allow("").description('Condição do filtro de identificação do cliente'),
                    cliente_nome: Joi.string().allow("").allow(null).description('Filtro por nome do cliente'),
                }
            }
        }
    },
    {
        method: 'POST',
        path: '/api/v4/filas/ligacoesTagsIds',
        handler: handlers.getLigacoesTagsIds,
        config: {
            description: 'Obtem as ligações das filas',
            tags: ['api', 'v4', 'Filas'],
            auth: {
                strategy: 'bearer',
                scope: ['app.cdrfilas', 'admin']
            },
            validate: {
                payload: {
                    max: Joi.number().max(100).default(15).integer(),
                    page: Joi.number().integer().default(1),
                    callids: Joi.array().items(Joi.number()).example(['7a8cc770-e9f1-11ed-a696-c50f2fa66f8e', '1691510914.336']),
                }
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/filas/gravacao',
        handler: handlers.getFilaGravacao,
        config: {
            description: 'Ouviu a gravação de filas',
            tags: ['api', 'Gravação'],
            auth: {
                strategy: 'bearer',
                scope: ['app.cdrfilas', 'admin', 'user']
            },
            validate: {
                query: {
                    access_token: Joi.string().description('Token'),
                    caminho: Joi.string().required().description('Caminho do arquivo'),
                }
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/filas/agente_web/gravacao',
        handler: handlers.getGravacaoAgenteWeb,
        config: {
            description: 'Ouviu a gravação de filas',
            tags: ['api', 'Gravação'],
            validate: {
                query: {
                    access_token: Joi.string().description('Token'),
                    caminho: Joi.string().required().description('Caminho do arquivo'),
                }
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/filas/downloadgravacao',
        handler: handlers.getFilaGravacao,
        config: {
            description: 'Fez download da gravação de filas',
            tags: ['api', 'Gravação'],
            auth: {
                strategy: 'bearer',
                scope: ['app.cdrfilas']
            },
            validate: {
                query: {
                    access_token: Joi.string().description('Token'),
                    caminho: Joi.string().required().description('Caminho do arquivo'),
                }
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/gruposFilas',
        handler: handlers.getGruposFilas,
        config: {
            description: 'Busca os grupos de fila',
            tags: ['api', 'v4', 'Filas'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                query: {
                    max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
                    page: Joi.number().integer().default(1).description('Página'),
                    order: Joi.array().items(
                        Joi.object().keys({
                            campo: Joi.string().description('Nome da coluna'),
                            ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
                        })
                    ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
                    nome: Joi.string().description('Nome do grupo'),
                    nomes: Joi.array().items(Joi.string()).example(['GrupoA', 'GrupoB']).description('Informe os grupos que deseja buscar'),
                    filas: Joi.array().items(Joi.string()).example(['FilaA', 'FilaB']).description('Informe as filas que o grupo pode ter.'),
                }
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/grupoFilas/{idGrupo}',
        handler: handlers.getGrupoFilas,
        config: {
            description: 'Obtem os dados da fila',
            tags: ['api', 'v4', 'Filas'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    idGrupo: Joi.number().required().description('ID do grupo')
                }
            }
        }
    },
    {
        method: 'POST',
        path: '/api/v4/grupoFilas',
        handler: handlers.createGrupoFilas,
        config: {
            description: 'Cria um grupo de filas',
            tags: ['api', 'v4', 'Filas'],
            auth: {
                strategy: 'bearer',
                scope: ['app.filas']
            },
            validate: {
                payload: {
                    nome: Joi.string().description('Nome do grupo'),
                    filas: Joi.array().items(Joi.string()).example(['FilaA', 'FilaB']).required().description('Todas as filas que esse grupo terá'),
                }
            }
        }
    },
    {
        method: 'PUT',
        path: '/api/v4/grupoFilas/{idGrupo}',
        handler: handlers.updateGrupoFilas,
        config: {
            description: 'Edita um grupo de filas',
            tags: ['api', 'v4', 'Filas'],
            auth: {
                strategy: 'bearer',
                scope: ['app.filas']
            },
            validate: {
                params: {
                    idGrupo: Joi.number().required().description('ID do grupo')
                },
                payload: {
                    nome: Joi.string().description('Nome do grupo'),
                    filas: Joi.array().items(Joi.string()).example(['FilaA', 'FilaB']).required().description('Todas as filas que esse grupo terá'),
                }
            }
        }
    },
    {
        method: 'DELETE',
        path: '/api/v4/grupoFilas/{idGrupo}',
        handler: handlers.deleteGrupoFilas,
        config: {
            description: 'Obtem os dados da fila',
            tags: ['api', 'v4', 'Filas'],
            auth: {
                strategy: 'bearer',
                scope: ['app.filas']
            },
            validate: {
                params: {
                    idGrupo: Joi.number().required().description('ID do grupo')
                }
            }
        }
    },
]