const Joi = require('joi');
const handlers = require('./handlers');

module.exports = [
    {
        method: 'POST',
        path: '/api/v4/lista_transmissao_arquivos/{id}',
        handler: handlers.uploadArquivoLista,
        config: {
          description: 'Envia um arquivo para lista de transmissão',
          tags: ['api', 'Lista de Transmissão'],
          auth: {
            strategy: 'bearer',
            scope: ['user']
          },
          plugins: {
            'hapi-swagger': {
              payloadType: 'form'
            }
          },
          payload: {
            output: 'stream',
            allow: 'multipart/form-data',
            maxBytes: 10000000, // 10mb
            parse: true,
          },
          validate: {
            params: {
                id: Joi.number().integer().required()
            },
            payload: {
              file: Joi.any().meta({ swaggerType: 'file' }).required(),
              legenda: Joi.string().optional().allow(null).allow(''),
            }
          }
        }
    },
    {
        method: 'POST',
        path: '/api/v4/lista_transmissao_import/{id}',
        handler: handlers.importNumeros,
        config: {
          description: 'Importa um arquivo CSV de números para lista',
          notes: 'Espera um arquivo CSV com delimitador "," com os seguintes campos em sequência: "Número telefone","Nome"',
          tags: ['api', 'Lista de Transmissão'],
          auth: {
            strategy: 'bearer',
            scope: ['user']
          },
          plugins: {
            'hapi-swagger': {
              payloadType: 'form'
            }
          },
          payload: {
            output: 'stream',
            allow: 'multipart/form-data',
            maxBytes: 104857600, // 100mb
            parse: true,
          },
          validate: {
            params: {
                id: Joi.number().integer().required()
            },
            payload: {
              file: Joi.any().meta({ swaggerType: 'file' }).required()
            }
          }
        }
    },
    {
        method: 'POST',
        path: '/api/v4/lista_transmissao',
        handler: handlers.createLista,
        config: {
            description: 'Cria uma Lista de Transmissão',
            tags: ['api', 'Lista de Transmissão'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                payload: Joi.object({
                    nome: Joi.string().required(),
                    ativa: Joi.boolean().required(),
                    idTemplate: Joi.string().optional().allow(null).description('Caso a lista use algum tamplete de API oficial'),
                    texto: Joi.string().optional().allow(null).allow(''),
                    idAgente: Joi.number().integer().required(),
                    status_atendimento: Joi.string().optional().allow(null).allow(''),
                    instancia: Joi.string().required(),
                    agendaData: Joi.date().iso().example('2018-01-01 15:00:00').description('Data especifica para a lista executar'),
                    agendaHora: Joi.string().optional().allow(null).example('15:00:00'),
                    agendaDias: Joi.array().optional().allow(null).items(Joi.number().integer().min(1).max(31)).description('Dias do mes 1-31'),
                    agendaMesses: Joi.array().optional().allow(null).items(Joi.number().integer().min(1).max(12)).description('Meses do ano 1-12 (1 = Janeiro)'),
                    agendaDiasSemana: Joi.array().optional().allow(null).items(Joi.number().integer().min(0).max(6)).description('Dias da semana 0-6 (0 = Domingo, 6 = Sábado)')
                })
            }
        }
    },
    {
        method: 'PUT',
        path: '/api/v4/lista_transmissao/{id}',
        handler: handlers.updateLista,
        config: {
            description: 'Atualiza uma Lista de Transmissão',
            tags: ['api', 'Lista de Transmissão'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                },
                payload: Joi.object({
                    nome: Joi.string().optional(),
                    ativa: Joi.boolean().optional(),
                    idTemplate: Joi.string().optional().description('Caso a lista use algum tamplete de API oficial'),
                    texto: Joi.string().optional().allow(null).allow(''),
                    idAgente: Joi.number().integer().optional(),
                    status_atendimento: Joi.string().optional().allow(null).allow(''),
                    instancia: Joi.string().optional(),
                    agendaData: Joi.date().optional().iso().example('2018-01-01 15:00:00').description('Data especifica para a lista executar'),
                    agendaHora: Joi.string().optional().example('15:00:00'),
                    agendaDias: Joi.array().optional().allow(null).items(Joi.number().integer().min(1).max(31)).description('Dias do mes 1-31'),
                    agendaMesses: Joi.array().optional().allow(null).items(Joi.number().integer().min(1).max(12)).description('Meses do ano 1-12 (1 = Janeiro)'),
                    agendaDiasSemana: Joi.array().optional().allow(null).items(Joi.number().integer().min(0).max(6)).description('Dias da semana 0-6 (0 = Domingo, 6 = Sábado)')
                })
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/lista_transmissao/{id}',
        handler: handlers.getLista,
        config: {
            description: 'Obtem uma lista de Transmissão',
            tags: ['api', 'Lista de Transmissão'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                }
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/listas_transmissoes',
        handler: handlers.getListas,
        config: {
            description: 'Obtem todas as listas de Transmissões',
            tags: ['api', 'Lista de Transmissão'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/listas_transmissoes_agente/{idAgente}',
        handler: handlers.getListasAgente,
        config: {
            description: 'Obtem todas as listas de Transmissões do agente',
            tags: ['api', 'Lista de Transmissão'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    idAgente: Joi.number().integer().required()
                }
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/lista_transmissao_arquivos/{id}',
        handler: handlers.getListaArquivos,
        config: {
            description: 'Obtem os arquivos de uma lista de Transmissão',
            tags: ['api', 'Lista de Transmissão'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                }
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/lista_transmissao_campos/{id}',
        handler: handlers.getListaCampos,
        config: {
            description: 'Obtem os campos e valores de uma lista de Transmissão',
            tags: ['api', 'Lista de Transmissão'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                }
            }
        }
    },
    {
        method: 'POST',
        path: '/api/v4/lista_transmissao_numero/{id}',
        handler: handlers.createNumero,
        config: {
            description: 'Cria um número na Lista de Transmissão',
            tags: ['api', 'Lista de Transmissão'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                },
                payload: Joi.object({
                    numero: Joi.string().required(),
                    nome:  Joi.string().optional().allow(null).allow('')                    
                })
            }
        }
    },
    {
        method: 'PUT',
        path: '/api/v4/lista_transmissao_numero/{id}',
        handler: handlers.updateNumero,
        config: {
            description: 'Atualiza uma Lista de Transmissão',
            tags: ['api', 'Lista de Transmissão'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                },
                payload: Joi.object({
                    numero: Joi.string().optional(),
                    nome:  Joi.string().optional().allow(null).allow('')    
                })
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/lista_transmissao_numeros/{id}',
        handler: handlers.getListaNumeros,
        config: {
            description: 'Obtem os números de uma lista de Transmissão',
            tags: ['api', 'Lista de Transmissão'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                }
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/lista_transmissao_regras/{id}',
        handler: handlers.getListaRegras,
        config: {
            description: 'Obtem as regras de transbordo de uma lista de Transmissão',
            tags: ['api', 'Lista de Transmissão'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                }
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/regras_transbordo',
        handler: handlers.getRegrasTransbordos,
        config: {
            description: 'Obtem as regras de transbordo',
            tags: ['api', 'Lista de Transmissão'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            }
        }
    },
    {
        method: 'DELETE',
        path: '/api/v4/lista_transmissao_numeros/{idLista}',
        handler: handlers.delAllNumeros,
        config: {
            description: 'Remove TODOS os número de uma lista de Transmissão',
            tags: ['api', 'Lista de Transmissão'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    idLista: Joi.number().integer().required()
                }
            }
        }
    },
    {
        method: 'DELETE',
        path: '/api/v4/lista_transmissao_numero/{id}',
        handler: handlers.delNumero,
        config: {
            description: 'Remove um número de uma lista de Transmissão',
            tags: ['api', 'Lista de Transmissão'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                }
            }
        }
    },
    {
        method: 'DELETE',
        path: '/api/v4/lista_transmissao_arquivos/{idLista}',
        handler: handlers.delAllArquivos,
        config: {
            description: 'Remove TODOS os arquivos de uma lista de Transmissão',
            tags: ['api', 'Lista de Transmissão'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    idLista: Joi.number().integer().required()
                }
            }
        }
    },
    {
        method: 'DELETE',
        path: '/api/v4/lista_transmissao_arquivo/{id}',
        handler: handlers.delArquivo,
        config: {
            description: 'Remove um arquivo de uma lista de Transmissão',
            tags: ['api', 'Lista de Transmissão'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                }
            }
        }
    },
    {
        method: 'DELETE',
        path: '/api/v4/lista_transmissao/{id}',
        handler: handlers.delLista,
        config: {
            description: 'Remove lista de Transmissão',
            tags: ['api', 'Lista de Transmissão'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                }
            }
        }
    },
    {
        method: 'POST',
        path: '/api/v4/lista_transmissao_executar/{id}',
        handler: handlers.executarLista,
        config: {
            description: 'Executa uma Lista de Transmissão',
            tags: ['api', 'Lista de Transmissão'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                }                
            }
        }
    },

]