const Boom = require('boom');
const Supervisores = require('../../../models/Supervisores');
const AutenticacaoFontes = require('../../../models/AutenticacaoFontes');
const Modulos = require('../../../models/Modulos');
const SupervisoresModulos = require('../../../models/SupervisoresModulos');
const SupervisoresFilas = require('../../../models/SupervisoresFilas');
const SupervisoresGruposFila = require('../../../models/SupervisoresGruposFila');
const uuid = require('uuid/v4');
const md5 = require('md5');
const LdapAuth = require('ldapauth-fork');
const twoFactor = require('node-2fa');
const { XCache } = require('../../../util/auth')

function ldapLogin (user, pass, config) {
    return new Promise((resolve, reject) => {
        const ldap = new LdapAuth(config);
        ldap.authenticate(user, pass, function (err, user) {
            if (err) {
                ldap.close();
                reject(err);
            }
            else {
                ldap.close();
                resolve(user);
            }
        });
    });
}

exports.logOff = async (request, reply) => {
    try {
        const user = await Supervisores.fetchById(request.params.id);
        if (user.token) {
            XCache.del(user.token)
        }
        user.session_id = null;
        await user.save();
        reply(user);
    } catch (error) {
        reply(Boom.badData(error));
    }
}

exports.getSupervisor = async (request, reply) => {
    try {
        // Busca usário pelo nome para verificar qual método de autenticação.
        const query = Supervisores.query;
        query.where({ nome: request.payload.nome });

        let sup = await query.require(true).first().fetch();

        if (!request.payload.duplo_fator && sup.duplo_fator_secret) {
            const error = Boom.forbidden("Seu usuário precisa de duplo fator");
            error.output.statusCode = 499;
            error.output.payload.duplo_fator = true;
            error.reformat();
            return reply(error);
        }

        if (request.payload.duplo_fator) {
            if (sup.duplo_fator_secret) {
                //console.log(`2fa secret ${sup.duplo_fator_secret} autenticando `)
                const tf = twoFactor.verifyToken(sup.duplo_fator_secret, request.payload.duplo_fator);
                if (tf) {
                    if (tf.delta != 0) {
                        const error = Boom.forbidden("Código de duplo fator inválido");
                        error.output.statusCode = 498;
                        error.output.payload.duplo_fator = true;
                        error.reformat();
                        return reply(error);
                    }
                }
                else {
                    const error = Boom.forbidden("Código de duplo fator inválido");
                    error.output.statusCode = 498;
                    error.output.payload.duplo_fator = true;
                    error.reformat();
                    return reply(error);
                }
            }
            else {
                const error = Boom.forbidden("Código de duplo fator inválido");
                error.output.statusCode = 498;
                error.output.payload.duplo_fator = true;
                error.reformat();
                return reply(error);
            }
        }


        // Autenticação local
        if (!sup.autenticacao_fonte) {
            if (sup.senha !== md5(request.payload.senha)) {
                reply(Boom.badData('[Erro3] Dados para login inválidos'));
                return;
            }
        }
        else {
            const fonte = await AutenticacaoFontes.query
                .where({ id: sup.autenticacao_fonte })
                .where({ ativo: true })
                .first()
                .require(true)
                .fetch();

            switch (fonte.tipo) {
                case 1: // Xcontact
                    if (sup.senha !== md5(request.payload.senha)) {
                        reply(Boom.badData('[Erro4] Dados para login inválidos'));
                        return;
                    }
                    break;
                case 2: // LDAP/AD
                    // Seta para aceitar certificado auto assinado no caso de TLS
                    fonte.config.tlsOptions = { rejectUnauthorized: false };
                    try {
                        sup.ldap_user = await ldapLogin(request.payload.nome, request.payload.senha, fonte.config);
                    } catch (error) {
                        reply(Boom.badData(error));
                        return;
                    }
                    break;
                default:
                    reply(Boom.badData('[Erro5] Metodo de autenticação não implementado'));
                    return;
            }
        }

        // Remove token antigo do cache
        if (sup.token) {
            XCache.del(sup.token)
        }

        // Gera o token
        sup.token = uuid();
        sup.session_id = uuid();
        sup.token_create = new Date();
        sup.pc_login = (request.id && request.id.split(':').length > 0) ? request.id.split(':')[1] : request.id;
        sup.ip_login = request.headers['x-real-ip'] ? request.headers['x-real-ip'] : request.info.remoteAddress;
        await sup.save();


        const supfilas = await SupervisoresFilas.query
            .where({ id_supervisor: sup.id })
            .require(false)
            .fetch();

        if (supfilas) {
            sup.filas = supfilas.map(f => {
                return f.id_fila;
            })
        }

        const supgruposfilas = await SupervisoresGruposFila.query
            .where({ id_supervisor: sup.id })
            .require(false)
            .fetch();

        if (supgruposfilas) {
            sup.gruposFila = supgruposfilas.map(f => {
                return f.fila_grupo;
            })
        }

        const supModulos = await SupervisoresModulos.query
            .innerJoin(Modulos)
            .require(false)
            .where({ id_supervisor: sup.id })
            .fetch();

        if (supModulos) {

            const somodulos = supModulos.map(modulo => {
                let mod = modulo.modulos[0];
                mod.extra = modulo.extra_usuario;
                return mod;
            })

            const parents = somodulos.filter(pa => pa.parent == null).slice(0)
                .sort((a, b) => {
                    return a.ordem > b.ordem ? 1 : -1
                });;
            const childrens = somodulos.filter(pa => pa.parent != null).slice(0)
                .sort((a, b) => {
                    return a.ordem > b.ordem ? 1 : -1
                });;

            const ordemm = parents.concat(childrens);

            const nest = (items, id = null, link = 'parent') =>
                items
                    .filter(item => item[link] === id)
                    .map(item => ({ ...item, modulos: nest(items, item.id) }));

            sup.modulos = nest(ordemm);
        }
        reply(sup);
    } catch (error) {
        console.log(error)
        reply(Boom.badData('[Erro6] Dados para login inválidos'));
    }
};