'use strict'

const Boom = require('boom');
const db = require('../../../util/db');
const config = require('../../../config.json');
const moment = require('moment-timezone');
moment.tz.setDefault(config.servidor.timezone);

exports.getPesquisaRegistros = async (request, reply) => {
    try {
        const max = request.query.max ? (request.query.max === -1 ? null : parseInt(request.query.max)) : 15
        const page = parseInt(request.query.page) || 1;
        const offset = (page - 1) * max;

        const knex = db.knex('public.pesquisa_satisfacao_registros AS psr')

        // Filtros
        if (request.query.dataini && !request.query.horaini) knex.where('psr.data_registro', '>=', moment(request.query.dataini, 'DD/MM/YYYY').format('YYYY/MM/DD') + ' 00:00:00');
        if (request.query.datafim && !request.query.horafim) knex.where('psr.data_registro', '<=', moment(request.query.datafim, 'DD/MM/YYYY').format('YYYY/MM/DD') + ' 23:59:59');
        if (request.query.dataini && request.query.horaini) knex.where('psr.data_registro', '>=', moment(request.query.dataini, 'DD/MM/YYYY').format('YYYY/MM/DD') + ` ` + request.query.horaini);
        if (request.query.datafim && request.query.horafim) knex.where('psr.data_registro', '<=', moment(request.query.datafim, 'DD/MM/YYYY').format('YYYY/MM/DD') + ` ` + request.query.horafim);
        if (request.query.fila) knex.whereIn('psr.fila', request.query.fila);
        if (request.query.agente) knex.whereIn('psr.agente', request.query.agente);
        if (request.query.pesquisa) knex.whereIn('psr.id_pesquisa', request.query.pesquisa);
        if (request.query.resposta) knex.whereIn('psr.nota', request.query.resposta);
        if (request.query.protocolo) {
            query.where('psr.callid', 'like', `%${request.query.protocolo}%`);
        }
        if (request.query.tags_id && request.query.tags_id.length > 0) {
            const tagsSubquery = db.knex('tags_atendimentos')
                .select('id_atendimento')
                .whereIn('id_tag', request.query.tags_id)
                .groupBy('id_atendimento')
            knex.whereIn('psr.uniqueid', tagsSubquery);
        }
        if (request.query.tipo || (request.query.camposAdicionais && request.query.camposAdicionais.length > 0)) {
            const queryCiente = db.knex('cliente').select('numero')
            if (request.query.tipo) {
                const isBusiness = request.query.tipo === 'CNPJ';
                queryCiente.where('isBusiness', isBusiness)
            }
            if (request.query.camposAdicionais && request.query.camposAdicionais.length > 0) {
                const camposAdicionais = request.query.camposAdicionais
                const idsClientes = db.knex('clienteCamposAdicionais')
                    .select('clienteID')
                    .whereIn('nome', camposAdicionais.map(campo => campo.nome))
                    .whereIn('valor', camposAdicionais.map(campo => campo.valor))
                    .groupBy('clienteID')
                queryCiente.whereIn('id', idsClientes)
            }
            knex.whereIn('psr.src', queryCiente)
        }

        // Adicionar ordenação
        if (request.query.order && request.query.order.length > 0) {
            request.query.order.forEach(orderItem => {
                knex.orderBy(`queue_calls.${orderItem.campo}`, orderItem.ordem);
            });
        } else {
            knex.orderBy('psr.data_registro', 'asc');
        }

        // Contar o total de registros
        const totalResult = await knex.clone().clearSelect().clearOrder().count('* as total').first();

        // Executar consulta
        const result = max ? await knex.clone().limit(max).offset(offset) : await knex.clone()
        const resultProtocolos = result.map(row => row.uniqueid)
        const protOnlyNumb = resultProtocolos.filter(id => /^[0-9]+$/.test(id)).map(p => parseInt(p));

        const clientesIdChats = await db.knex('queueChat').select('idCliente', 'id').whereIn('id', protOnlyNumb)
        const clienteIdCall = await db.knex('queue_calls').select('idCliente', 'callid as id').whereIn('callid', resultProtocolos)
        const clientesAtendimento = [...clientesIdChats, ...clienteIdCall]
        const idClientes = [...new Set([...clientesIdChats.map(c => c.idCliente), ...clienteIdCall.map(c => c.idCliente)])]
        const clientes = await db.knex('v_cliente').select('*').whereIn('id', idClientes)

        const gravacoes = await db.knex('cdr_gravacao').select('*').whereIn('uniqueid', resultProtocolos)
        const tagsAtendimento = await db.knex('tags_atendimentos').select('*').whereIn('id_atendimento', resultProtocolos)
        const tags = await db.knex('tags').select('*').whereIn('id', tagsAtendimento.map(tagAt => tagAt.id_tag))
        tagsAtendimento.forEach(tagAt => { tagAt.tags = tags.filter(tag => tag.id === tagAt.id_tag) })

        result.forEach(row => {
            const gravacao = gravacoes.find(grav => grav.uniqueid === row.uniqueid)
            const tagsId = tagsAtendimento.filter(ta => ta.id_atendimento === row.uniqueid).map(ta => ta.id_tag)
            const atendimento = clientesAtendimento.find(cli => cli.id === row.uniqueid)
            row.gravacao = gravacao ? gravacao.link : null
            row.tags = tags.filter(tag => tagsId.includes(tag.id))
            row.vCliente = atendimento ? clientes.find(cli => cli.id === atendimento.idCliente) : null
        })

        reply({
            page,
            max,
            rows: result,
            pages: max ? Math.ceil(totalResult.total / max) : 1,
            total: totalResult.total
        });
    } catch (error) {
        console.error(error);
        reply(Boom.badData(error));
    }
};

exports.getPesquisaRegistrosExport = async (request, reply) => {
    try {
        const knex = db.knex('public.pesquisa_satisfacao_registros AS psr')

        // Filtros
        if (request.query.dataini && !request.query.horaini) knex.where('psr.data_registro', '>=', moment(request.query.dataini, 'DD/MM/YYYY').format('YYYY/MM/DD') + ' 00:00:00');
        if (request.query.datafim && !request.query.horafim) knex.where('psr.data_registro', '<=', moment(request.query.datafim, 'DD/MM/YYYY').format('YYYY/MM/DD') + ' 23:59:59');
        if (request.query.dataini && request.query.horaini) knex.where('psr.data_registro', '>=', moment(request.query.dataini, 'DD/MM/YYYY').format('YYYY/MM/DD') + ` ` + request.query.horaini);
        if (request.query.datafim && request.query.horafim) knex.where('psr.data_registro', '<=', moment(request.query.datafim, 'DD/MM/YYYY').format('YYYY/MM/DD') + ` ` + request.query.horafim);
        if (request.query.fila) knex.whereIn('psr.fila', request.query.fila);
        if (request.query.agente) knex.whereIn('psr.agente', request.query.agente);
        if (request.query.pesquisa) knex.whereIn('psr.id_pesquisa', request.query.pesquisa);
        if (request.query.resposta) knex.whereIn('psr.nota', request.query.resposta);
        if (request.query.protocolo) {
            query.where('psr.callid', 'like', `%${request.query.protocolo}%`);
        }
        if (request.query.tags_id && request.query.tags_id.length > 0) {
            const tagsSubquery = db.knex('tags_atendimentos')
                .select('id_atendimento')
                .whereIn('id_tag', request.query.tags_id)
                .groupBy('id_atendimento')
            knex.whereIn('psr.uniqueid', tagsSubquery);
        }
        if (request.query.tipo || (request.query.camposAdicionais && request.query.camposAdicionais.length > 0)) {
            const queryCiente = db.knex('cliente').select('numero')
            if (request.query.tipo) {
                const isBusiness = request.query.tipo === 'CNPJ';
                queryCiente.where('isBusiness', isBusiness)
            }
            if (request.query.camposAdicionais && request.query.camposAdicionais.length > 0) {
                const camposAdicionais = request.query.camposAdicionais
                const idsClientes = db.knex('clienteCamposAdicionais')
                    .select('clienteID')
                    .whereIn('nome', camposAdicionais.map(campo => campo.nome))
                    .whereIn('valor', camposAdicionais.map(campo => campo.valor))
                    .groupBy('clienteID')
                queryCiente.whereIn('id', idsClientes)
            }
            knex.whereIn('psr.src', queryCiente)
        }

        // Executar consulta
        const result = await knex.clone()
        const resultProtocolos = result.map(row => row.uniqueid)
        const protOnlyNumb = resultProtocolos.filter(id => /^[0-9]+$/.test(id)).map(p => parseInt(p));

        const clientesIdChats = await db.knex('queueChat').select('idCliente', 'id').whereIn('id', protOnlyNumb)
        const clienteIdCall = await db.knex('queue_calls').select('idCliente', 'callid as id').whereIn('callid', resultProtocolos)
        const clientesAtendimento = [...clientesIdChats, ...clienteIdCall]
        const idClientes = [...new Set([...clientesIdChats.map(c => c.idCliente), ...clienteIdCall.map(c => c.idCliente)])]

        let clientes
        if (request.query.camposAdicionais) clientes = await db.knex('v_cliente').select('id', 'isBusiness', 'CamposAdicionais').whereIn('id', idClientes)
        else clientes = await db.knex('cliente').select('id', 'isBusiness').whereIn('id', idClientes)

        const tagsAtendimento = await db.knex('tags_atendimentos').select('*').whereIn('id_atendimento', resultProtocolos)
        const tags = await db.knex('tags').select('*').whereIn('id', tagsAtendimento.map(tagAt => tagAt.id_tag))
        tagsAtendimento.forEach(tagAt => { tagAt.tags = tags.filter(tag => tag.id === tagAt.id_tag) })

        result.forEach(row => {
            const tagsId = tagsAtendimento.filter(ta => ta.id_atendimento === row.uniqueid).map(ta => ta.id_tag)
            const atendimento = clientesAtendimento.find(cli => cli.id === row.uniqueid)
            row.tags = tags.filter(tag => tagsId.includes(tag.id))
            row.vCliente = atendimento ? clientes.find(cli => cli.id === atendimento.idCliente) : null
        })

        reply({
            rows: result,
        });
    } catch (error) {
        console.error(error);
        reply(Boom.badData(error));
    }
};

exports.getPesquisaRegistrosQuantidades = async (request, reply) => {
    try {
        const knex = db.knex('public.pesquisa_satisfacao_registros AS psr');

        // Filtros
        if (request.query.dataini && !request.query.horaini) knex.where('psr.data_registro', '>=', moment(request.query.dataini, 'DD/MM/YYYY').format('YYYY/MM/DD') + ' 00:00:00');
        if (request.query.datafim && !request.query.horafim) knex.where('psr.data_registro', '<=', moment(request.query.datafim, 'DD/MM/YYYY').format('YYYY/MM/DD') + ' 23:59:59');
        if (request.query.dataini && request.query.horaini) knex.where('psr.data_registro', '>=', moment(request.query.dataini, 'DD/MM/YYYY').format('YYYY/MM/DD') + ` ` + request.query.horaini);
        if (request.query.datafim && request.query.horafim) knex.where('psr.data_registro', '<=', moment(request.query.datafim, 'DD/MM/YYYY').format('YYYY/MM/DD') + ` ` + request.query.horafim);
        if (request.query.fila) knex.whereIn('psr.fila', request.query.fila);
        if (request.query.agente) knex.whereIn('psr.agente', request.query.agente);
        if (request.query.pesquisa) knex.whereIn('psr.id_pesquisa', request.query.pesquisa);
        if (request.query.resposta) knex.whereIn('psr.nota', request.query.resposta);
        if (request.query.protocolo) {
            query.where('psr.callid', 'like', `%${request.query.protocolo}%`);
        }
        if (request.query.tags_id && request.query.tags_id.length > 0) {
            const tagsSubquery = db.knex('tags_atendimentos')
                .select('id_atendimento')
                .whereIn('id_tag', request.query.tags_id)
                .groupBy('id_atendimento');
            knex.whereIn('psr.uniqueid', tagsSubquery);
        }
        if (request.query.tipo || request.query.camposAdicionais) {
            const queryCiente = db.knex('cliente').select('numero');
            if (request.query.tipo) {
                const isBusiness = request.query.tipo === 'CNPJ';
                queryCiente.where('isBusiness', isBusiness);
            }
            if (request.query.camposAdicionais && request.query.camposAdicionais.length > 0) {
                const camposAdicionais = request.query.camposAdicionais;
                const idsClientes = db.knex('clienteCamposAdicionais')
                    .select('clienteID')
                    .whereIn('nome', camposAdicionais.map(campo => campo.nome))
                    .whereIn('valor', camposAdicionais.map(campo => campo.valor))
                    .groupBy('clienteID')
                    .havingRaw(`count(distinct nome) = ${camposAdicionais.length}`);
                queryCiente.whereIn('id', idsClientes);
            }
            knex.whereIn('psr.src', queryCiente);
        }

        // Agora, contagem de respostas por nota
        const respostaCounts = await knex
            .select('nota')
            .count('* as count')
            .groupBy('nota');


        const total = respostaCounts.reduce((acc, res) => acc + parseInt(res.count), 0);
        const totalValidas = respostaCounts.reduce((acc, res) => {
            if (parseInt(res.nota) >= 0 && parseInt(res.nota) <= 10) {
                acc.total += parseInt(res.count);
            }
            return acc;
        }, { total: 0 });

        let results = [
            { nps: '0', count: 0, percentual: 0 },
            { nps: '1', count: 0, percentual: 0 },
            { nps: '2', count: 0, percentual: 0 },
            { nps: '3', count: 0, percentual: 0 },
            { nps: '4', count: 0, percentual: 0 },
            { nps: '5', count: 0, percentual: 0 },
            { nps: '6', count: 0, percentual: 0 },
            { nps: '7', count: 0, percentual: 0 },
            { nps: '8', count: 0, percentual: 0 },
            { nps: '9', count: 0, percentual: 0 },
            { nps: '10', count: 0, percentual: 0 },
            { nps: 'Fora', count: 0, percentual: null },
            { nps: 'Total', count: total, percentual: null } // Adiciona o total
        ];

        respostaCounts.forEach(res => {
            if (!res.nota || res.nota.replace(' ', '') === '') res.nota = 'Fora'; // Se a nota for nula ou vazia, considera como 'Fora'
            const notaString = res.nota.toString();
            const count = parseInt(res.count);

            results[12].count += count; // Soma total

            for (let i = 0; i <= 10; i++) {
                if (i === 0) {
                    const regex = new RegExp(`\\b${i}(?!0)\\b`); // Para evitar "10" ao contar "0"
                    if (regex.test(notaString)) {
                        results[i].count += count;
                        results[i].percentual = parseFloat(((results[i].count / totalValidas.total) * 100).toFixed(2));
                    }
                } else {
                    const regex = new RegExp(`\\b${i}\\b`);
                    if (regex.test(notaString)) {
                        results[i].count += count;
                        results[i].percentual = parseFloat(((results[i].count / totalValidas.total) * 100).toFixed(2));
                    }
                }
            }

            const nota = parseInt(notaString);
            if (isNaN(nota) || nota < 0 || nota > 10) {
                results[11].count += count; // Soma todas as notas 'Fora'
            }
        });

        reply(results);
    } catch (error) {
        console.error(error);
        reply(Boom.badData(error));
    }
};

exports.getPesquisaRegistrosQuantidadesPeriodo = async (request, reply) => {
    try {
        const knex = db.knex('public.pesquisa_satisfacao_registros AS psr');
        const period = request.query.periodo; // dia, dia_semana, mes, ano

        if (!['dia', 'dia_semana', 'mes', 'ano'].includes(period)) {
            return reply(Boom.badRequest('Parâmetro de período inválido.'));
        }

        // Filtros
        if (request.query.dataini && !request.query.horaini) knex.where('psr.data_registro', '>=', moment(request.query.dataini, 'DD/MM/YYYY').format('YYYY/MM/DD') + ' 00:00:00');
        if (request.query.datafim && !request.query.horafim) knex.where('psr.data_registro', '<=', moment(request.query.datafim, 'DD/MM/YYYY').format('YYYY/MM/DD') + ' 23:59:59');
        if (request.query.dataini && request.query.horaini) knex.where('psr.data_registro', '>=', moment(request.query.dataini, 'DD/MM/YYYY').format('YYYY/MM/DD') + ` ` + request.query.horaini);
        if (request.query.datafim && request.query.horafim) knex.where('psr.data_registro', '<=', moment(request.query.datafim, 'DD/MM/YYYY').format('YYYY/MM/DD') + ` ` + request.query.horafim);
        if (request.query.fila) knex.whereIn('psr.fila', request.query.fila);
        if (request.query.agente) knex.whereIn('psr.agente', request.query.agente);
        if (request.query.pesquisa) knex.whereIn('psr.id_pesquisa', request.query.pesquisa);
        if (request.query.resposta) knex.whereIn('psr.nota', request.query.resposta);
        if (request.query.protocolo) {
            query.where('psr.callid', 'like', `%${request.query.protocolo}%`);
        }
        if (request.query.tags_id && request.query.tags_id.length > 0) {
            const tagsSubquery = db.knex('tags_atendimentos')
                .select('id_atendimento')
                .whereIn('id_tag', request.query.tags_id)
                .groupBy('id_atendimento');
            knex.whereIn('psr.uniqueid', tagsSubquery);
        }
        if (request.query.tipo || request.query.camposAdicionais) {
            const queryCiente = db.knex('cliente').select('numero');
            if (request.query.tipo) {
                const isBusiness = request.query.tipo === 'CNPJ';
                queryCiente.where('isBusiness', isBusiness);
            }
            if (request.query.camposAdicionais && request.query.camposAdicionais.length > 0) {
                const camposAdicionais = request.query.camposAdicionais;
                const idsClientes = db.knex('clienteCamposAdicionais')
                    .select('clienteID')
                    .whereIn('nome', camposAdicionais.map(campo => campo.nome))
                    .whereIn('valor', camposAdicionais.map(campo => campo.valor))
                    .groupBy('clienteID')
                    .havingRaw(`count(distinct nome) = ${camposAdicionais.length}`);
                queryCiente.whereIn('id', idsClientes);
            }
            knex.whereIn('psr.src', queryCiente);
        }

        let periodFormat;
        switch (period) {
            case 'dia':
                periodFormat = "to_char(psr.data_registro, 'DD')";
                break;
            case 'dia_semana':
                periodFormat = "to_char(psr.data_registro, 'ID')"; // ID: Dia da semana (1-7, 1=Segunda)
                break;
            case 'mes':
                periodFormat = "to_char(psr.data_registro, 'MM')";
                break;
            case 'ano':
                periodFormat = "to_char(psr.data_registro, 'YYYY')";
                break;
        }

        const respostaCounts = await knex
            .select(db.knex.raw(`${periodFormat} as periodo`), 'nota')
            .count('* as count')
            .groupByRaw(`${periodFormat}, nota`);

        let groupedResults = {};

        respostaCounts.forEach(res => {
            if (!res.nota || res.nota.replace(' ', '') === '') res.nota = 'Fora'; // Se a nota for nula ou vazia, considera como 'Fora'
            const periodo = res.periodo;
            const notaString = res.nota.toString();
            const count = parseInt(res.count);

            if (!groupedResults[periodo]) {
                groupedResults[periodo] = [
                    { nps: '0', count: 0, percentual: 0 },
                    { nps: '1', count: 0, percentual: 0 },
                    { nps: '2', count: 0, percentual: 0 },
                    { nps: '3', count: 0, percentual: 0 },
                    { nps: '4', count: 0, percentual: 0 },
                    { nps: '5', count: 0, percentual: 0 },
                    { nps: '6', count: 0, percentual: 0 },
                    { nps: '7', count: 0, percentual: 0 },
                    { nps: '8', count: 0, percentual: 0 },
                    { nps: '9', count: 0, percentual: 0 },
                    { nps: '10', count: 0, percentual: 0 },
                    { nps: 'Fora', count: 0, percentual: null },
                    { nps: 'Total', count: 0, percentual: null } // Adiciona o total
                ];
            }

            groupedResults[periodo][12].count += count; // Soma total

            for (let i = 0; i <= 10; i++) {
                if (i === 0) {
                    const regex = new RegExp(`\\b${i}(?!0)\\b`); // Para evitar "10" ao contar "0"
                    if (regex.test(notaString)) {
                        groupedResults[periodo][i].count += count;
                        groupedResults[periodo][i].percentual = parseFloat(((groupedResults[periodo][i].count / groupedResults[periodo][12].count) * 100).toFixed(2));
                    }
                } else {
                    const regex = new RegExp(`\\b${i}\\b`);
                    if (regex.test(notaString)) {
                        groupedResults[periodo][i].count += count;
                        groupedResults[periodo][i].percentual = parseFloat(((groupedResults[periodo][i].count / groupedResults[periodo][12].count) * 100).toFixed(2));
                    }
                }
            }

            const nota = parseInt(notaString);
            if (isNaN(nota) || nota < 0 || nota > 10) {
                groupedResults[periodo][11].count += count; // Soma todas as notas 'Fora'
            }
        });

        reply(groupedResults);
    } catch (error) {
        console.error(error);
        reply(Boom.badData(error));
    }
};