'use strict'

const Joi = require('joi');
const handlers = require('./handlers');
const CRUD = require('../../../util/CRUD');
const util = require('../../../util/restbase');

module.exports = [
  {
    method: 'GET',
    path: '/api/v4/pesquisas/registros',
    handler: handlers.getPesquisaRegistros,
    config: {
      description: 'Busca os totais de registros para cada nota de pesquisa de satisfação',
      tags: ['api', 'Pesquisa de Satisfação'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          max: Joi.number().max(100).default(15).description('Máximo de registros, max=-1 traz todos os registros.').integer(),
          page: Joi.number().integer().default(1).description('Página'),
          order: Joi.array().items(
            Joi.object().keys({
              campo: Joi.string().description('Nome da coluna'),
              ordem: Joi.string().valid('ASC', 'DESC').description('ASC ou DESC')
            })
          ).description('Ordenação, informe um array de objetos com um ou mais campos para ordenar. O objeto deve ser no formato {"campo":"nome_coluna","ordem":"ASC ou DESC"}'),
          dataini: Joi.string().description('Filtra por data (início do período)'),
          datafim: Joi.string().description('Filtra por data (fim do período)'),
          horaini: Joi.string().description(`Filtra por hora inicial`).optional().allow(null).allow(``),
          horafim: Joi.string().description(`Filtra por hora final`).optional().allow(null).allow(``),
          fila: Joi.array().items(Joi.string()).optional().allow(null).description('Filtra por fila'),
          agente: Joi.array().items(Joi.string()).optional().allow(null).description('Filtra por agente'),
          pesquisa: Joi.array().items(Joi.number().integer()).optional().allow(null).description('Filtra por pesquisa'),
          resposta: Joi.array().items(Joi.number().integer()).optional().allow(null).description('Filtra respostas de pesquisas'),
          protocolo: Joi.string().description('Filtro por protocolo'),
          tags_id: Joi.array().items(Joi.number()).example([1, 2]).description('Filtra por um ou mais id de tags'),
          camposAdicionais: Joi.array().items(
            Joi.object().keys({
              nome: Joi.string().description('Nome do campo'),
              valor: Joi.string().description('Valor do campo')
            })
          ).description('Campos adicionais no formato {nome: "nome_campo", valor: "valor_campo"}'),
          tipo: Joi.string().valid('CPF', 'CNPJ').description('cliente é CPF ou CNPJ')
        }
      }
    }
  },

  {
    method: 'GET',
    path: '/api/v4/pesquisas/registrosExport',
    handler: handlers.getPesquisaRegistrosExport,
    config: {
      description: 'Busca os registros aplicando os filtros para exportação',
      tags: ['api', 'Pesquisa de Satisfação'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          dataini: Joi.string().description('Filtra por data (início do período)').required(),
          datafim: Joi.string().description('Filtra por data (fim do período)').required(),
          horaini: Joi.string().description(`Filtra por hora inicial`).optional().allow(null).allow(``),
          horafim: Joi.string().description(`Filtra por hora final`).optional().allow(null).allow(``),
          fila: Joi.array().items(Joi.string()).optional().allow(null).description('Filtra por fila'),
          agente: Joi.array().items(Joi.string()).optional().allow(null).description('Filtra por agente'),
          pesquisa: Joi.array().items(Joi.number().integer()).optional().allow(null).description('Filtra por pesquisa'),
          resposta: Joi.array().items(Joi.number().integer()).optional().allow(null).description('Filtra respostas de pesquisas'),
          protocolo: Joi.string().description('Filtro por protocolo'),
          tags_id: Joi.array().items(Joi.number()).example([1, 2]).description('Filtra por um ou mais id de tags'),
          camposAdicionais: Joi.array().items(
            Joi.object().keys({
              nome: Joi.string().description('Nome do campo'),
              valor: Joi.string().description('Valor do campo')
            })
          ).description('Campos adicionais no formato {nome: "nome_campo", valor: "valor_campo"}'),
          tipo: Joi.string().valid('CPF', 'CNPJ').description('cliente é CPF ou CNPJ')
        }
      }
    }
  },

  {
    method: 'GET',
    path: '/api/v4/pesquisas/registros/totais',
    handler: handlers.getPesquisaRegistrosQuantidades,
    config: {
      description: 'Busca os totais de registros para cada nota de pesquisa de satisfação',
      tags: ['api', 'Pesquisa de Satisfação'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          dataini: Joi.string().description('Filtra por data (início do período)'),
          datafim: Joi.string().description('Filtra por data (fim do período)'),
          horaini: Joi.string().description(`Filtra por hora inicial`).optional().allow(null).allow(``),
          horafim: Joi.string().description(`Filtra por hora final`).optional().allow(null).allow(``),
          fila: Joi.array().items(Joi.string()).optional().allow(null).description('Filtra por fila'),
          agente: Joi.array().items(Joi.string()).optional().allow(null).description('Filtra por agente'),
          pesquisa: Joi.array().items(Joi.number().integer()).optional().allow(null).description('Filtra por pesquisa'),
          resposta: Joi.array().items(Joi.number().integer()).optional().allow(null).description('Filtra respostas de pesquisas'),
          protocolo: Joi.string().description('Filtro por protocolo'),
          tags_id: Joi.array().items(Joi.number()).example([1, 2]).description('Filtra por um ou mais id de tags'),
          camposAdicionais: Joi.array().items(
            Joi.object().keys({
              nome: Joi.string().description('Nome do campo'),
              valor: Joi.string().description('Valor do campo')
            })
          ).description('Campos adicionais no formato {nome: "nome_campo", valor: "valor_campo"}'),
          tipo: Joi.string().valid('CPF', 'CNPJ').description('cliente é CPF ou CNPJ')
        }
      }
    }
  },

  {
    method: 'GET',
    path: '/api/v4/pesquisas/registros/totaisPeriodo',
    handler: handlers.getPesquisaRegistrosQuantidadesPeriodo,
    config: {
      description: 'Busca os totais de registros para cada nota de pesquisa de satisfação',
      tags: ['api', 'Pesquisa de Satisfação'],
      auth: {
        strategy: 'bearer',
        scope: 'user' // or [ 'user', 'admin' ]
      },
      validate: {
        query: {
          dataini: Joi.string().description('Filtra por data (início do período)'),
          datafim: Joi.string().description('Filtra por data (fim do período)'),
          horaini: Joi.string().description(`Filtra por hora inicial`).optional().allow(null).allow(``),
          horafim: Joi.string().description(`Filtra por hora final`).optional().allow(null).allow(``),
          periodo: Joi.string().valid('dia', 'dia_semana', 'mes', 'ano').description('Filtra por período'),
          fila: Joi.array().items(Joi.string()).optional().allow(null).description('Filtra por fila'),
          agente: Joi.array().items(Joi.string()).optional().allow(null).description('Filtra por agente'),
          pesquisa: Joi.array().items(Joi.number().integer()).optional().allow(null).description('Filtra por pesquisa'),
          resposta: Joi.array().items(Joi.number().integer()).optional().allow(null).description('Filtra respostas de pesquisas'),
          protocolo: Joi.string().description('Filtro por protocolo'),
          tags_id: Joi.array().items(Joi.number()).example([1, 2]).description('Filtra por um ou mais id de tags'),
          camposAdicionais: Joi.array().items(
            Joi.object().keys({
              nome: Joi.string().description('Nome do campo'),
              valor: Joi.string().description('Valor do campo')
            })
          ).description('Campos adicionais no formato {nome: "nome_campo", valor: "valor_campo"}'),
          tipo: Joi.string().valid('CPF', 'CNPJ').description('cliente é CPF ou CNPJ')
        }
      }
    }
  },
];

