const Joi = require('joi');
const handlers = require('./handlers');

module.exports = [
    {
        method: 'GET',
        path: '/api/v4/rotas',
        handler: handlers.getAll,
        config: {
            description: 'Retorna as rotas cadastradas',
            tags: ['api', 'Rotas'],
            auth: {
                strategy: 'bearer',
                scope: ['admin', 'app.rotas']
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/rota/{id}',
        handler: handlers.getOne,
        config: {
            description: 'Retorna a rota cadastrada',
            tags: ['api', 'Rotas'],
            auth: {
                strategy: 'bearer',
                scope: ['admin', 'app.rotas']
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                }
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/rota_prefixo_grupo/{prefixo}/{grupo}',
        handler: handlers.getPorPrefixo,
        config: {
            description: 'Retorna as com o prefixo e grupo informado',
            tags: ['api', 'Rotas'],
            auth: {
                strategy: 'bearer',
                scope: ['admin', 'app.rotas']
            },
            validate: {
                params: {
                    prefixo: Joi.string().required(),
                    grupo: Joi.string().required()
                }
            }
        }
    },
    {
        method: 'PUT',
        path: '/api/v4/rota/{id}',
        handler: handlers.update,
        config: {
            description: 'Atualiza uma rota',
            tags: ['api', 'Rotas'],
            auth: {
                strategy: 'bearer',
                scope: ['admin', 'app.rotas']
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                },
                payload: Joi.object({
                    nome: Joi.string().optional().allow(null),
                    grupo: Joi.string().optional().allow(null),
                    exten: Joi.string().optional().allow(null),
                    priority: Joi.number().integer().optional().allow(null),
                    app: Joi.string().optional().allow(null),
                    appdata: Joi.string().optional().allow([null, ''])
                })
            }
        }
    },
    {
        method: 'PUT',
        path: '/api/v4/rotareplace/{prefixo}/{grupo}',
        handler: handlers.replace,
        config: {
            description: 'Substitui os dados de uma rota por prefixo e grupo',
            tags: ['api', 'Rotas'],
            auth: {
                strategy: 'bearer',
                scope: ['admin', 'app.rotas']
            },
            validate: {
                params: {
                    prefixo: Joi.string().required(),
                    grupo: Joi.string().required()
                },
                payload: Joi.array().items(
                    Joi.object({
                        nome: Joi.string().optional().allow(null),
                        grupo: Joi.string().required(),
                        exten: Joi.string().required(),
                        priority: Joi.number().integer().required(),
                        app: Joi.string().required(),
                        appdata: Joi.string().optional().allow([null, ''])
                    })
                ).required()
            }
        }
    },
    {
        method: 'POST',
        path: '/api/v4/rota',
        handler: handlers.create,
        config: {
            description: 'Cria uma rota',
            tags: ['api', 'Rotas'],
            auth: {
                strategy: 'bearer',
                scope: ['admin', 'app.rotas']
            },
            validate: {
                payload: Joi.object({
                    nome: Joi.string().required(),
                    grupo: Joi.string().required(),
                    exten: Joi.string().required(),
                    priority: Joi.number().integer().required(),
                    app: Joi.string().required(),
                    appdata: Joi.string().optional().allow([null, ''])
                })
            }
        }
    },
    {
        method: 'POST',
        path: '/api/v4/rota_grupo_clonar',
        handler: handlers.clone,
        config: {
            description: 'Copia todas as rotas de um grupo para outro',
            tags: ['api', 'Rotas'],
            auth: {
                strategy: 'bearer',
                scope: ['admin', 'app.rotas']
            },
            validate: {
                payload: Joi.object({
                    grupo_origem: Joi.string().required(),
                    grupo_destino: Joi.string().required(),
                })
            }
        }
    },
    {
        method: 'POST',
        path: '/api/v4/rota_clonar',
        handler: handlers.cloneRota,
        config: {
            description: 'Copia os dados de uma rota de um grupo para outro',
            tags: ['api', 'Rotas'],
            auth: {
                strategy: 'bearer',
                scope: ['admin', 'app.rotas']
            },
            validate: {
                payload: Joi.object({
                    rota: Joi.string().required(),
                    grupo_origem: Joi.string().required(),
                    grupo_destino: Joi.string().required(),
                })
            }
        }
    },
    {
        method: 'DELETE',
        path: '/api/v4/rota/{id}',
        handler: handlers.delete,
        config: {
            description: 'Remove uma rota',
            tags: ['api', 'Rotas'],
            auth: {
                strategy: 'bearer',
                scope: ['admin', 'app.rotas']
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                }
            }
        }
    },
    {
        method: 'DELETE',
        path: '/api/v4/rota_prefixo_grupo/{prefixo}/{grupo}',
        handler: handlers.deletePrefixo,
        config: {
            description: 'Remove todas as rotas de um prefixo do grupo',
            tags: ['api', 'Rotas'],
            auth: {
                strategy: 'bearer',
                scope: ['admin', 'app.rotas']
            },
            validate: {
                params: {
                    prefixo: Joi.string().required(),
                    grupo: Joi.string().required()
                }
            }
        }
    },
    {
        method: 'DELETE',
        path: '/api/v4/limpa_grupo/{grupo}',
        handler: handlers.limpaGrupo,
        config: {
            description: 'Remove todas as rotas do grupo',
            tags: ['api', 'Rotas'],
            auth: {
                strategy: 'bearer',
                scope: ['admin', 'app.rotas']
            },
            validate: {
                params: {
                    grupo: Joi.string().required()
                }
            }
        }
    },
];
