const Joi = require('joi');
const handlers = require('./handlers');

const uraCampos = Joi.object({
    nome: Joi.string().required().description('Nome da ura'),
    descricao: Joi.string().optional().allow('').allow(null).description('Descrição'),
    expire: Joi.number().integer().required().description('Tempo de expiração da sessão da URA'),
    expireMsg: Joi.string().required().description('Mensagem enviada para o cliente informando que a sessão expirou'),
});

const blocoCampos = Joi.object({
    tipo: Joi.string().valid('mensagem', 'menu', 'api', 'pergunta').required().description('Tipo do bloco, permetido: mensagem, menu, api e pergunta'),
    descricao: Joi.string().optional().allow('').allow(null).description('Descrição'),
    sequencia: Joi.number().integer().required().description('Identificação do bloco'),
    bloco: Joi.object().required().description('Bloco, JSON'),
    app: Joi.string().optional().allow('').allow(null).valid('goto', 'goto_if', 'goto_agente', 'goto_fila').description('Aplicação a ser executada, permitido: goto, goto_if, goto_agente e goto_fila'),
    appDados: Joi.string().optional().allow('').allow(null).description('Dados para App'),
    falha_goto: 'integer'
})

module.exports = [
    {
        method: 'GET',
        path: '/api/v4/ura_texto',
        handler: handlers.getAll,
        config: {
            description: 'Obtem as uras de texto cadastradas',
            tags: ['api', 'v4', 'Ura Texto'],
            auth: {
                strategy: 'bearer',
                scope: ['admin']
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/ura_texto/{ID}',
        handler: handlers.getOne,
        config: {
            description: 'Obtem os dados de uma URA',
            tags: ['api', 'v4', 'Ura Texto'],
            auth: {
                strategy: 'bearer',
                scope: ['admin']
            },
            validate: {
                params: {
                    ID: Joi.number().integer().required()
                }
            }
        }
    },
    {
        method: 'DELETE',
        path: '/api/v4/ura_texto/{ID}',
        handler: handlers.delete,
        config: {
            description: 'Apaga uma ura arquivo',
            tags: ['api', 'v4', 'Ura Texto'],
            auth: {
                strategy: 'bearer',
                scope: ['admin']
            },
            validate: {
                params: {
                    ID: Joi.number().integer().required()
                }
            }
        }
    },
    {
        method: 'POST',
        path: '/api/v4/ura_texto',
        handler: handlers.create,
        config: {
            description: 'Cria uma nova ura de texto',
            tags: ['api', 'v4', 'Ura Texto'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                payload: uraCampos
            }
        }
    },
    {
        method: 'PUT',
        path: '/api/v4/ura_texto/{ID}',
        handler: handlers.update,
        config: {
            description: 'Atualiza uma ura de texto',
            tags: ['api', 'v4', 'Ura Texto'],
            auth: {
                strategy: 'bearer',
                scope: 'user' // or [ 'user', 'admin' ]
            },
            validate: {
                payload: uraCampos,
                params: {
                    ID: Joi.number().integer().required()
                }
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/ura_texto/{uraTextoID}/blocos',
        handler: handlers.getBlocos,
        config: {
            description: 'Obtem os blocos da Ura de texto',
            tags: ['api', 'v4', 'Ura Texto'],
            auth: {
                strategy: 'bearer',
                scope: ['admin']
            },
            validate: {
                params: {
                    uraTextoID: Joi.number().integer().required()
                }
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/ura_texto/{uraTextoID}/blocos/{ID}',
        handler: handlers.getBloco,
        config: {
            description: 'Obtem os dados de um bloco da Ura de texto',
            tags: ['api', 'v4', 'Ura Texto'],
            auth: {
                strategy: 'bearer',
                scope: ['admin']
            },
            validate: {
                params: {
                    uraTextoID: Joi.number().integer().required().description('ID da Ura'),
                    ID: Joi.number().integer().required().description('ID do bloco')
                }
            }
        }
    },
    {
        method: 'PUT',
        path: '/api/v4/ura_texto/{uraTextoID}/blocos/{ID}',
        handler: handlers.upBloco,
        config: {
            description: 'Atualiza os dados um bloco da Ura de texto',
            tags: ['api', 'v4', 'Ura Texto'],
            auth: {
                strategy: 'bearer',
                scope: ['admin']
            },
            validate: {
                params: {
                    uraTextoID: Joi.number().integer().required().description('ID da Ura'),
                    ID: Joi.number().integer().required().description('ID do bloco')
                },
                payload: blocoCampos,
            }
        }
    },
    {
        method: 'DELETE',
        path: '/api/v4/ura_texto/{uraTextoID}/blocos/{ID}',
        handler: handlers.delBloco,
        config: {
            description: 'Apaga um bloco da Ura de texto',
            tags: ['api', 'v4', 'Ura Texto'],
            auth: {
                strategy: 'bearer',
                scope: ['admin']
            },
            validate: {
                params: {
                    uraTextoID: Joi.number().integer().required().description('ID da Ura'),
                    ID: Joi.number().integer().required().description('ID do bloco')
                }
            }
        }
    }
];
