'use strict'
const Joi = require('joi');
const handlers = require('./handlers');

module.exports = [
    {
        method: 'POST',
        path: '/api/v4/usuario',
        handler: handlers.create,
        config: {
            description: 'Cria um usuario',
            tags: ['api', 'v4', 'Usuários'],
            auth: {
                strategy: 'bearer',
                scope: ['admin', 'app.usuarios']
            },
            validate: {
                payload: Joi.object({
                    nome: Joi.string().required(),
                    senha: Joi.string().required(),
                    fullname: Joi.string().optional().allow([null, '']),
                    autenticacao_fonte: Joi.number().integer().optional().allow(null)
                })
            }
        }
    },
    {
        method: 'PUT',
        path: '/api/v4/usuario/{id}',
        handler: handlers.update,
        config: {
            description: 'Atualiza um usuário',
            tags: ['api', 'v4', 'Usuários'],
            auth: {
                strategy: 'bearer',
                scope: ['admin', 'app.usuarios']
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                },
                payload: Joi.object({
                    nome: Joi.string().optional(),
                    senha: Joi.string().optional(),
                    fullname: Joi.string().optional().allow([null, '']),
                    autenticacao_fonte: Joi.number().integer().optional().allow(null)
                })
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/usuario/{id}',
        handler: handlers.getOne,
        config: {
            description: 'Obtem os dados de um usuário',
            tags: ['api', 'v4', 'Usuários'],
            auth: {
                strategy: 'bearer',
                scope: ['admin', 'app.usuarios']
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                }
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/usuario/{id}/permicoes',
        handler: handlers.getPermicoes,
        config: {
            description: 'Obtem as permissões do usuário',
            tags: ['api', 'v4', 'Usuários'],
            auth: {
                strategy: 'bearer',
                scope: ['app.usuarios']
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                }
            }
        }
    },
    {
        method: 'PUT',
        path: '/api/v4/usuario/{id}/permicoes',
        handler: handlers.setPermicoes,
        config: {
            description: 'Define as permissões do usuário',
            tags: ['api', 'v4', 'Usuários'],
            auth: {
                strategy: 'bearer',
                scope: ['app.usuarios']
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                },
                payload: Joi.object({
                    usa_grupo_fila: Joi.boolean().example(false),
                    modulos: Joi.array().items(
                        Joi.object({
                            id_modulo: Joi.number().integer().required(),
                            extra_usuario: Joi.string().optional().allow([null, ''])
                        })
                    ).required(),
                    filas: Joi.array().items(Joi.string()).required(),
                    grupos_fila: Joi.array().items(Joi.string()).required()
                })
            }
        }
    },
    {
        method: 'GET',
        path: '/api/v4/usuarios',
        handler: handlers.getAll,
        config: {
            description: 'Obtem os usuários',
            tags: ['api', 'v4', 'Usuários'],
            auth: {
                strategy: 'bearer',
                scope: ['admin', 'app.usuarios']
            }
        }
    },
    {
        method: 'DELETE',
        path: '/api/v4/usuario/{id}',
        handler: handlers.delete,
        config: {
            description: 'Apaga um usuário',
            tags: ['api', 'v4', 'Usuários'],
            auth: {
                strategy: 'bearer',
                scope: ['admin', 'app.usuarios']
            },
            validate: {
                params: {
                    id: Joi.number().integer().required()
                }
            }
        }
    },
]