/**
 * Este método limpa todos os caracteres que não são dígitos de um texto
 *
 * @param {Any} input Parâmetro de entrada, podendo ser texto ou numérico
 * @returns {String} O parâmetro {@link input} após o processamento
 *
 * @since 29/05/2020
 * @author Leandro Quevedo
 **/
exports.limparTextoNumerico = function (input) {
    try {
        if (!input) input = "";
        if (typeof input !== "string") return input;

        const regex = /[^0-9]+/g;

        return input.replace(regex, "");
    } catch (e) {
        console.error(`[TEXTO-UTIL] limparTextoNumerico -> ` + e);
        console.trace(e);
        return input;
    }
};

/**
 * Este método remove todos os acentos e caracteres especiais de um texto
 *
 * @param {Any} input Parâmetro de entrada, podendo ser texto ou numérico
 * @returns {String} O parâmetro {@link input} após o processamento
 *
 * @since 29/05/2020
 * @author Leandro Quevedo
 **/
exports.limparTexto = function (input) {
    try {
        if (!input) input = "";
        if (typeof input !== "string") return input;

        const accents = 'ÀÁÂÃÄÅàáâãäåßÒÓÔÕÕÖØòóôõöøÈÉÊËèéêëðÇçÐÌÍÎÏìíîïÙÚÛÜùúûüÑñŠšŸÿýŽž';
        const accentsOut = "AAAAAAaaaaaaBOOOOOOOooooooEEEEeeeeeCcDIIIIiiiiUUUUuuuuNnSsYyyZz";

        let chars = input.trim().split("");

        for (let i = 0; i < chars.length; i++) {
            let letter = chars[i];
            let accentsIndex = accents.indexOf(letter);

            if (accentsIndex !== -1) {
                chars[i] = accentsOut[accentsIndex];
            }
        }

        let after = chars.join("");

        if (input !== after) console.warn(`[TEXTO-UTIL] limparTexto -> Antes '${input}' Depois '${after}'`)

        return after;
    } catch (e) {
        console.error(`[TEXTO-UTIL] limparTexto -> ` + e);
        console.trace(e);
        return input;
    }
};

/**
 * Esta função valida um CPF dada uma string
 *
 * @param {string} input CPF a ser validado
 * @returns {boolean} True quando o {@link input} especificado for válido
 *
 * @since 14/10/2020
 * @author Leandro Quevedo
 */
exports.validarCPF = function (input) {
    if (!input) return false;

    // Limpa a string
    let cpf = input.replace(/[^\d]+/g, '');

    // Verifica se é vazia ou não coincide com o tamanho de um CPF
    if (cpf == '' || cpf.length != 11) return false;

    // Elimina CPFs invalidos conhecidos
    let falsosPositivos = ["00000000000", "11111111111", "22222222222", "33333333333", "44444444444", "55555555555", "66666666666", "77777777777", "88888888888", "99999999999"];
    if (falsosPositivos.indexOf(cpf) !== -1) return false;

    let soma = 0;
    let resto;

    for (let i = 1; i <= 9; i++) soma = soma + parseInt(cpf.substring(i - 1, i)) * (11 - i);
    resto = (soma * 10) % 11;

    if ((resto == 10) || (resto == 11)) resto = 0;

    if (resto != parseInt(cpf.substring(9, 10))) return false;
    soma = 0;

    for (let i = 1; i <= 10; i++) soma = soma + parseInt(cpf.substring(i - 1, i)) * (12 - i);
    resto = (soma * 10) % 11;

    if ((resto == 10) || (resto == 11)) resto = 0;

    if (resto != parseInt(cpf.substring(10, 11))) return false;
    return true;
};

/**
 * Esta função valida um CNPJ dada uma string
 *
 * @param {string} input CNPJ a ser validado
 * @returns {boolean} True quando o {@link input} especificado for válido
 *
 * @since 13/10/2020
 * @author Leandro Quevedo
 */
exports.validarCNPJ = function (input) {
    if (!input) return false;

    // Limpa a string
    let cnpj = input.replace(/[^\d]+/g, '');

    // Verifica se é vazia ou não coincide com o tamanho de um CNPJ
    if (cnpj == '' || cnpj.length != 14) return false;

    // Elimina CNPJs invalidos conhecidos
    let falsosPositivos = ["00000000000000", "11111111111111", "22222222222222", "33333333333333", "44444444444444", "55555555555555", "66666666666666", "77777777777777", "88888888888888", "99999999999999"];
    if (falsosPositivos.indexOf(cnpj) !== -1) return false;

    // Valida DVs
    tamanho = cnpj.length - 2;
    numeros = cnpj.substring(0, tamanho);
    digitos = cnpj.substring(tamanho);
    soma = 0;
    pos = tamanho - 7;
    for (i = tamanho; i >= 1; i--) {
        soma += numeros.charAt(tamanho - i) * pos--;
        if (pos < 2) pos = 9;
    }
    resultado = soma % 11 < 2 ? 0 : 11 - soma % 11;
    if (resultado != digitos.charAt(0)) return false;

    tamanho = tamanho + 1;
    numeros = cnpj.substring(0, tamanho);
    soma = 0;
    pos = tamanho - 7;
    for (i = tamanho; i >= 1; i--) {
        soma += numeros.charAt(tamanho - i) * pos--;
        if (pos < 2) pos = 9;
    }
    resultado = soma % 11 < 2 ? 0 : 11 - soma % 11;
    if (resultado != digitos.charAt(1)) return false;

    return true;
};