/*!
Classe utilizada para gerar os eventos de Chat.
Como o objetivo é integrar todos os serviços do Xcontact essa classe apenas monta os eventos para serem tratados pela AMI como se fossem eventos do Asterisk.
*/

const config = require('../../config');
const moment = require('moment-timezone');
moment.tz.setDefault(config.servidor.timezone);
const opts = {
    logFilePath:'/var/log/xcontact.log',
    timestampFormat:'DD-MM-YYYY HH:mm:ss.SSS'
};
const log = require('simple-node-logger').createSimpleLogger( opts );
const log_level = config.log ? config.log : 'error';
log.setLevel(log_level);
log.debug('CHATSTART: LOG_LEVEL = '+log_level);
log.info('CHATSTART: Modulo de chat Iniciado.');  
const EventEmitter = require('events');
var util = require('util');
const Message = require('./Message');
const Cliente = require('./Cliente');
const Ami = require('../../ami');

module.exports = class ChatEvents extends EventEmitter {
    constructor() {
        super();
    }
    _GeraEvento(Evento) {
        try {
            this.emit('ChatEvento', Evento);
            log.debug('CAHT_EVENTO: Emiti evento: '+ util.inspect(Evento));
        }catch (ex){
            log.error('CAHT_EVENTO: Erro ao tratar evento '+Evento.event+', motivo: '+ ex.toString());
        }
    }
    /**
     * Gera o evento de login do agente uma fila
     * @param {Agente} agente 
     * @param {string} filanome 
     */
    LoginEvent(agente, filanome) {
        const Evento = {
            event: 'QueueMemberAdded',
            queue: filanome,
            membername: agente.nome,
            interface: agente.ramal,
            membership: 'Chat',
            callstaken: 0,
            lastcall: 0,
            lastpause: 0,
            incall: 0,
            status: agente.Status,
            paused: agente.Status == 10 ? 1 : 0,
            pausedreason: agente.pausa_motivo,
            ringinuse: 0
        };
        this._GeraEvento(Evento);
    }
    /**
     * Gera o evento de Logoff do agente de uma fila
     * @param {Agente} agente 
     * @param {string} filanome 
     */
    LogoffEvent(agente, filanome) {
        const Evento = {
            event: 'QueueMemberRemoved',
            queue: filanome,
            membername: agente.nome,
            interface: agente.ramal,
            membership: 'Chat',
            callstaken: 0,
            lastcall: 0,
            lastpause: 0,
            incall: 0,
            status: agente.Status,
            paused: agente.Status == 10 ? 1 : 0,
            pausedreason: agente.pausa_motivo,
            ringinuse: 0
        };
        this._GeraEvento(Evento);
    }
    /**
     * Gera o evento de pausa do agente.
     * @param {Agente} agente 
     * @param {string} filanome 
     */
    PauseEvent(agente, filanome) {
        const Evento = {
            event: 'QueueMemberPause',
            queue: filanome,
            membername: agente.nome,
            interface: agente.ramal,
            membership: 'Chat',
            callstaken: agente.CallsTaken,
            lastcall: agente.LastCall,
            lastpause: agente.LastPause,
            incall: agente.InCall,
            status: agente.Status,
            paused: agente.Status == 10 ? 1 : 0,
            pausedreason: agente.pausa_motivo,
            reason: agente.pausa_motivo,
            ringinuse: 0
        };
        this._GeraEvento(Evento);
    }
    /**
     * Gera a entrada de um cliente na fila.
     * @param {Cliente} cliente 
     * @param {string} filanome 
     */
    ClientJoinEvent(cliente, filanome) {
        const fila = Ami.getFila(filanome);
        if(!fila) return;

        const total = fila.getTotalLigacoesAtivas();

        const Evento = {
            event: 'QueueCallerJoin',
            linkedid: cliente.Uid,
            calleridnum: cliente.From,
            calleridname: agente.FromName,
            queue: filanome,
            position: total ? total + 1 : 1,
            count: total
        };
        this._GeraEvento(Evento);
    }
    /**
     * Gera o abandono de um cliente da fila.
     * @param {Cliente} cliente 
     * @param {string} filanome 
     */
    ClientAbandonEvent(cliente, filanome) {
        const Evento = {
            event: 'QueueCallerAbandon',
            linkedid: cliente.Uid,
            calleridnum: cliente.From,
            calleridname: agente.FromName,
            queue: filanome,
            originalposition: cliente.OriginalPosition,
            position: cliente.Position,
            count: cliente.total,            
            holdtime: cliente.HoldTime,
            reason: cliente.AbandonReason ? cliente.AbandonReason : 'NORMAL',
        };
        this._GeraEvento(Evento);
    }
    /**
     * Gera o evento de saída do cliente da fila para ser atendido.
     * @param {Cliente} cliente 
     * @param {string} filanome 
     */
    ClientLeaveEvent(cliente, filanome) {
        const Evento = {
            event: 'QueueCallerLeave',
            linkedid: cliente.Uid,
            calleridnum: cliente.From,
            calleridname: agente.FromName,
            queue: filanome,
            originalposition: cliente.OriginalPosition,
            position: cliente.Position,
            count: cliente.total,            
            holdtime: cliente.HoldTime,
            reason: cliente.AbandonReason ? cliente.AbandonReason : 'NORMAL',
        };
        this._GeraEvento(Evento);
    }
  }