/**
 * Esse módulo se conecta no SimpleServer de WebRTCNativo como gestor das ligações solicitadas pelos clientes do SimpleServer.
 * A ideia deste módulo é receber as solicitações de invite dos clientes do SimpleServer enfilerar essas ligações, gerar o CDR e distribuir para os agentes.
 *
 */

const io = require('socket.io-client')
const Ami = require('../../../ami/index')
const { Estrategia } = require('../../EstrategiasDistribuicao')
const { getAgenteSocketByAgenteId } = require('../src/metodos_route/login/handlers')
const { Log } = require('../../../util/LogNew')
const log = new Log('WebRTCNativo-callManager', Log.levelDebug)
const dotenv = require('dotenv');
dotenv.config();
const config = require('../../../config.json')
let host = (config.video && config.video.host) ? config.video.host : 'ws://localhost:1000'
if (process.env && process.env.VIDEO_HOST) {
    host = process.env.VIDEO_HOST
    log.debug(`Usando servidor de vídeo ${host} definido no arquivo .env`)
}
const client = io(host, { autoConnect: false })

const QueueCalls = require('../../../models/QueueCalls')
const QueueEvents = require('../../../models/QueueEvents')

const csaHandler = require('./handlers/csaHandler')

// Define se deve enviar apenas o primeiro nome do agente para o atendimento
const shortName = true

let iowebrtc;


const LigacoesFila = []
const LigacoesAtivas = []

const RouterMetodos = new Map()
RouterMetodos.set('invite', _handlerInvite)
RouterMetodos.set('notificacao', _handlerNotificacao)
RouterMetodos.set('cancel', _handlerCancel)
RouterMetodos.set('bye', _handlerBye)

function getLigacaoNaFila (filaNome) {
    // console.log('----- Ligações na Fila: ', LigacoesFila)
    const ligacoes = LigacoesFila.filter(l => l.queue === filaNome && !l.processando)
    if (ligacoes && ligacoes.length > 0) {
        return ligacoes[0]
    }
    else {
        return undefined
    }
}

function getLigacaoNaFilaPorCallid (callid) {
    return LigacoesFila.find(l => l.callid == callid)
}

function RemoveLigacaoFila (call) {
    const idx = LigacoesFila.findIndex(l => l.callid == call.callid)
    // console.log(`LigacaoAtendida indice = ${idx} para ligacao ${call.callid}`)
    if (idx !== -1) {
        // console.log(`Achou indice = ${idx} para ligacao ${call.callid}, vai remover da fila de aguardando.`)
        return LigacoesFila.splice(idx, 1)
    } else {
        return null
    }
}

// Molhorar esses metodos, metodo que remove ligação deve apenas remover e retornar oque removeu, dai quem chamou trata a questão do agente
function RemoveLigacaoAtiva (call) {
    const idx = LigacoesAtivas.findIndex(l => l.callid == call.callid)
    // console.log(`LigacoesAtivas indice = ${idx} para ligacao ${call.callid}`)
    if (idx !== -1) {
        // console.log(`Achou indice = ${idx} para ligacao ${call.callid}, vai remover das ligacoes ativas.`)
        if(LigacoesAtivas[idx].agente) {
            const agente = LigacoesAtivas[idx].agente
            // console.log(`O agente ${agente.nome} estava na ligação ${call.callid}, liberando ele...`)
            // agente.status = 1
            if(agente.pausa && agente.motivo === 'VideoChamada') {
                Ami.QueueUnpause({ ramal: agente.ramal })
            }
        }
        return LigacoesAtivas.splice(idx, 1)
    } else {
        return null
    }
}

function cancelCallToAgente(call) {
    // Libera o agente
    // call.agente.status = 1
    console.log('Agente no cancelCallToAgente: ', call.agente)
    if(call.agente && call.agente.pausa && call.agente.motivo === 'VideoChamada') {
        Ami.QueueUnpause({ ramal: call.agente.ramal })
    }
    // Ve que socket o agente está e avisa ele
    const socket = getAgenteSocketByAgenteId(call.agente.id)
    if(socket) {
        const ev = {
            type: 'cancel',
            call: {
                callid: call.callid,
                from: call.from,
                to: call.to,
                dateIn: call.dateIn,
                queue: call.queue,
                room: call.room
            }
        }
        log.debug(`Enviando CANCEL para o agente ${call.agente.nome} da ligacao callid = ${call.callid}`)
        socket.emit('msg', ev)
    }    
}

function CancelLigacao(call) {
    clearTimeout(call.timeout)              
    const msg = `Ligação ${call.callid} cancelada com sucesso.`
    const cancelMsg = {
        type: 'canceled',
        callid: call.callid,
        room: call.room,
        msg
    }
    client.emit('msg', cancelMsg)
    if (call.agente) {        
        cancelCallToAgente(call)
    }
    log.debug(msg)
}

function LigacaoAbandono (call) {
    // Deve registrar o abandono
    const idx = LigacoesFila.findIndex(l => l.callid == call.callid)
    if (idx !== -1) {
        LigacoesFila.splice(idx, 1)
    }
    // Registra no banco
    try {
        const holdtime = Math.round((new Date().getTime() - call.dateIn)  / 1000)
        new QueueCalls({callid: call.callid, holdtime, status: 'Abandonada'}).update()
    } catch (error) {
        log.error('Falha ao atualizar ligação no banco, ERRO: ', error)
    }
}

function verificaFila (agente) {
    const call = getLigacaoNaFila(agente.fila)
    if (call) {
        log.info(`Temos ligação aguardando na fila ${agente.fila} vamos processar essa ligação`)
        call.processando = true
        call.distribuir()
    } else {
        log.info(`Não há ligações aguardando na fila ${agente.fila}`)
    }
}

Ami.eventos.on('AgenteAdicionado', (agente) => {
    try {
        log.info(`O agente ${agente.nome} entrou na fila ${agente.fila}, vamos buscar ligações aguardando nessa fila`)
        verificaFila(agente)
    } catch (error) {
        log.error('Erro ao processar evento QueueMemberAdded: ', error)
    }
})

Ami.eventos.on('AgentePausa', (agente) => {
    try {
        // So interessa se saiu de pausa
        if (agente.pausa === 1) return;

        log.info(`O agente ${agente.nome} saiu da pausa na fila ${agente.fila}, vamos buscar ligações aguardando nessa fila`)
        verificaFila(agente)
    } catch (error) {
        log.error('Erro ao processar evento QueueMemberPause: ' + error.toString())
    }
})

Ami.XCeventos.on('AgenteStatus', (agente) => {
    try {
        // So interessa se mudou para status 1
        if (agente.status !== 1) return;

        log.info(`O agente ${agente.nome} está disponível na fila ${agente.fila}, vamos buscar ligações aguardando nessa fila`)
        verificaFila(agente)
    } catch (error) {
        log.error('Erro ao processar evento QueueMemberStatus: ' + error.toString())
    }
})

Ami.XCeventos.on('AgenteDesligou', (dados) => {
    const { agente, ligacao } = dados
    try {
        // So interessa se mudou para status 1
        if (agente.status !== 1) return;

        log.info(`O agente ${agente.nome} finalizou uma ligação na fila ${agente.fila}, vamos buscar ligações aguardando nessa fila`)
        verificaFila(agente)
    } catch (error) {
        log.info('Erro ao processar evento QueueMemberStatus: ' + error.toString())
    }
})

function AddLigacao(call, fila) { 
    call.processando = true
    call.distribuir = () => {
       distribuir(call, fila) 
    }

    call.reject = async (motivo) => {
        log.info(`A ligação ${call.callid} na fila ${fila.nome} foi rejeitada pelo motivo ${motivo}`)
        // Quando uma ligação é rejeitada vou confirmar se ela ainda existe na origem, então ainda não libero ela
        call.processando = true
        try {
            log.info(`Solicitando confirmação de INVITE para callid = ${call.callid}`)
            const confirm = await csaHandler.confirmInvite(client, call)
            log.info(`A confirmação do INVITE retornou: `, confirm)
            // Aguarada 5s para tentar de novo
            setTimeout(() => {
                call.processando = true
                call.distribuir()
            }, 5 * 1000);
        } catch (error) {
            log.error('Erro ao confirmar se o INVITE ainda é válido, ERRO: ', error)
            csaHandler.desligar(client, call)
            _handlerCancel({call})
        }
    }

    // Registra no banco
    try {
        new QueueCalls({callid: call.callid, queuename: fila.nome, numero: call.from, status: 'Entrou'}).insert()
    } catch (error) {
        log.error('Falha ao registrar ligação no banco, ERRO: ', error)
    }

    call.distribuir()
    LigacoesFila.push(call)
}

function distribuir(call, fila) {
    try {
        log.info(`Iniciando distribuição da ligação ${call.callid} na fila ${fila.nome}`)
        let getAgente = Estrategia.get(fila.strategy)
        if (typeof getAgente !== 'function') {
            log.warn(`Estratégia ${fila.strategy} não implementada, usando sorteio`)
            getAgente = Estrategia.get('random')
        }
        const agente = getAgente(fila)
        if (agente) {
            // console.log('Vou chamar getAgenteSocketByAgenteId para agente: ', agente)
            const socket = getAgenteSocketByAgenteId(agente.id)
            if (socket) {
                call.agente = agente
                // Primeira coisa é colocar o agente em pausa para que ele não entre em novos sorteios nem receba ligações ou atendimentos de chat
                Ami.QueuePause({ ramal: agente.ramal, fila: undefined, motivo: 'VideoChamada'} )
                // agente.status = 6
                // Cria um timeout para que o agente não tenha todo o tempo do mundo para atender
                call.timeout = setTimeout(() => {
                    cancelCallToAgente(call)
                    // agente.status = 1
                    // Se um agente rejeitou uma ligação manda reprocessar ela para tentar outro agente.
                    call.processando = false
                    if(agente.pausa && agente.motivo === 'VideoChamada') {
                        Ami.QueueUnpause({ ramal: agente.ramal })
                    }
                    call.reject('Agente não atendeu')
                }, 30 * 1000);

                const ev = {
                    type: 'invite',
                    call: {
                        callid: call.callid,
                        from: call.from,
                        to: call.to,
                        dateIn: call.dateIn,
                        queue: call.queue,
                        room: call.room
                    }
                }
                log.info(`Enviando INVITE para o agente ${agente.nome} da ligacao callid = ${call.callid}`)
                socket.emit('msg', ev)
            } else {
                log.info(`Interessante! O Agente ${agente.nome} está disponível mas não achei o socket dele`)
                // Como esse agente está deveria estar logado na fila mas não achei o socket dele vou mudar o status dele para desconhecido
                agente.status = 0  
                // Se deu falha com esse agente reprocessa a ligação para tentar outro agente. 
                call.processando = false                 
                call.reject('Socket do agente não encontrado')
            }                
        } else {
            log.info(`Não há agentes disponiveis no momento na fila ${fila.nome} para realizar o atendimento`)
            // Se nao ha agentes na fila não tem pq ficar tentando reprocessar a ligação, espera um agente logar...
            call.processando = false
            // call.reject('Não há agentes disponiveis no momento')
        }
    } catch (error) {
        log.info('Erro ao tentar distribuir: ', error)
    }
}

client.on('connect', () => {
    log.info('Conectou no simpleServer, enviando JOIN para entrar na sala trunk')
    client.emit('join', { room: 'trunk', nome: 'Xcontact' })
    client.emit('login', { user: 'Xcontact', version: '1.5' })
})

/**
 * Processa mensagens do tipo INVITE, nova ligação
 * @param {Object} data Mensagem de sinalização recebida de clientes do CSA WebRTC
 */
function _handlerInvite(data) {
    try {
        log.info(`Processando INVITE de ${data.from} para ${data.to}, callid ${data.callid}`)
        // Roteamento direto para fila do tipo 9
        const fila = Ami.getFilas().find(f => f.nome === data.to && f.queue_type === 9)
        if (fila) {
            // cria sala no CSA para trafegar dados dessa ligação
            const room = `call_${data.callid}`
            client.emit('join', { room, nome: 'Xcontact' })
            log.debug(`Xcontact entrou na sala ${room}`)

            // Responde direto pro socket que enviou o INVITE que a ligação foi aceita e indica em que sala a partir de agora devem ser trocadas as mensagens.
            const acceptMsg = {
                type: 'accept',
                callid: data.callid,
                room: data.SocketID,
                msg: 'Queued',
                new_room: room
            }
            log.info(`Enviando para o CSA accept do INVITE de ${data.from} para ${data.to}, callid ${data.callid}`)
            client.emit('msg', acceptMsg)

            const call = {
                callid: data.callid,
                from: data.from,
                to: data.to,
                dateIn: new Date().getTime(),
                queue: fila.nome,
                room
            }            
            AddLigacao(call, fila)
        } else {
            const rejectMsg = {
                type: 'reject',
                callid: data.callid,
                room: data.SocketID,
                msg: 'Destination not found'
            }
            log.warn(`Enviando para o CSA reject do INVITE de ${data.from} para ${data.to}, callid ${data.callid}, motivo: ${rejectMsg.msg}`)
            client.emit('msg', rejectMsg)
        }        
    } catch (error) {
        log.error('Erro ao processar INVITE, ERROR: ', error)
    } 
}

function _handlerCancel(data) {
    try {
        log.info(`Processando cancel para callid ${data.call.callid}`)        
        const call = getLigacaoNaFilaPorCallid(data.call.callid)
        if (call) {
            call.processando = true
            CancelLigacao(call)
            LigacaoAbandono(call)               
        } else {
            const callAtiva = LigacoesAtivas.find(l => l.callid == data.call.callid)
            if (callAtiva) {
                log.debug(`A ligação de callid = ${call.callid} já foi para o agente ${call.agente.nome} vamos noticá-lo do cancelamento`)
                CancelLigacao(call)
                RemoveLigacaoAtiva(call)
            }
        }
    } catch (error) {
        log.error('Erro ao processar cancel, ERRO: ', error)
    }    
}

function _handlerBye(data) {
    try {
        log.info(`Processando bye para callid ${data.call.callid}`)
        // Envia o bye pro agente
        if(data.room) {
            iowebrtc.to(data.room).emit(`msg`, data)
        }
        // Ve se recebeu um bye com a ligação aguardando
        const call = getLigacaoNaFilaPorCallid(data.call.callid)
        if (call) {
            log.info('Pegou bye uma ligacao que estava na fila', call)
            call.processando = true
            CancelLigacao(call)
            LigacaoAbandono(call)               
        } else {
            log.info(`Bye nao estava na fila, vamos buscar ${data.call.callid} nas ligacoes ativas`)
            const callAtiva = LigacoesAtivas.find(l => l.callid == data.call.callid)
            if (callAtiva) {
                log.debug(`A ligação de callid = ${data.call.callid} estava ativa com o agente ${callAtiva.agente.nome} vamos removela das ativas e liberar o agente`)
                // CancelLigacao(call)
                RemoveLigacaoAtiva(callAtiva)

                // Registra no banco
                try {
                    const calltime = Math.round((new Date().getTime() - callAtiva.atendimento)  / 1000)
                    new QueueCalls({callid: callAtiva.callid, calltime, terminatedby: 2}).update()
                } catch (error) {
                    log.error('Falha ao atualizar QueueCalls no banco, ERRO: ', error)
                }

                try {
                    QueueEvents.query.where({event: 'Answer', event_data1 : callAtiva.callid, agent : callAtiva.agente.nome, queue: callAtiva.queue}).update({event_end: new Date(), event_data2 : 'Normal', event_data3 : 'Caller'})
                } catch (error) {
                    log.error('Falha ao atualizar QueueEvents no banco, ERRO: ', error)
                }
            }
        }       
    } catch (error) {
        log.error('Erro ao processar bye, ERRO: ', error)
    }    
}

function _handlerNotificacao(dados) {
    log.debug('Recebi notificacao: ', dados)
    iowebrtc.emit('notificacao', dados)
}

/**
 * Mensagem recebida do CSA
 */
client.on('msg', data => {
    // log.debug('Recebendo mensagem do CSA: ', data)
    
    if (data.type) {
        const fn = RouterMetodos.get(data.type)
        if (typeof fn === 'function') {
            fn(data)
        } else {            
            if(data.room) {
                log.info(`Mensagem do tipo ${data.type} enviando direto para a sala ${data.room}`)
                iowebrtc.to(data.room).emit(`msg`, data)                
            } else {
                log.error(`Mensagem do tipo ${data.type} desconhecida e sem room definida, ignorando...`)
            }
        }
    } else {
        log.error('Mensagem sem tipo definido')
    }
})

exports.atender = async (request) => {
    log.debug(`Agente ${request.agente.nome} solicitando atender ligação callid ${request.call.callid}`)
    const call = getLigacaoNaFilaPorCallid(request.call.callid)
    // console.log('Atender, call: ', call)
    if (call) {
        call.processando = true
        // Cancela o timeout para aguardar confirmação
        clearTimeout(call.timeout)

        // Confirma antes de iniciar o processo de atendimento
        try {
            log.info(`Solicitando confirmação de INVITE para callid = ${call.callid}`)
            const confirm = await csaHandler.confirmInvite(client, call)
            log.info(`A confirmação do INVITE retornou: `, confirm)

            // call.agente.status = 3
            call.atendimento = new Date().getTime()
            // Remove ligacao da fila
            RemoveLigacaoFila(call)
            // Add ligacao nas ativas
            LigacoesAtivas.push(call)
            // Coloca sala na mensagem que está no objeto da ligacao
            request.room = request.call.room
            const agn = request.agente.nome

            // Pega apenas primeiro nome do agente
            if(shortName){
                if (request.agente) {
                    if(request.agente.nome) {
                        const nome = request.agente.nome.split(' ');
                        if(nome.length > 1) {
                            request.agente.nome = nome[0]
                        }
                    }
                }
            }

            client.emit('msg', request)
            const msg = `Ligação ${request.call.callid} atendida pelo agente ${request.agente.nome} com sucesso.`
            log.info(msg)
            // Registra no banco
            try {
                const holdtime = Math.round((new Date().getTime() - call.dateIn)  / 1000)
                new QueueCalls({callid: call.callid, holdtime, agent: agn, status: 'Atendida'}).update()
            } catch (error) {
                log.error('Falha ao atualizar QueueCalls no banco, ERRO: ', error)
            }

            try {
                const holdtime = Math.round((new Date().getTime() - call.dateIn)  / 1000)
                new QueueEvents({event_start: new Date(), event: 'Answer', envent_group: 1, agent: agn, queue: call.queue, event_data1: call.callid, event_data4: holdtime + ''}).insert()
            } catch (error) {
                log.error('Falha ao inserir na QueueEvents, ERRO: ', error)
            }

            return {error: false, msg}
        } catch (error) {
            log.error('Erro ao confirmar se o INVITE ainda é válido, ERRO: ', error)
            csaHandler.desligar(client, call)
            _handlerCancel({call})
            return { error: true, msg: 'Erro ao confirmar se o INVITE ainda é válido' }
        }
    } else {
        const msg = `Ligação ${request.call.callid} não encontrado na lista de ligações aguardando`
        log.error(msg)
        return { error: true, msg }
    }
}

exports.rejeitar = (request) => {
    log.debug(`Agente ${request.agente.nome} solicitando REJEITAR ligação callid ${request.call.callid}`)
    const call = getLigacaoNaFilaPorCallid(request.call.callid)
    // console.log('Rejeitar, call: ', call)
    if (call) {
        const agente = Ami.getAgentePeloID(request.agente.id)
        if(agente) {
            if(agente.pausa && agente.motivo === 'VideoChamada') {
                Ami.QueueUnpause({ ramal: agente.ramal })
            }
            // agente.status = 1
        }
        // call.processando = false
        clearTimeout(call.timeout)
        call.reject('Agente REJEITOU')

        const msg = `Ligação ${request.call.callid} REJEITADA pelo agente ${request.agente.nome}`
        log.info(msg)
        return {error: false, msg}
    } else {
        const msg = `Ligação ${request.call.callid} não encontrado na lista de ligações aguardando`
        log.error(msg)
        return {error: true, msg}
    }
}

exports.desligar = (request) => {
    log.debug(`Agente ${request.agente.nome} solicitando desligar ligação callid ${request.call.callid}`)
    const agente = Ami.getAgentePeloID(request.agente.id)
    if (agente) {
        // agente.status = 1
        if(agente.pausa && agente.motivo === 'VideoChamada') {
            Ami.QueueUnpause({ ramal: agente.ramal })
        }
        request.room = request.call.room
        client.emit('msg', request)
        const msg = `Ligação ${request.call.callid} finalizada pelo agente ${request.agente.nome} com sucesso.`
        log.info(msg)
        // Registra no banco
        try {
            const callAtiva = LigacoesAtivas.find(l => l.callid == request.call.callid)
            let calltime = 0
            if (callAtiva) {
                calltime = Math.round((new Date().getTime() - callAtiva.atendimento)  / 1000)
                RemoveLigacaoAtiva(callAtiva)
            }
            new QueueCalls({callid: request.call.callid, calltime, terminatedby: 1}).update()

            // .require(false) caso nao encontre nao retorna erro
            QueueEvents.query.require(false).where({event: 'Answer', event_data1 : request.call.callid, agent : request.agente.nome}).update({event_end: new Date(), event_data2 : 'Normal', event_data3 : 'Agent'})
        } catch (error) {
            log.error('Falha ao atualizar ligação no banco, ERRO: ', error)
        }
        return {error: false, msg}
    } else {
        const msg = `Agente ID = ${request.agente.id} não encontrado`
        log.error(msg)
        return {error: true, msg}
    }
}

exports.start = (ioWebRTCNativo) => {
    const videoEnv = (process.env && process.env.VIDEO) ? process.env.VIDEO : false
    if((config.video && config.video.ativado) || videoEnv) {
        client.connect()
    } else {
        log.warn('Suporte a vídeo desativado')
    }    
    iowebrtc = ioWebRTCNativo
}

exports.client = client;

