/**
 * Classe para controle de ligações aguardando e ligações ativas.
 * Eventos emitidos:
 * - NovaLigacaoAguardando
 * - LigacaoSaiuDoAguardando
 * - NovaLigacaoAtiva
 * - LigacaoSaiuDasAtivas
 */
const { EventEmitter } = require('events')

module.exports = class Fila extends EventEmitter {
    constructor(nome) {
        super()
        this.nome = nome
        this.aguardando = []
        this.ativas = []
    }
    /**
     * Adiciona uma ligação na fila
     * @param {Object} call Objeto Ligação
     */
    addLigacaoAguardando(call) {
        this.aguardando.push(call)
        this.emit('NovaLigacaoAguardando', call)
    }
    /**
     * Chama o proximo da fila
     * Nao podemos remover as ligações da fila pois nesse momento estamos pegando o proximo da fila e vamos TENTAR entregar para um agente, então marcamos a ligação como processando = true
     * caso o agente não aceite marcamos a ligação como processando = false e ela "volta" pra fila.
     * Executando desta forma não mudamos a ordem da fila e caso a ligação que foi chamada não seja atendida será a proxima a ser chamada no proximo fluxo de processamento.
     * Exemplo: Se tivermos 5 ligações aguardando e 5 agentes logarem vamos tentar distribuir uma ligação para cada agente. Caso um dos agentes não aceite e enquanto estamos tentando distribuir
     * entra mais ligações precisamos entregar essa ligação antes das novas que entraram então remover para depois colocar de volta dá mais trabalho e pode mudar a ordem da fila.
     * @returns Ligacao
     */
    getProximoDaFila() {
        // Pega ligações aguardando que NÃO estão em processamento, o metodo filter não muda a order do array
        const ligacoes = this.aguardando.filter(l => !l.processando)
        // Se tem algo retorna a primeira desta lista
        if (ligacoes && ligacoes.length > 0) {
            const ligacao = ligacoes[0]
            // Marca ligacao como processando para evitar que outro evento pegue a mesma ligacao
            ligacao.processando = true
            return ligacao
        }
        else {
            return undefined
        }
    }
    /**
     * Faz com que uma ligação volte para a fila, ela manterá a ordem que ela estava na fila.
     * @param {*} callid 
     * @returns Retorna a ligação ou undefined caso a ligação já não esteja mais na fila
     */
    voltaPraFila(callid) {
        const call = this.aguardando.find(l => l.callid == callid)
        if (call) {
            call.processando = false
            this.emit('NovaLigacaoAguardando', call)
            return call
        } else {
            // Ligacao não está mais na fila
            return undefined
        }
    }
    /**
     * Verifica pelo callid se uma ligação está aguardando
     * @param {String} callid Callid da ligação
     * @returns Boolean
     */    
    estaAguardando(callid) {
        return this.aguardando.some(l => l.callid === callid)
    }
    /**
     * Retorna a posição que a ligação está na fila
     * @param {*} callid 
     * @returns 
     */
    posicaoNaFila(callid) {
        // if com == pois em alguns casos temos callid numericos armazenados como string
        const idx = this.aguardando.findIndex(l => l.callid == callid)
        if (idx !== -1) {
            return idx + 1
        } else {
            return undefined
        }
    }
    /**
     * Remove uma ligação da fila
     * @param {String} callid Callid da ligação
     * @returns O a ligação removida ou undefined
     */
    removeDaFila(callid) {
        // if com == pois em alguns casos temos callid numericos armazenados como string
        const idx = this.aguardando.findIndex(l => l.callid == callid)
        if (idx !== -1) {
            this.emit('LigacaoSaiuDoAguardando', call)
            return this.aguardando.splice(idx, 1)
        } else {
            return undefined
        }
    }
    /**
     * Adiciona uma ligação nas ativas
     * @param {Object} call Objeto Ligação
     */
    addLigacaoAtiva(call) {        
        this.ativas.push(call)
        this.emit('NovaLigacaoAtiva', call)
    }
    /**
     * Verifica pelo callid se uma ligação está na lista de ativas
     * @param {String} callid Callid da ligação
     * @returns Boolean
     */    
    estaAtiva(callid) {
        return this.ativas.some(l => l.callid === callid)
    }
    /**
     * Remove uma ligação da lista de ligações ativas
     * @param {String} callid Callid da ligação
     * @returns O a ligação removida ou undefined
     */
    removeDasAtivas(callid) {
        // if com == pois em alguns casos temos callid numericos armazenados como string
        const idx = this.ativas.findIndex(l => l.callid == callid)
        if (idx !== -1) {
            this.emit('LigacaoSaiuDasAtivas', call)
            return this.ativas.splice(idx, 1)
        } else {
            return undefined
        }
    }
}