/**
 * Por enquanto use a callManager.js, essa classe ira fazer parte do módulo de conexões quando ele existir.
 */

const io = require('socket.io-client')
const { EventEmitter } = require('events')

module.exports = class WebrtcNativo extends EventEmitter {
    constructor(config) {
        super()
        this._default_timeout = config.timeout || 15
        const io_config = { 
            transports: config.io && config.io.transports || ['websocket'],
            forceNew: config.io && config.io.forceNew || false,
            upgrade: config.io && config.io.upgrade || false,
            autoConnect: config.io && config.io.autoConnect || false,
            reconnectionAttempts: config.io && config.io.reconnectionAttempts || 15,
            ...config.io
        }

        console.log(io_config)

        this._conn = io(config.server, io_config)
    }

    /**
     * Inicia a conexão e cria os listeners para os eventos
     */
    async start() {
        this._status = {}
        this._conn.connect()

        this._conn.on('connect', () => {
            console.log('...:: Conectou.')
            this._status.conectado = true
            this._status.erro = false
            this._status.msg = `Conectado no servidor`
            this._status.dataStatus = new Date()
            this.emit('status', this._status)
        }) 

        this._conn.on('disconnect', (e) => {
            this._status.conectado = false
            this._status.msg = e
            this._status.dataStatus = new Date()
            this.emit('status', this._status)
        })
        
        this._conn.on('connect_timeout', (timeout) => {
            this._status.conectado = false
            this._status.msg = 'Connect timeout'
            this._status.dataStatus = new Date()
            this.emit('status', this._status)
        })

        this._conn.on('error', (error) => {
            this._status.conectado = false
            this._status.msg = error
            this._status.dataStatus = new Date()
            this.emit('status', this._status)
        })

        this._conn.on('connect_error', (error) => {
            this._status.conectado = false
            this._status.msg = error
            this._status.dataStatus = new Date()
            this.emit('status', this._status)
        })

        this._conn.on('reconnecting', (attemptNumber) => {
            this._status.conectado = false
            this._status.msg = `Tentando reconectar. Tentativa: ${attemptNumber}`
            this._status.dataStatus = new Date()            
            this.emit('status', this._status)
        })

        this._conn.on('reconnect_failed', () => {
            this._status.conectado = false
            this._status.msg = `Falha ao tentar reconectar`
            this._status.dataStatus = new Date()
            this.emit('status', this._status) 
        })

        this._conn.on('unauthorized', (err) => {
            this._status.conectado = false
            this._status.msg = err
            this._status.dataStatus = new Date() 
            this.emit('status', this._status)
        })

        this._conn.on('msg', data => {
            // console.log('Recebendo msg = ', data)
            this.emit('msg', data)

            // Esse formato coloca o socket na sala, teria que ver como funciona em multi salas pois em tese podemos ter várias chamadas simultâneas
            // if (data.type === 'INVITE') {
            //   const room = `call_${data.callid}`
            //   this._conn.emit('join', { room, nome: 'Xcontact' })
            //   // A msg de accept vai na sala public, quem enviou o INVITE deve processar esse evento e entrar na nova sala criada para essa ligacao
            //   const acceptMsg = {
            //     type: 'accept',
            //     callid: data.callid,
            //     new_room: room
            //   }
            //   this._conn.emit('msg', acceptMsg)
            // }
          })
    }

    /**
     * Retorna se está ou não conectado
     * @returns Bolean
     */
    isConnected() {
        return this._status.conectado
    }
    /**
     * Fecha a conexão e remove os listeners
     */
    async stop () {
        console.log('Fechando a conexao')
        this._conn.close()
        this.removeAllListeners()
        this._status.conectado = false
        this._status.msg = 'Conexão encerrada pelo comando stop'
        this._status.dataStatus = new Date() 
    }
    /**
     * Executa uma verificação na conexão
     * @returns Object or error
     */
    async check () {
        return new Error('Not implemented yet')
    }
    /**
     * Retorna o último status reportado pela conexão
     * @returns Object
     */
    async status () {
        return this._status
    }

    /**
     * Metodo privado que envia comandos diretos para o CSA implementando timeout
     * @param {*} metodo 
     * @param {*} dados 
     * @returns 
     */
    async _sendSync (metodo, dados) {
        if(!this._status.conectado) return new Error(`Não está conectado`)

        return new Promise((resolve, reject) => {            
            let timeout = setTimeout(() => {
                console.log(`***** CAIU NO TIMEOUT para o METODO ${metodo}`)
                    reject(new Error('Timeout, o servidor não respondeu sua chamada'))
                },
                this._default_timeout * 1000
            )
            this._conn.emit(metodo, dados, response => {
                console.log(`***** Metodo ${metodo} retornou: `, response)
                clearTimeout(timeout)
                if(response && response.error) {
                    reject(new Error(response.errorDetails))
                }
                else {
                    resolve(response || true)
                }                
            })
        })
    }

    /**
     * Metodo privado que envia comandos diretos para o CSA sem aguardar retorno
     * @param {*} metodo 
     * @param {*} dados 
     * @returns 
     */
    async _send (metodo, dados) {
        if(!this._status.conectado) return new Error(`Não está conectado`)
        this._conn.emit(metodo, dados)
    }
}